// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/trace/v3/dynamic_ot.proto

package tracev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/go-control-plane/envoy/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	structpb "google.golang.org/protobuf/types/known/structpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// DynamicOtConfig was used to dynamically load a tracer from a shared library
// that implements the `OpenTracing dynamic loading API
// <https://github.com/opentracing/opentracing-cpp>`_.
// [#not-implemented-hide:]
type DynamicOtConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Dynamic library implementing the `OpenTracing API
	// <https://github.com/opentracing/opentracing-cpp>`_.
	//
	// Deprecated: Marked as deprecated in envoy/config/trace/v3/dynamic_ot.proto.
	Library string `protobuf:"bytes,1,opt,name=library,proto3" json:"library,omitempty"`
	// The configuration to use when creating a tracer from the given dynamic
	// library.
	//
	// Deprecated: Marked as deprecated in envoy/config/trace/v3/dynamic_ot.proto.
	Config        *structpb.Struct `protobuf:"bytes,2,opt,name=config,proto3" json:"config,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DynamicOtConfig) Reset() {
	*x = DynamicOtConfig{}
	mi := &file_envoy_config_trace_v3_dynamic_ot_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DynamicOtConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DynamicOtConfig) ProtoMessage() {}

func (x *DynamicOtConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_trace_v3_dynamic_ot_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DynamicOtConfig.ProtoReflect.Descriptor instead.
func (*DynamicOtConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_trace_v3_dynamic_ot_proto_rawDescGZIP(), []int{0}
}

// Deprecated: Marked as deprecated in envoy/config/trace/v3/dynamic_ot.proto.
func (x *DynamicOtConfig) GetLibrary() string {
	if x != nil {
		return x.Library
	}
	return ""
}

// Deprecated: Marked as deprecated in envoy/config/trace/v3/dynamic_ot.proto.
func (x *DynamicOtConfig) GetConfig() *structpb.Struct {
	if x != nil {
		return x.Config
	}
	return nil
}

var File_envoy_config_trace_v3_dynamic_ot_proto protoreflect.FileDescriptor

const file_envoy_config_trace_v3_dynamic_ot_proto_rawDesc = "" +
	"\n" +
	"&envoy/config/trace/v3/dynamic_ot.proto\x12\x15envoy.config.trace.v3\x1a\x1cgoogle/protobuf/struct.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xb7\x01\n" +
	"\x0fDynamicOtConfig\x122\n" +
	"\alibrary\x18\x01 \x01(\tB\x18\xfaB\x04r\x02\x10\x01\x92ǆ\xd8\x04\x033.0\xb8\xee\xf2\xd2\x05\x01\x18\x01R\alibrary\x12B\n" +
	"\x06config\x18\x02 \x01(\v2\x17.google.protobuf.StructB\x11\x92ǆ\xd8\x04\x033.0\xb8\xee\xf2\xd2\x05\x01\x18\x01R\x06config:,\x9aň\x1e'\n" +
	"%envoy.config.trace.v2.DynamicOtConfigB\xb8\x01\xf2\x98\xfe\x8f\x05-\x12+envoy.extensions.tracers.dynamic_ot.v4alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"#io.envoyproxy.envoy.config.trace.v3B\x0eDynamicOtProtoP\x01ZDgithub.com/envoyproxy/go-control-plane/envoy/config/trace/v3;tracev3b\x06proto3"

var (
	file_envoy_config_trace_v3_dynamic_ot_proto_rawDescOnce sync.Once
	file_envoy_config_trace_v3_dynamic_ot_proto_rawDescData []byte
)

func file_envoy_config_trace_v3_dynamic_ot_proto_rawDescGZIP() []byte {
	file_envoy_config_trace_v3_dynamic_ot_proto_rawDescOnce.Do(func() {
		file_envoy_config_trace_v3_dynamic_ot_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_trace_v3_dynamic_ot_proto_rawDesc), len(file_envoy_config_trace_v3_dynamic_ot_proto_rawDesc)))
	})
	return file_envoy_config_trace_v3_dynamic_ot_proto_rawDescData
}

var file_envoy_config_trace_v3_dynamic_ot_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_config_trace_v3_dynamic_ot_proto_goTypes = []any{
	(*DynamicOtConfig)(nil), // 0: envoy.config.trace.v3.DynamicOtConfig
	(*structpb.Struct)(nil), // 1: google.protobuf.Struct
}
var file_envoy_config_trace_v3_dynamic_ot_proto_depIdxs = []int32{
	1, // 0: envoy.config.trace.v3.DynamicOtConfig.config:type_name -> google.protobuf.Struct
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_config_trace_v3_dynamic_ot_proto_init() }
func file_envoy_config_trace_v3_dynamic_ot_proto_init() {
	if File_envoy_config_trace_v3_dynamic_ot_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_trace_v3_dynamic_ot_proto_rawDesc), len(file_envoy_config_trace_v3_dynamic_ot_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_trace_v3_dynamic_ot_proto_goTypes,
		DependencyIndexes: file_envoy_config_trace_v3_dynamic_ot_proto_depIdxs,
		MessageInfos:      file_envoy_config_trace_v3_dynamic_ot_proto_msgTypes,
	}.Build()
	File_envoy_config_trace_v3_dynamic_ot_proto = out.File
	file_envoy_config_trace_v3_dynamic_ot_proto_goTypes = nil
	file_envoy_config_trace_v3_dynamic_ot_proto_depIdxs = nil
}
