// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.1
// source: xds/type/matcher/v3/domain.proto

package v3

import (
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type ServerNameMatcher struct {
	state          protoimpl.MessageState             `protogen:"open.v1"`
	DomainMatchers []*ServerNameMatcher_DomainMatcher `protobuf:"bytes,1,rep,name=domain_matchers,json=domainMatchers,proto3" json:"domain_matchers,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *ServerNameMatcher) Reset() {
	*x = ServerNameMatcher{}
	mi := &file_xds_type_matcher_v3_domain_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServerNameMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServerNameMatcher) ProtoMessage() {}

func (x *ServerNameMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_domain_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServerNameMatcher.ProtoReflect.Descriptor instead.
func (*ServerNameMatcher) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_domain_proto_rawDescGZIP(), []int{0}
}

func (x *ServerNameMatcher) GetDomainMatchers() []*ServerNameMatcher_DomainMatcher {
	if x != nil {
		return x.DomainMatchers
	}
	return nil
}

type ServerNameMatcher_DomainMatcher struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Domains       []string               `protobuf:"bytes,1,rep,name=domains,proto3" json:"domains,omitempty"`
	OnMatch       *Matcher_OnMatch       `protobuf:"bytes,2,opt,name=on_match,json=onMatch,proto3" json:"on_match,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServerNameMatcher_DomainMatcher) Reset() {
	*x = ServerNameMatcher_DomainMatcher{}
	mi := &file_xds_type_matcher_v3_domain_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServerNameMatcher_DomainMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServerNameMatcher_DomainMatcher) ProtoMessage() {}

func (x *ServerNameMatcher_DomainMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_domain_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServerNameMatcher_DomainMatcher.ProtoReflect.Descriptor instead.
func (*ServerNameMatcher_DomainMatcher) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_domain_proto_rawDescGZIP(), []int{0, 0}
}

func (x *ServerNameMatcher_DomainMatcher) GetDomains() []string {
	if x != nil {
		return x.Domains
	}
	return nil
}

func (x *ServerNameMatcher_DomainMatcher) GetOnMatch() *Matcher_OnMatch {
	if x != nil {
		return x.OnMatch
	}
	return nil
}

var File_xds_type_matcher_v3_domain_proto protoreflect.FileDescriptor

const file_xds_type_matcher_v3_domain_proto_rawDesc = "" +
	"\n" +
	" xds/type/matcher/v3/domain.proto\x12\x13xds.type.matcher.v3\x1a\x1fxds/annotations/v3/status.proto\x1a!xds/type/matcher/v3/matcher.proto\x1a\x17validate/validate.proto\"\xe8\x01\n" +
	"\x11ServerNameMatcher\x12]\n" +
	"\x0fdomain_matchers\x18\x01 \x03(\v24.xds.type.matcher.v3.ServerNameMatcher.DomainMatcherR\x0edomainMatchers\x1at\n" +
	"\rDomainMatcher\x12\"\n" +
	"\adomains\x18\x01 \x03(\tB\b\xfaB\x05\x92\x01\x02\b\x01R\adomains\x12?\n" +
	"\bon_match\x18\x02 \x01(\v2$.xds.type.matcher.v3.Matcher.OnMatchR\aonMatchBn\xd2Ƥ\xe1\x06\x02\b\x01\n" +
	"\x1ecom.github.xds.type.matcher.v3B\x16ServerNameMatcherProtoP\x01Z*github.com/cncf/xds/go/xds/type/matcher/v3b\x06proto3"

var (
	file_xds_type_matcher_v3_domain_proto_rawDescOnce sync.Once
	file_xds_type_matcher_v3_domain_proto_rawDescData []byte
)

func file_xds_type_matcher_v3_domain_proto_rawDescGZIP() []byte {
	file_xds_type_matcher_v3_domain_proto_rawDescOnce.Do(func() {
		file_xds_type_matcher_v3_domain_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_xds_type_matcher_v3_domain_proto_rawDesc), len(file_xds_type_matcher_v3_domain_proto_rawDesc)))
	})
	return file_xds_type_matcher_v3_domain_proto_rawDescData
}

var file_xds_type_matcher_v3_domain_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_xds_type_matcher_v3_domain_proto_goTypes = []any{
	(*ServerNameMatcher)(nil),               // 0: xds.type.matcher.v3.ServerNameMatcher
	(*ServerNameMatcher_DomainMatcher)(nil), // 1: xds.type.matcher.v3.ServerNameMatcher.DomainMatcher
	(*Matcher_OnMatch)(nil),                 // 2: xds.type.matcher.v3.Matcher.OnMatch
}
var file_xds_type_matcher_v3_domain_proto_depIdxs = []int32{
	1, // 0: xds.type.matcher.v3.ServerNameMatcher.domain_matchers:type_name -> xds.type.matcher.v3.ServerNameMatcher.DomainMatcher
	2, // 1: xds.type.matcher.v3.ServerNameMatcher.DomainMatcher.on_match:type_name -> xds.type.matcher.v3.Matcher.OnMatch
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_xds_type_matcher_v3_domain_proto_init() }
func file_xds_type_matcher_v3_domain_proto_init() {
	if File_xds_type_matcher_v3_domain_proto != nil {
		return
	}
	file_xds_type_matcher_v3_matcher_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_xds_type_matcher_v3_domain_proto_rawDesc), len(file_xds_type_matcher_v3_domain_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_xds_type_matcher_v3_domain_proto_goTypes,
		DependencyIndexes: file_xds_type_matcher_v3_domain_proto_depIdxs,
		MessageInfos:      file_xds_type_matcher_v3_domain_proto_msgTypes,
	}.Build()
	File_xds_type_matcher_v3_domain_proto = out.File
	file_xds_type_matcher_v3_domain_proto_goTypes = nil
	file_xds_type_matcher_v3_domain_proto_depIdxs = nil
}
