// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11-devel
// 	protoc        (unknown)
// source: common/common.proto

package common

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	status "google.golang.org/genproto/googleapis/rpc/status"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	descriptorpb "google.golang.org/protobuf/types/descriptorpb"
	anypb "google.golang.org/protobuf/types/known/anypb"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Code int32

const (
	Code_FATAL    Code = 0
	Code_LOCKED   Code = 1
	Code_CANCELED Code = 2
)

// Enum value maps for Code.
var (
	Code_name = map[int32]string{
		0: "FATAL",
		1: "LOCKED",
		2: "CANCELED",
	}
	Code_value = map[string]int32{
		"FATAL":    0,
		"LOCKED":   1,
		"CANCELED": 2,
	}
)

func (x Code) Enum() *Code {
	p := new(Code)
	*p = x
	return p
}

func (x Code) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (Code) Descriptor() protoreflect.EnumDescriptor {
	return file_common_common_proto_enumTypes[0].Descriptor()
}

func (Code) Type() protoreflect.EnumType {
	return &file_common_common_proto_enumTypes[0]
}

func (x Code) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use Code.Descriptor instead.
func (Code) EnumDescriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{0}
}

type ContainerDriver int32

const (
	ContainerDriver_CONTAINERD ContainerDriver = 0
	ContainerDriver_CRI        ContainerDriver = 1
)

// Enum value maps for ContainerDriver.
var (
	ContainerDriver_name = map[int32]string{
		0: "CONTAINERD",
		1: "CRI",
	}
	ContainerDriver_value = map[string]int32{
		"CONTAINERD": 0,
		"CRI":        1,
	}
)

func (x ContainerDriver) Enum() *ContainerDriver {
	p := new(ContainerDriver)
	*p = x
	return p
}

func (x ContainerDriver) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ContainerDriver) Descriptor() protoreflect.EnumDescriptor {
	return file_common_common_proto_enumTypes[1].Descriptor()
}

func (ContainerDriver) Type() protoreflect.EnumType {
	return &file_common_common_proto_enumTypes[1]
}

func (x ContainerDriver) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ContainerDriver.Descriptor instead.
func (ContainerDriver) EnumDescriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{1}
}

type ContainerdNamespace int32

const (
	ContainerdNamespace_NS_UNKNOWN ContainerdNamespace = 0
	ContainerdNamespace_NS_SYSTEM  ContainerdNamespace = 1
	ContainerdNamespace_NS_CRI     ContainerdNamespace = 2
)

// Enum value maps for ContainerdNamespace.
var (
	ContainerdNamespace_name = map[int32]string{
		0: "NS_UNKNOWN",
		1: "NS_SYSTEM",
		2: "NS_CRI",
	}
	ContainerdNamespace_value = map[string]int32{
		"NS_UNKNOWN": 0,
		"NS_SYSTEM":  1,
		"NS_CRI":     2,
	}
)

func (x ContainerdNamespace) Enum() *ContainerdNamespace {
	p := new(ContainerdNamespace)
	*p = x
	return p
}

func (x ContainerdNamespace) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ContainerdNamespace) Descriptor() protoreflect.EnumDescriptor {
	return file_common_common_proto_enumTypes[2].Descriptor()
}

func (ContainerdNamespace) Type() protoreflect.EnumType {
	return &file_common_common_proto_enumTypes[2]
}

func (x ContainerdNamespace) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ContainerdNamespace.Descriptor instead.
func (ContainerdNamespace) EnumDescriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{2}
}

type Error struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Code          Code                   `protobuf:"varint,1,opt,name=code,proto3,enum=common.Code" json:"code,omitempty"`
	Message       string                 `protobuf:"bytes,2,opt,name=message,proto3" json:"message,omitempty"`
	Details       []*anypb.Any           `protobuf:"bytes,3,rep,name=details,proto3" json:"details,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Error) Reset() {
	*x = Error{}
	mi := &file_common_common_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Error) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Error) ProtoMessage() {}

func (x *Error) ProtoReflect() protoreflect.Message {
	mi := &file_common_common_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Error.ProtoReflect.Descriptor instead.
func (*Error) Descriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{0}
}

func (x *Error) GetCode() Code {
	if x != nil {
		return x.Code
	}
	return Code_FATAL
}

func (x *Error) GetMessage() string {
	if x != nil {
		return x.Message
	}
	return ""
}

func (x *Error) GetDetails() []*anypb.Any {
	if x != nil {
		return x.Details
	}
	return nil
}

// Common metadata message nested in all reply message types
type Metadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// hostname of the server response comes from (injected by proxy)
	Hostname string `protobuf:"bytes,1,opt,name=hostname,proto3" json:"hostname,omitempty"`
	// error is set if request failed to the upstream (rest of response is
	// undefined)
	Error string `protobuf:"bytes,2,opt,name=error,proto3" json:"error,omitempty"`
	// error as gRPC Status
	Status        *status.Status `protobuf:"bytes,3,opt,name=status,proto3" json:"status,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Metadata) Reset() {
	*x = Metadata{}
	mi := &file_common_common_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Metadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Metadata) ProtoMessage() {}

func (x *Metadata) ProtoReflect() protoreflect.Message {
	mi := &file_common_common_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Metadata.ProtoReflect.Descriptor instead.
func (*Metadata) Descriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{1}
}

func (x *Metadata) GetHostname() string {
	if x != nil {
		return x.Hostname
	}
	return ""
}

func (x *Metadata) GetError() string {
	if x != nil {
		return x.Error
	}
	return ""
}

func (x *Metadata) GetStatus() *status.Status {
	if x != nil {
		return x.Status
	}
	return nil
}

type Data struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *Metadata              `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Bytes         []byte                 `protobuf:"bytes,2,opt,name=bytes,proto3" json:"bytes,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Data) Reset() {
	*x = Data{}
	mi := &file_common_common_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Data) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Data) ProtoMessage() {}

func (x *Data) ProtoReflect() protoreflect.Message {
	mi := &file_common_common_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Data.ProtoReflect.Descriptor instead.
func (*Data) Descriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{2}
}

func (x *Data) GetMetadata() *Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *Data) GetBytes() []byte {
	if x != nil {
		return x.Bytes
	}
	return nil
}

type DataResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Data                `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DataResponse) Reset() {
	*x = DataResponse{}
	mi := &file_common_common_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DataResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DataResponse) ProtoMessage() {}

func (x *DataResponse) ProtoReflect() protoreflect.Message {
	mi := &file_common_common_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DataResponse.ProtoReflect.Descriptor instead.
func (*DataResponse) Descriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{3}
}

func (x *DataResponse) GetMessages() []*Data {
	if x != nil {
		return x.Messages
	}
	return nil
}

type Empty struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *Metadata              `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Empty) Reset() {
	*x = Empty{}
	mi := &file_common_common_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Empty) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Empty) ProtoMessage() {}

func (x *Empty) ProtoReflect() protoreflect.Message {
	mi := &file_common_common_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Empty.ProtoReflect.Descriptor instead.
func (*Empty) Descriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{4}
}

func (x *Empty) GetMetadata() *Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

type EmptyResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Empty               `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EmptyResponse) Reset() {
	*x = EmptyResponse{}
	mi := &file_common_common_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EmptyResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EmptyResponse) ProtoMessage() {}

func (x *EmptyResponse) ProtoReflect() protoreflect.Message {
	mi := &file_common_common_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EmptyResponse.ProtoReflect.Descriptor instead.
func (*EmptyResponse) Descriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{5}
}

func (x *EmptyResponse) GetMessages() []*Empty {
	if x != nil {
		return x.Messages
	}
	return nil
}

type ContainerdInstance struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Containerd instance to use.
	Driver ContainerDriver `protobuf:"varint,1,opt,name=driver,proto3,enum=common.ContainerDriver" json:"driver,omitempty"`
	// Containerd namespace to use.
	Namespace     ContainerdNamespace `protobuf:"varint,2,opt,name=namespace,proto3,enum=common.ContainerdNamespace" json:"namespace,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ContainerdInstance) Reset() {
	*x = ContainerdInstance{}
	mi := &file_common_common_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ContainerdInstance) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ContainerdInstance) ProtoMessage() {}

func (x *ContainerdInstance) ProtoReflect() protoreflect.Message {
	mi := &file_common_common_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ContainerdInstance.ProtoReflect.Descriptor instead.
func (*ContainerdInstance) Descriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{6}
}

func (x *ContainerdInstance) GetDriver() ContainerDriver {
	if x != nil {
		return x.Driver
	}
	return ContainerDriver_CONTAINERD
}

func (x *ContainerdInstance) GetNamespace() ContainerdNamespace {
	if x != nil {
		return x.Namespace
	}
	return ContainerdNamespace_NS_UNKNOWN
}

type URL struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	FullPath      string                 `protobuf:"bytes,1,opt,name=full_path,json=fullPath,proto3" json:"full_path,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *URL) Reset() {
	*x = URL{}
	mi := &file_common_common_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *URL) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*URL) ProtoMessage() {}

func (x *URL) ProtoReflect() protoreflect.Message {
	mi := &file_common_common_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use URL.ProtoReflect.Descriptor instead.
func (*URL) Descriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{7}
}

func (x *URL) GetFullPath() string {
	if x != nil {
		return x.FullPath
	}
	return ""
}

type PEMEncodedCertificateAndKey struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Crt           []byte                 `protobuf:"bytes,1,opt,name=crt,proto3" json:"crt,omitempty"`
	Key           []byte                 `protobuf:"bytes,2,opt,name=key,proto3" json:"key,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PEMEncodedCertificateAndKey) Reset() {
	*x = PEMEncodedCertificateAndKey{}
	mi := &file_common_common_proto_msgTypes[8]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PEMEncodedCertificateAndKey) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PEMEncodedCertificateAndKey) ProtoMessage() {}

func (x *PEMEncodedCertificateAndKey) ProtoReflect() protoreflect.Message {
	mi := &file_common_common_proto_msgTypes[8]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PEMEncodedCertificateAndKey.ProtoReflect.Descriptor instead.
func (*PEMEncodedCertificateAndKey) Descriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{8}
}

func (x *PEMEncodedCertificateAndKey) GetCrt() []byte {
	if x != nil {
		return x.Crt
	}
	return nil
}

func (x *PEMEncodedCertificateAndKey) GetKey() []byte {
	if x != nil {
		return x.Key
	}
	return nil
}

type PEMEncodedKey struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Key           []byte                 `protobuf:"bytes,1,opt,name=key,proto3" json:"key,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PEMEncodedKey) Reset() {
	*x = PEMEncodedKey{}
	mi := &file_common_common_proto_msgTypes[9]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PEMEncodedKey) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PEMEncodedKey) ProtoMessage() {}

func (x *PEMEncodedKey) ProtoReflect() protoreflect.Message {
	mi := &file_common_common_proto_msgTypes[9]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PEMEncodedKey.ProtoReflect.Descriptor instead.
func (*PEMEncodedKey) Descriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{9}
}

func (x *PEMEncodedKey) GetKey() []byte {
	if x != nil {
		return x.Key
	}
	return nil
}

type PEMEncodedCertificate struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Crt           []byte                 `protobuf:"bytes,1,opt,name=crt,proto3" json:"crt,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PEMEncodedCertificate) Reset() {
	*x = PEMEncodedCertificate{}
	mi := &file_common_common_proto_msgTypes[10]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PEMEncodedCertificate) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PEMEncodedCertificate) ProtoMessage() {}

func (x *PEMEncodedCertificate) ProtoReflect() protoreflect.Message {
	mi := &file_common_common_proto_msgTypes[10]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PEMEncodedCertificate.ProtoReflect.Descriptor instead.
func (*PEMEncodedCertificate) Descriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{10}
}

func (x *PEMEncodedCertificate) GetCrt() []byte {
	if x != nil {
		return x.Crt
	}
	return nil
}

type NetIP struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Ip            []byte                 `protobuf:"bytes,1,opt,name=ip,proto3" json:"ip,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *NetIP) Reset() {
	*x = NetIP{}
	mi := &file_common_common_proto_msgTypes[11]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *NetIP) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*NetIP) ProtoMessage() {}

func (x *NetIP) ProtoReflect() protoreflect.Message {
	mi := &file_common_common_proto_msgTypes[11]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use NetIP.ProtoReflect.Descriptor instead.
func (*NetIP) Descriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{11}
}

func (x *NetIP) GetIp() []byte {
	if x != nil {
		return x.Ip
	}
	return nil
}

type NetIPPort struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Ip            []byte                 `protobuf:"bytes,1,opt,name=ip,proto3" json:"ip,omitempty"`
	Port          int32                  `protobuf:"varint,2,opt,name=port,proto3" json:"port,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *NetIPPort) Reset() {
	*x = NetIPPort{}
	mi := &file_common_common_proto_msgTypes[12]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *NetIPPort) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*NetIPPort) ProtoMessage() {}

func (x *NetIPPort) ProtoReflect() protoreflect.Message {
	mi := &file_common_common_proto_msgTypes[12]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use NetIPPort.ProtoReflect.Descriptor instead.
func (*NetIPPort) Descriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{12}
}

func (x *NetIPPort) GetIp() []byte {
	if x != nil {
		return x.Ip
	}
	return nil
}

func (x *NetIPPort) GetPort() int32 {
	if x != nil {
		return x.Port
	}
	return 0
}

type NetIPPrefix struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Ip            []byte                 `protobuf:"bytes,1,opt,name=ip,proto3" json:"ip,omitempty"`
	PrefixLength  int32                  `protobuf:"varint,2,opt,name=prefix_length,json=prefixLength,proto3" json:"prefix_length,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *NetIPPrefix) Reset() {
	*x = NetIPPrefix{}
	mi := &file_common_common_proto_msgTypes[13]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *NetIPPrefix) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*NetIPPrefix) ProtoMessage() {}

func (x *NetIPPrefix) ProtoReflect() protoreflect.Message {
	mi := &file_common_common_proto_msgTypes[13]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use NetIPPrefix.ProtoReflect.Descriptor instead.
func (*NetIPPrefix) Descriptor() ([]byte, []int) {
	return file_common_common_proto_rawDescGZIP(), []int{13}
}

func (x *NetIPPrefix) GetIp() []byte {
	if x != nil {
		return x.Ip
	}
	return nil
}

func (x *NetIPPrefix) GetPrefixLength() int32 {
	if x != nil {
		return x.PrefixLength
	}
	return 0
}

var file_common_common_proto_extTypes = []protoimpl.ExtensionInfo{
	{
		ExtendedType:  (*descriptorpb.MessageOptions)(nil),
		ExtensionType: (*string)(nil),
		Field:         93117,
		Name:          "common.remove_deprecated_message",
		Tag:           "bytes,93117,opt,name=remove_deprecated_message",
		Filename:      "common/common.proto",
	},
	{
		ExtendedType:  (*descriptorpb.FieldOptions)(nil),
		ExtensionType: (*string)(nil),
		Field:         93117,
		Name:          "common.remove_deprecated_field",
		Tag:           "bytes,93117,opt,name=remove_deprecated_field",
		Filename:      "common/common.proto",
	},
	{
		ExtendedType:  (*descriptorpb.EnumOptions)(nil),
		ExtensionType: (*string)(nil),
		Field:         93117,
		Name:          "common.remove_deprecated_enum",
		Tag:           "bytes,93117,opt,name=remove_deprecated_enum",
		Filename:      "common/common.proto",
	},
	{
		ExtendedType:  (*descriptorpb.EnumValueOptions)(nil),
		ExtensionType: (*string)(nil),
		Field:         93117,
		Name:          "common.remove_deprecated_enum_value",
		Tag:           "bytes,93117,opt,name=remove_deprecated_enum_value",
		Filename:      "common/common.proto",
	},
	{
		ExtendedType:  (*descriptorpb.MethodOptions)(nil),
		ExtensionType: (*string)(nil),
		Field:         93117,
		Name:          "common.remove_deprecated_method",
		Tag:           "bytes,93117,opt,name=remove_deprecated_method",
		Filename:      "common/common.proto",
	},
	{
		ExtendedType:  (*descriptorpb.ServiceOptions)(nil),
		ExtensionType: (*string)(nil),
		Field:         93117,
		Name:          "common.remove_deprecated_service",
		Tag:           "bytes,93117,opt,name=remove_deprecated_service",
		Filename:      "common/common.proto",
	},
}

// Extension fields to descriptorpb.MessageOptions.
var (
	// Indicates the Talos version when this deprecated message will be removed from API.
	//
	// optional string remove_deprecated_message = 93117;
	E_RemoveDeprecatedMessage = &file_common_common_proto_extTypes[0]
)

// Extension fields to descriptorpb.FieldOptions.
var (
	// Indicates the Talos version when this deprecated filed will be removed from API.
	//
	// optional string remove_deprecated_field = 93117;
	E_RemoveDeprecatedField = &file_common_common_proto_extTypes[1]
)

// Extension fields to descriptorpb.EnumOptions.
var (
	// Indicates the Talos version when this deprecated enum will be removed from API.
	//
	// optional string remove_deprecated_enum = 93117;
	E_RemoveDeprecatedEnum = &file_common_common_proto_extTypes[2]
)

// Extension fields to descriptorpb.EnumValueOptions.
var (
	// Indicates the Talos version when this deprecated enum value will be removed from API.
	//
	// optional string remove_deprecated_enum_value = 93117;
	E_RemoveDeprecatedEnumValue = &file_common_common_proto_extTypes[3]
)

// Extension fields to descriptorpb.MethodOptions.
var (
	// Indicates the Talos version when this deprecated method will be removed from API.
	//
	// optional string remove_deprecated_method = 93117;
	E_RemoveDeprecatedMethod = &file_common_common_proto_extTypes[4]
)

// Extension fields to descriptorpb.ServiceOptions.
var (
	// Indicates the Talos version when this deprecated service will be removed from API.
	//
	// optional string remove_deprecated_service = 93117;
	E_RemoveDeprecatedService = &file_common_common_proto_extTypes[5]
)

var File_common_common_proto protoreflect.FileDescriptor

const file_common_common_proto_rawDesc = "" +
	"\n" +
	"\x13common/common.proto\x12\x06common\x1a\x19google/protobuf/any.proto\x1a google/protobuf/descriptor.proto\x1a\x17google/rpc/status.proto\"s\n" +
	"\x05Error\x12 \n" +
	"\x04code\x18\x01 \x01(\x0e2\f.common.CodeR\x04code\x12\x18\n" +
	"\amessage\x18\x02 \x01(\tR\amessage\x12.\n" +
	"\adetails\x18\x03 \x03(\v2\x14.google.protobuf.AnyR\adetails\"h\n" +
	"\bMetadata\x12\x1a\n" +
	"\bhostname\x18\x01 \x01(\tR\bhostname\x12\x14\n" +
	"\x05error\x18\x02 \x01(\tR\x05error\x12*\n" +
	"\x06status\x18\x03 \x01(\v2\x12.google.rpc.StatusR\x06status\"J\n" +
	"\x04Data\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x14\n" +
	"\x05bytes\x18\x02 \x01(\fR\x05bytes\"8\n" +
	"\fDataResponse\x12(\n" +
	"\bmessages\x18\x01 \x03(\v2\f.common.DataR\bmessages\"5\n" +
	"\x05Empty\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\":\n" +
	"\rEmptyResponse\x12)\n" +
	"\bmessages\x18\x01 \x03(\v2\r.common.EmptyR\bmessages\"\x80\x01\n" +
	"\x12ContainerdInstance\x12/\n" +
	"\x06driver\x18\x01 \x01(\x0e2\x17.common.ContainerDriverR\x06driver\x129\n" +
	"\tnamespace\x18\x02 \x01(\x0e2\x1b.common.ContainerdNamespaceR\tnamespace\"\"\n" +
	"\x03URL\x12\x1b\n" +
	"\tfull_path\x18\x01 \x01(\tR\bfullPath\"A\n" +
	"\x1bPEMEncodedCertificateAndKey\x12\x10\n" +
	"\x03crt\x18\x01 \x01(\fR\x03crt\x12\x10\n" +
	"\x03key\x18\x02 \x01(\fR\x03key\"!\n" +
	"\rPEMEncodedKey\x12\x10\n" +
	"\x03key\x18\x01 \x01(\fR\x03key\")\n" +
	"\x15PEMEncodedCertificate\x12\x10\n" +
	"\x03crt\x18\x01 \x01(\fR\x03crt\"\x17\n" +
	"\x05NetIP\x12\x0e\n" +
	"\x02ip\x18\x01 \x01(\fR\x02ip\"/\n" +
	"\tNetIPPort\x12\x0e\n" +
	"\x02ip\x18\x01 \x01(\fR\x02ip\x12\x12\n" +
	"\x04port\x18\x02 \x01(\x05R\x04port\"B\n" +
	"\vNetIPPrefix\x12\x0e\n" +
	"\x02ip\x18\x01 \x01(\fR\x02ip\x12#\n" +
	"\rprefix_length\x18\x02 \x01(\x05R\fprefixLength*+\n" +
	"\x04Code\x12\t\n" +
	"\x05FATAL\x10\x00\x12\n" +
	"\n" +
	"\x06LOCKED\x10\x01\x12\f\n" +
	"\bCANCELED\x10\x02**\n" +
	"\x0fContainerDriver\x12\x0e\n" +
	"\n" +
	"CONTAINERD\x10\x00\x12\a\n" +
	"\x03CRI\x10\x01*@\n" +
	"\x13ContainerdNamespace\x12\x0e\n" +
	"\n" +
	"NS_UNKNOWN\x10\x00\x12\r\n" +
	"\tNS_SYSTEM\x10\x01\x12\n" +
	"\n" +
	"\x06NS_CRI\x10\x02:]\n" +
	"\x19remove_deprecated_message\x12\x1f.google.protobuf.MessageOptions\x18\xbd\xd7\x05 \x01(\tR\x17removeDeprecatedMessage:W\n" +
	"\x17remove_deprecated_field\x12\x1d.google.protobuf.FieldOptions\x18\xbd\xd7\x05 \x01(\tR\x15removeDeprecatedField:T\n" +
	"\x16remove_deprecated_enum\x12\x1c.google.protobuf.EnumOptions\x18\xbd\xd7\x05 \x01(\tR\x14removeDeprecatedEnum:d\n" +
	"\x1cremove_deprecated_enum_value\x12!.google.protobuf.EnumValueOptions\x18\xbd\xd7\x05 \x01(\tR\x19removeDeprecatedEnumValue:Z\n" +
	"\x18remove_deprecated_method\x12\x1e.google.protobuf.MethodOptions\x18\xbd\xd7\x05 \x01(\tR\x16removeDeprecatedMethod:]\n" +
	"\x19remove_deprecated_service\x12\x1f.google.protobuf.ServiceOptions\x18\xbd\xd7\x05 \x01(\tR\x17removeDeprecatedServiceBL\n" +
	"\x14dev.talos.api.commonZ4github.com/siderolabs/talos/pkg/machinery/api/commonb\x06proto3"

var (
	file_common_common_proto_rawDescOnce sync.Once
	file_common_common_proto_rawDescData []byte
)

func file_common_common_proto_rawDescGZIP() []byte {
	file_common_common_proto_rawDescOnce.Do(func() {
		file_common_common_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_common_common_proto_rawDesc), len(file_common_common_proto_rawDesc)))
	})
	return file_common_common_proto_rawDescData
}

var file_common_common_proto_enumTypes = make([]protoimpl.EnumInfo, 3)
var file_common_common_proto_msgTypes = make([]protoimpl.MessageInfo, 14)
var file_common_common_proto_goTypes = []any{
	(Code)(0),                             // 0: common.Code
	(ContainerDriver)(0),                  // 1: common.ContainerDriver
	(ContainerdNamespace)(0),              // 2: common.ContainerdNamespace
	(*Error)(nil),                         // 3: common.Error
	(*Metadata)(nil),                      // 4: common.Metadata
	(*Data)(nil),                          // 5: common.Data
	(*DataResponse)(nil),                  // 6: common.DataResponse
	(*Empty)(nil),                         // 7: common.Empty
	(*EmptyResponse)(nil),                 // 8: common.EmptyResponse
	(*ContainerdInstance)(nil),            // 9: common.ContainerdInstance
	(*URL)(nil),                           // 10: common.URL
	(*PEMEncodedCertificateAndKey)(nil),   // 11: common.PEMEncodedCertificateAndKey
	(*PEMEncodedKey)(nil),                 // 12: common.PEMEncodedKey
	(*PEMEncodedCertificate)(nil),         // 13: common.PEMEncodedCertificate
	(*NetIP)(nil),                         // 14: common.NetIP
	(*NetIPPort)(nil),                     // 15: common.NetIPPort
	(*NetIPPrefix)(nil),                   // 16: common.NetIPPrefix
	(*anypb.Any)(nil),                     // 17: google.protobuf.Any
	(*status.Status)(nil),                 // 18: google.rpc.Status
	(*descriptorpb.MessageOptions)(nil),   // 19: google.protobuf.MessageOptions
	(*descriptorpb.FieldOptions)(nil),     // 20: google.protobuf.FieldOptions
	(*descriptorpb.EnumOptions)(nil),      // 21: google.protobuf.EnumOptions
	(*descriptorpb.EnumValueOptions)(nil), // 22: google.protobuf.EnumValueOptions
	(*descriptorpb.MethodOptions)(nil),    // 23: google.protobuf.MethodOptions
	(*descriptorpb.ServiceOptions)(nil),   // 24: google.protobuf.ServiceOptions
}
var file_common_common_proto_depIdxs = []int32{
	0,  // 0: common.Error.code:type_name -> common.Code
	17, // 1: common.Error.details:type_name -> google.protobuf.Any
	18, // 2: common.Metadata.status:type_name -> google.rpc.Status
	4,  // 3: common.Data.metadata:type_name -> common.Metadata
	5,  // 4: common.DataResponse.messages:type_name -> common.Data
	4,  // 5: common.Empty.metadata:type_name -> common.Metadata
	7,  // 6: common.EmptyResponse.messages:type_name -> common.Empty
	1,  // 7: common.ContainerdInstance.driver:type_name -> common.ContainerDriver
	2,  // 8: common.ContainerdInstance.namespace:type_name -> common.ContainerdNamespace
	19, // 9: common.remove_deprecated_message:extendee -> google.protobuf.MessageOptions
	20, // 10: common.remove_deprecated_field:extendee -> google.protobuf.FieldOptions
	21, // 11: common.remove_deprecated_enum:extendee -> google.protobuf.EnumOptions
	22, // 12: common.remove_deprecated_enum_value:extendee -> google.protobuf.EnumValueOptions
	23, // 13: common.remove_deprecated_method:extendee -> google.protobuf.MethodOptions
	24, // 14: common.remove_deprecated_service:extendee -> google.protobuf.ServiceOptions
	15, // [15:15] is the sub-list for method output_type
	15, // [15:15] is the sub-list for method input_type
	15, // [15:15] is the sub-list for extension type_name
	9,  // [9:15] is the sub-list for extension extendee
	0,  // [0:9] is the sub-list for field type_name
}

func init() { file_common_common_proto_init() }
func file_common_common_proto_init() {
	if File_common_common_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_common_common_proto_rawDesc), len(file_common_common_proto_rawDesc)),
			NumEnums:      3,
			NumMessages:   14,
			NumExtensions: 6,
			NumServices:   0,
		},
		GoTypes:           file_common_common_proto_goTypes,
		DependencyIndexes: file_common_common_proto_depIdxs,
		EnumInfos:         file_common_common_proto_enumTypes,
		MessageInfos:      file_common_common_proto_msgTypes,
		ExtensionInfos:    file_common_common_proto_extTypes,
	}.Build()
	File_common_common_proto = out.File
	file_common_common_proto_goTypes = nil
	file_common_common_proto_depIdxs = nil
}
