// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.1
// source: xds/service/orca/v3/orca.proto

package v3

import (
	v3 "github.com/cncf/xds/go/xds/data/orca/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type OrcaLoadReportRequest struct {
	state            protoimpl.MessageState `protogen:"open.v1"`
	ReportInterval   *durationpb.Duration   `protobuf:"bytes,1,opt,name=report_interval,json=reportInterval,proto3" json:"report_interval,omitempty"`
	RequestCostNames []string               `protobuf:"bytes,2,rep,name=request_cost_names,json=requestCostNames,proto3" json:"request_cost_names,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *OrcaLoadReportRequest) Reset() {
	*x = OrcaLoadReportRequest{}
	mi := &file_xds_service_orca_v3_orca_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OrcaLoadReportRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OrcaLoadReportRequest) ProtoMessage() {}

func (x *OrcaLoadReportRequest) ProtoReflect() protoreflect.Message {
	mi := &file_xds_service_orca_v3_orca_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OrcaLoadReportRequest.ProtoReflect.Descriptor instead.
func (*OrcaLoadReportRequest) Descriptor() ([]byte, []int) {
	return file_xds_service_orca_v3_orca_proto_rawDescGZIP(), []int{0}
}

func (x *OrcaLoadReportRequest) GetReportInterval() *durationpb.Duration {
	if x != nil {
		return x.ReportInterval
	}
	return nil
}

func (x *OrcaLoadReportRequest) GetRequestCostNames() []string {
	if x != nil {
		return x.RequestCostNames
	}
	return nil
}

var File_xds_service_orca_v3_orca_proto protoreflect.FileDescriptor

const file_xds_service_orca_v3_orca_proto_rawDesc = "" +
	"\n" +
	"\x1exds/service/orca/v3/orca.proto\x12\x13xds.service.orca.v3\x1a'xds/data/orca/v3/orca_load_report.proto\x1a\x1egoogle/protobuf/duration.proto\"\x89\x01\n" +
	"\x15OrcaLoadReportRequest\x12B\n" +
	"\x0freport_interval\x18\x01 \x01(\v2\x19.google.protobuf.DurationR\x0ereportInterval\x12,\n" +
	"\x12request_cost_names\x18\x02 \x03(\tR\x10requestCostNames2u\n" +
	"\x0eOpenRcaService\x12c\n" +
	"\x11StreamCoreMetrics\x12*.xds.service.orca.v3.OrcaLoadReportRequest\x1a .xds.data.orca.v3.OrcaLoadReport0\x01BY\n" +
	"\x1ecom.github.xds.service.orca.v3B\tOrcaProtoP\x01Z*github.com/cncf/xds/go/xds/service/orca/v3b\x06proto3"

var (
	file_xds_service_orca_v3_orca_proto_rawDescOnce sync.Once
	file_xds_service_orca_v3_orca_proto_rawDescData []byte
)

func file_xds_service_orca_v3_orca_proto_rawDescGZIP() []byte {
	file_xds_service_orca_v3_orca_proto_rawDescOnce.Do(func() {
		file_xds_service_orca_v3_orca_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_xds_service_orca_v3_orca_proto_rawDesc), len(file_xds_service_orca_v3_orca_proto_rawDesc)))
	})
	return file_xds_service_orca_v3_orca_proto_rawDescData
}

var file_xds_service_orca_v3_orca_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_xds_service_orca_v3_orca_proto_goTypes = []any{
	(*OrcaLoadReportRequest)(nil), // 0: xds.service.orca.v3.OrcaLoadReportRequest
	(*durationpb.Duration)(nil),   // 1: google.protobuf.Duration
	(*v3.OrcaLoadReport)(nil),     // 2: xds.data.orca.v3.OrcaLoadReport
}
var file_xds_service_orca_v3_orca_proto_depIdxs = []int32{
	1, // 0: xds.service.orca.v3.OrcaLoadReportRequest.report_interval:type_name -> google.protobuf.Duration
	0, // 1: xds.service.orca.v3.OpenRcaService.StreamCoreMetrics:input_type -> xds.service.orca.v3.OrcaLoadReportRequest
	2, // 2: xds.service.orca.v3.OpenRcaService.StreamCoreMetrics:output_type -> xds.data.orca.v3.OrcaLoadReport
	2, // [2:3] is the sub-list for method output_type
	1, // [1:2] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_xds_service_orca_v3_orca_proto_init() }
func file_xds_service_orca_v3_orca_proto_init() {
	if File_xds_service_orca_v3_orca_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_xds_service_orca_v3_orca_proto_rawDesc), len(file_xds_service_orca_v3_orca_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_xds_service_orca_v3_orca_proto_goTypes,
		DependencyIndexes: file_xds_service_orca_v3_orca_proto_depIdxs,
		MessageInfos:      file_xds_service_orca_v3_orca_proto_msgTypes,
	}.Build()
	File_xds_service_orca_v3_orca_proto = out.File
	file_xds_service_orca_v3_orca_proto_goTypes = nil
	file_xds_service_orca_v3_orca_proto_depIdxs = nil
}
