// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/admin/v3/init_dump.proto

package adminv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Dumps of unready targets of envoy init managers. Envoy's admin fills this message with init managers,
// which provides the information of their unready targets.
// The :ref:`/init_dump <operations_admin_interface_init_dump>` will dump all unready targets information.
type UnreadyTargetsDumps struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// You can choose specific component to dump unready targets with mask query parameter.
	// See :ref:`/init_dump?mask={} <operations_admin_interface_init_dump_by_mask>` for more information.
	// The dumps of unready targets of all init managers.
	UnreadyTargetsDumps []*UnreadyTargetsDumps_UnreadyTargetsDump `protobuf:"bytes,1,rep,name=unready_targets_dumps,json=unreadyTargetsDumps,proto3" json:"unready_targets_dumps,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *UnreadyTargetsDumps) Reset() {
	*x = UnreadyTargetsDumps{}
	mi := &file_envoy_admin_v3_init_dump_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UnreadyTargetsDumps) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UnreadyTargetsDumps) ProtoMessage() {}

func (x *UnreadyTargetsDumps) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_init_dump_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UnreadyTargetsDumps.ProtoReflect.Descriptor instead.
func (*UnreadyTargetsDumps) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_init_dump_proto_rawDescGZIP(), []int{0}
}

func (x *UnreadyTargetsDumps) GetUnreadyTargetsDumps() []*UnreadyTargetsDumps_UnreadyTargetsDump {
	if x != nil {
		return x.UnreadyTargetsDumps
	}
	return nil
}

// Message of unready targets information of an init manager.
type UnreadyTargetsDumps_UnreadyTargetsDump struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Name of the init manager. Example: "init_manager_xxx".
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// Names of unready targets of the init manager. Example: "target_xxx".
	TargetNames   []string `protobuf:"bytes,2,rep,name=target_names,json=targetNames,proto3" json:"target_names,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UnreadyTargetsDumps_UnreadyTargetsDump) Reset() {
	*x = UnreadyTargetsDumps_UnreadyTargetsDump{}
	mi := &file_envoy_admin_v3_init_dump_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UnreadyTargetsDumps_UnreadyTargetsDump) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UnreadyTargetsDumps_UnreadyTargetsDump) ProtoMessage() {}

func (x *UnreadyTargetsDumps_UnreadyTargetsDump) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_init_dump_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UnreadyTargetsDumps_UnreadyTargetsDump.ProtoReflect.Descriptor instead.
func (*UnreadyTargetsDumps_UnreadyTargetsDump) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_init_dump_proto_rawDescGZIP(), []int{0, 0}
}

func (x *UnreadyTargetsDumps_UnreadyTargetsDump) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *UnreadyTargetsDumps_UnreadyTargetsDump) GetTargetNames() []string {
	if x != nil {
		return x.TargetNames
	}
	return nil
}

var File_envoy_admin_v3_init_dump_proto protoreflect.FileDescriptor

const file_envoy_admin_v3_init_dump_proto_rawDesc = "" +
	"\n" +
	"\x1eenvoy/admin/v3/init_dump.proto\x12\x0eenvoy.admin.v3\x1a\x1dudpa/annotations/status.proto\"\xce\x01\n" +
	"\x13UnreadyTargetsDumps\x12j\n" +
	"\x15unready_targets_dumps\x18\x01 \x03(\v26.envoy.admin.v3.UnreadyTargetsDumps.UnreadyTargetsDumpR\x13unreadyTargetsDumps\x1aK\n" +
	"\x12UnreadyTargetsDump\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12!\n" +
	"\ftarget_names\x18\x02 \x03(\tR\vtargetNamesBv\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"\x1cio.envoyproxy.envoy.admin.v3B\rInitDumpProtoP\x01Z=github.com/envoyproxy/go-control-plane/envoy/admin/v3;adminv3b\x06proto3"

var (
	file_envoy_admin_v3_init_dump_proto_rawDescOnce sync.Once
	file_envoy_admin_v3_init_dump_proto_rawDescData []byte
)

func file_envoy_admin_v3_init_dump_proto_rawDescGZIP() []byte {
	file_envoy_admin_v3_init_dump_proto_rawDescOnce.Do(func() {
		file_envoy_admin_v3_init_dump_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_admin_v3_init_dump_proto_rawDesc), len(file_envoy_admin_v3_init_dump_proto_rawDesc)))
	})
	return file_envoy_admin_v3_init_dump_proto_rawDescData
}

var file_envoy_admin_v3_init_dump_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_admin_v3_init_dump_proto_goTypes = []any{
	(*UnreadyTargetsDumps)(nil),                    // 0: envoy.admin.v3.UnreadyTargetsDumps
	(*UnreadyTargetsDumps_UnreadyTargetsDump)(nil), // 1: envoy.admin.v3.UnreadyTargetsDumps.UnreadyTargetsDump
}
var file_envoy_admin_v3_init_dump_proto_depIdxs = []int32{
	1, // 0: envoy.admin.v3.UnreadyTargetsDumps.unready_targets_dumps:type_name -> envoy.admin.v3.UnreadyTargetsDumps.UnreadyTargetsDump
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_admin_v3_init_dump_proto_init() }
func file_envoy_admin_v3_init_dump_proto_init() {
	if File_envoy_admin_v3_init_dump_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_admin_v3_init_dump_proto_rawDesc), len(file_envoy_admin_v3_init_dump_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_admin_v3_init_dump_proto_goTypes,
		DependencyIndexes: file_envoy_admin_v3_init_dump_proto_depIdxs,
		MessageInfos:      file_envoy_admin_v3_init_dump_proto_msgTypes,
	}.Build()
	File_envoy_admin_v3_init_dump_proto = out.File
	file_envoy_admin_v3_init_dump_proto_goTypes = nil
	file_envoy_admin_v3_init_dump_proto_depIdxs = nil
}
