// Copyright 2025 Huawei Technologies Co.,Ltd.
//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package internal

const stsEndpoint = `
{
    "cn-north-4": "https://sts.cn-north-4.myhuaweicloud.com",
    "cn-south-1": "https://sts.cn-south-1.myhuaweicloud.com",
    "cn-east-3": "https://sts.cn-east-3.myhuaweicloud.com",
    "cn-southwest-2": "https://sts.cn-southwest-2.myhuaweicloud.com",
    "ap-southeast-1": "https://sts.ap-southeast-1.myhuaweicloud.com",
    "ap-southeast-3": "https://sts.ap-southeast-3.myhuaweicloud.com",
    "ap-southeast-2": "https://sts.ap-southeast-2.myhuaweicloud.com",
    "sa-brazil-1": "https://sts.sa-brazil-1.myhuaweicloud.com",
    "la-south-2": "https://sts.la-south-2.myhuaweicloud.com",
    "af-south-1": "https://sts.af-south-1.myhuaweicloud.com",
    "ap-southeast-4": "https://sts.ap-southeast-4.myhuaweicloud.com",
    "tr-west-1": "https://sts.tr-west-1.myhuaweicloud.com"
}
`
