// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/type/v3/ratelimit_unit.proto

package typev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Identifies the unit of of time for rate limit.
type RateLimitUnit int32

const (
	// The time unit is not known.
	RateLimitUnit_UNKNOWN RateLimitUnit = 0
	// The time unit representing a second.
	RateLimitUnit_SECOND RateLimitUnit = 1
	// The time unit representing a minute.
	RateLimitUnit_MINUTE RateLimitUnit = 2
	// The time unit representing an hour.
	RateLimitUnit_HOUR RateLimitUnit = 3
	// The time unit representing a day.
	RateLimitUnit_DAY RateLimitUnit = 4
	// The time unit representing a month.
	RateLimitUnit_MONTH RateLimitUnit = 5
	// The time unit representing a year.
	RateLimitUnit_YEAR RateLimitUnit = 6
)

// Enum value maps for RateLimitUnit.
var (
	RateLimitUnit_name = map[int32]string{
		0: "UNKNOWN",
		1: "SECOND",
		2: "MINUTE",
		3: "HOUR",
		4: "DAY",
		5: "MONTH",
		6: "YEAR",
	}
	RateLimitUnit_value = map[string]int32{
		"UNKNOWN": 0,
		"SECOND":  1,
		"MINUTE":  2,
		"HOUR":    3,
		"DAY":     4,
		"MONTH":   5,
		"YEAR":    6,
	}
)

func (x RateLimitUnit) Enum() *RateLimitUnit {
	p := new(RateLimitUnit)
	*p = x
	return p
}

func (x RateLimitUnit) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (RateLimitUnit) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_type_v3_ratelimit_unit_proto_enumTypes[0].Descriptor()
}

func (RateLimitUnit) Type() protoreflect.EnumType {
	return &file_envoy_type_v3_ratelimit_unit_proto_enumTypes[0]
}

func (x RateLimitUnit) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use RateLimitUnit.Descriptor instead.
func (RateLimitUnit) EnumDescriptor() ([]byte, []int) {
	return file_envoy_type_v3_ratelimit_unit_proto_rawDescGZIP(), []int{0}
}

var File_envoy_type_v3_ratelimit_unit_proto protoreflect.FileDescriptor

const file_envoy_type_v3_ratelimit_unit_proto_rawDesc = "" +
	"\n" +
	"\"envoy/type/v3/ratelimit_unit.proto\x12\renvoy.type.v3\x1a\x1dudpa/annotations/status.proto*\\\n" +
	"\rRateLimitUnit\x12\v\n" +
	"\aUNKNOWN\x10\x00\x12\n" +
	"\n" +
	"\x06SECOND\x10\x01\x12\n" +
	"\n" +
	"\x06MINUTE\x10\x02\x12\b\n" +
	"\x04HOUR\x10\x03\x12\a\n" +
	"\x03DAY\x10\x04\x12\t\n" +
	"\x05MONTH\x10\x05\x12\b\n" +
	"\x04YEAR\x10\x06Bx\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"\x1bio.envoyproxy.envoy.type.v3B\x12RatelimitUnitProtoP\x01Z;github.com/envoyproxy/go-control-plane/envoy/type/v3;typev3b\x06proto3"

var (
	file_envoy_type_v3_ratelimit_unit_proto_rawDescOnce sync.Once
	file_envoy_type_v3_ratelimit_unit_proto_rawDescData []byte
)

func file_envoy_type_v3_ratelimit_unit_proto_rawDescGZIP() []byte {
	file_envoy_type_v3_ratelimit_unit_proto_rawDescOnce.Do(func() {
		file_envoy_type_v3_ratelimit_unit_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_type_v3_ratelimit_unit_proto_rawDesc), len(file_envoy_type_v3_ratelimit_unit_proto_rawDesc)))
	})
	return file_envoy_type_v3_ratelimit_unit_proto_rawDescData
}

var file_envoy_type_v3_ratelimit_unit_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_envoy_type_v3_ratelimit_unit_proto_goTypes = []any{
	(RateLimitUnit)(0), // 0: envoy.type.v3.RateLimitUnit
}
var file_envoy_type_v3_ratelimit_unit_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_envoy_type_v3_ratelimit_unit_proto_init() }
func file_envoy_type_v3_ratelimit_unit_proto_init() {
	if File_envoy_type_v3_ratelimit_unit_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_type_v3_ratelimit_unit_proto_rawDesc), len(file_envoy_type_v3_ratelimit_unit_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   0,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_type_v3_ratelimit_unit_proto_goTypes,
		DependencyIndexes: file_envoy_type_v3_ratelimit_unit_proto_depIdxs,
		EnumInfos:         file_envoy_type_v3_ratelimit_unit_proto_enumTypes,
	}.Build()
	File_envoy_type_v3_ratelimit_unit_proto = out.File
	file_envoy_type_v3_ratelimit_unit_proto_goTypes = nil
	file_envoy_type_v3_ratelimit_unit_proto_depIdxs = nil
}
