// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/endpoint/v3/load_report.proto

package endpointv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	structpb "google.golang.org/protobuf/types/known/structpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// These are stats Envoy reports to the management server at a frequency defined by
// :ref:`LoadStatsResponse.load_reporting_interval<envoy_v3_api_field_service.load_stats.v3.LoadStatsResponse.load_reporting_interval>`.
// Stats per upstream region/zone and optionally per subzone.
// [#next-free-field: 15]
type UpstreamLocalityStats struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Name of zone, region and optionally endpoint group these metrics were
	// collected from. Zone and region names could be empty if unknown.
	Locality *v3.Locality `protobuf:"bytes,1,opt,name=locality,proto3" json:"locality,omitempty"`
	// The total number of requests successfully completed by the endpoints in the
	// locality.
	TotalSuccessfulRequests uint64 `protobuf:"varint,2,opt,name=total_successful_requests,json=totalSuccessfulRequests,proto3" json:"total_successful_requests,omitempty"`
	// The total number of unfinished requests. A request can be an HTTP request
	// or a TCP connection for a TCP connection pool.
	TotalRequestsInProgress uint64 `protobuf:"varint,3,opt,name=total_requests_in_progress,json=totalRequestsInProgress,proto3" json:"total_requests_in_progress,omitempty"`
	// The total number of requests that failed due to errors at the endpoint,
	// aggregated over all endpoints in the locality.
	TotalErrorRequests uint64 `protobuf:"varint,4,opt,name=total_error_requests,json=totalErrorRequests,proto3" json:"total_error_requests,omitempty"`
	// The total number of requests that were issued by this Envoy since
	// the last report. This information is aggregated over all the
	// upstream endpoints in the locality. A request can be an HTTP request
	// or a TCP connection for a TCP connection pool.
	TotalIssuedRequests uint64 `protobuf:"varint,8,opt,name=total_issued_requests,json=totalIssuedRequests,proto3" json:"total_issued_requests,omitempty"`
	// The total number of connections in an established state at the time of the
	// report. This field is aggregated over all the upstream endpoints in the
	// locality.
	// In Envoy, this information may be based on “upstream_cx_active metric“.
	// [#not-implemented-hide:]
	TotalActiveConnections uint64 `protobuf:"varint,9,opt,name=total_active_connections,json=totalActiveConnections,proto3" json:"total_active_connections,omitempty"`
	// The total number of connections opened since the last report.
	// This field is aggregated over all the upstream endpoints in the locality.
	// In Envoy, this information may be based on “upstream_cx_total“ metric
	// compared to itself between start and end of an interval, i.e.
	// “upstream_cx_total“(now) - “upstream_cx_total“(now -
	// load_report_interval).
	// [#not-implemented-hide:]
	TotalNewConnections uint64 `protobuf:"varint,10,opt,name=total_new_connections,json=totalNewConnections,proto3" json:"total_new_connections,omitempty"`
	// The total number of connection failures since the last report.
	// This field is aggregated over all the upstream endpoints in the locality.
	// In Envoy, this information may be based on “upstream_cx_connect_fail“
	// metric compared to itself between start and end of an interval, i.e.
	// “upstream_cx_connect_fail“(now) - “upstream_cx_connect_fail“(now -
	// load_report_interval).
	// [#not-implemented-hide:]
	TotalFailConnections uint64 `protobuf:"varint,11,opt,name=total_fail_connections,json=totalFailConnections,proto3" json:"total_fail_connections,omitempty"`
	// CPU utilization stats for multi-dimensional load balancing.
	// This typically comes from endpoint metrics reported via ORCA.
	CpuUtilization *UnnamedEndpointLoadMetricStats `protobuf:"bytes,12,opt,name=cpu_utilization,json=cpuUtilization,proto3" json:"cpu_utilization,omitempty"`
	// Memory utilization for multi-dimensional load balancing.
	// This typically comes from endpoint metrics reported via ORCA.
	MemUtilization *UnnamedEndpointLoadMetricStats `protobuf:"bytes,13,opt,name=mem_utilization,json=memUtilization,proto3" json:"mem_utilization,omitempty"`
	// Blended application-defined utilization for multi-dimensional load balancing.
	// This typically comes from endpoint metrics reported via ORCA.
	ApplicationUtilization *UnnamedEndpointLoadMetricStats `protobuf:"bytes,14,opt,name=application_utilization,json=applicationUtilization,proto3" json:"application_utilization,omitempty"`
	// Named stats for multi-dimensional load balancing.
	// These typically come from endpoint metrics reported via ORCA.
	LoadMetricStats []*EndpointLoadMetricStats `protobuf:"bytes,5,rep,name=load_metric_stats,json=loadMetricStats,proto3" json:"load_metric_stats,omitempty"`
	// Endpoint granularity stats information for this locality. This information
	// is populated if the Server requests it by setting
	// :ref:`LoadStatsResponse.report_endpoint_granularity<envoy_v3_api_field_service.load_stats.v3.LoadStatsResponse.report_endpoint_granularity>`.
	UpstreamEndpointStats []*UpstreamEndpointStats `protobuf:"bytes,7,rep,name=upstream_endpoint_stats,json=upstreamEndpointStats,proto3" json:"upstream_endpoint_stats,omitempty"`
	// [#not-implemented-hide:] The priority of the endpoint group these metrics
	// were collected from.
	Priority      uint32 `protobuf:"varint,6,opt,name=priority,proto3" json:"priority,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpstreamLocalityStats) Reset() {
	*x = UpstreamLocalityStats{}
	mi := &file_envoy_config_endpoint_v3_load_report_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpstreamLocalityStats) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpstreamLocalityStats) ProtoMessage() {}

func (x *UpstreamLocalityStats) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_endpoint_v3_load_report_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpstreamLocalityStats.ProtoReflect.Descriptor instead.
func (*UpstreamLocalityStats) Descriptor() ([]byte, []int) {
	return file_envoy_config_endpoint_v3_load_report_proto_rawDescGZIP(), []int{0}
}

func (x *UpstreamLocalityStats) GetLocality() *v3.Locality {
	if x != nil {
		return x.Locality
	}
	return nil
}

func (x *UpstreamLocalityStats) GetTotalSuccessfulRequests() uint64 {
	if x != nil {
		return x.TotalSuccessfulRequests
	}
	return 0
}

func (x *UpstreamLocalityStats) GetTotalRequestsInProgress() uint64 {
	if x != nil {
		return x.TotalRequestsInProgress
	}
	return 0
}

func (x *UpstreamLocalityStats) GetTotalErrorRequests() uint64 {
	if x != nil {
		return x.TotalErrorRequests
	}
	return 0
}

func (x *UpstreamLocalityStats) GetTotalIssuedRequests() uint64 {
	if x != nil {
		return x.TotalIssuedRequests
	}
	return 0
}

func (x *UpstreamLocalityStats) GetTotalActiveConnections() uint64 {
	if x != nil {
		return x.TotalActiveConnections
	}
	return 0
}

func (x *UpstreamLocalityStats) GetTotalNewConnections() uint64 {
	if x != nil {
		return x.TotalNewConnections
	}
	return 0
}

func (x *UpstreamLocalityStats) GetTotalFailConnections() uint64 {
	if x != nil {
		return x.TotalFailConnections
	}
	return 0
}

func (x *UpstreamLocalityStats) GetCpuUtilization() *UnnamedEndpointLoadMetricStats {
	if x != nil {
		return x.CpuUtilization
	}
	return nil
}

func (x *UpstreamLocalityStats) GetMemUtilization() *UnnamedEndpointLoadMetricStats {
	if x != nil {
		return x.MemUtilization
	}
	return nil
}

func (x *UpstreamLocalityStats) GetApplicationUtilization() *UnnamedEndpointLoadMetricStats {
	if x != nil {
		return x.ApplicationUtilization
	}
	return nil
}

func (x *UpstreamLocalityStats) GetLoadMetricStats() []*EndpointLoadMetricStats {
	if x != nil {
		return x.LoadMetricStats
	}
	return nil
}

func (x *UpstreamLocalityStats) GetUpstreamEndpointStats() []*UpstreamEndpointStats {
	if x != nil {
		return x.UpstreamEndpointStats
	}
	return nil
}

func (x *UpstreamLocalityStats) GetPriority() uint32 {
	if x != nil {
		return x.Priority
	}
	return 0
}

// [#next-free-field: 8]
type UpstreamEndpointStats struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Upstream host address.
	Address *v3.Address `protobuf:"bytes,1,opt,name=address,proto3" json:"address,omitempty"`
	// Opaque and implementation dependent metadata of the
	// endpoint. Envoy will pass this directly to the management server.
	Metadata *structpb.Struct `protobuf:"bytes,6,opt,name=metadata,proto3" json:"metadata,omitempty"`
	// The total number of requests successfully completed by the endpoints in the
	// locality. These include non-5xx responses for HTTP, where errors
	// originate at the client and the endpoint responded successfully. For gRPC,
	// the grpc-status values are those not covered by total_error_requests below.
	TotalSuccessfulRequests uint64 `protobuf:"varint,2,opt,name=total_successful_requests,json=totalSuccessfulRequests,proto3" json:"total_successful_requests,omitempty"`
	// The total number of unfinished requests for this endpoint.
	TotalRequestsInProgress uint64 `protobuf:"varint,3,opt,name=total_requests_in_progress,json=totalRequestsInProgress,proto3" json:"total_requests_in_progress,omitempty"`
	// The total number of requests that failed due to errors at the endpoint.
	// For HTTP these are responses with 5xx status codes and for gRPC the
	// grpc-status values:
	//
	//   - DeadlineExceeded
	//   - Unimplemented
	//   - Internal
	//   - Unavailable
	//   - Unknown
	//   - DataLoss
	TotalErrorRequests uint64 `protobuf:"varint,4,opt,name=total_error_requests,json=totalErrorRequests,proto3" json:"total_error_requests,omitempty"`
	// The total number of requests that were issued to this endpoint
	// since the last report. A single TCP connection, HTTP or gRPC
	// request or stream is counted as one request.
	TotalIssuedRequests uint64 `protobuf:"varint,7,opt,name=total_issued_requests,json=totalIssuedRequests,proto3" json:"total_issued_requests,omitempty"`
	// Stats for multi-dimensional load balancing.
	LoadMetricStats []*EndpointLoadMetricStats `protobuf:"bytes,5,rep,name=load_metric_stats,json=loadMetricStats,proto3" json:"load_metric_stats,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *UpstreamEndpointStats) Reset() {
	*x = UpstreamEndpointStats{}
	mi := &file_envoy_config_endpoint_v3_load_report_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpstreamEndpointStats) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpstreamEndpointStats) ProtoMessage() {}

func (x *UpstreamEndpointStats) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_endpoint_v3_load_report_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpstreamEndpointStats.ProtoReflect.Descriptor instead.
func (*UpstreamEndpointStats) Descriptor() ([]byte, []int) {
	return file_envoy_config_endpoint_v3_load_report_proto_rawDescGZIP(), []int{1}
}

func (x *UpstreamEndpointStats) GetAddress() *v3.Address {
	if x != nil {
		return x.Address
	}
	return nil
}

func (x *UpstreamEndpointStats) GetMetadata() *structpb.Struct {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *UpstreamEndpointStats) GetTotalSuccessfulRequests() uint64 {
	if x != nil {
		return x.TotalSuccessfulRequests
	}
	return 0
}

func (x *UpstreamEndpointStats) GetTotalRequestsInProgress() uint64 {
	if x != nil {
		return x.TotalRequestsInProgress
	}
	return 0
}

func (x *UpstreamEndpointStats) GetTotalErrorRequests() uint64 {
	if x != nil {
		return x.TotalErrorRequests
	}
	return 0
}

func (x *UpstreamEndpointStats) GetTotalIssuedRequests() uint64 {
	if x != nil {
		return x.TotalIssuedRequests
	}
	return 0
}

func (x *UpstreamEndpointStats) GetLoadMetricStats() []*EndpointLoadMetricStats {
	if x != nil {
		return x.LoadMetricStats
	}
	return nil
}

type EndpointLoadMetricStats struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Name of the metric; may be empty.
	MetricName string `protobuf:"bytes,1,opt,name=metric_name,json=metricName,proto3" json:"metric_name,omitempty"`
	// Number of calls that finished and included this metric.
	NumRequestsFinishedWithMetric uint64 `protobuf:"varint,2,opt,name=num_requests_finished_with_metric,json=numRequestsFinishedWithMetric,proto3" json:"num_requests_finished_with_metric,omitempty"`
	// Sum of metric values across all calls that finished with this metric for
	// load_reporting_interval.
	TotalMetricValue float64 `protobuf:"fixed64,3,opt,name=total_metric_value,json=totalMetricValue,proto3" json:"total_metric_value,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *EndpointLoadMetricStats) Reset() {
	*x = EndpointLoadMetricStats{}
	mi := &file_envoy_config_endpoint_v3_load_report_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EndpointLoadMetricStats) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EndpointLoadMetricStats) ProtoMessage() {}

func (x *EndpointLoadMetricStats) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_endpoint_v3_load_report_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EndpointLoadMetricStats.ProtoReflect.Descriptor instead.
func (*EndpointLoadMetricStats) Descriptor() ([]byte, []int) {
	return file_envoy_config_endpoint_v3_load_report_proto_rawDescGZIP(), []int{2}
}

func (x *EndpointLoadMetricStats) GetMetricName() string {
	if x != nil {
		return x.MetricName
	}
	return ""
}

func (x *EndpointLoadMetricStats) GetNumRequestsFinishedWithMetric() uint64 {
	if x != nil {
		return x.NumRequestsFinishedWithMetric
	}
	return 0
}

func (x *EndpointLoadMetricStats) GetTotalMetricValue() float64 {
	if x != nil {
		return x.TotalMetricValue
	}
	return 0
}

// Same as EndpointLoadMetricStats, except without the metric_name field.
type UnnamedEndpointLoadMetricStats struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Number of calls that finished and included this metric.
	NumRequestsFinishedWithMetric uint64 `protobuf:"varint,1,opt,name=num_requests_finished_with_metric,json=numRequestsFinishedWithMetric,proto3" json:"num_requests_finished_with_metric,omitempty"`
	// Sum of metric values across all calls that finished with this metric for
	// load_reporting_interval.
	TotalMetricValue float64 `protobuf:"fixed64,2,opt,name=total_metric_value,json=totalMetricValue,proto3" json:"total_metric_value,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *UnnamedEndpointLoadMetricStats) Reset() {
	*x = UnnamedEndpointLoadMetricStats{}
	mi := &file_envoy_config_endpoint_v3_load_report_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UnnamedEndpointLoadMetricStats) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UnnamedEndpointLoadMetricStats) ProtoMessage() {}

func (x *UnnamedEndpointLoadMetricStats) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_endpoint_v3_load_report_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UnnamedEndpointLoadMetricStats.ProtoReflect.Descriptor instead.
func (*UnnamedEndpointLoadMetricStats) Descriptor() ([]byte, []int) {
	return file_envoy_config_endpoint_v3_load_report_proto_rawDescGZIP(), []int{3}
}

func (x *UnnamedEndpointLoadMetricStats) GetNumRequestsFinishedWithMetric() uint64 {
	if x != nil {
		return x.NumRequestsFinishedWithMetric
	}
	return 0
}

func (x *UnnamedEndpointLoadMetricStats) GetTotalMetricValue() float64 {
	if x != nil {
		return x.TotalMetricValue
	}
	return 0
}

// Per cluster load stats. Envoy reports these stats a management server in a
// :ref:`LoadStatsRequest<envoy_v3_api_msg_service.load_stats.v3.LoadStatsRequest>`
// Next ID: 7
// [#next-free-field: 7]
type ClusterStats struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name of the cluster.
	ClusterName string `protobuf:"bytes,1,opt,name=cluster_name,json=clusterName,proto3" json:"cluster_name,omitempty"`
	// The eds_cluster_config service_name of the cluster.
	// It's possible that two clusters send the same service_name to EDS,
	// in that case, the management server is supposed to do aggregation on the load reports.
	ClusterServiceName string `protobuf:"bytes,6,opt,name=cluster_service_name,json=clusterServiceName,proto3" json:"cluster_service_name,omitempty"`
	// Need at least one.
	UpstreamLocalityStats []*UpstreamLocalityStats `protobuf:"bytes,2,rep,name=upstream_locality_stats,json=upstreamLocalityStats,proto3" json:"upstream_locality_stats,omitempty"`
	// Cluster-level stats such as total_successful_requests may be computed by
	// summing upstream_locality_stats. In addition, below there are additional
	// cluster-wide stats.
	//
	// The total number of dropped requests. This covers requests
	// deliberately dropped by the drop_overload policy and circuit breaking.
	TotalDroppedRequests uint64 `protobuf:"varint,3,opt,name=total_dropped_requests,json=totalDroppedRequests,proto3" json:"total_dropped_requests,omitempty"`
	// Information about deliberately dropped requests for each category specified
	// in the DropOverload policy.
	DroppedRequests []*ClusterStats_DroppedRequests `protobuf:"bytes,5,rep,name=dropped_requests,json=droppedRequests,proto3" json:"dropped_requests,omitempty"`
	// Period over which the actual load report occurred. This will be guaranteed to include every
	// request reported. Due to system load and delays between the “LoadStatsRequest“ sent from Envoy
	// and the “LoadStatsResponse“ message sent from the management server, this may be longer than
	// the requested load reporting interval in the “LoadStatsResponse“.
	LoadReportInterval *durationpb.Duration `protobuf:"bytes,4,opt,name=load_report_interval,json=loadReportInterval,proto3" json:"load_report_interval,omitempty"`
	unknownFields      protoimpl.UnknownFields
	sizeCache          protoimpl.SizeCache
}

func (x *ClusterStats) Reset() {
	*x = ClusterStats{}
	mi := &file_envoy_config_endpoint_v3_load_report_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClusterStats) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClusterStats) ProtoMessage() {}

func (x *ClusterStats) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_endpoint_v3_load_report_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClusterStats.ProtoReflect.Descriptor instead.
func (*ClusterStats) Descriptor() ([]byte, []int) {
	return file_envoy_config_endpoint_v3_load_report_proto_rawDescGZIP(), []int{4}
}

func (x *ClusterStats) GetClusterName() string {
	if x != nil {
		return x.ClusterName
	}
	return ""
}

func (x *ClusterStats) GetClusterServiceName() string {
	if x != nil {
		return x.ClusterServiceName
	}
	return ""
}

func (x *ClusterStats) GetUpstreamLocalityStats() []*UpstreamLocalityStats {
	if x != nil {
		return x.UpstreamLocalityStats
	}
	return nil
}

func (x *ClusterStats) GetTotalDroppedRequests() uint64 {
	if x != nil {
		return x.TotalDroppedRequests
	}
	return 0
}

func (x *ClusterStats) GetDroppedRequests() []*ClusterStats_DroppedRequests {
	if x != nil {
		return x.DroppedRequests
	}
	return nil
}

func (x *ClusterStats) GetLoadReportInterval() *durationpb.Duration {
	if x != nil {
		return x.LoadReportInterval
	}
	return nil
}

type ClusterStats_DroppedRequests struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Identifier for the policy specifying the drop.
	Category string `protobuf:"bytes,1,opt,name=category,proto3" json:"category,omitempty"`
	// Total number of deliberately dropped requests for the category.
	DroppedCount  uint64 `protobuf:"varint,2,opt,name=dropped_count,json=droppedCount,proto3" json:"dropped_count,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ClusterStats_DroppedRequests) Reset() {
	*x = ClusterStats_DroppedRequests{}
	mi := &file_envoy_config_endpoint_v3_load_report_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClusterStats_DroppedRequests) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClusterStats_DroppedRequests) ProtoMessage() {}

func (x *ClusterStats_DroppedRequests) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_endpoint_v3_load_report_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClusterStats_DroppedRequests.ProtoReflect.Descriptor instead.
func (*ClusterStats_DroppedRequests) Descriptor() ([]byte, []int) {
	return file_envoy_config_endpoint_v3_load_report_proto_rawDescGZIP(), []int{4, 0}
}

func (x *ClusterStats_DroppedRequests) GetCategory() string {
	if x != nil {
		return x.Category
	}
	return ""
}

func (x *ClusterStats_DroppedRequests) GetDroppedCount() uint64 {
	if x != nil {
		return x.DroppedCount
	}
	return 0
}

var File_envoy_config_endpoint_v3_load_report_proto protoreflect.FileDescriptor

const file_envoy_config_endpoint_v3_load_report_proto_rawDesc = "" +
	"\n" +
	"*envoy/config/endpoint/v3/load_report.proto\x12\x18envoy.config.endpoint.v3\x1a\"envoy/config/core/v3/address.proto\x1a\x1fenvoy/config/core/v3/base.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xc5\b\n" +
	"\x15UpstreamLocalityStats\x12:\n" +
	"\blocality\x18\x01 \x01(\v2\x1e.envoy.config.core.v3.LocalityR\blocality\x12:\n" +
	"\x19total_successful_requests\x18\x02 \x01(\x04R\x17totalSuccessfulRequests\x12;\n" +
	"\x1atotal_requests_in_progress\x18\x03 \x01(\x04R\x17totalRequestsInProgress\x120\n" +
	"\x14total_error_requests\x18\x04 \x01(\x04R\x12totalErrorRequests\x122\n" +
	"\x15total_issued_requests\x18\b \x01(\x04R\x13totalIssuedRequests\x12B\n" +
	"\x18total_active_connections\x18\t \x01(\x04B\b\xd2Ƥ\xe1\x06\x02\b\x01R\x16totalActiveConnections\x12<\n" +
	"\x15total_new_connections\x18\n" +
	" \x01(\x04B\b\xd2Ƥ\xe1\x06\x02\b\x01R\x13totalNewConnections\x12>\n" +
	"\x16total_fail_connections\x18\v \x01(\x04B\b\xd2Ƥ\xe1\x06\x02\b\x01R\x14totalFailConnections\x12a\n" +
	"\x0fcpu_utilization\x18\f \x01(\v28.envoy.config.endpoint.v3.UnnamedEndpointLoadMetricStatsR\x0ecpuUtilization\x12a\n" +
	"\x0fmem_utilization\x18\r \x01(\v28.envoy.config.endpoint.v3.UnnamedEndpointLoadMetricStatsR\x0ememUtilization\x12q\n" +
	"\x17application_utilization\x18\x0e \x01(\v28.envoy.config.endpoint.v3.UnnamedEndpointLoadMetricStatsR\x16applicationUtilization\x12]\n" +
	"\x11load_metric_stats\x18\x05 \x03(\v21.envoy.config.endpoint.v3.EndpointLoadMetricStatsR\x0floadMetricStats\x12g\n" +
	"\x17upstream_endpoint_stats\x18\a \x03(\v2/.envoy.config.endpoint.v3.UpstreamEndpointStatsR\x15upstreamEndpointStats\x12\x1a\n" +
	"\bpriority\x18\x06 \x01(\rR\bpriority:2\x9aň\x1e-\n" +
	"+envoy.api.v2.endpoint.UpstreamLocalityStats\"\xf7\x03\n" +
	"\x15UpstreamEndpointStats\x127\n" +
	"\aaddress\x18\x01 \x01(\v2\x1d.envoy.config.core.v3.AddressR\aaddress\x123\n" +
	"\bmetadata\x18\x06 \x01(\v2\x17.google.protobuf.StructR\bmetadata\x12:\n" +
	"\x19total_successful_requests\x18\x02 \x01(\x04R\x17totalSuccessfulRequests\x12;\n" +
	"\x1atotal_requests_in_progress\x18\x03 \x01(\x04R\x17totalRequestsInProgress\x120\n" +
	"\x14total_error_requests\x18\x04 \x01(\x04R\x12totalErrorRequests\x122\n" +
	"\x15total_issued_requests\x18\a \x01(\x04R\x13totalIssuedRequests\x12]\n" +
	"\x11load_metric_stats\x18\x05 \x03(\v21.envoy.config.endpoint.v3.EndpointLoadMetricStatsR\x0floadMetricStats:2\x9aň\x1e-\n" +
	"+envoy.api.v2.endpoint.UpstreamEndpointStats\"\xe8\x01\n" +
	"\x17EndpointLoadMetricStats\x12\x1f\n" +
	"\vmetric_name\x18\x01 \x01(\tR\n" +
	"metricName\x12H\n" +
	"!num_requests_finished_with_metric\x18\x02 \x01(\x04R\x1dnumRequestsFinishedWithMetric\x12,\n" +
	"\x12total_metric_value\x18\x03 \x01(\x01R\x10totalMetricValue:4\x9aň\x1e/\n" +
	"-envoy.api.v2.endpoint.EndpointLoadMetricStats\"\x98\x01\n" +
	"\x1eUnnamedEndpointLoadMetricStats\x12H\n" +
	"!num_requests_finished_with_metric\x18\x01 \x01(\x04R\x1dnumRequestsFinishedWithMetric\x12,\n" +
	"\x12total_metric_value\x18\x02 \x01(\x01R\x10totalMetricValue\"\x89\x05\n" +
	"\fClusterStats\x12*\n" +
	"\fcluster_name\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\vclusterName\x120\n" +
	"\x14cluster_service_name\x18\x06 \x01(\tR\x12clusterServiceName\x12q\n" +
	"\x17upstream_locality_stats\x18\x02 \x03(\v2/.envoy.config.endpoint.v3.UpstreamLocalityStatsB\b\xfaB\x05\x92\x01\x02\b\x01R\x15upstreamLocalityStats\x124\n" +
	"\x16total_dropped_requests\x18\x03 \x01(\x04R\x14totalDroppedRequests\x12a\n" +
	"\x10dropped_requests\x18\x05 \x03(\v26.envoy.config.endpoint.v3.ClusterStats.DroppedRequestsR\x0fdroppedRequests\x12K\n" +
	"\x14load_report_interval\x18\x04 \x01(\v2\x19.google.protobuf.DurationR\x12loadReportInterval\x1a\x96\x01\n" +
	"\x0fDroppedRequests\x12#\n" +
	"\bcategory\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\bcategory\x12#\n" +
	"\rdropped_count\x18\x02 \x01(\x04R\fdroppedCount:9\x9aň\x1e4\n" +
	"2envoy.api.v2.endpoint.ClusterStats.DroppedRequests:)\x9aň\x1e$\n" +
	"\"envoy.api.v2.endpoint.ClusterStatsB\x8f\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"&io.envoyproxy.envoy.config.endpoint.v3B\x0fLoadReportProtoP\x01ZJgithub.com/envoyproxy/go-control-plane/envoy/config/endpoint/v3;endpointv3b\x06proto3"

var (
	file_envoy_config_endpoint_v3_load_report_proto_rawDescOnce sync.Once
	file_envoy_config_endpoint_v3_load_report_proto_rawDescData []byte
)

func file_envoy_config_endpoint_v3_load_report_proto_rawDescGZIP() []byte {
	file_envoy_config_endpoint_v3_load_report_proto_rawDescOnce.Do(func() {
		file_envoy_config_endpoint_v3_load_report_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_endpoint_v3_load_report_proto_rawDesc), len(file_envoy_config_endpoint_v3_load_report_proto_rawDesc)))
	})
	return file_envoy_config_endpoint_v3_load_report_proto_rawDescData
}

var file_envoy_config_endpoint_v3_load_report_proto_msgTypes = make([]protoimpl.MessageInfo, 6)
var file_envoy_config_endpoint_v3_load_report_proto_goTypes = []any{
	(*UpstreamLocalityStats)(nil),          // 0: envoy.config.endpoint.v3.UpstreamLocalityStats
	(*UpstreamEndpointStats)(nil),          // 1: envoy.config.endpoint.v3.UpstreamEndpointStats
	(*EndpointLoadMetricStats)(nil),        // 2: envoy.config.endpoint.v3.EndpointLoadMetricStats
	(*UnnamedEndpointLoadMetricStats)(nil), // 3: envoy.config.endpoint.v3.UnnamedEndpointLoadMetricStats
	(*ClusterStats)(nil),                   // 4: envoy.config.endpoint.v3.ClusterStats
	(*ClusterStats_DroppedRequests)(nil),   // 5: envoy.config.endpoint.v3.ClusterStats.DroppedRequests
	(*v3.Locality)(nil),                    // 6: envoy.config.core.v3.Locality
	(*v3.Address)(nil),                     // 7: envoy.config.core.v3.Address
	(*structpb.Struct)(nil),                // 8: google.protobuf.Struct
	(*durationpb.Duration)(nil),            // 9: google.protobuf.Duration
}
var file_envoy_config_endpoint_v3_load_report_proto_depIdxs = []int32{
	6,  // 0: envoy.config.endpoint.v3.UpstreamLocalityStats.locality:type_name -> envoy.config.core.v3.Locality
	3,  // 1: envoy.config.endpoint.v3.UpstreamLocalityStats.cpu_utilization:type_name -> envoy.config.endpoint.v3.UnnamedEndpointLoadMetricStats
	3,  // 2: envoy.config.endpoint.v3.UpstreamLocalityStats.mem_utilization:type_name -> envoy.config.endpoint.v3.UnnamedEndpointLoadMetricStats
	3,  // 3: envoy.config.endpoint.v3.UpstreamLocalityStats.application_utilization:type_name -> envoy.config.endpoint.v3.UnnamedEndpointLoadMetricStats
	2,  // 4: envoy.config.endpoint.v3.UpstreamLocalityStats.load_metric_stats:type_name -> envoy.config.endpoint.v3.EndpointLoadMetricStats
	1,  // 5: envoy.config.endpoint.v3.UpstreamLocalityStats.upstream_endpoint_stats:type_name -> envoy.config.endpoint.v3.UpstreamEndpointStats
	7,  // 6: envoy.config.endpoint.v3.UpstreamEndpointStats.address:type_name -> envoy.config.core.v3.Address
	8,  // 7: envoy.config.endpoint.v3.UpstreamEndpointStats.metadata:type_name -> google.protobuf.Struct
	2,  // 8: envoy.config.endpoint.v3.UpstreamEndpointStats.load_metric_stats:type_name -> envoy.config.endpoint.v3.EndpointLoadMetricStats
	0,  // 9: envoy.config.endpoint.v3.ClusterStats.upstream_locality_stats:type_name -> envoy.config.endpoint.v3.UpstreamLocalityStats
	5,  // 10: envoy.config.endpoint.v3.ClusterStats.dropped_requests:type_name -> envoy.config.endpoint.v3.ClusterStats.DroppedRequests
	9,  // 11: envoy.config.endpoint.v3.ClusterStats.load_report_interval:type_name -> google.protobuf.Duration
	12, // [12:12] is the sub-list for method output_type
	12, // [12:12] is the sub-list for method input_type
	12, // [12:12] is the sub-list for extension type_name
	12, // [12:12] is the sub-list for extension extendee
	0,  // [0:12] is the sub-list for field type_name
}

func init() { file_envoy_config_endpoint_v3_load_report_proto_init() }
func file_envoy_config_endpoint_v3_load_report_proto_init() {
	if File_envoy_config_endpoint_v3_load_report_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_endpoint_v3_load_report_proto_rawDesc), len(file_envoy_config_endpoint_v3_load_report_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   6,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_endpoint_v3_load_report_proto_goTypes,
		DependencyIndexes: file_envoy_config_endpoint_v3_load_report_proto_depIdxs,
		MessageInfos:      file_envoy_config_endpoint_v3_load_report_proto_msgTypes,
	}.Build()
	File_envoy_config_endpoint_v3_load_report_proto = out.File
	file_envoy_config_endpoint_v3_load_report_proto_goTypes = nil
	file_envoy_config_endpoint_v3_load_report_proto_depIdxs = nil
}
