// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11-devel
// 	protoc        (unknown)
// source: resource/definitions/runtime/runtime.proto

package runtime

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"

	common "github.com/siderolabs/talos/pkg/machinery/api/common"
	enums "github.com/siderolabs/talos/pkg/machinery/api/resource/definitions/enums"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// APIServiceConfigSpec describes configuration for Talos API service (apid).
type APIServiceConfigSpec struct {
	state                   protoimpl.MessageState `protogen:"open.v1"`
	ListenAddress           string                 `protobuf:"bytes,1,opt,name=listen_address,json=listenAddress,proto3" json:"listen_address,omitempty"`
	NodeRoutingDisabled     bool                   `protobuf:"varint,2,opt,name=node_routing_disabled,json=nodeRoutingDisabled,proto3" json:"node_routing_disabled,omitempty"`
	ReadonlyRoleMode        bool                   `protobuf:"varint,3,opt,name=readonly_role_mode,json=readonlyRoleMode,proto3" json:"readonly_role_mode,omitempty"`
	SkipVerifyingClientCert bool                   `protobuf:"varint,4,opt,name=skip_verifying_client_cert,json=skipVerifyingClientCert,proto3" json:"skip_verifying_client_cert,omitempty"`
	unknownFields           protoimpl.UnknownFields
	sizeCache               protoimpl.SizeCache
}

func (x *APIServiceConfigSpec) Reset() {
	*x = APIServiceConfigSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *APIServiceConfigSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*APIServiceConfigSpec) ProtoMessage() {}

func (x *APIServiceConfigSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use APIServiceConfigSpec.ProtoReflect.Descriptor instead.
func (*APIServiceConfigSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{0}
}

func (x *APIServiceConfigSpec) GetListenAddress() string {
	if x != nil {
		return x.ListenAddress
	}
	return ""
}

func (x *APIServiceConfigSpec) GetNodeRoutingDisabled() bool {
	if x != nil {
		return x.NodeRoutingDisabled
	}
	return false
}

func (x *APIServiceConfigSpec) GetReadonlyRoleMode() bool {
	if x != nil {
		return x.ReadonlyRoleMode
	}
	return false
}

func (x *APIServiceConfigSpec) GetSkipVerifyingClientCert() bool {
	if x != nil {
		return x.SkipVerifyingClientCert
	}
	return false
}

// BootedEntrySpec describes the booted entry resource properties.
type BootedEntrySpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	BootedEntry   string                 `protobuf:"bytes,1,opt,name=booted_entry,json=bootedEntry,proto3" json:"booted_entry,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *BootedEntrySpec) Reset() {
	*x = BootedEntrySpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *BootedEntrySpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*BootedEntrySpec) ProtoMessage() {}

func (x *BootedEntrySpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use BootedEntrySpec.ProtoReflect.Descriptor instead.
func (*BootedEntrySpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{1}
}

func (x *BootedEntrySpec) GetBootedEntry() string {
	if x != nil {
		return x.BootedEntry
	}
	return ""
}

// DevicesStatusSpec is the spec for devices status.
type DevicesStatusSpec struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Devices are settled down and ready to be used.
	Ready         bool `protobuf:"varint,1,opt,name=ready,proto3" json:"ready,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DevicesStatusSpec) Reset() {
	*x = DevicesStatusSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DevicesStatusSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DevicesStatusSpec) ProtoMessage() {}

func (x *DevicesStatusSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DevicesStatusSpec.ProtoReflect.Descriptor instead.
func (*DevicesStatusSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{2}
}

func (x *DevicesStatusSpec) GetReady() bool {
	if x != nil {
		return x.Ready
	}
	return false
}

// DiagnosticSpec is the spec for devices status.
type DiagnosticSpec struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Short message describing the problem.
	Message string `protobuf:"bytes,1,opt,name=message,proto3" json:"message,omitempty"`
	// Details about the problem.
	Details       []string `protobuf:"bytes,2,rep,name=details,proto3" json:"details,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DiagnosticSpec) Reset() {
	*x = DiagnosticSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DiagnosticSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DiagnosticSpec) ProtoMessage() {}

func (x *DiagnosticSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DiagnosticSpec.ProtoReflect.Descriptor instead.
func (*DiagnosticSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{3}
}

func (x *DiagnosticSpec) GetMessage() string {
	if x != nil {
		return x.Message
	}
	return ""
}

func (x *DiagnosticSpec) GetDetails() []string {
	if x != nil {
		return x.Details
	}
	return nil
}

// EnvironmentSpec describes the specification of Environment resource.
type EnvironmentSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Variables     []string               `protobuf:"bytes,1,rep,name=variables,proto3" json:"variables,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EnvironmentSpec) Reset() {
	*x = EnvironmentSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EnvironmentSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EnvironmentSpec) ProtoMessage() {}

func (x *EnvironmentSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EnvironmentSpec.ProtoReflect.Descriptor instead.
func (*EnvironmentSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{4}
}

func (x *EnvironmentSpec) GetVariables() []string {
	if x != nil {
		return x.Variables
	}
	return nil
}

// EventSinkConfigSpec describes configuration of Talos event log streaming.
type EventSinkConfigSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Endpoint      string                 `protobuf:"bytes,1,opt,name=endpoint,proto3" json:"endpoint,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EventSinkConfigSpec) Reset() {
	*x = EventSinkConfigSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EventSinkConfigSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EventSinkConfigSpec) ProtoMessage() {}

func (x *EventSinkConfigSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EventSinkConfigSpec.ProtoReflect.Descriptor instead.
func (*EventSinkConfigSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{5}
}

func (x *EventSinkConfigSpec) GetEndpoint() string {
	if x != nil {
		return x.Endpoint
	}
	return ""
}

// ExtensionServiceConfigFile describes extensions service config files.
type ExtensionServiceConfigFile struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Content       string                 `protobuf:"bytes,1,opt,name=content,proto3" json:"content,omitempty"`
	MountPath     string                 `protobuf:"bytes,2,opt,name=mount_path,json=mountPath,proto3" json:"mount_path,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ExtensionServiceConfigFile) Reset() {
	*x = ExtensionServiceConfigFile{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ExtensionServiceConfigFile) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ExtensionServiceConfigFile) ProtoMessage() {}

func (x *ExtensionServiceConfigFile) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ExtensionServiceConfigFile.ProtoReflect.Descriptor instead.
func (*ExtensionServiceConfigFile) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{6}
}

func (x *ExtensionServiceConfigFile) GetContent() string {
	if x != nil {
		return x.Content
	}
	return ""
}

func (x *ExtensionServiceConfigFile) GetMountPath() string {
	if x != nil {
		return x.MountPath
	}
	return ""
}

// ExtensionServiceConfigSpec describes status of rendered extensions service config files.
type ExtensionServiceConfigSpec struct {
	state         protoimpl.MessageState        `protogen:"open.v1"`
	Files         []*ExtensionServiceConfigFile `protobuf:"bytes,1,rep,name=files,proto3" json:"files,omitempty"`
	Environment   []string                      `protobuf:"bytes,2,rep,name=environment,proto3" json:"environment,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ExtensionServiceConfigSpec) Reset() {
	*x = ExtensionServiceConfigSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ExtensionServiceConfigSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ExtensionServiceConfigSpec) ProtoMessage() {}

func (x *ExtensionServiceConfigSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ExtensionServiceConfigSpec.ProtoReflect.Descriptor instead.
func (*ExtensionServiceConfigSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{7}
}

func (x *ExtensionServiceConfigSpec) GetFiles() []*ExtensionServiceConfigFile {
	if x != nil {
		return x.Files
	}
	return nil
}

func (x *ExtensionServiceConfigSpec) GetEnvironment() []string {
	if x != nil {
		return x.Environment
	}
	return nil
}

// ExtensionServiceConfigStatusSpec describes status of rendered extensions service config files.
type ExtensionServiceConfigStatusSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	SpecVersion   string                 `protobuf:"bytes,1,opt,name=spec_version,json=specVersion,proto3" json:"spec_version,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ExtensionServiceConfigStatusSpec) Reset() {
	*x = ExtensionServiceConfigStatusSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[8]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ExtensionServiceConfigStatusSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ExtensionServiceConfigStatusSpec) ProtoMessage() {}

func (x *ExtensionServiceConfigStatusSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[8]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ExtensionServiceConfigStatusSpec.ProtoReflect.Descriptor instead.
func (*ExtensionServiceConfigStatusSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{8}
}

func (x *ExtensionServiceConfigStatusSpec) GetSpecVersion() string {
	if x != nil {
		return x.SpecVersion
	}
	return ""
}

// KernelCmdlineSpec presents kernel command line (contents of /proc/cmdline).
type KernelCmdlineSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Cmdline       string                 `protobuf:"bytes,1,opt,name=cmdline,proto3" json:"cmdline,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *KernelCmdlineSpec) Reset() {
	*x = KernelCmdlineSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[9]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *KernelCmdlineSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*KernelCmdlineSpec) ProtoMessage() {}

func (x *KernelCmdlineSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[9]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use KernelCmdlineSpec.ProtoReflect.Descriptor instead.
func (*KernelCmdlineSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{9}
}

func (x *KernelCmdlineSpec) GetCmdline() string {
	if x != nil {
		return x.Cmdline
	}
	return ""
}

// KernelModuleSpecSpec describes Linux kernel module to load.
type KernelModuleSpecSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Name          string                 `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	Parameters    []string               `protobuf:"bytes,2,rep,name=parameters,proto3" json:"parameters,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *KernelModuleSpecSpec) Reset() {
	*x = KernelModuleSpecSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[10]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *KernelModuleSpecSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*KernelModuleSpecSpec) ProtoMessage() {}

func (x *KernelModuleSpecSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[10]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use KernelModuleSpecSpec.ProtoReflect.Descriptor instead.
func (*KernelModuleSpecSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{10}
}

func (x *KernelModuleSpecSpec) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *KernelModuleSpecSpec) GetParameters() []string {
	if x != nil {
		return x.Parameters
	}
	return nil
}

// KernelModuleStatusSpec represents the status of a Linux kernel module.
type KernelModuleStatusSpec struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Type indicates whether the kernel module is built-in or dynamically loaded.
	Type enums.RuntimeKernelModuleType `protobuf:"varint,1,opt,name=type,proto3,enum=talos.resource.definitions.enums.RuntimeKernelModuleType" json:"type,omitempty"`
	// Size is the size of the kernel module in bytes.
	Size int64 `protobuf:"varint,2,opt,name=size,proto3" json:"size,omitempty"`
	// ReferenceCount is the number of references to this kernel module.
	ReferenceCount int64 `protobuf:"varint,3,opt,name=reference_count,json=referenceCount,proto3" json:"reference_count,omitempty"`
	// Dependencies lists the names of kernel modules that this module depends on.
	Dependencies []string `protobuf:"bytes,4,rep,name=dependencies,proto3" json:"dependencies,omitempty"`
	// State is the operational state of the kernel module.
	State enums.RuntimeKernelModuleState `protobuf:"varint,5,opt,name=state,proto3,enum=talos.resource.definitions.enums.RuntimeKernelModuleState" json:"state,omitempty"`
	// Address is the memory address where the kernel module is loaded (if applicable).
	Address       string `protobuf:"bytes,6,opt,name=address,proto3" json:"address,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *KernelModuleStatusSpec) Reset() {
	*x = KernelModuleStatusSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[11]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *KernelModuleStatusSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*KernelModuleStatusSpec) ProtoMessage() {}

func (x *KernelModuleStatusSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[11]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use KernelModuleStatusSpec.ProtoReflect.Descriptor instead.
func (*KernelModuleStatusSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{11}
}

func (x *KernelModuleStatusSpec) GetType() enums.RuntimeKernelModuleType {
	if x != nil {
		return x.Type
	}
	return enums.RuntimeKernelModuleType(0)
}

func (x *KernelModuleStatusSpec) GetSize() int64 {
	if x != nil {
		return x.Size
	}
	return 0
}

func (x *KernelModuleStatusSpec) GetReferenceCount() int64 {
	if x != nil {
		return x.ReferenceCount
	}
	return 0
}

func (x *KernelModuleStatusSpec) GetDependencies() []string {
	if x != nil {
		return x.Dependencies
	}
	return nil
}

func (x *KernelModuleStatusSpec) GetState() enums.RuntimeKernelModuleState {
	if x != nil {
		return x.State
	}
	return enums.RuntimeKernelModuleState(0)
}

func (x *KernelModuleStatusSpec) GetAddress() string {
	if x != nil {
		return x.Address
	}
	return ""
}

// KernelParamSpecSpec describes status of the defined sysctls.
type KernelParamSpecSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Value         string                 `protobuf:"bytes,1,opt,name=value,proto3" json:"value,omitempty"`
	IgnoreErrors  bool                   `protobuf:"varint,2,opt,name=ignore_errors,json=ignoreErrors,proto3" json:"ignore_errors,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *KernelParamSpecSpec) Reset() {
	*x = KernelParamSpecSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[12]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *KernelParamSpecSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*KernelParamSpecSpec) ProtoMessage() {}

func (x *KernelParamSpecSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[12]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use KernelParamSpecSpec.ProtoReflect.Descriptor instead.
func (*KernelParamSpecSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{12}
}

func (x *KernelParamSpecSpec) GetValue() string {
	if x != nil {
		return x.Value
	}
	return ""
}

func (x *KernelParamSpecSpec) GetIgnoreErrors() bool {
	if x != nil {
		return x.IgnoreErrors
	}
	return false
}

// KernelParamStatusSpec describes status of the defined sysctls.
type KernelParamStatusSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Current       string                 `protobuf:"bytes,1,opt,name=current,proto3" json:"current,omitempty"`
	Default       string                 `protobuf:"bytes,2,opt,name=default,proto3" json:"default,omitempty"`
	Unsupported   bool                   `protobuf:"varint,3,opt,name=unsupported,proto3" json:"unsupported,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *KernelParamStatusSpec) Reset() {
	*x = KernelParamStatusSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[13]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *KernelParamStatusSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*KernelParamStatusSpec) ProtoMessage() {}

func (x *KernelParamStatusSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[13]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use KernelParamStatusSpec.ProtoReflect.Descriptor instead.
func (*KernelParamStatusSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{13}
}

func (x *KernelParamStatusSpec) GetCurrent() string {
	if x != nil {
		return x.Current
	}
	return ""
}

func (x *KernelParamStatusSpec) GetDefault() string {
	if x != nil {
		return x.Default
	}
	return ""
}

func (x *KernelParamStatusSpec) GetUnsupported() bool {
	if x != nil {
		return x.Unsupported
	}
	return false
}

// KmsgLogConfigSpec describes configuration for kmsg log streaming.
type KmsgLogConfigSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Destinations  []*common.URL          `protobuf:"bytes,1,rep,name=destinations,proto3" json:"destinations,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *KmsgLogConfigSpec) Reset() {
	*x = KmsgLogConfigSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[14]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *KmsgLogConfigSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*KmsgLogConfigSpec) ProtoMessage() {}

func (x *KmsgLogConfigSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[14]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use KmsgLogConfigSpec.ProtoReflect.Descriptor instead.
func (*KmsgLogConfigSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{14}
}

func (x *KmsgLogConfigSpec) GetDestinations() []*common.URL {
	if x != nil {
		return x.Destinations
	}
	return nil
}

// LoadedKernelModuleSpec describes loaded Linux kernel modules.
type LoadedKernelModuleSpec struct {
	state          protoimpl.MessageState `protogen:"open.v1"`
	Size           int64                  `protobuf:"varint,1,opt,name=size,proto3" json:"size,omitempty"`
	ReferenceCount int64                  `protobuf:"varint,2,opt,name=reference_count,json=referenceCount,proto3" json:"reference_count,omitempty"`
	Dependencies   []string               `protobuf:"bytes,3,rep,name=dependencies,proto3" json:"dependencies,omitempty"`
	State          string                 `protobuf:"bytes,4,opt,name=state,proto3" json:"state,omitempty"`
	Address        string                 `protobuf:"bytes,5,opt,name=address,proto3" json:"address,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *LoadedKernelModuleSpec) Reset() {
	*x = LoadedKernelModuleSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[15]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *LoadedKernelModuleSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*LoadedKernelModuleSpec) ProtoMessage() {}

func (x *LoadedKernelModuleSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[15]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use LoadedKernelModuleSpec.ProtoReflect.Descriptor instead.
func (*LoadedKernelModuleSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{15}
}

func (x *LoadedKernelModuleSpec) GetSize() int64 {
	if x != nil {
		return x.Size
	}
	return 0
}

func (x *LoadedKernelModuleSpec) GetReferenceCount() int64 {
	if x != nil {
		return x.ReferenceCount
	}
	return 0
}

func (x *LoadedKernelModuleSpec) GetDependencies() []string {
	if x != nil {
		return x.Dependencies
	}
	return nil
}

func (x *LoadedKernelModuleSpec) GetState() string {
	if x != nil {
		return x.State
	}
	return ""
}

func (x *LoadedKernelModuleSpec) GetAddress() string {
	if x != nil {
		return x.Address
	}
	return ""
}

// MachineStatusSpec describes status of the defined sysctls.
type MachineStatusSpec struct {
	state         protoimpl.MessageState    `protogen:"open.v1"`
	Stage         enums.RuntimeMachineStage `protobuf:"varint,1,opt,name=stage,proto3,enum=talos.resource.definitions.enums.RuntimeMachineStage" json:"stage,omitempty"`
	Status        *MachineStatusStatus      `protobuf:"bytes,2,opt,name=status,proto3" json:"status,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MachineStatusSpec) Reset() {
	*x = MachineStatusSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[16]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MachineStatusSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MachineStatusSpec) ProtoMessage() {}

func (x *MachineStatusSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[16]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MachineStatusSpec.ProtoReflect.Descriptor instead.
func (*MachineStatusSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{16}
}

func (x *MachineStatusSpec) GetStage() enums.RuntimeMachineStage {
	if x != nil {
		return x.Stage
	}
	return enums.RuntimeMachineStage(0)
}

func (x *MachineStatusSpec) GetStatus() *MachineStatusStatus {
	if x != nil {
		return x.Status
	}
	return nil
}

// MachineStatusStatus describes machine current status at the stage.
type MachineStatusStatus struct {
	state           protoimpl.MessageState `protogen:"open.v1"`
	Ready           bool                   `protobuf:"varint,1,opt,name=ready,proto3" json:"ready,omitempty"`
	UnmetConditions []*UnmetCondition      `protobuf:"bytes,2,rep,name=unmet_conditions,json=unmetConditions,proto3" json:"unmet_conditions,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *MachineStatusStatus) Reset() {
	*x = MachineStatusStatus{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[17]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MachineStatusStatus) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MachineStatusStatus) ProtoMessage() {}

func (x *MachineStatusStatus) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[17]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MachineStatusStatus.ProtoReflect.Descriptor instead.
func (*MachineStatusStatus) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{17}
}

func (x *MachineStatusStatus) GetReady() bool {
	if x != nil {
		return x.Ready
	}
	return false
}

func (x *MachineStatusStatus) GetUnmetConditions() []*UnmetCondition {
	if x != nil {
		return x.UnmetConditions
	}
	return nil
}

// MaintenanceServiceConfigSpec describes configuration for maintenance service API.
type MaintenanceServiceConfigSpec struct {
	state              protoimpl.MessageState `protogen:"open.v1"`
	ListenAddress      string                 `protobuf:"bytes,1,opt,name=listen_address,json=listenAddress,proto3" json:"listen_address,omitempty"`
	ReachableAddresses []*common.NetIP        `protobuf:"bytes,2,rep,name=reachable_addresses,json=reachableAddresses,proto3" json:"reachable_addresses,omitempty"`
	unknownFields      protoimpl.UnknownFields
	sizeCache          protoimpl.SizeCache
}

func (x *MaintenanceServiceConfigSpec) Reset() {
	*x = MaintenanceServiceConfigSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[18]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MaintenanceServiceConfigSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MaintenanceServiceConfigSpec) ProtoMessage() {}

func (x *MaintenanceServiceConfigSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[18]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MaintenanceServiceConfigSpec.ProtoReflect.Descriptor instead.
func (*MaintenanceServiceConfigSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{18}
}

func (x *MaintenanceServiceConfigSpec) GetListenAddress() string {
	if x != nil {
		return x.ListenAddress
	}
	return ""
}

func (x *MaintenanceServiceConfigSpec) GetReachableAddresses() []*common.NetIP {
	if x != nil {
		return x.ReachableAddresses
	}
	return nil
}

// MetaKeySpec describes status of the defined sysctls.
type MetaKeySpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Value         string                 `protobuf:"bytes,1,opt,name=value,proto3" json:"value,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MetaKeySpec) Reset() {
	*x = MetaKeySpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[19]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MetaKeySpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MetaKeySpec) ProtoMessage() {}

func (x *MetaKeySpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[19]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MetaKeySpec.ProtoReflect.Descriptor instead.
func (*MetaKeySpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{19}
}

func (x *MetaKeySpec) GetValue() string {
	if x != nil {
		return x.Value
	}
	return ""
}

// MetaLoadedSpec is the spec for meta loaded. The Done field is always true when resource exists.
type MetaLoadedSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Done          bool                   `protobuf:"varint,1,opt,name=done,proto3" json:"done,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MetaLoadedSpec) Reset() {
	*x = MetaLoadedSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[20]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MetaLoadedSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MetaLoadedSpec) ProtoMessage() {}

func (x *MetaLoadedSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[20]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MetaLoadedSpec.ProtoReflect.Descriptor instead.
func (*MetaLoadedSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{20}
}

func (x *MetaLoadedSpec) GetDone() bool {
	if x != nil {
		return x.Done
	}
	return false
}

// MountStatusSpec describes status of the defined sysctls.
type MountStatusSpec struct {
	state               protoimpl.MessageState `protogen:"open.v1"`
	Source              string                 `protobuf:"bytes,1,opt,name=source,proto3" json:"source,omitempty"`
	Target              string                 `protobuf:"bytes,2,opt,name=target,proto3" json:"target,omitempty"`
	FilesystemType      string                 `protobuf:"bytes,3,opt,name=filesystem_type,json=filesystemType,proto3" json:"filesystem_type,omitempty"`
	Options             []string               `protobuf:"bytes,4,rep,name=options,proto3" json:"options,omitempty"`
	Encrypted           bool                   `protobuf:"varint,5,opt,name=encrypted,proto3" json:"encrypted,omitempty"`
	EncryptionProviders []string               `protobuf:"bytes,6,rep,name=encryption_providers,json=encryptionProviders,proto3" json:"encryption_providers,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *MountStatusSpec) Reset() {
	*x = MountStatusSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[21]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MountStatusSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MountStatusSpec) ProtoMessage() {}

func (x *MountStatusSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[21]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MountStatusSpec.ProtoReflect.Descriptor instead.
func (*MountStatusSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{21}
}

func (x *MountStatusSpec) GetSource() string {
	if x != nil {
		return x.Source
	}
	return ""
}

func (x *MountStatusSpec) GetTarget() string {
	if x != nil {
		return x.Target
	}
	return ""
}

func (x *MountStatusSpec) GetFilesystemType() string {
	if x != nil {
		return x.FilesystemType
	}
	return ""
}

func (x *MountStatusSpec) GetOptions() []string {
	if x != nil {
		return x.Options
	}
	return nil
}

func (x *MountStatusSpec) GetEncrypted() bool {
	if x != nil {
		return x.Encrypted
	}
	return false
}

func (x *MountStatusSpec) GetEncryptionProviders() []string {
	if x != nil {
		return x.EncryptionProviders
	}
	return nil
}

// OOMActionSpec describes the OOM action record resource properties.
type OOMActionSpec struct {
	state          protoimpl.MessageState `protogen:"open.v1"`
	TriggerContext string                 `protobuf:"bytes,1,opt,name=trigger_context,json=triggerContext,proto3" json:"trigger_context,omitempty"`
	Score          float64                `protobuf:"fixed64,2,opt,name=score,proto3" json:"score,omitempty"`
	Processes      []string               `protobuf:"bytes,3,rep,name=processes,proto3" json:"processes,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *OOMActionSpec) Reset() {
	*x = OOMActionSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[22]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OOMActionSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OOMActionSpec) ProtoMessage() {}

func (x *OOMActionSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[22]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OOMActionSpec.ProtoReflect.Descriptor instead.
func (*OOMActionSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{22}
}

func (x *OOMActionSpec) GetTriggerContext() string {
	if x != nil {
		return x.TriggerContext
	}
	return ""
}

func (x *OOMActionSpec) GetScore() float64 {
	if x != nil {
		return x.Score
	}
	return 0
}

func (x *OOMActionSpec) GetProcesses() []string {
	if x != nil {
		return x.Processes
	}
	return nil
}

// PlatformMetadataSpec describes platform metadata properties.
type PlatformMetadataSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Platform      string                 `protobuf:"bytes,1,opt,name=platform,proto3" json:"platform,omitempty"`
	Hostname      string                 `protobuf:"bytes,2,opt,name=hostname,proto3" json:"hostname,omitempty"`
	Region        string                 `protobuf:"bytes,3,opt,name=region,proto3" json:"region,omitempty"`
	Zone          string                 `protobuf:"bytes,4,opt,name=zone,proto3" json:"zone,omitempty"`
	InstanceType  string                 `protobuf:"bytes,5,opt,name=instance_type,json=instanceType,proto3" json:"instance_type,omitempty"`
	InstanceId    string                 `protobuf:"bytes,6,opt,name=instance_id,json=instanceId,proto3" json:"instance_id,omitempty"`
	ProviderId    string                 `protobuf:"bytes,7,opt,name=provider_id,json=providerId,proto3" json:"provider_id,omitempty"`
	Spot          bool                   `protobuf:"varint,8,opt,name=spot,proto3" json:"spot,omitempty"`
	InternalDns   string                 `protobuf:"bytes,9,opt,name=internal_dns,json=internalDns,proto3" json:"internal_dns,omitempty"`
	ExternalDns   string                 `protobuf:"bytes,10,opt,name=external_dns,json=externalDns,proto3" json:"external_dns,omitempty"`
	Tags          map[string]string      `protobuf:"bytes,11,rep,name=tags,proto3" json:"tags,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PlatformMetadataSpec) Reset() {
	*x = PlatformMetadataSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[23]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PlatformMetadataSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PlatformMetadataSpec) ProtoMessage() {}

func (x *PlatformMetadataSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[23]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PlatformMetadataSpec.ProtoReflect.Descriptor instead.
func (*PlatformMetadataSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{23}
}

func (x *PlatformMetadataSpec) GetPlatform() string {
	if x != nil {
		return x.Platform
	}
	return ""
}

func (x *PlatformMetadataSpec) GetHostname() string {
	if x != nil {
		return x.Hostname
	}
	return ""
}

func (x *PlatformMetadataSpec) GetRegion() string {
	if x != nil {
		return x.Region
	}
	return ""
}

func (x *PlatformMetadataSpec) GetZone() string {
	if x != nil {
		return x.Zone
	}
	return ""
}

func (x *PlatformMetadataSpec) GetInstanceType() string {
	if x != nil {
		return x.InstanceType
	}
	return ""
}

func (x *PlatformMetadataSpec) GetInstanceId() string {
	if x != nil {
		return x.InstanceId
	}
	return ""
}

func (x *PlatformMetadataSpec) GetProviderId() string {
	if x != nil {
		return x.ProviderId
	}
	return ""
}

func (x *PlatformMetadataSpec) GetSpot() bool {
	if x != nil {
		return x.Spot
	}
	return false
}

func (x *PlatformMetadataSpec) GetInternalDns() string {
	if x != nil {
		return x.InternalDns
	}
	return ""
}

func (x *PlatformMetadataSpec) GetExternalDns() string {
	if x != nil {
		return x.ExternalDns
	}
	return ""
}

func (x *PlatformMetadataSpec) GetTags() map[string]string {
	if x != nil {
		return x.Tags
	}
	return nil
}

// SBOMItemSpec describes the SBOM item resource properties.
type SBOMItemSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Name          string                 `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	Version       string                 `protobuf:"bytes,2,opt,name=version,proto3" json:"version,omitempty"`
	License       string                 `protobuf:"bytes,3,opt,name=license,proto3" json:"license,omitempty"`
	CpEs          []string               `protobuf:"bytes,4,rep,name=cp_es,json=cpEs,proto3" json:"cp_es,omitempty"`
	PurLs         []string               `protobuf:"bytes,5,rep,name=pur_ls,json=purLs,proto3" json:"pur_ls,omitempty"`
	Extension     bool                   `protobuf:"varint,6,opt,name=extension,proto3" json:"extension,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *SBOMItemSpec) Reset() {
	*x = SBOMItemSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[24]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SBOMItemSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SBOMItemSpec) ProtoMessage() {}

func (x *SBOMItemSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[24]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SBOMItemSpec.ProtoReflect.Descriptor instead.
func (*SBOMItemSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{24}
}

func (x *SBOMItemSpec) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *SBOMItemSpec) GetVersion() string {
	if x != nil {
		return x.Version
	}
	return ""
}

func (x *SBOMItemSpec) GetLicense() string {
	if x != nil {
		return x.License
	}
	return ""
}

func (x *SBOMItemSpec) GetCpEs() []string {
	if x != nil {
		return x.CpEs
	}
	return nil
}

func (x *SBOMItemSpec) GetPurLs() []string {
	if x != nil {
		return x.PurLs
	}
	return nil
}

func (x *SBOMItemSpec) GetExtension() bool {
	if x != nil {
		return x.Extension
	}
	return false
}

// SecurityStateSpec describes the security state resource properties.
type SecurityStateSpec struct {
	state                    protoimpl.MessageState    `protogen:"open.v1"`
	SecureBoot               bool                      `protobuf:"varint,1,opt,name=secure_boot,json=secureBoot,proto3" json:"secure_boot,omitempty"`
	UkiSigningKeyFingerprint string                    `protobuf:"bytes,2,opt,name=uki_signing_key_fingerprint,json=ukiSigningKeyFingerprint,proto3" json:"uki_signing_key_fingerprint,omitempty"`
	PcrSigningKeyFingerprint string                    `protobuf:"bytes,3,opt,name=pcr_signing_key_fingerprint,json=pcrSigningKeyFingerprint,proto3" json:"pcr_signing_key_fingerprint,omitempty"`
	SeLinuxState             enums.RuntimeSELinuxState `protobuf:"varint,4,opt,name=se_linux_state,json=seLinuxState,proto3,enum=talos.resource.definitions.enums.RuntimeSELinuxState" json:"se_linux_state,omitempty"`
	BootedWithUki            bool                      `protobuf:"varint,5,opt,name=booted_with_uki,json=bootedWithUki,proto3" json:"booted_with_uki,omitempty"`
	FipsState                enums.RuntimeFIPSState    `protobuf:"varint,6,opt,name=fips_state,json=fipsState,proto3,enum=talos.resource.definitions.enums.RuntimeFIPSState" json:"fips_state,omitempty"`
	ModuleSignatureEnforced  bool                      `protobuf:"varint,7,opt,name=module_signature_enforced,json=moduleSignatureEnforced,proto3" json:"module_signature_enforced,omitempty"`
	unknownFields            protoimpl.UnknownFields
	sizeCache                protoimpl.SizeCache
}

func (x *SecurityStateSpec) Reset() {
	*x = SecurityStateSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[25]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SecurityStateSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SecurityStateSpec) ProtoMessage() {}

func (x *SecurityStateSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[25]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SecurityStateSpec.ProtoReflect.Descriptor instead.
func (*SecurityStateSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{25}
}

func (x *SecurityStateSpec) GetSecureBoot() bool {
	if x != nil {
		return x.SecureBoot
	}
	return false
}

func (x *SecurityStateSpec) GetUkiSigningKeyFingerprint() string {
	if x != nil {
		return x.UkiSigningKeyFingerprint
	}
	return ""
}

func (x *SecurityStateSpec) GetPcrSigningKeyFingerprint() string {
	if x != nil {
		return x.PcrSigningKeyFingerprint
	}
	return ""
}

func (x *SecurityStateSpec) GetSeLinuxState() enums.RuntimeSELinuxState {
	if x != nil {
		return x.SeLinuxState
	}
	return enums.RuntimeSELinuxState(0)
}

func (x *SecurityStateSpec) GetBootedWithUki() bool {
	if x != nil {
		return x.BootedWithUki
	}
	return false
}

func (x *SecurityStateSpec) GetFipsState() enums.RuntimeFIPSState {
	if x != nil {
		return x.FipsState
	}
	return enums.RuntimeFIPSState(0)
}

func (x *SecurityStateSpec) GetModuleSignatureEnforced() bool {
	if x != nil {
		return x.ModuleSignatureEnforced
	}
	return false
}

// ServicePIDSpec is the spec for the service PID.
type ServicePIDSpec struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// PID is the host PID of the service.
	Pid int32 `protobuf:"varint,1,opt,name=pid,proto3" json:"pid,omitempty"`
	// MountNamespace is the mount namespace of the service.
	MountNamespace string `protobuf:"bytes,2,opt,name=mount_namespace,json=mountNamespace,proto3" json:"mount_namespace,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *ServicePIDSpec) Reset() {
	*x = ServicePIDSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[26]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServicePIDSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServicePIDSpec) ProtoMessage() {}

func (x *ServicePIDSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[26]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServicePIDSpec.ProtoReflect.Descriptor instead.
func (*ServicePIDSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{26}
}

func (x *ServicePIDSpec) GetPid() int32 {
	if x != nil {
		return x.Pid
	}
	return 0
}

func (x *ServicePIDSpec) GetMountNamespace() string {
	if x != nil {
		return x.MountNamespace
	}
	return ""
}

// UniqueMachineTokenSpec is the spec for the machine unique token. Token can be empty if machine wasn't assigned any.
type UniqueMachineTokenSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Token         string                 `protobuf:"bytes,1,opt,name=token,proto3" json:"token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UniqueMachineTokenSpec) Reset() {
	*x = UniqueMachineTokenSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[27]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UniqueMachineTokenSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UniqueMachineTokenSpec) ProtoMessage() {}

func (x *UniqueMachineTokenSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[27]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UniqueMachineTokenSpec.ProtoReflect.Descriptor instead.
func (*UniqueMachineTokenSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{27}
}

func (x *UniqueMachineTokenSpec) GetToken() string {
	if x != nil {
		return x.Token
	}
	return ""
}

// UnmetCondition is a failure which prevents machine from being ready at the stage.
type UnmetCondition struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Name          string                 `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	Reason        string                 `protobuf:"bytes,2,opt,name=reason,proto3" json:"reason,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UnmetCondition) Reset() {
	*x = UnmetCondition{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[28]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UnmetCondition) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UnmetCondition) ProtoMessage() {}

func (x *UnmetCondition) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[28]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UnmetCondition.ProtoReflect.Descriptor instead.
func (*UnmetCondition) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{28}
}

func (x *UnmetCondition) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *UnmetCondition) GetReason() string {
	if x != nil {
		return x.Reason
	}
	return ""
}

// VersionSpec describes version of Talos.
type VersionSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Version       string                 `protobuf:"bytes,1,opt,name=version,proto3" json:"version,omitempty"`
	Name          string                 `protobuf:"bytes,2,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *VersionSpec) Reset() {
	*x = VersionSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[29]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *VersionSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*VersionSpec) ProtoMessage() {}

func (x *VersionSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[29]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use VersionSpec.ProtoReflect.Descriptor instead.
func (*VersionSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{29}
}

func (x *VersionSpec) GetVersion() string {
	if x != nil {
		return x.Version
	}
	return ""
}

func (x *VersionSpec) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

// WatchdogTimerConfigSpec describes configuration of watchdog timer.
type WatchdogTimerConfigSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Device        string                 `protobuf:"bytes,1,opt,name=device,proto3" json:"device,omitempty"`
	Timeout       *durationpb.Duration   `protobuf:"bytes,2,opt,name=timeout,proto3" json:"timeout,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *WatchdogTimerConfigSpec) Reset() {
	*x = WatchdogTimerConfigSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[30]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *WatchdogTimerConfigSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*WatchdogTimerConfigSpec) ProtoMessage() {}

func (x *WatchdogTimerConfigSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[30]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use WatchdogTimerConfigSpec.ProtoReflect.Descriptor instead.
func (*WatchdogTimerConfigSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{30}
}

func (x *WatchdogTimerConfigSpec) GetDevice() string {
	if x != nil {
		return x.Device
	}
	return ""
}

func (x *WatchdogTimerConfigSpec) GetTimeout() *durationpb.Duration {
	if x != nil {
		return x.Timeout
	}
	return nil
}

// WatchdogTimerStatusSpec describes configuration of watchdog timer.
type WatchdogTimerStatusSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Device        string                 `protobuf:"bytes,1,opt,name=device,proto3" json:"device,omitempty"`
	Timeout       *durationpb.Duration   `protobuf:"bytes,2,opt,name=timeout,proto3" json:"timeout,omitempty"`
	FeedInterval  *durationpb.Duration   `protobuf:"bytes,3,opt,name=feed_interval,json=feedInterval,proto3" json:"feed_interval,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *WatchdogTimerStatusSpec) Reset() {
	*x = WatchdogTimerStatusSpec{}
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[31]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *WatchdogTimerStatusSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*WatchdogTimerStatusSpec) ProtoMessage() {}

func (x *WatchdogTimerStatusSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_definitions_runtime_runtime_proto_msgTypes[31]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use WatchdogTimerStatusSpec.ProtoReflect.Descriptor instead.
func (*WatchdogTimerStatusSpec) Descriptor() ([]byte, []int) {
	return file_resource_definitions_runtime_runtime_proto_rawDescGZIP(), []int{31}
}

func (x *WatchdogTimerStatusSpec) GetDevice() string {
	if x != nil {
		return x.Device
	}
	return ""
}

func (x *WatchdogTimerStatusSpec) GetTimeout() *durationpb.Duration {
	if x != nil {
		return x.Timeout
	}
	return nil
}

func (x *WatchdogTimerStatusSpec) GetFeedInterval() *durationpb.Duration {
	if x != nil {
		return x.FeedInterval
	}
	return nil
}

var File_resource_definitions_runtime_runtime_proto protoreflect.FileDescriptor

const file_resource_definitions_runtime_runtime_proto_rawDesc = "" +
	"\n" +
	"*resource/definitions/runtime/runtime.proto\x12\"talos.resource.definitions.runtime\x1a\x13common/common.proto\x1a\x1egoogle/protobuf/duration.proto\x1a&resource/definitions/enums/enums.proto\"\xdc\x01\n" +
	"\x14APIServiceConfigSpec\x12%\n" +
	"\x0elisten_address\x18\x01 \x01(\tR\rlistenAddress\x122\n" +
	"\x15node_routing_disabled\x18\x02 \x01(\bR\x13nodeRoutingDisabled\x12,\n" +
	"\x12readonly_role_mode\x18\x03 \x01(\bR\x10readonlyRoleMode\x12;\n" +
	"\x1askip_verifying_client_cert\x18\x04 \x01(\bR\x17skipVerifyingClientCert\"4\n" +
	"\x0fBootedEntrySpec\x12!\n" +
	"\fbooted_entry\x18\x01 \x01(\tR\vbootedEntry\")\n" +
	"\x11DevicesStatusSpec\x12\x14\n" +
	"\x05ready\x18\x01 \x01(\bR\x05ready\"D\n" +
	"\x0eDiagnosticSpec\x12\x18\n" +
	"\amessage\x18\x01 \x01(\tR\amessage\x12\x18\n" +
	"\adetails\x18\x02 \x03(\tR\adetails\"/\n" +
	"\x0fEnvironmentSpec\x12\x1c\n" +
	"\tvariables\x18\x01 \x03(\tR\tvariables\"1\n" +
	"\x13EventSinkConfigSpec\x12\x1a\n" +
	"\bendpoint\x18\x01 \x01(\tR\bendpoint\"U\n" +
	"\x1aExtensionServiceConfigFile\x12\x18\n" +
	"\acontent\x18\x01 \x01(\tR\acontent\x12\x1d\n" +
	"\n" +
	"mount_path\x18\x02 \x01(\tR\tmountPath\"\x94\x01\n" +
	"\x1aExtensionServiceConfigSpec\x12T\n" +
	"\x05files\x18\x01 \x03(\v2>.talos.resource.definitions.runtime.ExtensionServiceConfigFileR\x05files\x12 \n" +
	"\venvironment\x18\x02 \x03(\tR\venvironment\"E\n" +
	" ExtensionServiceConfigStatusSpec\x12!\n" +
	"\fspec_version\x18\x01 \x01(\tR\vspecVersion\"-\n" +
	"\x11KernelCmdlineSpec\x12\x18\n" +
	"\acmdline\x18\x01 \x01(\tR\acmdline\"J\n" +
	"\x14KernelModuleSpecSpec\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12\x1e\n" +
	"\n" +
	"parameters\x18\x02 \x03(\tR\n" +
	"parameters\"\xb4\x02\n" +
	"\x16KernelModuleStatusSpec\x12M\n" +
	"\x04type\x18\x01 \x01(\x0e29.talos.resource.definitions.enums.RuntimeKernelModuleTypeR\x04type\x12\x12\n" +
	"\x04size\x18\x02 \x01(\x03R\x04size\x12'\n" +
	"\x0freference_count\x18\x03 \x01(\x03R\x0ereferenceCount\x12\"\n" +
	"\fdependencies\x18\x04 \x03(\tR\fdependencies\x12P\n" +
	"\x05state\x18\x05 \x01(\x0e2:.talos.resource.definitions.enums.RuntimeKernelModuleStateR\x05state\x12\x18\n" +
	"\aaddress\x18\x06 \x01(\tR\aaddress\"P\n" +
	"\x13KernelParamSpecSpec\x12\x14\n" +
	"\x05value\x18\x01 \x01(\tR\x05value\x12#\n" +
	"\rignore_errors\x18\x02 \x01(\bR\fignoreErrors\"m\n" +
	"\x15KernelParamStatusSpec\x12\x18\n" +
	"\acurrent\x18\x01 \x01(\tR\acurrent\x12\x18\n" +
	"\adefault\x18\x02 \x01(\tR\adefault\x12 \n" +
	"\vunsupported\x18\x03 \x01(\bR\vunsupported\"D\n" +
	"\x11KmsgLogConfigSpec\x12/\n" +
	"\fdestinations\x18\x01 \x03(\v2\v.common.URLR\fdestinations\"\xa9\x01\n" +
	"\x16LoadedKernelModuleSpec\x12\x12\n" +
	"\x04size\x18\x01 \x01(\x03R\x04size\x12'\n" +
	"\x0freference_count\x18\x02 \x01(\x03R\x0ereferenceCount\x12\"\n" +
	"\fdependencies\x18\x03 \x03(\tR\fdependencies\x12\x14\n" +
	"\x05state\x18\x04 \x01(\tR\x05state\x12\x18\n" +
	"\aaddress\x18\x05 \x01(\tR\aaddress\"\xb1\x01\n" +
	"\x11MachineStatusSpec\x12K\n" +
	"\x05stage\x18\x01 \x01(\x0e25.talos.resource.definitions.enums.RuntimeMachineStageR\x05stage\x12O\n" +
	"\x06status\x18\x02 \x01(\v27.talos.resource.definitions.runtime.MachineStatusStatusR\x06status\"\x8a\x01\n" +
	"\x13MachineStatusStatus\x12\x14\n" +
	"\x05ready\x18\x01 \x01(\bR\x05ready\x12]\n" +
	"\x10unmet_conditions\x18\x02 \x03(\v22.talos.resource.definitions.runtime.UnmetConditionR\x0funmetConditions\"\x85\x01\n" +
	"\x1cMaintenanceServiceConfigSpec\x12%\n" +
	"\x0elisten_address\x18\x01 \x01(\tR\rlistenAddress\x12>\n" +
	"\x13reachable_addresses\x18\x02 \x03(\v2\r.common.NetIPR\x12reachableAddresses\"#\n" +
	"\vMetaKeySpec\x12\x14\n" +
	"\x05value\x18\x01 \x01(\tR\x05value\"$\n" +
	"\x0eMetaLoadedSpec\x12\x12\n" +
	"\x04done\x18\x01 \x01(\bR\x04done\"\xd5\x01\n" +
	"\x0fMountStatusSpec\x12\x16\n" +
	"\x06source\x18\x01 \x01(\tR\x06source\x12\x16\n" +
	"\x06target\x18\x02 \x01(\tR\x06target\x12'\n" +
	"\x0ffilesystem_type\x18\x03 \x01(\tR\x0efilesystemType\x12\x18\n" +
	"\aoptions\x18\x04 \x03(\tR\aoptions\x12\x1c\n" +
	"\tencrypted\x18\x05 \x01(\bR\tencrypted\x121\n" +
	"\x14encryption_providers\x18\x06 \x03(\tR\x13encryptionProviders\"l\n" +
	"\rOOMActionSpec\x12'\n" +
	"\x0ftrigger_context\x18\x01 \x01(\tR\x0etriggerContext\x12\x14\n" +
	"\x05score\x18\x02 \x01(\x01R\x05score\x12\x1c\n" +
	"\tprocesses\x18\x03 \x03(\tR\tprocesses\"\xcc\x03\n" +
	"\x14PlatformMetadataSpec\x12\x1a\n" +
	"\bplatform\x18\x01 \x01(\tR\bplatform\x12\x1a\n" +
	"\bhostname\x18\x02 \x01(\tR\bhostname\x12\x16\n" +
	"\x06region\x18\x03 \x01(\tR\x06region\x12\x12\n" +
	"\x04zone\x18\x04 \x01(\tR\x04zone\x12#\n" +
	"\rinstance_type\x18\x05 \x01(\tR\finstanceType\x12\x1f\n" +
	"\vinstance_id\x18\x06 \x01(\tR\n" +
	"instanceId\x12\x1f\n" +
	"\vprovider_id\x18\a \x01(\tR\n" +
	"providerId\x12\x12\n" +
	"\x04spot\x18\b \x01(\bR\x04spot\x12!\n" +
	"\finternal_dns\x18\t \x01(\tR\vinternalDns\x12!\n" +
	"\fexternal_dns\x18\n" +
	" \x01(\tR\vexternalDns\x12V\n" +
	"\x04tags\x18\v \x03(\v2B.talos.resource.definitions.runtime.PlatformMetadataSpec.TagsEntryR\x04tags\x1a7\n" +
	"\tTagsEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\tR\x05value:\x028\x01\"\xa0\x01\n" +
	"\fSBOMItemSpec\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12\x18\n" +
	"\aversion\x18\x02 \x01(\tR\aversion\x12\x18\n" +
	"\alicense\x18\x03 \x01(\tR\alicense\x12\x13\n" +
	"\x05cp_es\x18\x04 \x03(\tR\x04cpEs\x12\x15\n" +
	"\x06pur_ls\x18\x05 \x03(\tR\x05purLs\x12\x1c\n" +
	"\textension\x18\x06 \x01(\bR\textension\"\xc6\x03\n" +
	"\x11SecurityStateSpec\x12\x1f\n" +
	"\vsecure_boot\x18\x01 \x01(\bR\n" +
	"secureBoot\x12=\n" +
	"\x1buki_signing_key_fingerprint\x18\x02 \x01(\tR\x18ukiSigningKeyFingerprint\x12=\n" +
	"\x1bpcr_signing_key_fingerprint\x18\x03 \x01(\tR\x18pcrSigningKeyFingerprint\x12[\n" +
	"\x0ese_linux_state\x18\x04 \x01(\x0e25.talos.resource.definitions.enums.RuntimeSELinuxStateR\fseLinuxState\x12&\n" +
	"\x0fbooted_with_uki\x18\x05 \x01(\bR\rbootedWithUki\x12Q\n" +
	"\n" +
	"fips_state\x18\x06 \x01(\x0e22.talos.resource.definitions.enums.RuntimeFIPSStateR\tfipsState\x12:\n" +
	"\x19module_signature_enforced\x18\a \x01(\bR\x17moduleSignatureEnforced\"K\n" +
	"\x0eServicePIDSpec\x12\x10\n" +
	"\x03pid\x18\x01 \x01(\x05R\x03pid\x12'\n" +
	"\x0fmount_namespace\x18\x02 \x01(\tR\x0emountNamespace\".\n" +
	"\x16UniqueMachineTokenSpec\x12\x14\n" +
	"\x05token\x18\x01 \x01(\tR\x05token\"<\n" +
	"\x0eUnmetCondition\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12\x16\n" +
	"\x06reason\x18\x02 \x01(\tR\x06reason\";\n" +
	"\vVersionSpec\x12\x18\n" +
	"\aversion\x18\x01 \x01(\tR\aversion\x12\x12\n" +
	"\x04name\x18\x02 \x01(\tR\x04name\"f\n" +
	"\x17WatchdogTimerConfigSpec\x12\x16\n" +
	"\x06device\x18\x01 \x01(\tR\x06device\x123\n" +
	"\atimeout\x18\x02 \x01(\v2\x19.google.protobuf.DurationR\atimeout\"\xa6\x01\n" +
	"\x17WatchdogTimerStatusSpec\x12\x16\n" +
	"\x06device\x18\x01 \x01(\tR\x06device\x123\n" +
	"\atimeout\x18\x02 \x01(\v2\x19.google.protobuf.DurationR\atimeout\x12>\n" +
	"\rfeed_interval\x18\x03 \x01(\v2\x19.google.protobuf.DurationR\ffeedIntervalBx\n" +
	"*dev.talos.api.resource.definitions.runtimeZJgithub.com/siderolabs/talos/pkg/machinery/api/resource/definitions/runtimeb\x06proto3"

var (
	file_resource_definitions_runtime_runtime_proto_rawDescOnce sync.Once
	file_resource_definitions_runtime_runtime_proto_rawDescData []byte
)

func file_resource_definitions_runtime_runtime_proto_rawDescGZIP() []byte {
	file_resource_definitions_runtime_runtime_proto_rawDescOnce.Do(func() {
		file_resource_definitions_runtime_runtime_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_resource_definitions_runtime_runtime_proto_rawDesc), len(file_resource_definitions_runtime_runtime_proto_rawDesc)))
	})
	return file_resource_definitions_runtime_runtime_proto_rawDescData
}

var file_resource_definitions_runtime_runtime_proto_msgTypes = make([]protoimpl.MessageInfo, 33)
var file_resource_definitions_runtime_runtime_proto_goTypes = []any{
	(*APIServiceConfigSpec)(nil),             // 0: talos.resource.definitions.runtime.APIServiceConfigSpec
	(*BootedEntrySpec)(nil),                  // 1: talos.resource.definitions.runtime.BootedEntrySpec
	(*DevicesStatusSpec)(nil),                // 2: talos.resource.definitions.runtime.DevicesStatusSpec
	(*DiagnosticSpec)(nil),                   // 3: talos.resource.definitions.runtime.DiagnosticSpec
	(*EnvironmentSpec)(nil),                  // 4: talos.resource.definitions.runtime.EnvironmentSpec
	(*EventSinkConfigSpec)(nil),              // 5: talos.resource.definitions.runtime.EventSinkConfigSpec
	(*ExtensionServiceConfigFile)(nil),       // 6: talos.resource.definitions.runtime.ExtensionServiceConfigFile
	(*ExtensionServiceConfigSpec)(nil),       // 7: talos.resource.definitions.runtime.ExtensionServiceConfigSpec
	(*ExtensionServiceConfigStatusSpec)(nil), // 8: talos.resource.definitions.runtime.ExtensionServiceConfigStatusSpec
	(*KernelCmdlineSpec)(nil),                // 9: talos.resource.definitions.runtime.KernelCmdlineSpec
	(*KernelModuleSpecSpec)(nil),             // 10: talos.resource.definitions.runtime.KernelModuleSpecSpec
	(*KernelModuleStatusSpec)(nil),           // 11: talos.resource.definitions.runtime.KernelModuleStatusSpec
	(*KernelParamSpecSpec)(nil),              // 12: talos.resource.definitions.runtime.KernelParamSpecSpec
	(*KernelParamStatusSpec)(nil),            // 13: talos.resource.definitions.runtime.KernelParamStatusSpec
	(*KmsgLogConfigSpec)(nil),                // 14: talos.resource.definitions.runtime.KmsgLogConfigSpec
	(*LoadedKernelModuleSpec)(nil),           // 15: talos.resource.definitions.runtime.LoadedKernelModuleSpec
	(*MachineStatusSpec)(nil),                // 16: talos.resource.definitions.runtime.MachineStatusSpec
	(*MachineStatusStatus)(nil),              // 17: talos.resource.definitions.runtime.MachineStatusStatus
	(*MaintenanceServiceConfigSpec)(nil),     // 18: talos.resource.definitions.runtime.MaintenanceServiceConfigSpec
	(*MetaKeySpec)(nil),                      // 19: talos.resource.definitions.runtime.MetaKeySpec
	(*MetaLoadedSpec)(nil),                   // 20: talos.resource.definitions.runtime.MetaLoadedSpec
	(*MountStatusSpec)(nil),                  // 21: talos.resource.definitions.runtime.MountStatusSpec
	(*OOMActionSpec)(nil),                    // 22: talos.resource.definitions.runtime.OOMActionSpec
	(*PlatformMetadataSpec)(nil),             // 23: talos.resource.definitions.runtime.PlatformMetadataSpec
	(*SBOMItemSpec)(nil),                     // 24: talos.resource.definitions.runtime.SBOMItemSpec
	(*SecurityStateSpec)(nil),                // 25: talos.resource.definitions.runtime.SecurityStateSpec
	(*ServicePIDSpec)(nil),                   // 26: talos.resource.definitions.runtime.ServicePIDSpec
	(*UniqueMachineTokenSpec)(nil),           // 27: talos.resource.definitions.runtime.UniqueMachineTokenSpec
	(*UnmetCondition)(nil),                   // 28: talos.resource.definitions.runtime.UnmetCondition
	(*VersionSpec)(nil),                      // 29: talos.resource.definitions.runtime.VersionSpec
	(*WatchdogTimerConfigSpec)(nil),          // 30: talos.resource.definitions.runtime.WatchdogTimerConfigSpec
	(*WatchdogTimerStatusSpec)(nil),          // 31: talos.resource.definitions.runtime.WatchdogTimerStatusSpec
	nil,                                      // 32: talos.resource.definitions.runtime.PlatformMetadataSpec.TagsEntry
	(enums.RuntimeKernelModuleType)(0),       // 33: talos.resource.definitions.enums.RuntimeKernelModuleType
	(enums.RuntimeKernelModuleState)(0),      // 34: talos.resource.definitions.enums.RuntimeKernelModuleState
	(*common.URL)(nil),                       // 35: common.URL
	(enums.RuntimeMachineStage)(0),           // 36: talos.resource.definitions.enums.RuntimeMachineStage
	(*common.NetIP)(nil),                     // 37: common.NetIP
	(enums.RuntimeSELinuxState)(0),           // 38: talos.resource.definitions.enums.RuntimeSELinuxState
	(enums.RuntimeFIPSState)(0),              // 39: talos.resource.definitions.enums.RuntimeFIPSState
	(*durationpb.Duration)(nil),              // 40: google.protobuf.Duration
}
var file_resource_definitions_runtime_runtime_proto_depIdxs = []int32{
	6,  // 0: talos.resource.definitions.runtime.ExtensionServiceConfigSpec.files:type_name -> talos.resource.definitions.runtime.ExtensionServiceConfigFile
	33, // 1: talos.resource.definitions.runtime.KernelModuleStatusSpec.type:type_name -> talos.resource.definitions.enums.RuntimeKernelModuleType
	34, // 2: talos.resource.definitions.runtime.KernelModuleStatusSpec.state:type_name -> talos.resource.definitions.enums.RuntimeKernelModuleState
	35, // 3: talos.resource.definitions.runtime.KmsgLogConfigSpec.destinations:type_name -> common.URL
	36, // 4: talos.resource.definitions.runtime.MachineStatusSpec.stage:type_name -> talos.resource.definitions.enums.RuntimeMachineStage
	17, // 5: talos.resource.definitions.runtime.MachineStatusSpec.status:type_name -> talos.resource.definitions.runtime.MachineStatusStatus
	28, // 6: talos.resource.definitions.runtime.MachineStatusStatus.unmet_conditions:type_name -> talos.resource.definitions.runtime.UnmetCondition
	37, // 7: talos.resource.definitions.runtime.MaintenanceServiceConfigSpec.reachable_addresses:type_name -> common.NetIP
	32, // 8: talos.resource.definitions.runtime.PlatformMetadataSpec.tags:type_name -> talos.resource.definitions.runtime.PlatformMetadataSpec.TagsEntry
	38, // 9: talos.resource.definitions.runtime.SecurityStateSpec.se_linux_state:type_name -> talos.resource.definitions.enums.RuntimeSELinuxState
	39, // 10: talos.resource.definitions.runtime.SecurityStateSpec.fips_state:type_name -> talos.resource.definitions.enums.RuntimeFIPSState
	40, // 11: talos.resource.definitions.runtime.WatchdogTimerConfigSpec.timeout:type_name -> google.protobuf.Duration
	40, // 12: talos.resource.definitions.runtime.WatchdogTimerStatusSpec.timeout:type_name -> google.protobuf.Duration
	40, // 13: talos.resource.definitions.runtime.WatchdogTimerStatusSpec.feed_interval:type_name -> google.protobuf.Duration
	14, // [14:14] is the sub-list for method output_type
	14, // [14:14] is the sub-list for method input_type
	14, // [14:14] is the sub-list for extension type_name
	14, // [14:14] is the sub-list for extension extendee
	0,  // [0:14] is the sub-list for field type_name
}

func init() { file_resource_definitions_runtime_runtime_proto_init() }
func file_resource_definitions_runtime_runtime_proto_init() {
	if File_resource_definitions_runtime_runtime_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_resource_definitions_runtime_runtime_proto_rawDesc), len(file_resource_definitions_runtime_runtime_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   33,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_resource_definitions_runtime_runtime_proto_goTypes,
		DependencyIndexes: file_resource_definitions_runtime_runtime_proto_depIdxs,
		MessageInfos:      file_resource_definitions_runtime_runtime_proto_msgTypes,
	}.Build()
	File_resource_definitions_runtime_runtime_proto = out.File
	file_resource_definitions_runtime_runtime_proto_goTypes = nil
	file_resource_definitions_runtime_runtime_proto_depIdxs = nil
}
