// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/listener/v3/udp_listener_config.proto

package listenerv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// [#next-free-field: 9]
type UdpListenerConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// UDP socket configuration for the listener. The default for
	// :ref:`prefer_gro <envoy_v3_api_field_config.core.v3.UdpSocketConfig.prefer_gro>` is false for
	// listener sockets. If receiving a large amount of datagrams from a small number of sources, it
	// may be worthwhile to enable this option after performance testing.
	DownstreamSocketConfig *v3.UdpSocketConfig `protobuf:"bytes,5,opt,name=downstream_socket_config,json=downstreamSocketConfig,proto3" json:"downstream_socket_config,omitempty"`
	// Configuration for QUIC protocol. If empty, QUIC will not be enabled on this listener. Set
	// to the default object to enable QUIC without modifying any additional options.
	QuicOptions *QuicProtocolOptions `protobuf:"bytes,7,opt,name=quic_options,json=quicOptions,proto3" json:"quic_options,omitempty"`
	// Configuration for the UDP packet writer. If empty, HTTP/3 will use GSO if available
	// (:ref:`UdpDefaultWriterFactory <envoy_v3_api_msg_extensions.udp_packet_writer.v3.UdpGsoBatchWriterFactory>`)
	// or the default kernel sendmsg if not,
	// (:ref:`UdpDefaultWriterFactory <envoy_v3_api_msg_extensions.udp_packet_writer.v3.UdpDefaultWriterFactory>`)
	// and raw UDP will use kernel sendmsg.
	// [#extension-category: envoy.udp_packet_writer]
	UdpPacketPacketWriterConfig *v3.TypedExtensionConfig `protobuf:"bytes,8,opt,name=udp_packet_packet_writer_config,json=udpPacketPacketWriterConfig,proto3" json:"udp_packet_packet_writer_config,omitempty"`
	unknownFields               protoimpl.UnknownFields
	sizeCache                   protoimpl.SizeCache
}

func (x *UdpListenerConfig) Reset() {
	*x = UdpListenerConfig{}
	mi := &file_envoy_config_listener_v3_udp_listener_config_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UdpListenerConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UdpListenerConfig) ProtoMessage() {}

func (x *UdpListenerConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_listener_v3_udp_listener_config_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UdpListenerConfig.ProtoReflect.Descriptor instead.
func (*UdpListenerConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_listener_v3_udp_listener_config_proto_rawDescGZIP(), []int{0}
}

func (x *UdpListenerConfig) GetDownstreamSocketConfig() *v3.UdpSocketConfig {
	if x != nil {
		return x.DownstreamSocketConfig
	}
	return nil
}

func (x *UdpListenerConfig) GetQuicOptions() *QuicProtocolOptions {
	if x != nil {
		return x.QuicOptions
	}
	return nil
}

func (x *UdpListenerConfig) GetUdpPacketPacketWriterConfig() *v3.TypedExtensionConfig {
	if x != nil {
		return x.UdpPacketPacketWriterConfig
	}
	return nil
}

type ActiveRawUdpListenerConfig struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ActiveRawUdpListenerConfig) Reset() {
	*x = ActiveRawUdpListenerConfig{}
	mi := &file_envoy_config_listener_v3_udp_listener_config_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ActiveRawUdpListenerConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ActiveRawUdpListenerConfig) ProtoMessage() {}

func (x *ActiveRawUdpListenerConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_listener_v3_udp_listener_config_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ActiveRawUdpListenerConfig.ProtoReflect.Descriptor instead.
func (*ActiveRawUdpListenerConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_listener_v3_udp_listener_config_proto_rawDescGZIP(), []int{1}
}

var File_envoy_config_listener_v3_udp_listener_config_proto protoreflect.FileDescriptor

const file_envoy_config_listener_v3_udp_listener_config_proto_rawDesc = "" +
	"\n" +
	"2envoy/config/listener/v3/udp_listener_config.proto\x12\x18envoy.config.listener.v3\x1a$envoy/config/core/v3/extension.proto\x1a,envoy/config/core/v3/udp_socket_config.proto\x1a*envoy/config/listener/v3/quic_config.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"\x8e\x03\n" +
	"\x11UdpListenerConfig\x12_\n" +
	"\x18downstream_socket_config\x18\x05 \x01(\v2%.envoy.config.core.v3.UdpSocketConfigR\x16downstreamSocketConfig\x12P\n" +
	"\fquic_options\x18\a \x01(\v2-.envoy.config.listener.v3.QuicProtocolOptionsR\vquicOptions\x12p\n" +
	"\x1fudp_packet_packet_writer_config\x18\b \x01(\v2*.envoy.config.core.v3.TypedExtensionConfigR\x1budpPacketPacketWriterConfig:.\x9aň\x1e)\n" +
	"'envoy.api.v2.listener.UdpListenerConfigJ\x04\b\x01\x10\x02J\x04\b\x02\x10\x03J\x04\b\x03\x10\x04J\x04\b\x04\x10\x05J\x04\b\x06\x10\aR\x06config\"U\n" +
	"\x1aActiveRawUdpListenerConfig:7\x9aň\x1e2\n" +
	"0envoy.api.v2.listener.ActiveRawUdpListenerConfigB\x96\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"&io.envoyproxy.envoy.config.listener.v3B\x16UdpListenerConfigProtoP\x01ZJgithub.com/envoyproxy/go-control-plane/envoy/config/listener/v3;listenerv3b\x06proto3"

var (
	file_envoy_config_listener_v3_udp_listener_config_proto_rawDescOnce sync.Once
	file_envoy_config_listener_v3_udp_listener_config_proto_rawDescData []byte
)

func file_envoy_config_listener_v3_udp_listener_config_proto_rawDescGZIP() []byte {
	file_envoy_config_listener_v3_udp_listener_config_proto_rawDescOnce.Do(func() {
		file_envoy_config_listener_v3_udp_listener_config_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_listener_v3_udp_listener_config_proto_rawDesc), len(file_envoy_config_listener_v3_udp_listener_config_proto_rawDesc)))
	})
	return file_envoy_config_listener_v3_udp_listener_config_proto_rawDescData
}

var file_envoy_config_listener_v3_udp_listener_config_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_config_listener_v3_udp_listener_config_proto_goTypes = []any{
	(*UdpListenerConfig)(nil),          // 0: envoy.config.listener.v3.UdpListenerConfig
	(*ActiveRawUdpListenerConfig)(nil), // 1: envoy.config.listener.v3.ActiveRawUdpListenerConfig
	(*v3.UdpSocketConfig)(nil),         // 2: envoy.config.core.v3.UdpSocketConfig
	(*QuicProtocolOptions)(nil),        // 3: envoy.config.listener.v3.QuicProtocolOptions
	(*v3.TypedExtensionConfig)(nil),    // 4: envoy.config.core.v3.TypedExtensionConfig
}
var file_envoy_config_listener_v3_udp_listener_config_proto_depIdxs = []int32{
	2, // 0: envoy.config.listener.v3.UdpListenerConfig.downstream_socket_config:type_name -> envoy.config.core.v3.UdpSocketConfig
	3, // 1: envoy.config.listener.v3.UdpListenerConfig.quic_options:type_name -> envoy.config.listener.v3.QuicProtocolOptions
	4, // 2: envoy.config.listener.v3.UdpListenerConfig.udp_packet_packet_writer_config:type_name -> envoy.config.core.v3.TypedExtensionConfig
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() { file_envoy_config_listener_v3_udp_listener_config_proto_init() }
func file_envoy_config_listener_v3_udp_listener_config_proto_init() {
	if File_envoy_config_listener_v3_udp_listener_config_proto != nil {
		return
	}
	file_envoy_config_listener_v3_quic_config_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_listener_v3_udp_listener_config_proto_rawDesc), len(file_envoy_config_listener_v3_udp_listener_config_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_listener_v3_udp_listener_config_proto_goTypes,
		DependencyIndexes: file_envoy_config_listener_v3_udp_listener_config_proto_depIdxs,
		MessageInfos:      file_envoy_config_listener_v3_udp_listener_config_proto_msgTypes,
	}.Build()
	File_envoy_config_listener_v3_udp_listener_config_proto = out.File
	file_envoy_config_listener_v3_udp_listener_config_proto_goTypes = nil
	file_envoy_config_listener_v3_udp_listener_config_proto_depIdxs = nil
}
