// Copyright 2025 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.31.0
// source: google/monitoring/v3/dropped_labels.proto

package monitoringpb

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// A set of (label, value) pairs that were removed from a Distribution
// time series during aggregation and then added as an attachment to a
// Distribution.Exemplar.
//
// The full label set for the exemplars is constructed by using the dropped
// pairs in combination with the label values that remain on the aggregated
// Distribution time series. The constructed full label set can be used to
// identify the specific entity, such as the instance or job, which might be
// contributing to a long-tail. However, with dropped labels, the storage
// requirements are reduced because only the aggregated distribution values for
// a large group of time series are stored.
//
// Note that there are no guarantees on ordering of the labels from
// exemplar-to-exemplar and from distribution-to-distribution in the same
// stream, and there may be duplicates.  It is up to clients to resolve any
// ambiguities.
type DroppedLabels struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Map from label to its value, for all labels dropped in any aggregation.
	Label         map[string]string `protobuf:"bytes,1,rep,name=label,proto3" json:"label,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DroppedLabels) Reset() {
	*x = DroppedLabels{}
	mi := &file_google_monitoring_v3_dropped_labels_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DroppedLabels) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DroppedLabels) ProtoMessage() {}

func (x *DroppedLabels) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_dropped_labels_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DroppedLabels.ProtoReflect.Descriptor instead.
func (*DroppedLabels) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_dropped_labels_proto_rawDescGZIP(), []int{0}
}

func (x *DroppedLabels) GetLabel() map[string]string {
	if x != nil {
		return x.Label
	}
	return nil
}

var File_google_monitoring_v3_dropped_labels_proto protoreflect.FileDescriptor

const file_google_monitoring_v3_dropped_labels_proto_rawDesc = "" +
	"\n" +
	")google/monitoring/v3/dropped_labels.proto\x12\x14google.monitoring.v3\"\x8f\x01\n" +
	"\rDroppedLabels\x12D\n" +
	"\x05label\x18\x01 \x03(\v2..google.monitoring.v3.DroppedLabels.LabelEntryR\x05label\x1a8\n" +
	"\n" +
	"LabelEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\tR\x05value:\x028\x01B\xcd\x01\n" +
	"\x18com.google.monitoring.v3B\x12DroppedLabelsProtoP\x01ZAcloud.google.com/go/monitoring/apiv3/v2/monitoringpb;monitoringpb\xaa\x02\x1aGoogle.Cloud.Monitoring.V3\xca\x02\x1aGoogle\\Cloud\\Monitoring\\V3\xea\x02\x1dGoogle::Cloud::Monitoring::V3b\x06proto3"

var (
	file_google_monitoring_v3_dropped_labels_proto_rawDescOnce sync.Once
	file_google_monitoring_v3_dropped_labels_proto_rawDescData []byte
)

func file_google_monitoring_v3_dropped_labels_proto_rawDescGZIP() []byte {
	file_google_monitoring_v3_dropped_labels_proto_rawDescOnce.Do(func() {
		file_google_monitoring_v3_dropped_labels_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_google_monitoring_v3_dropped_labels_proto_rawDesc), len(file_google_monitoring_v3_dropped_labels_proto_rawDesc)))
	})
	return file_google_monitoring_v3_dropped_labels_proto_rawDescData
}

var file_google_monitoring_v3_dropped_labels_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_google_monitoring_v3_dropped_labels_proto_goTypes = []any{
	(*DroppedLabels)(nil), // 0: google.monitoring.v3.DroppedLabels
	nil,                   // 1: google.monitoring.v3.DroppedLabels.LabelEntry
}
var file_google_monitoring_v3_dropped_labels_proto_depIdxs = []int32{
	1, // 0: google.monitoring.v3.DroppedLabels.label:type_name -> google.monitoring.v3.DroppedLabels.LabelEntry
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_google_monitoring_v3_dropped_labels_proto_init() }
func file_google_monitoring_v3_dropped_labels_proto_init() {
	if File_google_monitoring_v3_dropped_labels_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_monitoring_v3_dropped_labels_proto_rawDesc), len(file_google_monitoring_v3_dropped_labels_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_google_monitoring_v3_dropped_labels_proto_goTypes,
		DependencyIndexes: file_google_monitoring_v3_dropped_labels_proto_depIdxs,
		MessageInfos:      file_google_monitoring_v3_dropped_labels_proto_msgTypes,
	}.Build()
	File_google_monitoring_v3_dropped_labels_proto = out.File
	file_google_monitoring_v3_dropped_labels_proto_goTypes = nil
	file_google_monitoring_v3_dropped_labels_proto_depIdxs = nil
}
