// Copyright 2022 Huawei Technologies Co.,Ltd.
//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package retry

const (
	BaseDelay = 10        // base delay is 10ms
	MaxDelay  = 60 * 1000 // max delay is 60s
)

type strategyBase struct {
	baseDelay int32
}

// newStrategyBase new struct with default base delay: BaseDelay
func newStrategyBase() *strategyBase {
	return &strategyBase{baseDelay: BaseDelay}
}

func (s *strategyBase) SetBaseDelay(baseDelay int32) {
	if s != nil {
		s.baseDelay = baseDelay
	}
}

func (s *strategyBase) GetBaseDelay() int32 {
	if s != nil {
		return s.baseDelay
	}
	return BaseDelay
}

type Strategy interface {
	SetBaseDelay(int32)
	ComputeDelayBeforeNextRetry(int32) int32
}
