// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.1
// source: udpa/annotations/security.proto

package annotations

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	descriptorpb "google.golang.org/protobuf/types/descriptorpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type FieldSecurityAnnotation struct {
	state                           protoimpl.MessageState `protogen:"open.v1"`
	ConfigureForUntrustedDownstream bool                   `protobuf:"varint,1,opt,name=configure_for_untrusted_downstream,json=configureForUntrustedDownstream,proto3" json:"configure_for_untrusted_downstream,omitempty"`
	ConfigureForUntrustedUpstream   bool                   `protobuf:"varint,2,opt,name=configure_for_untrusted_upstream,json=configureForUntrustedUpstream,proto3" json:"configure_for_untrusted_upstream,omitempty"`
	unknownFields                   protoimpl.UnknownFields
	sizeCache                       protoimpl.SizeCache
}

func (x *FieldSecurityAnnotation) Reset() {
	*x = FieldSecurityAnnotation{}
	mi := &file_udpa_annotations_security_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FieldSecurityAnnotation) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FieldSecurityAnnotation) ProtoMessage() {}

func (x *FieldSecurityAnnotation) ProtoReflect() protoreflect.Message {
	mi := &file_udpa_annotations_security_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FieldSecurityAnnotation.ProtoReflect.Descriptor instead.
func (*FieldSecurityAnnotation) Descriptor() ([]byte, []int) {
	return file_udpa_annotations_security_proto_rawDescGZIP(), []int{0}
}

func (x *FieldSecurityAnnotation) GetConfigureForUntrustedDownstream() bool {
	if x != nil {
		return x.ConfigureForUntrustedDownstream
	}
	return false
}

func (x *FieldSecurityAnnotation) GetConfigureForUntrustedUpstream() bool {
	if x != nil {
		return x.ConfigureForUntrustedUpstream
	}
	return false
}

var file_udpa_annotations_security_proto_extTypes = []protoimpl.ExtensionInfo{
	{
		ExtendedType:  (*descriptorpb.FieldOptions)(nil),
		ExtensionType: (*FieldSecurityAnnotation)(nil),
		Field:         11122993,
		Name:          "udpa.annotations.security",
		Tag:           "bytes,11122993,opt,name=security",
		Filename:      "udpa/annotations/security.proto",
	},
}

// Extension fields to descriptorpb.FieldOptions.
var (
	// optional udpa.annotations.FieldSecurityAnnotation security = 11122993;
	E_Security = &file_udpa_annotations_security_proto_extTypes[0]
)

var File_udpa_annotations_security_proto protoreflect.FileDescriptor

const file_udpa_annotations_security_proto_rawDesc = "" +
	"\n" +
	"\x1fudpa/annotations/security.proto\x12\x10udpa.annotations\x1a\x1dudpa/annotations/status.proto\x1a google/protobuf/descriptor.proto\"\xaf\x01\n" +
	"\x17FieldSecurityAnnotation\x12K\n" +
	"\"configure_for_untrusted_downstream\x18\x01 \x01(\bR\x1fconfigureForUntrustedDownstream\x12G\n" +
	" configure_for_untrusted_upstream\x18\x02 \x01(\bR\x1dconfigureForUntrustedUpstream:g\n" +
	"\bsecurity\x12\x1d.google.protobuf.FieldOptions\x18\xb1\xf2\xa6\x05 \x01(\v2).udpa.annotations.FieldSecurityAnnotationR\bsecurityB1\xba\x80\xc8\xd1\x06\x02\b\x01Z'github.com/cncf/xds/go/udpa/annotationsb\x06proto3"

var (
	file_udpa_annotations_security_proto_rawDescOnce sync.Once
	file_udpa_annotations_security_proto_rawDescData []byte
)

func file_udpa_annotations_security_proto_rawDescGZIP() []byte {
	file_udpa_annotations_security_proto_rawDescOnce.Do(func() {
		file_udpa_annotations_security_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_udpa_annotations_security_proto_rawDesc), len(file_udpa_annotations_security_proto_rawDesc)))
	})
	return file_udpa_annotations_security_proto_rawDescData
}

var file_udpa_annotations_security_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_udpa_annotations_security_proto_goTypes = []any{
	(*FieldSecurityAnnotation)(nil),   // 0: udpa.annotations.FieldSecurityAnnotation
	(*descriptorpb.FieldOptions)(nil), // 1: google.protobuf.FieldOptions
}
var file_udpa_annotations_security_proto_depIdxs = []int32{
	1, // 0: udpa.annotations.security:extendee -> google.protobuf.FieldOptions
	0, // 1: udpa.annotations.security:type_name -> udpa.annotations.FieldSecurityAnnotation
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	1, // [1:2] is the sub-list for extension type_name
	0, // [0:1] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_udpa_annotations_security_proto_init() }
func file_udpa_annotations_security_proto_init() {
	if File_udpa_annotations_security_proto != nil {
		return
	}
	file_udpa_annotations_status_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_udpa_annotations_security_proto_rawDesc), len(file_udpa_annotations_security_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 1,
			NumServices:   0,
		},
		GoTypes:           file_udpa_annotations_security_proto_goTypes,
		DependencyIndexes: file_udpa_annotations_security_proto_depIdxs,
		MessageInfos:      file_udpa_annotations_security_proto_msgTypes,
		ExtensionInfos:    file_udpa_annotations_security_proto_extTypes,
	}.Build()
	File_udpa_annotations_security_proto = out.File
	file_udpa_annotations_security_proto_goTypes = nil
	file_udpa_annotations_security_proto_depIdxs = nil
}
