// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

// Code generated by "deep-copy -type Layer -header-file ../../../hack/boilerplate.txt -o deep_copy.generated.go ."; DO NOT EDIT.

package extensions

// DeepCopy generates a deep copy of Layer.
func (o Layer) DeepCopy() Layer {
	var cp Layer = o
	return cp
}
