// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/clusters/aggregate/v3/cluster.proto

package aggregatev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the aggregate cluster. See the :ref:`architecture overview
// <arch_overview_aggregate_cluster>` for more information.
// [#extension: envoy.clusters.aggregate]
type ClusterConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Load balancing clusters in aggregate cluster. Clusters are prioritized based on the order they
	// appear in this list.
	Clusters      []string `protobuf:"bytes,1,rep,name=clusters,proto3" json:"clusters,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ClusterConfig) Reset() {
	*x = ClusterConfig{}
	mi := &file_envoy_extensions_clusters_aggregate_v3_cluster_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClusterConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClusterConfig) ProtoMessage() {}

func (x *ClusterConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_clusters_aggregate_v3_cluster_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClusterConfig.ProtoReflect.Descriptor instead.
func (*ClusterConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_clusters_aggregate_v3_cluster_proto_rawDescGZIP(), []int{0}
}

func (x *ClusterConfig) GetClusters() []string {
	if x != nil {
		return x.Clusters
	}
	return nil
}

// Configures an aggregate cluster whose
// :ref:`ClusterConfig <envoy_v3_api_msg_extensions.clusters.aggregate.v3.ClusterConfig>`
// is to be fetched from a separate xDS resource.
// [#extension: envoy.clusters.aggregate_resource]
// [#not-implemented-hide:]
type AggregateClusterResource struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Configuration source specifier for the ClusterConfig resource.
	// Only the aggregated protocol variants are supported; if configured
	// otherwise, the cluster resource will be NACKed.
	ConfigSource *v3.ConfigSource `protobuf:"bytes,1,opt,name=config_source,json=configSource,proto3" json:"config_source,omitempty"`
	// The name of the ClusterConfig resource to subscribe to.
	ResourceName  string `protobuf:"bytes,2,opt,name=resource_name,json=resourceName,proto3" json:"resource_name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AggregateClusterResource) Reset() {
	*x = AggregateClusterResource{}
	mi := &file_envoy_extensions_clusters_aggregate_v3_cluster_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AggregateClusterResource) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AggregateClusterResource) ProtoMessage() {}

func (x *AggregateClusterResource) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_clusters_aggregate_v3_cluster_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AggregateClusterResource.ProtoReflect.Descriptor instead.
func (*AggregateClusterResource) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_clusters_aggregate_v3_cluster_proto_rawDescGZIP(), []int{1}
}

func (x *AggregateClusterResource) GetConfigSource() *v3.ConfigSource {
	if x != nil {
		return x.ConfigSource
	}
	return nil
}

func (x *AggregateClusterResource) GetResourceName() string {
	if x != nil {
		return x.ResourceName
	}
	return ""
}

var File_envoy_extensions_clusters_aggregate_v3_cluster_proto protoreflect.FileDescriptor

const file_envoy_extensions_clusters_aggregate_v3_cluster_proto_rawDesc = "" +
	"\n" +
	"4envoy/extensions/clusters/aggregate/v3/cluster.proto\x12&envoy.extensions.clusters.aggregate.v3\x1a(envoy/config/core/v3/config_source.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"r\n" +
	"\rClusterConfig\x12$\n" +
	"\bclusters\x18\x01 \x03(\tB\b\xfaB\x05\x92\x01\x02\b\x01R\bclusters:;\x9aň\x1e6\n" +
	"4envoy.config.cluster.aggregate.v2alpha.ClusterConfig\"\x9b\x01\n" +
	"\x18AggregateClusterResource\x12Q\n" +
	"\rconfig_source\x18\x01 \x01(\v2\".envoy.config.core.v3.ConfigSourceB\b\xfaB\x05\x8a\x01\x02\x10\x01R\fconfigSource\x12,\n" +
	"\rresource_name\x18\x02 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\fresourceNameB\xa9\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"4io.envoyproxy.envoy.extensions.clusters.aggregate.v3B\fClusterProtoP\x01ZYgithub.com/envoyproxy/go-control-plane/envoy/extensions/clusters/aggregate/v3;aggregatev3b\x06proto3"

var (
	file_envoy_extensions_clusters_aggregate_v3_cluster_proto_rawDescOnce sync.Once
	file_envoy_extensions_clusters_aggregate_v3_cluster_proto_rawDescData []byte
)

func file_envoy_extensions_clusters_aggregate_v3_cluster_proto_rawDescGZIP() []byte {
	file_envoy_extensions_clusters_aggregate_v3_cluster_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_clusters_aggregate_v3_cluster_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_clusters_aggregate_v3_cluster_proto_rawDesc), len(file_envoy_extensions_clusters_aggregate_v3_cluster_proto_rawDesc)))
	})
	return file_envoy_extensions_clusters_aggregate_v3_cluster_proto_rawDescData
}

var file_envoy_extensions_clusters_aggregate_v3_cluster_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_clusters_aggregate_v3_cluster_proto_goTypes = []any{
	(*ClusterConfig)(nil),            // 0: envoy.extensions.clusters.aggregate.v3.ClusterConfig
	(*AggregateClusterResource)(nil), // 1: envoy.extensions.clusters.aggregate.v3.AggregateClusterResource
	(*v3.ConfigSource)(nil),          // 2: envoy.config.core.v3.ConfigSource
}
var file_envoy_extensions_clusters_aggregate_v3_cluster_proto_depIdxs = []int32{
	2, // 0: envoy.extensions.clusters.aggregate.v3.AggregateClusterResource.config_source:type_name -> envoy.config.core.v3.ConfigSource
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_extensions_clusters_aggregate_v3_cluster_proto_init() }
func file_envoy_extensions_clusters_aggregate_v3_cluster_proto_init() {
	if File_envoy_extensions_clusters_aggregate_v3_cluster_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_clusters_aggregate_v3_cluster_proto_rawDesc), len(file_envoy_extensions_clusters_aggregate_v3_cluster_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_clusters_aggregate_v3_cluster_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_clusters_aggregate_v3_cluster_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_clusters_aggregate_v3_cluster_proto_msgTypes,
	}.Build()
	File_envoy_extensions_clusters_aggregate_v3_cluster_proto = out.File
	file_envoy_extensions_clusters_aggregate_v3_cluster_proto_goTypes = nil
	file_envoy_extensions_clusters_aggregate_v3_cluster_proto_depIdxs = nil
}
