// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/trace/v3/opentelemetry.proto

package tracev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the OpenTelemetry tracer.
//
//	[#extension: envoy.tracers.opentelemetry]
//
// [#next-free-field: 7]
type OpenTelemetryConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The upstream gRPC cluster that will receive OTLP traces.
	// Note that the tracer drops traces if the server does not read data fast enough.
	// This field can be left empty to disable reporting traces to the gRPC service.
	// Only one of “grpc_service“, “http_service“ may be used.
	GrpcService *v3.GrpcService `protobuf:"bytes,1,opt,name=grpc_service,json=grpcService,proto3" json:"grpc_service,omitempty"`
	// The upstream HTTP cluster that will receive OTLP traces.
	// This field can be left empty to disable reporting traces to the HTTP service.
	// Only one of “grpc_service“, “http_service“ may be used.
	//
	// .. note::
	//
	//	Note: The ``request_headers_to_add`` property in the OTLP HTTP exporter service
	//	does not support the :ref:`format specifier <config_access_log_format>` as used for
	//	:ref:`HTTP access logging <config_access_log>`.
	//	The values configured are added as HTTP headers on the OTLP export request
	//	without any formatting applied.
	HttpService *v3.HttpService `protobuf:"bytes,3,opt,name=http_service,json=httpService,proto3" json:"http_service,omitempty"`
	// The name for the service. This will be populated in the ResourceSpan Resource attributes.
	// If it is not provided, it will default to "unknown_service:envoy".
	ServiceName string `protobuf:"bytes,2,opt,name=service_name,json=serviceName,proto3" json:"service_name,omitempty"`
	// An ordered list of resource detectors
	// [#extension-category: envoy.tracers.opentelemetry.resource_detectors]
	ResourceDetectors []*v3.TypedExtensionConfig `protobuf:"bytes,4,rep,name=resource_detectors,json=resourceDetectors,proto3" json:"resource_detectors,omitempty"`
	// Specifies the sampler to be used by the OpenTelemetry tracer.
	// The configured sampler implements the Sampler interface defined by the OpenTelemetry specification.
	// This field can be left empty. In this case, the default Envoy sampling decision is used.
	//
	// See: `OpenTelemetry sampler specification <https://opentelemetry.io/docs/specs/otel/trace/sdk/#sampler>`_
	// [#extension-category: envoy.tracers.opentelemetry.samplers]
	Sampler *v3.TypedExtensionConfig `protobuf:"bytes,5,opt,name=sampler,proto3" json:"sampler,omitempty"`
	// Envoy caches the span in memory when the OpenTelemetry backend service is temporarily unavailable.
	// This field specifies the maximum number of spans that can be cached. If not specified, the
	// default is 1024.
	MaxCacheSize  *wrapperspb.UInt32Value `protobuf:"bytes,6,opt,name=max_cache_size,json=maxCacheSize,proto3" json:"max_cache_size,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *OpenTelemetryConfig) Reset() {
	*x = OpenTelemetryConfig{}
	mi := &file_envoy_config_trace_v3_opentelemetry_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OpenTelemetryConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OpenTelemetryConfig) ProtoMessage() {}

func (x *OpenTelemetryConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_trace_v3_opentelemetry_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OpenTelemetryConfig.ProtoReflect.Descriptor instead.
func (*OpenTelemetryConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_trace_v3_opentelemetry_proto_rawDescGZIP(), []int{0}
}

func (x *OpenTelemetryConfig) GetGrpcService() *v3.GrpcService {
	if x != nil {
		return x.GrpcService
	}
	return nil
}

func (x *OpenTelemetryConfig) GetHttpService() *v3.HttpService {
	if x != nil {
		return x.HttpService
	}
	return nil
}

func (x *OpenTelemetryConfig) GetServiceName() string {
	if x != nil {
		return x.ServiceName
	}
	return ""
}

func (x *OpenTelemetryConfig) GetResourceDetectors() []*v3.TypedExtensionConfig {
	if x != nil {
		return x.ResourceDetectors
	}
	return nil
}

func (x *OpenTelemetryConfig) GetSampler() *v3.TypedExtensionConfig {
	if x != nil {
		return x.Sampler
	}
	return nil
}

func (x *OpenTelemetryConfig) GetMaxCacheSize() *wrapperspb.UInt32Value {
	if x != nil {
		return x.MaxCacheSize
	}
	return nil
}

var File_envoy_config_trace_v3_opentelemetry_proto protoreflect.FileDescriptor

const file_envoy_config_trace_v3_opentelemetry_proto_rawDesc = "" +
	"\n" +
	")envoy/config/trace/v3/opentelemetry.proto\x12\x15envoy.config.trace.v3\x1a$envoy/config/core/v3/extension.proto\x1a'envoy/config/core/v3/grpc_service.proto\x1a'envoy/config/core/v3/http_service.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\"\xd7\x03\n" +
	"\x13OpenTelemetryConfig\x12[\n" +
	"\fgrpc_service\x18\x01 \x01(\v2!.envoy.config.core.v3.GrpcServiceB\x15\xf2\x98\xfe\x8f\x05\x0f\x12\rotlp_exporterR\vgrpcService\x12[\n" +
	"\fhttp_service\x18\x03 \x01(\v2!.envoy.config.core.v3.HttpServiceB\x15\xf2\x98\xfe\x8f\x05\x0f\x12\rotlp_exporterR\vhttpService\x12!\n" +
	"\fservice_name\x18\x02 \x01(\tR\vserviceName\x12Y\n" +
	"\x12resource_detectors\x18\x04 \x03(\v2*.envoy.config.core.v3.TypedExtensionConfigR\x11resourceDetectors\x12D\n" +
	"\asampler\x18\x05 \x01(\v2*.envoy.config.core.v3.TypedExtensionConfigR\asampler\x12B\n" +
	"\x0emax_cache_size\x18\x06 \x01(\v2\x1c.google.protobuf.UInt32ValueR\fmaxCacheSizeB\x89\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"#io.envoyproxy.envoy.config.trace.v3B\x12OpentelemetryProtoP\x01ZDgithub.com/envoyproxy/go-control-plane/envoy/config/trace/v3;tracev3b\x06proto3"

var (
	file_envoy_config_trace_v3_opentelemetry_proto_rawDescOnce sync.Once
	file_envoy_config_trace_v3_opentelemetry_proto_rawDescData []byte
)

func file_envoy_config_trace_v3_opentelemetry_proto_rawDescGZIP() []byte {
	file_envoy_config_trace_v3_opentelemetry_proto_rawDescOnce.Do(func() {
		file_envoy_config_trace_v3_opentelemetry_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_trace_v3_opentelemetry_proto_rawDesc), len(file_envoy_config_trace_v3_opentelemetry_proto_rawDesc)))
	})
	return file_envoy_config_trace_v3_opentelemetry_proto_rawDescData
}

var file_envoy_config_trace_v3_opentelemetry_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_config_trace_v3_opentelemetry_proto_goTypes = []any{
	(*OpenTelemetryConfig)(nil),     // 0: envoy.config.trace.v3.OpenTelemetryConfig
	(*v3.GrpcService)(nil),          // 1: envoy.config.core.v3.GrpcService
	(*v3.HttpService)(nil),          // 2: envoy.config.core.v3.HttpService
	(*v3.TypedExtensionConfig)(nil), // 3: envoy.config.core.v3.TypedExtensionConfig
	(*wrapperspb.UInt32Value)(nil),  // 4: google.protobuf.UInt32Value
}
var file_envoy_config_trace_v3_opentelemetry_proto_depIdxs = []int32{
	1, // 0: envoy.config.trace.v3.OpenTelemetryConfig.grpc_service:type_name -> envoy.config.core.v3.GrpcService
	2, // 1: envoy.config.trace.v3.OpenTelemetryConfig.http_service:type_name -> envoy.config.core.v3.HttpService
	3, // 2: envoy.config.trace.v3.OpenTelemetryConfig.resource_detectors:type_name -> envoy.config.core.v3.TypedExtensionConfig
	3, // 3: envoy.config.trace.v3.OpenTelemetryConfig.sampler:type_name -> envoy.config.core.v3.TypedExtensionConfig
	4, // 4: envoy.config.trace.v3.OpenTelemetryConfig.max_cache_size:type_name -> google.protobuf.UInt32Value
	5, // [5:5] is the sub-list for method output_type
	5, // [5:5] is the sub-list for method input_type
	5, // [5:5] is the sub-list for extension type_name
	5, // [5:5] is the sub-list for extension extendee
	0, // [0:5] is the sub-list for field type_name
}

func init() { file_envoy_config_trace_v3_opentelemetry_proto_init() }
func file_envoy_config_trace_v3_opentelemetry_proto_init() {
	if File_envoy_config_trace_v3_opentelemetry_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_trace_v3_opentelemetry_proto_rawDesc), len(file_envoy_config_trace_v3_opentelemetry_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_trace_v3_opentelemetry_proto_goTypes,
		DependencyIndexes: file_envoy_config_trace_v3_opentelemetry_proto_depIdxs,
		MessageInfos:      file_envoy_config_trace_v3_opentelemetry_proto_msgTypes,
	}.Build()
	File_envoy_config_trace_v3_opentelemetry_proto = out.File
	file_envoy_config_trace_v3_opentelemetry_proto_goTypes = nil
	file_envoy_config_trace_v3_opentelemetry_proto_depIdxs = nil
}
