// Copyright 2025 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.31.0
// source: google/monitoring/v3/snooze.proto

package monitoringpb

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// A `Snooze` will prevent any alerts from being opened, and close any that
// are already open. The `Snooze` will work on alerts that match the
// criteria defined in the `Snooze`. The `Snooze` will be active from
// `interval.start_time` through `interval.end_time`.
type Snooze struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Identifier. The name of the `Snooze`. The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]/snoozes/[SNOOZE_ID]
	//
	// The ID of the `Snooze` will be generated by the system.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// Required. This defines the criteria for applying the `Snooze`. See
	// `Criteria` for more information.
	Criteria *Snooze_Criteria `protobuf:"bytes,3,opt,name=criteria,proto3" json:"criteria,omitempty"`
	// Required. The `Snooze` will be active from `interval.start_time` through
	// `interval.end_time`.
	// `interval.start_time` cannot be in the past. There is a 15 second clock
	// skew to account for the time it takes for a request to reach the API from
	// the UI.
	Interval *TimeInterval `protobuf:"bytes,4,opt,name=interval,proto3" json:"interval,omitempty"`
	// Required. A display name for the `Snooze`. This can be, at most, 512
	// unicode characters.
	DisplayName   string `protobuf:"bytes,5,opt,name=display_name,json=displayName,proto3" json:"display_name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Snooze) Reset() {
	*x = Snooze{}
	mi := &file_google_monitoring_v3_snooze_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Snooze) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Snooze) ProtoMessage() {}

func (x *Snooze) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_snooze_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Snooze.ProtoReflect.Descriptor instead.
func (*Snooze) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_snooze_proto_rawDescGZIP(), []int{0}
}

func (x *Snooze) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *Snooze) GetCriteria() *Snooze_Criteria {
	if x != nil {
		return x.Criteria
	}
	return nil
}

func (x *Snooze) GetInterval() *TimeInterval {
	if x != nil {
		return x.Interval
	}
	return nil
}

func (x *Snooze) GetDisplayName() string {
	if x != nil {
		return x.DisplayName
	}
	return ""
}

// Criteria specific to the `AlertPolicy`s that this `Snooze` applies to. The
// `Snooze` will suppress alerts that come from one of the `AlertPolicy`s
// whose names are supplied.
type Snooze_Criteria struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The specific `AlertPolicy` names for the alert that should be snoozed.
	// The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]/alertPolicies/[POLICY_ID]
	//
	// There is a limit of 16 policies per snooze. This limit is checked during
	// snooze creation.
	// Exactly 1 alert policy is required if `filter` is specified at the same
	// time.
	Policies []string `protobuf:"bytes,1,rep,name=policies,proto3" json:"policies,omitempty"`
	// Optional. The filter string to match on Alert fields when silencing the
	// alerts. It follows the standard https://google.aip.dev/160 syntax.
	// A filter string used to apply the snooze to specific incidents
	// that have matching filter values.
	// Filters can be defined for snoozes that apply to one alerting
	// policy.
	// Filters must be a string formatted as one or more resource labels with
	// specific label values. If multiple resource labels are used, then they
	// must be connected with an AND operator. For example, the following filter
	// applies the snooze to incidents that have an instance ID of
	// `1234567890` and a zone of `us-central1-a`:
	//
	//	resource.labels.instance_id="1234567890" AND
	//	resource.labels.zone="us-central1-a"
	Filter        string `protobuf:"bytes,2,opt,name=filter,proto3" json:"filter,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Snooze_Criteria) Reset() {
	*x = Snooze_Criteria{}
	mi := &file_google_monitoring_v3_snooze_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Snooze_Criteria) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Snooze_Criteria) ProtoMessage() {}

func (x *Snooze_Criteria) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_snooze_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Snooze_Criteria.ProtoReflect.Descriptor instead.
func (*Snooze_Criteria) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_snooze_proto_rawDescGZIP(), []int{0, 0}
}

func (x *Snooze_Criteria) GetPolicies() []string {
	if x != nil {
		return x.Policies
	}
	return nil
}

func (x *Snooze_Criteria) GetFilter() string {
	if x != nil {
		return x.Filter
	}
	return ""
}

var File_google_monitoring_v3_snooze_proto protoreflect.FileDescriptor

const file_google_monitoring_v3_snooze_proto_rawDesc = "" +
	"\n" +
	"!google/monitoring/v3/snooze.proto\x12\x14google.monitoring.v3\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a!google/monitoring/v3/common.proto\"\x8e\x03\n" +
	"\x06Snooze\x12\x17\n" +
	"\x04name\x18\x01 \x01(\tB\x03\xe0A\bR\x04name\x12F\n" +
	"\bcriteria\x18\x03 \x01(\v2%.google.monitoring.v3.Snooze.CriteriaB\x03\xe0A\x02R\bcriteria\x12C\n" +
	"\binterval\x18\x04 \x01(\v2\".google.monitoring.v3.TimeIntervalB\x03\xe0A\x02R\binterval\x12&\n" +
	"\fdisplay_name\x18\x05 \x01(\tB\x03\xe0A\x02R\vdisplayName\x1aj\n" +
	"\bCriteria\x12F\n" +
	"\bpolicies\x18\x01 \x03(\tB*\xfaA'\n" +
	"%monitoring.googleapis.com/AlertPolicyR\bpolicies\x12\x16\n" +
	"\x06filter\x18\x02 \x01(\tR\x06filter:J\xeaAG\n" +
	" monitoring.googleapis.com/Snooze\x12#projects/{project}/snoozes/{snooze}B\xc6\x01\n" +
	"\x18com.google.monitoring.v3B\vSnoozeProtoP\x01ZAcloud.google.com/go/monitoring/apiv3/v2/monitoringpb;monitoringpb\xaa\x02\x1aGoogle.Cloud.Monitoring.V3\xca\x02\x1aGoogle\\Cloud\\Monitoring\\V3\xea\x02\x1dGoogle::Cloud::Monitoring::V3b\x06proto3"

var (
	file_google_monitoring_v3_snooze_proto_rawDescOnce sync.Once
	file_google_monitoring_v3_snooze_proto_rawDescData []byte
)

func file_google_monitoring_v3_snooze_proto_rawDescGZIP() []byte {
	file_google_monitoring_v3_snooze_proto_rawDescOnce.Do(func() {
		file_google_monitoring_v3_snooze_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_google_monitoring_v3_snooze_proto_rawDesc), len(file_google_monitoring_v3_snooze_proto_rawDesc)))
	})
	return file_google_monitoring_v3_snooze_proto_rawDescData
}

var file_google_monitoring_v3_snooze_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_google_monitoring_v3_snooze_proto_goTypes = []any{
	(*Snooze)(nil),          // 0: google.monitoring.v3.Snooze
	(*Snooze_Criteria)(nil), // 1: google.monitoring.v3.Snooze.Criteria
	(*TimeInterval)(nil),    // 2: google.monitoring.v3.TimeInterval
}
var file_google_monitoring_v3_snooze_proto_depIdxs = []int32{
	1, // 0: google.monitoring.v3.Snooze.criteria:type_name -> google.monitoring.v3.Snooze.Criteria
	2, // 1: google.monitoring.v3.Snooze.interval:type_name -> google.monitoring.v3.TimeInterval
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_google_monitoring_v3_snooze_proto_init() }
func file_google_monitoring_v3_snooze_proto_init() {
	if File_google_monitoring_v3_snooze_proto != nil {
		return
	}
	file_google_monitoring_v3_common_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_monitoring_v3_snooze_proto_rawDesc), len(file_google_monitoring_v3_snooze_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_google_monitoring_v3_snooze_proto_goTypes,
		DependencyIndexes: file_google_monitoring_v3_snooze_proto_depIdxs,
		MessageInfos:      file_google_monitoring_v3_snooze_proto_msgTypes,
	}.Build()
	File_google_monitoring_v3_snooze_proto = out.File
	file_google_monitoring_v3_snooze_proto_goTypes = nil
	file_google_monitoring_v3_snooze_proto_depIdxs = nil
}
