// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/core/v3/grpc_method_list.proto

package corev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// A list of gRPC methods which can be used as an allowlist, for example.
type GrpcMethodList struct {
	state         protoimpl.MessageState    `protogen:"open.v1"`
	Services      []*GrpcMethodList_Service `protobuf:"bytes,1,rep,name=services,proto3" json:"services,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GrpcMethodList) Reset() {
	*x = GrpcMethodList{}
	mi := &file_envoy_config_core_v3_grpc_method_list_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GrpcMethodList) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GrpcMethodList) ProtoMessage() {}

func (x *GrpcMethodList) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_core_v3_grpc_method_list_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GrpcMethodList.ProtoReflect.Descriptor instead.
func (*GrpcMethodList) Descriptor() ([]byte, []int) {
	return file_envoy_config_core_v3_grpc_method_list_proto_rawDescGZIP(), []int{0}
}

func (x *GrpcMethodList) GetServices() []*GrpcMethodList_Service {
	if x != nil {
		return x.Services
	}
	return nil
}

type GrpcMethodList_Service struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name of the gRPC service.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// The names of the gRPC methods in this service.
	MethodNames   []string `protobuf:"bytes,2,rep,name=method_names,json=methodNames,proto3" json:"method_names,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GrpcMethodList_Service) Reset() {
	*x = GrpcMethodList_Service{}
	mi := &file_envoy_config_core_v3_grpc_method_list_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GrpcMethodList_Service) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GrpcMethodList_Service) ProtoMessage() {}

func (x *GrpcMethodList_Service) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_core_v3_grpc_method_list_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GrpcMethodList_Service.ProtoReflect.Descriptor instead.
func (*GrpcMethodList_Service) Descriptor() ([]byte, []int) {
	return file_envoy_config_core_v3_grpc_method_list_proto_rawDescGZIP(), []int{0, 0}
}

func (x *GrpcMethodList_Service) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *GrpcMethodList_Service) GetMethodNames() []string {
	if x != nil {
		return x.MethodNames
	}
	return nil
}

var File_envoy_config_core_v3_grpc_method_list_proto protoreflect.FileDescriptor

const file_envoy_config_core_v3_grpc_method_list_proto_rawDesc = "" +
	"\n" +
	"+envoy/config/core/v3/grpc_method_list.proto\x12\x14envoy.config.core.v3\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\x8a\x02\n" +
	"\x0eGrpcMethodList\x12H\n" +
	"\bservices\x18\x01 \x03(\v2,.envoy.config.core.v3.GrpcMethodList.ServiceR\bservices\x1a\x84\x01\n" +
	"\aService\x12\x1b\n" +
	"\x04name\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x04name\x12+\n" +
	"\fmethod_names\x18\x02 \x03(\tB\b\xfaB\x05\x92\x01\x02\b\x01R\vmethodNames:/\x9aň\x1e*\n" +
	"(envoy.api.v2.core.GrpcMethodList.Service:'\x9aň\x1e\"\n" +
	" envoy.api.v2.core.GrpcMethodListB\x87\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"\"io.envoyproxy.envoy.config.core.v3B\x13GrpcMethodListProtoP\x01ZBgithub.com/envoyproxy/go-control-plane/envoy/config/core/v3;corev3b\x06proto3"

var (
	file_envoy_config_core_v3_grpc_method_list_proto_rawDescOnce sync.Once
	file_envoy_config_core_v3_grpc_method_list_proto_rawDescData []byte
)

func file_envoy_config_core_v3_grpc_method_list_proto_rawDescGZIP() []byte {
	file_envoy_config_core_v3_grpc_method_list_proto_rawDescOnce.Do(func() {
		file_envoy_config_core_v3_grpc_method_list_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_core_v3_grpc_method_list_proto_rawDesc), len(file_envoy_config_core_v3_grpc_method_list_proto_rawDesc)))
	})
	return file_envoy_config_core_v3_grpc_method_list_proto_rawDescData
}

var file_envoy_config_core_v3_grpc_method_list_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_config_core_v3_grpc_method_list_proto_goTypes = []any{
	(*GrpcMethodList)(nil),         // 0: envoy.config.core.v3.GrpcMethodList
	(*GrpcMethodList_Service)(nil), // 1: envoy.config.core.v3.GrpcMethodList.Service
}
var file_envoy_config_core_v3_grpc_method_list_proto_depIdxs = []int32{
	1, // 0: envoy.config.core.v3.GrpcMethodList.services:type_name -> envoy.config.core.v3.GrpcMethodList.Service
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_config_core_v3_grpc_method_list_proto_init() }
func file_envoy_config_core_v3_grpc_method_list_proto_init() {
	if File_envoy_config_core_v3_grpc_method_list_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_core_v3_grpc_method_list_proto_rawDesc), len(file_envoy_config_core_v3_grpc_method_list_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_core_v3_grpc_method_list_proto_goTypes,
		DependencyIndexes: file_envoy_config_core_v3_grpc_method_list_proto_depIdxs,
		MessageInfos:      file_envoy_config_core_v3_grpc_method_list_proto_msgTypes,
	}.Build()
	File_envoy_config_core_v3_grpc_method_list_proto = out.File
	file_envoy_config_core_v3_grpc_method_list_proto_goTypes = nil
	file_envoy_config_core_v3_grpc_method_list_proto_depIdxs = nil
}
