// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/annotations/deprecation.proto

package annotations

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	descriptorpb "google.golang.org/protobuf/types/descriptorpb"
	reflect "reflect"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

var file_envoy_annotations_deprecation_proto_extTypes = []protoimpl.ExtensionInfo{
	{
		ExtendedType:  (*descriptorpb.FieldOptions)(nil),
		ExtensionType: (*bool)(nil),
		Field:         189503207,
		Name:          "envoy.annotations.disallowed_by_default",
		Tag:           "varint,189503207,opt,name=disallowed_by_default",
		Filename:      "envoy/annotations/deprecation.proto",
	},
	{
		ExtendedType:  (*descriptorpb.FieldOptions)(nil),
		ExtensionType: (*string)(nil),
		Field:         157299826,
		Name:          "envoy.annotations.deprecated_at_minor_version",
		Tag:           "bytes,157299826,opt,name=deprecated_at_minor_version",
		Filename:      "envoy/annotations/deprecation.proto",
	},
	{
		ExtendedType:  (*descriptorpb.EnumValueOptions)(nil),
		ExtensionType: (*bool)(nil),
		Field:         70100853,
		Name:          "envoy.annotations.disallowed_by_default_enum",
		Tag:           "varint,70100853,opt,name=disallowed_by_default_enum",
		Filename:      "envoy/annotations/deprecation.proto",
	},
	{
		ExtendedType:  (*descriptorpb.EnumValueOptions)(nil),
		ExtensionType: (*string)(nil),
		Field:         181198657,
		Name:          "envoy.annotations.deprecated_at_minor_version_enum",
		Tag:           "bytes,181198657,opt,name=deprecated_at_minor_version_enum",
		Filename:      "envoy/annotations/deprecation.proto",
	},
}

// Extension fields to descriptorpb.FieldOptions.
var (
	// optional bool disallowed_by_default = 189503207;
	E_DisallowedByDefault = &file_envoy_annotations_deprecation_proto_extTypes[0]
	// The API major and minor version on which the field was deprecated
	// (e.g., "3.5" for major version 3 and minor version 5).
	//
	// optional string deprecated_at_minor_version = 157299826;
	E_DeprecatedAtMinorVersion = &file_envoy_annotations_deprecation_proto_extTypes[1]
)

// Extension fields to descriptorpb.EnumValueOptions.
var (
	// optional bool disallowed_by_default_enum = 70100853;
	E_DisallowedByDefaultEnum = &file_envoy_annotations_deprecation_proto_extTypes[2]
	// The API major and minor version on which the enum value was deprecated
	// (e.g., "3.5" for major version 3 and minor version 5).
	//
	// optional string deprecated_at_minor_version_enum = 181198657;
	E_DeprecatedAtMinorVersionEnum = &file_envoy_annotations_deprecation_proto_extTypes[3]
)

var File_envoy_annotations_deprecation_proto protoreflect.FileDescriptor

const file_envoy_annotations_deprecation_proto_rawDesc = "" +
	"\n" +
	"#envoy/annotations/deprecation.proto\x12\x11envoy.annotations\x1a google/protobuf/descriptor.proto:T\n" +
	"\x15disallowed_by_default\x12\x1d.google.protobuf.FieldOptions\x18筮Z \x01(\bR\x13disallowedByDefault:_\n" +
	"\x1bdeprecated_at_minor_version\x12\x1d.google.protobuf.FieldOptions\x18\xf2\xe8\x80K \x01(\tR\x18deprecatedAtMinorVersion:a\n" +
	"\x1adisallowed_by_default_enum\x12!.google.protobuf.EnumValueOptions\x18\xf5ζ! \x01(\bR\x17disallowedByDefaultEnum:l\n" +
	" deprecated_at_minor_version_enum\x12!.google.protobuf.EnumValueOptions\x18\xc1\xbe\xb3V \x01(\tR\x1cdeprecatedAtMinorVersionEnumB:Z8github.com/envoyproxy/go-control-plane/envoy/annotationsb\x06proto3"

var file_envoy_annotations_deprecation_proto_goTypes = []any{
	(*descriptorpb.FieldOptions)(nil),     // 0: google.protobuf.FieldOptions
	(*descriptorpb.EnumValueOptions)(nil), // 1: google.protobuf.EnumValueOptions
}
var file_envoy_annotations_deprecation_proto_depIdxs = []int32{
	0, // 0: envoy.annotations.disallowed_by_default:extendee -> google.protobuf.FieldOptions
	0, // 1: envoy.annotations.deprecated_at_minor_version:extendee -> google.protobuf.FieldOptions
	1, // 2: envoy.annotations.disallowed_by_default_enum:extendee -> google.protobuf.EnumValueOptions
	1, // 3: envoy.annotations.deprecated_at_minor_version_enum:extendee -> google.protobuf.EnumValueOptions
	4, // [4:4] is the sub-list for method output_type
	4, // [4:4] is the sub-list for method input_type
	4, // [4:4] is the sub-list for extension type_name
	0, // [0:4] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_envoy_annotations_deprecation_proto_init() }
func file_envoy_annotations_deprecation_proto_init() {
	if File_envoy_annotations_deprecation_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_annotations_deprecation_proto_rawDesc), len(file_envoy_annotations_deprecation_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   0,
			NumExtensions: 4,
			NumServices:   0,
		},
		GoTypes:           file_envoy_annotations_deprecation_proto_goTypes,
		DependencyIndexes: file_envoy_annotations_deprecation_proto_depIdxs,
		ExtensionInfos:    file_envoy_annotations_deprecation_proto_extTypes,
	}.Build()
	File_envoy_annotations_deprecation_proto = out.File
	file_envoy_annotations_deprecation_proto_goTypes = nil
	file_envoy_annotations_deprecation_proto_depIdxs = nil
}
