// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.1
// source: xds/annotations/v3/migrate.proto

package v3

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	descriptorpb "google.golang.org/protobuf/types/descriptorpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type MigrateAnnotation struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Rename        string                 `protobuf:"bytes,1,opt,name=rename,proto3" json:"rename,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MigrateAnnotation) Reset() {
	*x = MigrateAnnotation{}
	mi := &file_xds_annotations_v3_migrate_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MigrateAnnotation) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MigrateAnnotation) ProtoMessage() {}

func (x *MigrateAnnotation) ProtoReflect() protoreflect.Message {
	mi := &file_xds_annotations_v3_migrate_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MigrateAnnotation.ProtoReflect.Descriptor instead.
func (*MigrateAnnotation) Descriptor() ([]byte, []int) {
	return file_xds_annotations_v3_migrate_proto_rawDescGZIP(), []int{0}
}

func (x *MigrateAnnotation) GetRename() string {
	if x != nil {
		return x.Rename
	}
	return ""
}

type FieldMigrateAnnotation struct {
	state          protoimpl.MessageState `protogen:"open.v1"`
	Rename         string                 `protobuf:"bytes,1,opt,name=rename,proto3" json:"rename,omitempty"`
	OneofPromotion string                 `protobuf:"bytes,2,opt,name=oneof_promotion,json=oneofPromotion,proto3" json:"oneof_promotion,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *FieldMigrateAnnotation) Reset() {
	*x = FieldMigrateAnnotation{}
	mi := &file_xds_annotations_v3_migrate_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FieldMigrateAnnotation) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FieldMigrateAnnotation) ProtoMessage() {}

func (x *FieldMigrateAnnotation) ProtoReflect() protoreflect.Message {
	mi := &file_xds_annotations_v3_migrate_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FieldMigrateAnnotation.ProtoReflect.Descriptor instead.
func (*FieldMigrateAnnotation) Descriptor() ([]byte, []int) {
	return file_xds_annotations_v3_migrate_proto_rawDescGZIP(), []int{1}
}

func (x *FieldMigrateAnnotation) GetRename() string {
	if x != nil {
		return x.Rename
	}
	return ""
}

func (x *FieldMigrateAnnotation) GetOneofPromotion() string {
	if x != nil {
		return x.OneofPromotion
	}
	return ""
}

type FileMigrateAnnotation struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	MoveToPackage string                 `protobuf:"bytes,2,opt,name=move_to_package,json=moveToPackage,proto3" json:"move_to_package,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FileMigrateAnnotation) Reset() {
	*x = FileMigrateAnnotation{}
	mi := &file_xds_annotations_v3_migrate_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FileMigrateAnnotation) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FileMigrateAnnotation) ProtoMessage() {}

func (x *FileMigrateAnnotation) ProtoReflect() protoreflect.Message {
	mi := &file_xds_annotations_v3_migrate_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FileMigrateAnnotation.ProtoReflect.Descriptor instead.
func (*FileMigrateAnnotation) Descriptor() ([]byte, []int) {
	return file_xds_annotations_v3_migrate_proto_rawDescGZIP(), []int{2}
}

func (x *FileMigrateAnnotation) GetMoveToPackage() string {
	if x != nil {
		return x.MoveToPackage
	}
	return ""
}

var file_xds_annotations_v3_migrate_proto_extTypes = []protoimpl.ExtensionInfo{
	{
		ExtendedType:  (*descriptorpb.MessageOptions)(nil),
		ExtensionType: (*MigrateAnnotation)(nil),
		Field:         112948430,
		Name:          "xds.annotations.v3.message_migrate",
		Tag:           "bytes,112948430,opt,name=message_migrate",
		Filename:      "xds/annotations/v3/migrate.proto",
	},
	{
		ExtendedType:  (*descriptorpb.FieldOptions)(nil),
		ExtensionType: (*FieldMigrateAnnotation)(nil),
		Field:         112948430,
		Name:          "xds.annotations.v3.field_migrate",
		Tag:           "bytes,112948430,opt,name=field_migrate",
		Filename:      "xds/annotations/v3/migrate.proto",
	},
	{
		ExtendedType:  (*descriptorpb.EnumOptions)(nil),
		ExtensionType: (*MigrateAnnotation)(nil),
		Field:         112948430,
		Name:          "xds.annotations.v3.enum_migrate",
		Tag:           "bytes,112948430,opt,name=enum_migrate",
		Filename:      "xds/annotations/v3/migrate.proto",
	},
	{
		ExtendedType:  (*descriptorpb.EnumValueOptions)(nil),
		ExtensionType: (*MigrateAnnotation)(nil),
		Field:         112948430,
		Name:          "xds.annotations.v3.enum_value_migrate",
		Tag:           "bytes,112948430,opt,name=enum_value_migrate",
		Filename:      "xds/annotations/v3/migrate.proto",
	},
	{
		ExtendedType:  (*descriptorpb.FileOptions)(nil),
		ExtensionType: (*FileMigrateAnnotation)(nil),
		Field:         112948430,
		Name:          "xds.annotations.v3.file_migrate",
		Tag:           "bytes,112948430,opt,name=file_migrate",
		Filename:      "xds/annotations/v3/migrate.proto",
	},
}

// Extension fields to descriptorpb.MessageOptions.
var (
	// optional xds.annotations.v3.MigrateAnnotation message_migrate = 112948430;
	E_MessageMigrate = &file_xds_annotations_v3_migrate_proto_extTypes[0]
)

// Extension fields to descriptorpb.FieldOptions.
var (
	// optional xds.annotations.v3.FieldMigrateAnnotation field_migrate = 112948430;
	E_FieldMigrate = &file_xds_annotations_v3_migrate_proto_extTypes[1]
)

// Extension fields to descriptorpb.EnumOptions.
var (
	// optional xds.annotations.v3.MigrateAnnotation enum_migrate = 112948430;
	E_EnumMigrate = &file_xds_annotations_v3_migrate_proto_extTypes[2]
)

// Extension fields to descriptorpb.EnumValueOptions.
var (
	// optional xds.annotations.v3.MigrateAnnotation enum_value_migrate = 112948430;
	E_EnumValueMigrate = &file_xds_annotations_v3_migrate_proto_extTypes[3]
)

// Extension fields to descriptorpb.FileOptions.
var (
	// optional xds.annotations.v3.FileMigrateAnnotation file_migrate = 112948430;
	E_FileMigrate = &file_xds_annotations_v3_migrate_proto_extTypes[4]
)

var File_xds_annotations_v3_migrate_proto protoreflect.FileDescriptor

const file_xds_annotations_v3_migrate_proto_rawDesc = "" +
	"\n" +
	" xds/annotations/v3/migrate.proto\x12\x12xds.annotations.v3\x1a google/protobuf/descriptor.proto\"+\n" +
	"\x11MigrateAnnotation\x12\x16\n" +
	"\x06rename\x18\x01 \x01(\tR\x06rename\"Y\n" +
	"\x16FieldMigrateAnnotation\x12\x16\n" +
	"\x06rename\x18\x01 \x01(\tR\x06rename\x12'\n" +
	"\x0foneof_promotion\x18\x02 \x01(\tR\x0eoneofPromotion\"?\n" +
	"\x15FileMigrateAnnotation\x12&\n" +
	"\x0fmove_to_package\x18\x02 \x01(\tR\rmoveToPackage:r\n" +
	"\x0fmessage_migrate\x12\x1f.google.protobuf.MessageOptions\x18\xce\xe9\xed5 \x01(\v2%.xds.annotations.v3.MigrateAnnotationR\x0emessageMigrate:q\n" +
	"\rfield_migrate\x12\x1d.google.protobuf.FieldOptions\x18\xce\xe9\xed5 \x01(\v2*.xds.annotations.v3.FieldMigrateAnnotationR\ffieldMigrate:i\n" +
	"\fenum_migrate\x12\x1c.google.protobuf.EnumOptions\x18\xce\xe9\xed5 \x01(\v2%.xds.annotations.v3.MigrateAnnotationR\venumMigrate:y\n" +
	"\x12enum_value_migrate\x12!.google.protobuf.EnumValueOptions\x18\xce\xe9\xed5 \x01(\v2%.xds.annotations.v3.MigrateAnnotationR\x10enumValueMigrate:m\n" +
	"\ffile_migrate\x12\x1c.google.protobuf.FileOptions\x18\xce\xe9\xed5 \x01(\v2).xds.annotations.v3.FileMigrateAnnotationR\vfileMigrateB+Z)github.com/cncf/xds/go/xds/annotations/v3b\x06proto3"

var (
	file_xds_annotations_v3_migrate_proto_rawDescOnce sync.Once
	file_xds_annotations_v3_migrate_proto_rawDescData []byte
)

func file_xds_annotations_v3_migrate_proto_rawDescGZIP() []byte {
	file_xds_annotations_v3_migrate_proto_rawDescOnce.Do(func() {
		file_xds_annotations_v3_migrate_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_xds_annotations_v3_migrate_proto_rawDesc), len(file_xds_annotations_v3_migrate_proto_rawDesc)))
	})
	return file_xds_annotations_v3_migrate_proto_rawDescData
}

var file_xds_annotations_v3_migrate_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_xds_annotations_v3_migrate_proto_goTypes = []any{
	(*MigrateAnnotation)(nil),             // 0: xds.annotations.v3.MigrateAnnotation
	(*FieldMigrateAnnotation)(nil),        // 1: xds.annotations.v3.FieldMigrateAnnotation
	(*FileMigrateAnnotation)(nil),         // 2: xds.annotations.v3.FileMigrateAnnotation
	(*descriptorpb.MessageOptions)(nil),   // 3: google.protobuf.MessageOptions
	(*descriptorpb.FieldOptions)(nil),     // 4: google.protobuf.FieldOptions
	(*descriptorpb.EnumOptions)(nil),      // 5: google.protobuf.EnumOptions
	(*descriptorpb.EnumValueOptions)(nil), // 6: google.protobuf.EnumValueOptions
	(*descriptorpb.FileOptions)(nil),      // 7: google.protobuf.FileOptions
}
var file_xds_annotations_v3_migrate_proto_depIdxs = []int32{
	3,  // 0: xds.annotations.v3.message_migrate:extendee -> google.protobuf.MessageOptions
	4,  // 1: xds.annotations.v3.field_migrate:extendee -> google.protobuf.FieldOptions
	5,  // 2: xds.annotations.v3.enum_migrate:extendee -> google.protobuf.EnumOptions
	6,  // 3: xds.annotations.v3.enum_value_migrate:extendee -> google.protobuf.EnumValueOptions
	7,  // 4: xds.annotations.v3.file_migrate:extendee -> google.protobuf.FileOptions
	0,  // 5: xds.annotations.v3.message_migrate:type_name -> xds.annotations.v3.MigrateAnnotation
	1,  // 6: xds.annotations.v3.field_migrate:type_name -> xds.annotations.v3.FieldMigrateAnnotation
	0,  // 7: xds.annotations.v3.enum_migrate:type_name -> xds.annotations.v3.MigrateAnnotation
	0,  // 8: xds.annotations.v3.enum_value_migrate:type_name -> xds.annotations.v3.MigrateAnnotation
	2,  // 9: xds.annotations.v3.file_migrate:type_name -> xds.annotations.v3.FileMigrateAnnotation
	10, // [10:10] is the sub-list for method output_type
	10, // [10:10] is the sub-list for method input_type
	5,  // [5:10] is the sub-list for extension type_name
	0,  // [0:5] is the sub-list for extension extendee
	0,  // [0:0] is the sub-list for field type_name
}

func init() { file_xds_annotations_v3_migrate_proto_init() }
func file_xds_annotations_v3_migrate_proto_init() {
	if File_xds_annotations_v3_migrate_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_xds_annotations_v3_migrate_proto_rawDesc), len(file_xds_annotations_v3_migrate_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   3,
			NumExtensions: 5,
			NumServices:   0,
		},
		GoTypes:           file_xds_annotations_v3_migrate_proto_goTypes,
		DependencyIndexes: file_xds_annotations_v3_migrate_proto_depIdxs,
		MessageInfos:      file_xds_annotations_v3_migrate_proto_msgTypes,
		ExtensionInfos:    file_xds_annotations_v3_migrate_proto_extTypes,
	}.Build()
	File_xds_annotations_v3_migrate_proto = out.File
	file_xds_annotations_v3_migrate_proto_goTypes = nil
	file_xds_annotations_v3_migrate_proto_depIdxs = nil
}
