// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.1
// source: xds/type/matcher/v3/string.proto

package v3

import (
	v3 "github.com/cncf/xds/go/xds/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type StringMatcher struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to MatchPattern:
	//
	//	*StringMatcher_Exact
	//	*StringMatcher_Prefix
	//	*StringMatcher_Suffix
	//	*StringMatcher_SafeRegex
	//	*StringMatcher_Contains
	//	*StringMatcher_Custom
	MatchPattern  isStringMatcher_MatchPattern `protobuf_oneof:"match_pattern"`
	IgnoreCase    bool                         `protobuf:"varint,6,opt,name=ignore_case,json=ignoreCase,proto3" json:"ignore_case,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *StringMatcher) Reset() {
	*x = StringMatcher{}
	mi := &file_xds_type_matcher_v3_string_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StringMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StringMatcher) ProtoMessage() {}

func (x *StringMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_string_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StringMatcher.ProtoReflect.Descriptor instead.
func (*StringMatcher) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_string_proto_rawDescGZIP(), []int{0}
}

func (x *StringMatcher) GetMatchPattern() isStringMatcher_MatchPattern {
	if x != nil {
		return x.MatchPattern
	}
	return nil
}

func (x *StringMatcher) GetExact() string {
	if x != nil {
		if x, ok := x.MatchPattern.(*StringMatcher_Exact); ok {
			return x.Exact
		}
	}
	return ""
}

func (x *StringMatcher) GetPrefix() string {
	if x != nil {
		if x, ok := x.MatchPattern.(*StringMatcher_Prefix); ok {
			return x.Prefix
		}
	}
	return ""
}

func (x *StringMatcher) GetSuffix() string {
	if x != nil {
		if x, ok := x.MatchPattern.(*StringMatcher_Suffix); ok {
			return x.Suffix
		}
	}
	return ""
}

func (x *StringMatcher) GetSafeRegex() *RegexMatcher {
	if x != nil {
		if x, ok := x.MatchPattern.(*StringMatcher_SafeRegex); ok {
			return x.SafeRegex
		}
	}
	return nil
}

func (x *StringMatcher) GetContains() string {
	if x != nil {
		if x, ok := x.MatchPattern.(*StringMatcher_Contains); ok {
			return x.Contains
		}
	}
	return ""
}

func (x *StringMatcher) GetCustom() *v3.TypedExtensionConfig {
	if x != nil {
		if x, ok := x.MatchPattern.(*StringMatcher_Custom); ok {
			return x.Custom
		}
	}
	return nil
}

func (x *StringMatcher) GetIgnoreCase() bool {
	if x != nil {
		return x.IgnoreCase
	}
	return false
}

type isStringMatcher_MatchPattern interface {
	isStringMatcher_MatchPattern()
}

type StringMatcher_Exact struct {
	Exact string `protobuf:"bytes,1,opt,name=exact,proto3,oneof"`
}

type StringMatcher_Prefix struct {
	Prefix string `protobuf:"bytes,2,opt,name=prefix,proto3,oneof"`
}

type StringMatcher_Suffix struct {
	Suffix string `protobuf:"bytes,3,opt,name=suffix,proto3,oneof"`
}

type StringMatcher_SafeRegex struct {
	SafeRegex *RegexMatcher `protobuf:"bytes,5,opt,name=safe_regex,json=safeRegex,proto3,oneof"`
}

type StringMatcher_Contains struct {
	Contains string `protobuf:"bytes,7,opt,name=contains,proto3,oneof"`
}

type StringMatcher_Custom struct {
	Custom *v3.TypedExtensionConfig `protobuf:"bytes,8,opt,name=custom,proto3,oneof"`
}

func (*StringMatcher_Exact) isStringMatcher_MatchPattern() {}

func (*StringMatcher_Prefix) isStringMatcher_MatchPattern() {}

func (*StringMatcher_Suffix) isStringMatcher_MatchPattern() {}

func (*StringMatcher_SafeRegex) isStringMatcher_MatchPattern() {}

func (*StringMatcher_Contains) isStringMatcher_MatchPattern() {}

func (*StringMatcher_Custom) isStringMatcher_MatchPattern() {}

type ListStringMatcher struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Patterns      []*StringMatcher       `protobuf:"bytes,1,rep,name=patterns,proto3" json:"patterns,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListStringMatcher) Reset() {
	*x = ListStringMatcher{}
	mi := &file_xds_type_matcher_v3_string_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListStringMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListStringMatcher) ProtoMessage() {}

func (x *ListStringMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_string_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListStringMatcher.ProtoReflect.Descriptor instead.
func (*ListStringMatcher) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_string_proto_rawDescGZIP(), []int{1}
}

func (x *ListStringMatcher) GetPatterns() []*StringMatcher {
	if x != nil {
		return x.Patterns
	}
	return nil
}

var File_xds_type_matcher_v3_string_proto protoreflect.FileDescriptor

const file_xds_type_matcher_v3_string_proto_rawDesc = "" +
	"\n" +
	" xds/type/matcher/v3/string.proto\x12\x13xds.type.matcher.v3\x1a\x1bxds/core/v3/extension.proto\x1a\x1fxds/type/matcher/v3/regex.proto\x1a\x17validate/validate.proto\"\xd6\x02\n" +
	"\rStringMatcher\x12\x16\n" +
	"\x05exact\x18\x01 \x01(\tH\x00R\x05exact\x12!\n" +
	"\x06prefix\x18\x02 \x01(\tB\a\xfaB\x04r\x02\x10\x01H\x00R\x06prefix\x12!\n" +
	"\x06suffix\x18\x03 \x01(\tB\a\xfaB\x04r\x02\x10\x01H\x00R\x06suffix\x12L\n" +
	"\n" +
	"safe_regex\x18\x05 \x01(\v2!.xds.type.matcher.v3.RegexMatcherB\b\xfaB\x05\x8a\x01\x02\x10\x01H\x00R\tsafeRegex\x12%\n" +
	"\bcontains\x18\a \x01(\tB\a\xfaB\x04r\x02\x10\x01H\x00R\bcontains\x12;\n" +
	"\x06custom\x18\b \x01(\v2!.xds.core.v3.TypedExtensionConfigH\x00R\x06custom\x12\x1f\n" +
	"\vignore_case\x18\x06 \x01(\bR\n" +
	"ignoreCaseB\x14\n" +
	"\rmatch_pattern\x12\x03\xf8B\x01\"]\n" +
	"\x11ListStringMatcher\x12H\n" +
	"\bpatterns\x18\x01 \x03(\v2\".xds.type.matcher.v3.StringMatcherB\b\xfaB\x05\x92\x01\x02\b\x01R\bpatternsB[\n" +
	"\x1ecom.github.xds.type.matcher.v3B\vStringProtoP\x01Z*github.com/cncf/xds/go/xds/type/matcher/v3b\x06proto3"

var (
	file_xds_type_matcher_v3_string_proto_rawDescOnce sync.Once
	file_xds_type_matcher_v3_string_proto_rawDescData []byte
)

func file_xds_type_matcher_v3_string_proto_rawDescGZIP() []byte {
	file_xds_type_matcher_v3_string_proto_rawDescOnce.Do(func() {
		file_xds_type_matcher_v3_string_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_xds_type_matcher_v3_string_proto_rawDesc), len(file_xds_type_matcher_v3_string_proto_rawDesc)))
	})
	return file_xds_type_matcher_v3_string_proto_rawDescData
}

var file_xds_type_matcher_v3_string_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_xds_type_matcher_v3_string_proto_goTypes = []any{
	(*StringMatcher)(nil),           // 0: xds.type.matcher.v3.StringMatcher
	(*ListStringMatcher)(nil),       // 1: xds.type.matcher.v3.ListStringMatcher
	(*RegexMatcher)(nil),            // 2: xds.type.matcher.v3.RegexMatcher
	(*v3.TypedExtensionConfig)(nil), // 3: xds.core.v3.TypedExtensionConfig
}
var file_xds_type_matcher_v3_string_proto_depIdxs = []int32{
	2, // 0: xds.type.matcher.v3.StringMatcher.safe_regex:type_name -> xds.type.matcher.v3.RegexMatcher
	3, // 1: xds.type.matcher.v3.StringMatcher.custom:type_name -> xds.core.v3.TypedExtensionConfig
	0, // 2: xds.type.matcher.v3.ListStringMatcher.patterns:type_name -> xds.type.matcher.v3.StringMatcher
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() { file_xds_type_matcher_v3_string_proto_init() }
func file_xds_type_matcher_v3_string_proto_init() {
	if File_xds_type_matcher_v3_string_proto != nil {
		return
	}
	file_xds_type_matcher_v3_regex_proto_init()
	file_xds_type_matcher_v3_string_proto_msgTypes[0].OneofWrappers = []any{
		(*StringMatcher_Exact)(nil),
		(*StringMatcher_Prefix)(nil),
		(*StringMatcher_Suffix)(nil),
		(*StringMatcher_SafeRegex)(nil),
		(*StringMatcher_Contains)(nil),
		(*StringMatcher_Custom)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_xds_type_matcher_v3_string_proto_rawDesc), len(file_xds_type_matcher_v3_string_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_xds_type_matcher_v3_string_proto_goTypes,
		DependencyIndexes: file_xds_type_matcher_v3_string_proto_depIdxs,
		MessageInfos:      file_xds_type_matcher_v3_string_proto_msgTypes,
	}.Build()
	File_xds_type_matcher_v3_string_proto = out.File
	file_xds_type_matcher_v3_string_proto_goTypes = nil
	file_xds_type_matcher_v3_string_proto_depIdxs = nil
}
