// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.1
// source: xds/core/v3/collection_entry.proto

package v3

import (
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	anypb "google.golang.org/protobuf/types/known/anypb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type CollectionEntry struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to ResourceSpecifier:
	//
	//	*CollectionEntry_Locator
	//	*CollectionEntry_InlineEntry_
	ResourceSpecifier isCollectionEntry_ResourceSpecifier `protobuf_oneof:"resource_specifier"`
	unknownFields     protoimpl.UnknownFields
	sizeCache         protoimpl.SizeCache
}

func (x *CollectionEntry) Reset() {
	*x = CollectionEntry{}
	mi := &file_xds_core_v3_collection_entry_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CollectionEntry) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CollectionEntry) ProtoMessage() {}

func (x *CollectionEntry) ProtoReflect() protoreflect.Message {
	mi := &file_xds_core_v3_collection_entry_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CollectionEntry.ProtoReflect.Descriptor instead.
func (*CollectionEntry) Descriptor() ([]byte, []int) {
	return file_xds_core_v3_collection_entry_proto_rawDescGZIP(), []int{0}
}

func (x *CollectionEntry) GetResourceSpecifier() isCollectionEntry_ResourceSpecifier {
	if x != nil {
		return x.ResourceSpecifier
	}
	return nil
}

func (x *CollectionEntry) GetLocator() *ResourceLocator {
	if x != nil {
		if x, ok := x.ResourceSpecifier.(*CollectionEntry_Locator); ok {
			return x.Locator
		}
	}
	return nil
}

func (x *CollectionEntry) GetInlineEntry() *CollectionEntry_InlineEntry {
	if x != nil {
		if x, ok := x.ResourceSpecifier.(*CollectionEntry_InlineEntry_); ok {
			return x.InlineEntry
		}
	}
	return nil
}

type isCollectionEntry_ResourceSpecifier interface {
	isCollectionEntry_ResourceSpecifier()
}

type CollectionEntry_Locator struct {
	Locator *ResourceLocator `protobuf:"bytes,1,opt,name=locator,proto3,oneof"`
}

type CollectionEntry_InlineEntry_ struct {
	InlineEntry *CollectionEntry_InlineEntry `protobuf:"bytes,2,opt,name=inline_entry,json=inlineEntry,proto3,oneof"`
}

func (*CollectionEntry_Locator) isCollectionEntry_ResourceSpecifier() {}

func (*CollectionEntry_InlineEntry_) isCollectionEntry_ResourceSpecifier() {}

type CollectionEntry_InlineEntry struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Name          string                 `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	Version       string                 `protobuf:"bytes,2,opt,name=version,proto3" json:"version,omitempty"`
	Resource      *anypb.Any             `protobuf:"bytes,3,opt,name=resource,proto3" json:"resource,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CollectionEntry_InlineEntry) Reset() {
	*x = CollectionEntry_InlineEntry{}
	mi := &file_xds_core_v3_collection_entry_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CollectionEntry_InlineEntry) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CollectionEntry_InlineEntry) ProtoMessage() {}

func (x *CollectionEntry_InlineEntry) ProtoReflect() protoreflect.Message {
	mi := &file_xds_core_v3_collection_entry_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CollectionEntry_InlineEntry.ProtoReflect.Descriptor instead.
func (*CollectionEntry_InlineEntry) Descriptor() ([]byte, []int) {
	return file_xds_core_v3_collection_entry_proto_rawDescGZIP(), []int{0, 0}
}

func (x *CollectionEntry_InlineEntry) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *CollectionEntry_InlineEntry) GetVersion() string {
	if x != nil {
		return x.Version
	}
	return ""
}

func (x *CollectionEntry_InlineEntry) GetResource() *anypb.Any {
	if x != nil {
		return x.Resource
	}
	return nil
}

var File_xds_core_v3_collection_entry_proto protoreflect.FileDescriptor

const file_xds_core_v3_collection_entry_proto_rawDesc = "" +
	"\n" +
	"\"xds/core/v3/collection_entry.proto\x12\vxds.core.v3\x1a\x19google/protobuf/any.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\"xds/core/v3/resource_locator.proto\x1a\x17validate/validate.proto\"\xc3\x02\n" +
	"\x0fCollectionEntry\x128\n" +
	"\alocator\x18\x01 \x01(\v2\x1c.xds.core.v3.ResourceLocatorH\x00R\alocator\x12M\n" +
	"\finline_entry\x18\x02 \x01(\v2(.xds.core.v3.CollectionEntry.InlineEntryH\x00R\vinlineEntry\x1a\x8b\x01\n" +
	"\vInlineEntry\x120\n" +
	"\x04name\x18\x01 \x01(\tB\x1c\xfaB\x19r\x172\x15^[0-9a-zA-Z_\\-\\.~:]+$R\x04name\x12\x18\n" +
	"\aversion\x18\x02 \x01(\tR\aversion\x120\n" +
	"\bresource\x18\x03 \x01(\v2\x14.google.protobuf.AnyR\bresourceB\x19\n" +
	"\x12resource_specifier\x12\x03\xf8B\x01B\\\xd2Ƥ\xe1\x06\x02\b\x01\n" +
	"\x16com.github.xds.core.v3B\x14CollectionEntryProtoP\x01Z\"github.com/cncf/xds/go/xds/core/v3b\x06proto3"

var (
	file_xds_core_v3_collection_entry_proto_rawDescOnce sync.Once
	file_xds_core_v3_collection_entry_proto_rawDescData []byte
)

func file_xds_core_v3_collection_entry_proto_rawDescGZIP() []byte {
	file_xds_core_v3_collection_entry_proto_rawDescOnce.Do(func() {
		file_xds_core_v3_collection_entry_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_xds_core_v3_collection_entry_proto_rawDesc), len(file_xds_core_v3_collection_entry_proto_rawDesc)))
	})
	return file_xds_core_v3_collection_entry_proto_rawDescData
}

var file_xds_core_v3_collection_entry_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_xds_core_v3_collection_entry_proto_goTypes = []any{
	(*CollectionEntry)(nil),             // 0: xds.core.v3.CollectionEntry
	(*CollectionEntry_InlineEntry)(nil), // 1: xds.core.v3.CollectionEntry.InlineEntry
	(*ResourceLocator)(nil),             // 2: xds.core.v3.ResourceLocator
	(*anypb.Any)(nil),                   // 3: google.protobuf.Any
}
var file_xds_core_v3_collection_entry_proto_depIdxs = []int32{
	2, // 0: xds.core.v3.CollectionEntry.locator:type_name -> xds.core.v3.ResourceLocator
	1, // 1: xds.core.v3.CollectionEntry.inline_entry:type_name -> xds.core.v3.CollectionEntry.InlineEntry
	3, // 2: xds.core.v3.CollectionEntry.InlineEntry.resource:type_name -> google.protobuf.Any
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() { file_xds_core_v3_collection_entry_proto_init() }
func file_xds_core_v3_collection_entry_proto_init() {
	if File_xds_core_v3_collection_entry_proto != nil {
		return
	}
	file_xds_core_v3_resource_locator_proto_init()
	file_xds_core_v3_collection_entry_proto_msgTypes[0].OneofWrappers = []any{
		(*CollectionEntry_Locator)(nil),
		(*CollectionEntry_InlineEntry_)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_xds_core_v3_collection_entry_proto_rawDesc), len(file_xds_core_v3_collection_entry_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_xds_core_v3_collection_entry_proto_goTypes,
		DependencyIndexes: file_xds_core_v3_collection_entry_proto_depIdxs,
		MessageInfos:      file_xds_core_v3_collection_entry_proto_msgTypes,
	}.Build()
	File_xds_core_v3_collection_entry_proto = out.File
	file_xds_core_v3_collection_entry_proto_goTypes = nil
	file_xds_core_v3_collection_entry_proto_depIdxs = nil
}
