// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/admin/v3/config_dump_shared.proto

package adminv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	anypb "google.golang.org/protobuf/types/known/anypb"
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Resource status from the view of a xDS client, which tells the synchronization
// status between the xDS client and the xDS server.
type ClientResourceStatus int32

const (
	// Resource status is not available/unknown.
	ClientResourceStatus_UNKNOWN ClientResourceStatus = 0
	// Client requested this resource but hasn't received any update from management
	// server. The client will not fail requests, but will queue them until update
	// arrives or the client times out waiting for the resource.
	ClientResourceStatus_REQUESTED ClientResourceStatus = 1
	// This resource has been requested by the client but has either not been
	// delivered by the server or was previously delivered by the server and then
	// subsequently removed from resources provided by the server. For more
	// information, please refer to the :ref:`"Knowing When a Requested Resource
	// Does Not Exist" <xds_protocol_resource_not_existed>` section.
	ClientResourceStatus_DOES_NOT_EXIST ClientResourceStatus = 2
	// Client received this resource and replied with ACK.
	ClientResourceStatus_ACKED ClientResourceStatus = 3
	// Client received this resource and replied with NACK.
	ClientResourceStatus_NACKED ClientResourceStatus = 4
	// Client received an error from the control plane. The attached config
	// dump is the most recent accepted one. If no config is accepted yet,
	// the attached config dump will be empty.
	ClientResourceStatus_RECEIVED_ERROR ClientResourceStatus = 5
	// Client timed out waiting for the resource from the control plane.
	ClientResourceStatus_TIMEOUT ClientResourceStatus = 6
)

// Enum value maps for ClientResourceStatus.
var (
	ClientResourceStatus_name = map[int32]string{
		0: "UNKNOWN",
		1: "REQUESTED",
		2: "DOES_NOT_EXIST",
		3: "ACKED",
		4: "NACKED",
		5: "RECEIVED_ERROR",
		6: "TIMEOUT",
	}
	ClientResourceStatus_value = map[string]int32{
		"UNKNOWN":        0,
		"REQUESTED":      1,
		"DOES_NOT_EXIST": 2,
		"ACKED":          3,
		"NACKED":         4,
		"RECEIVED_ERROR": 5,
		"TIMEOUT":        6,
	}
)

func (x ClientResourceStatus) Enum() *ClientResourceStatus {
	p := new(ClientResourceStatus)
	*p = x
	return p
}

func (x ClientResourceStatus) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ClientResourceStatus) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_admin_v3_config_dump_shared_proto_enumTypes[0].Descriptor()
}

func (ClientResourceStatus) Type() protoreflect.EnumType {
	return &file_envoy_admin_v3_config_dump_shared_proto_enumTypes[0]
}

func (x ClientResourceStatus) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ClientResourceStatus.Descriptor instead.
func (ClientResourceStatus) EnumDescriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{0}
}

type UpdateFailureState struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// What the component configuration would have been if the update had succeeded.
	// This field may not be populated by xDS clients due to storage overhead.
	FailedConfiguration *anypb.Any `protobuf:"bytes,1,opt,name=failed_configuration,json=failedConfiguration,proto3" json:"failed_configuration,omitempty"`
	// Time of the latest failed update attempt.
	LastUpdateAttempt *timestamppb.Timestamp `protobuf:"bytes,2,opt,name=last_update_attempt,json=lastUpdateAttempt,proto3" json:"last_update_attempt,omitempty"`
	// Details about the last failed update attempt.
	Details string `protobuf:"bytes,3,opt,name=details,proto3" json:"details,omitempty"`
	// This is the version of the rejected resource.
	// [#not-implemented-hide:]
	VersionInfo   string `protobuf:"bytes,4,opt,name=version_info,json=versionInfo,proto3" json:"version_info,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpdateFailureState) Reset() {
	*x = UpdateFailureState{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateFailureState) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateFailureState) ProtoMessage() {}

func (x *UpdateFailureState) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateFailureState.ProtoReflect.Descriptor instead.
func (*UpdateFailureState) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{0}
}

func (x *UpdateFailureState) GetFailedConfiguration() *anypb.Any {
	if x != nil {
		return x.FailedConfiguration
	}
	return nil
}

func (x *UpdateFailureState) GetLastUpdateAttempt() *timestamppb.Timestamp {
	if x != nil {
		return x.LastUpdateAttempt
	}
	return nil
}

func (x *UpdateFailureState) GetDetails() string {
	if x != nil {
		return x.Details
	}
	return ""
}

func (x *UpdateFailureState) GetVersionInfo() string {
	if x != nil {
		return x.VersionInfo
	}
	return ""
}

// Envoy's listener manager fills this message with all currently known listeners. Listener
// configuration information can be used to recreate an Envoy configuration by populating all
// listeners as static listeners or by returning them in a LDS response.
type ListenersConfigDump struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// This is the :ref:`version_info <envoy_v3_api_field_service.discovery.v3.DiscoveryResponse.version_info>` in the
	// last processed LDS discovery response. If there are only static bootstrap listeners, this field
	// will be "".
	VersionInfo string `protobuf:"bytes,1,opt,name=version_info,json=versionInfo,proto3" json:"version_info,omitempty"`
	// The statically loaded listener configs.
	StaticListeners []*ListenersConfigDump_StaticListener `protobuf:"bytes,2,rep,name=static_listeners,json=staticListeners,proto3" json:"static_listeners,omitempty"`
	// State for any warming, active, or draining listeners.
	DynamicListeners []*ListenersConfigDump_DynamicListener `protobuf:"bytes,3,rep,name=dynamic_listeners,json=dynamicListeners,proto3" json:"dynamic_listeners,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *ListenersConfigDump) Reset() {
	*x = ListenersConfigDump{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListenersConfigDump) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListenersConfigDump) ProtoMessage() {}

func (x *ListenersConfigDump) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListenersConfigDump.ProtoReflect.Descriptor instead.
func (*ListenersConfigDump) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{1}
}

func (x *ListenersConfigDump) GetVersionInfo() string {
	if x != nil {
		return x.VersionInfo
	}
	return ""
}

func (x *ListenersConfigDump) GetStaticListeners() []*ListenersConfigDump_StaticListener {
	if x != nil {
		return x.StaticListeners
	}
	return nil
}

func (x *ListenersConfigDump) GetDynamicListeners() []*ListenersConfigDump_DynamicListener {
	if x != nil {
		return x.DynamicListeners
	}
	return nil
}

// Envoy's cluster manager fills this message with all currently known clusters. Cluster
// configuration information can be used to recreate an Envoy configuration by populating all
// clusters as static clusters or by returning them in a CDS response.
type ClustersConfigDump struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// This is the :ref:`version_info <envoy_v3_api_field_service.discovery.v3.DiscoveryResponse.version_info>` in the
	// last processed CDS discovery response. If there are only static bootstrap clusters, this field
	// will be "".
	VersionInfo string `protobuf:"bytes,1,opt,name=version_info,json=versionInfo,proto3" json:"version_info,omitempty"`
	// The statically loaded cluster configs.
	StaticClusters []*ClustersConfigDump_StaticCluster `protobuf:"bytes,2,rep,name=static_clusters,json=staticClusters,proto3" json:"static_clusters,omitempty"`
	// The dynamically loaded active clusters. These are clusters that are available to service
	// data plane traffic.
	DynamicActiveClusters []*ClustersConfigDump_DynamicCluster `protobuf:"bytes,3,rep,name=dynamic_active_clusters,json=dynamicActiveClusters,proto3" json:"dynamic_active_clusters,omitempty"`
	// The dynamically loaded warming clusters. These are clusters that are currently undergoing
	// warming in preparation to service data plane traffic. Note that if attempting to recreate an
	// Envoy configuration from a configuration dump, the warming clusters should generally be
	// discarded.
	DynamicWarmingClusters []*ClustersConfigDump_DynamicCluster `protobuf:"bytes,4,rep,name=dynamic_warming_clusters,json=dynamicWarmingClusters,proto3" json:"dynamic_warming_clusters,omitempty"`
	unknownFields          protoimpl.UnknownFields
	sizeCache              protoimpl.SizeCache
}

func (x *ClustersConfigDump) Reset() {
	*x = ClustersConfigDump{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClustersConfigDump) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClustersConfigDump) ProtoMessage() {}

func (x *ClustersConfigDump) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClustersConfigDump.ProtoReflect.Descriptor instead.
func (*ClustersConfigDump) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{2}
}

func (x *ClustersConfigDump) GetVersionInfo() string {
	if x != nil {
		return x.VersionInfo
	}
	return ""
}

func (x *ClustersConfigDump) GetStaticClusters() []*ClustersConfigDump_StaticCluster {
	if x != nil {
		return x.StaticClusters
	}
	return nil
}

func (x *ClustersConfigDump) GetDynamicActiveClusters() []*ClustersConfigDump_DynamicCluster {
	if x != nil {
		return x.DynamicActiveClusters
	}
	return nil
}

func (x *ClustersConfigDump) GetDynamicWarmingClusters() []*ClustersConfigDump_DynamicCluster {
	if x != nil {
		return x.DynamicWarmingClusters
	}
	return nil
}

// Envoy's RDS implementation fills this message with all currently loaded routes, as described by
// their RouteConfiguration objects. Static routes that are either defined in the bootstrap configuration
// or defined inline while configuring listeners are separated from those configured dynamically via RDS.
// Route configuration information can be used to recreate an Envoy configuration by populating all routes
// as static routes or by returning them in RDS responses.
type RoutesConfigDump struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The statically loaded route configs.
	StaticRouteConfigs []*RoutesConfigDump_StaticRouteConfig `protobuf:"bytes,2,rep,name=static_route_configs,json=staticRouteConfigs,proto3" json:"static_route_configs,omitempty"`
	// The dynamically loaded route configs.
	DynamicRouteConfigs []*RoutesConfigDump_DynamicRouteConfig `protobuf:"bytes,3,rep,name=dynamic_route_configs,json=dynamicRouteConfigs,proto3" json:"dynamic_route_configs,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *RoutesConfigDump) Reset() {
	*x = RoutesConfigDump{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RoutesConfigDump) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RoutesConfigDump) ProtoMessage() {}

func (x *RoutesConfigDump) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RoutesConfigDump.ProtoReflect.Descriptor instead.
func (*RoutesConfigDump) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{3}
}

func (x *RoutesConfigDump) GetStaticRouteConfigs() []*RoutesConfigDump_StaticRouteConfig {
	if x != nil {
		return x.StaticRouteConfigs
	}
	return nil
}

func (x *RoutesConfigDump) GetDynamicRouteConfigs() []*RoutesConfigDump_DynamicRouteConfig {
	if x != nil {
		return x.DynamicRouteConfigs
	}
	return nil
}

// Envoy's scoped RDS implementation fills this message with all currently loaded route
// configuration scopes (defined via ScopedRouteConfigurationsSet protos). This message lists both
// the scopes defined inline with the higher order object (i.e., the HttpConnectionManager) and the
// dynamically obtained scopes via the SRDS API.
type ScopedRoutesConfigDump struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The statically loaded scoped route configs.
	InlineScopedRouteConfigs []*ScopedRoutesConfigDump_InlineScopedRouteConfigs `protobuf:"bytes,1,rep,name=inline_scoped_route_configs,json=inlineScopedRouteConfigs,proto3" json:"inline_scoped_route_configs,omitempty"`
	// The dynamically loaded scoped route configs.
	DynamicScopedRouteConfigs []*ScopedRoutesConfigDump_DynamicScopedRouteConfigs `protobuf:"bytes,2,rep,name=dynamic_scoped_route_configs,json=dynamicScopedRouteConfigs,proto3" json:"dynamic_scoped_route_configs,omitempty"`
	unknownFields             protoimpl.UnknownFields
	sizeCache                 protoimpl.SizeCache
}

func (x *ScopedRoutesConfigDump) Reset() {
	*x = ScopedRoutesConfigDump{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ScopedRoutesConfigDump) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ScopedRoutesConfigDump) ProtoMessage() {}

func (x *ScopedRoutesConfigDump) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ScopedRoutesConfigDump.ProtoReflect.Descriptor instead.
func (*ScopedRoutesConfigDump) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{4}
}

func (x *ScopedRoutesConfigDump) GetInlineScopedRouteConfigs() []*ScopedRoutesConfigDump_InlineScopedRouteConfigs {
	if x != nil {
		return x.InlineScopedRouteConfigs
	}
	return nil
}

func (x *ScopedRoutesConfigDump) GetDynamicScopedRouteConfigs() []*ScopedRoutesConfigDump_DynamicScopedRouteConfigs {
	if x != nil {
		return x.DynamicScopedRouteConfigs
	}
	return nil
}

// Envoy's admin fill this message with all currently known endpoints. Endpoint
// configuration information can be used to recreate an Envoy configuration by populating all
// endpoints as static endpoints or by returning them in an EDS response.
type EndpointsConfigDump struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The statically loaded endpoint configs.
	StaticEndpointConfigs []*EndpointsConfigDump_StaticEndpointConfig `protobuf:"bytes,2,rep,name=static_endpoint_configs,json=staticEndpointConfigs,proto3" json:"static_endpoint_configs,omitempty"`
	// The dynamically loaded endpoint configs.
	DynamicEndpointConfigs []*EndpointsConfigDump_DynamicEndpointConfig `protobuf:"bytes,3,rep,name=dynamic_endpoint_configs,json=dynamicEndpointConfigs,proto3" json:"dynamic_endpoint_configs,omitempty"`
	unknownFields          protoimpl.UnknownFields
	sizeCache              protoimpl.SizeCache
}

func (x *EndpointsConfigDump) Reset() {
	*x = EndpointsConfigDump{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EndpointsConfigDump) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EndpointsConfigDump) ProtoMessage() {}

func (x *EndpointsConfigDump) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EndpointsConfigDump.ProtoReflect.Descriptor instead.
func (*EndpointsConfigDump) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{5}
}

func (x *EndpointsConfigDump) GetStaticEndpointConfigs() []*EndpointsConfigDump_StaticEndpointConfig {
	if x != nil {
		return x.StaticEndpointConfigs
	}
	return nil
}

func (x *EndpointsConfigDump) GetDynamicEndpointConfigs() []*EndpointsConfigDump_DynamicEndpointConfig {
	if x != nil {
		return x.DynamicEndpointConfigs
	}
	return nil
}

// Envoy's ECDS service fills this message with all currently extension
// configuration. Extension configuration information can be used to recreate
// an Envoy ECDS listener and HTTP filters as static filters or by returning
// them in ECDS response.
type EcdsConfigDump struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The ECDS filter configs.
	EcdsFilters   []*EcdsConfigDump_EcdsFilterConfig `protobuf:"bytes,1,rep,name=ecds_filters,json=ecdsFilters,proto3" json:"ecds_filters,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EcdsConfigDump) Reset() {
	*x = EcdsConfigDump{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EcdsConfigDump) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EcdsConfigDump) ProtoMessage() {}

func (x *EcdsConfigDump) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EcdsConfigDump.ProtoReflect.Descriptor instead.
func (*EcdsConfigDump) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{6}
}

func (x *EcdsConfigDump) GetEcdsFilters() []*EcdsConfigDump_EcdsFilterConfig {
	if x != nil {
		return x.EcdsFilters
	}
	return nil
}

// Describes a statically loaded listener.
type ListenersConfigDump_StaticListener struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The listener config.
	Listener *anypb.Any `protobuf:"bytes,1,opt,name=listener,proto3" json:"listener,omitempty"`
	// The timestamp when the Listener was last successfully updated.
	LastUpdated   *timestamppb.Timestamp `protobuf:"bytes,2,opt,name=last_updated,json=lastUpdated,proto3" json:"last_updated,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListenersConfigDump_StaticListener) Reset() {
	*x = ListenersConfigDump_StaticListener{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListenersConfigDump_StaticListener) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListenersConfigDump_StaticListener) ProtoMessage() {}

func (x *ListenersConfigDump_StaticListener) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListenersConfigDump_StaticListener.ProtoReflect.Descriptor instead.
func (*ListenersConfigDump_StaticListener) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{1, 0}
}

func (x *ListenersConfigDump_StaticListener) GetListener() *anypb.Any {
	if x != nil {
		return x.Listener
	}
	return nil
}

func (x *ListenersConfigDump_StaticListener) GetLastUpdated() *timestamppb.Timestamp {
	if x != nil {
		return x.LastUpdated
	}
	return nil
}

type ListenersConfigDump_DynamicListenerState struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// This is the per-resource version information. This version is currently taken from the
	// :ref:`version_info <envoy_v3_api_field_service.discovery.v3.DiscoveryResponse.version_info>` field at the time
	// that the listener was loaded. In the future, discrete per-listener versions may be supported
	// by the API.
	VersionInfo string `protobuf:"bytes,1,opt,name=version_info,json=versionInfo,proto3" json:"version_info,omitempty"`
	// The listener config.
	Listener *anypb.Any `protobuf:"bytes,2,opt,name=listener,proto3" json:"listener,omitempty"`
	// The timestamp when the Listener was last successfully updated.
	LastUpdated   *timestamppb.Timestamp `protobuf:"bytes,3,opt,name=last_updated,json=lastUpdated,proto3" json:"last_updated,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListenersConfigDump_DynamicListenerState) Reset() {
	*x = ListenersConfigDump_DynamicListenerState{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[8]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListenersConfigDump_DynamicListenerState) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListenersConfigDump_DynamicListenerState) ProtoMessage() {}

func (x *ListenersConfigDump_DynamicListenerState) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[8]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListenersConfigDump_DynamicListenerState.ProtoReflect.Descriptor instead.
func (*ListenersConfigDump_DynamicListenerState) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{1, 1}
}

func (x *ListenersConfigDump_DynamicListenerState) GetVersionInfo() string {
	if x != nil {
		return x.VersionInfo
	}
	return ""
}

func (x *ListenersConfigDump_DynamicListenerState) GetListener() *anypb.Any {
	if x != nil {
		return x.Listener
	}
	return nil
}

func (x *ListenersConfigDump_DynamicListenerState) GetLastUpdated() *timestamppb.Timestamp {
	if x != nil {
		return x.LastUpdated
	}
	return nil
}

// Describes a dynamically loaded listener via the LDS API.
// [#next-free-field: 7]
type ListenersConfigDump_DynamicListener struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name or unique id of this listener, pulled from the DynamicListenerState config.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// The listener state for any active listener by this name.
	// These are listeners that are available to service data plane traffic.
	ActiveState *ListenersConfigDump_DynamicListenerState `protobuf:"bytes,2,opt,name=active_state,json=activeState,proto3" json:"active_state,omitempty"`
	// The listener state for any warming listener by this name.
	// These are listeners that are currently undergoing warming in preparation to service data
	// plane traffic. Note that if attempting to recreate an Envoy configuration from a
	// configuration dump, the warming listeners should generally be discarded.
	WarmingState *ListenersConfigDump_DynamicListenerState `protobuf:"bytes,3,opt,name=warming_state,json=warmingState,proto3" json:"warming_state,omitempty"`
	// The listener state for any draining listener by this name.
	// These are listeners that are currently undergoing draining in preparation to stop servicing
	// data plane traffic. Note that if attempting to recreate an Envoy configuration from a
	// configuration dump, the draining listeners should generally be discarded.
	DrainingState *ListenersConfigDump_DynamicListenerState `protobuf:"bytes,4,opt,name=draining_state,json=drainingState,proto3" json:"draining_state,omitempty"`
	// Set if the last update failed, cleared after the next successful update.
	// The “error_state“ field contains the rejected version of this particular
	// resource along with the reason and timestamp. For successfully updated or
	// acknowledged resource, this field should be empty.
	ErrorState *UpdateFailureState `protobuf:"bytes,5,opt,name=error_state,json=errorState,proto3" json:"error_state,omitempty"`
	// The client status of this resource.
	// [#not-implemented-hide:]
	ClientStatus  ClientResourceStatus `protobuf:"varint,6,opt,name=client_status,json=clientStatus,proto3,enum=envoy.admin.v3.ClientResourceStatus" json:"client_status,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListenersConfigDump_DynamicListener) Reset() {
	*x = ListenersConfigDump_DynamicListener{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[9]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListenersConfigDump_DynamicListener) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListenersConfigDump_DynamicListener) ProtoMessage() {}

func (x *ListenersConfigDump_DynamicListener) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[9]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListenersConfigDump_DynamicListener.ProtoReflect.Descriptor instead.
func (*ListenersConfigDump_DynamicListener) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{1, 2}
}

func (x *ListenersConfigDump_DynamicListener) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *ListenersConfigDump_DynamicListener) GetActiveState() *ListenersConfigDump_DynamicListenerState {
	if x != nil {
		return x.ActiveState
	}
	return nil
}

func (x *ListenersConfigDump_DynamicListener) GetWarmingState() *ListenersConfigDump_DynamicListenerState {
	if x != nil {
		return x.WarmingState
	}
	return nil
}

func (x *ListenersConfigDump_DynamicListener) GetDrainingState() *ListenersConfigDump_DynamicListenerState {
	if x != nil {
		return x.DrainingState
	}
	return nil
}

func (x *ListenersConfigDump_DynamicListener) GetErrorState() *UpdateFailureState {
	if x != nil {
		return x.ErrorState
	}
	return nil
}

func (x *ListenersConfigDump_DynamicListener) GetClientStatus() ClientResourceStatus {
	if x != nil {
		return x.ClientStatus
	}
	return ClientResourceStatus_UNKNOWN
}

// Describes a statically loaded cluster.
type ClustersConfigDump_StaticCluster struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The cluster config.
	Cluster *anypb.Any `protobuf:"bytes,1,opt,name=cluster,proto3" json:"cluster,omitempty"`
	// The timestamp when the Cluster was last updated.
	LastUpdated   *timestamppb.Timestamp `protobuf:"bytes,2,opt,name=last_updated,json=lastUpdated,proto3" json:"last_updated,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ClustersConfigDump_StaticCluster) Reset() {
	*x = ClustersConfigDump_StaticCluster{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[10]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClustersConfigDump_StaticCluster) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClustersConfigDump_StaticCluster) ProtoMessage() {}

func (x *ClustersConfigDump_StaticCluster) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[10]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClustersConfigDump_StaticCluster.ProtoReflect.Descriptor instead.
func (*ClustersConfigDump_StaticCluster) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{2, 0}
}

func (x *ClustersConfigDump_StaticCluster) GetCluster() *anypb.Any {
	if x != nil {
		return x.Cluster
	}
	return nil
}

func (x *ClustersConfigDump_StaticCluster) GetLastUpdated() *timestamppb.Timestamp {
	if x != nil {
		return x.LastUpdated
	}
	return nil
}

// Describes a dynamically loaded cluster via the CDS API.
// [#next-free-field: 6]
type ClustersConfigDump_DynamicCluster struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// This is the per-resource version information. This version is currently taken from the
	// :ref:`version_info <envoy_v3_api_field_service.discovery.v3.DiscoveryResponse.version_info>` field at the time
	// that the cluster was loaded. In the future, discrete per-cluster versions may be supported by
	// the API.
	VersionInfo string `protobuf:"bytes,1,opt,name=version_info,json=versionInfo,proto3" json:"version_info,omitempty"`
	// The cluster config.
	Cluster *anypb.Any `protobuf:"bytes,2,opt,name=cluster,proto3" json:"cluster,omitempty"`
	// The timestamp when the Cluster was last updated.
	LastUpdated *timestamppb.Timestamp `protobuf:"bytes,3,opt,name=last_updated,json=lastUpdated,proto3" json:"last_updated,omitempty"`
	// Set if the last update failed, cleared after the next successful update.
	// The “error_state“ field contains the rejected version of this particular
	// resource along with the reason and timestamp. For successfully updated or
	// acknowledged resource, this field should be empty.
	// [#not-implemented-hide:]
	ErrorState *UpdateFailureState `protobuf:"bytes,4,opt,name=error_state,json=errorState,proto3" json:"error_state,omitempty"`
	// The client status of this resource.
	// [#not-implemented-hide:]
	ClientStatus  ClientResourceStatus `protobuf:"varint,5,opt,name=client_status,json=clientStatus,proto3,enum=envoy.admin.v3.ClientResourceStatus" json:"client_status,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ClustersConfigDump_DynamicCluster) Reset() {
	*x = ClustersConfigDump_DynamicCluster{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[11]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClustersConfigDump_DynamicCluster) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClustersConfigDump_DynamicCluster) ProtoMessage() {}

func (x *ClustersConfigDump_DynamicCluster) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[11]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClustersConfigDump_DynamicCluster.ProtoReflect.Descriptor instead.
func (*ClustersConfigDump_DynamicCluster) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{2, 1}
}

func (x *ClustersConfigDump_DynamicCluster) GetVersionInfo() string {
	if x != nil {
		return x.VersionInfo
	}
	return ""
}

func (x *ClustersConfigDump_DynamicCluster) GetCluster() *anypb.Any {
	if x != nil {
		return x.Cluster
	}
	return nil
}

func (x *ClustersConfigDump_DynamicCluster) GetLastUpdated() *timestamppb.Timestamp {
	if x != nil {
		return x.LastUpdated
	}
	return nil
}

func (x *ClustersConfigDump_DynamicCluster) GetErrorState() *UpdateFailureState {
	if x != nil {
		return x.ErrorState
	}
	return nil
}

func (x *ClustersConfigDump_DynamicCluster) GetClientStatus() ClientResourceStatus {
	if x != nil {
		return x.ClientStatus
	}
	return ClientResourceStatus_UNKNOWN
}

type RoutesConfigDump_StaticRouteConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The route config.
	RouteConfig *anypb.Any `protobuf:"bytes,1,opt,name=route_config,json=routeConfig,proto3" json:"route_config,omitempty"`
	// The timestamp when the Route was last updated.
	LastUpdated   *timestamppb.Timestamp `protobuf:"bytes,2,opt,name=last_updated,json=lastUpdated,proto3" json:"last_updated,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RoutesConfigDump_StaticRouteConfig) Reset() {
	*x = RoutesConfigDump_StaticRouteConfig{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[12]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RoutesConfigDump_StaticRouteConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RoutesConfigDump_StaticRouteConfig) ProtoMessage() {}

func (x *RoutesConfigDump_StaticRouteConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[12]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RoutesConfigDump_StaticRouteConfig.ProtoReflect.Descriptor instead.
func (*RoutesConfigDump_StaticRouteConfig) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{3, 0}
}

func (x *RoutesConfigDump_StaticRouteConfig) GetRouteConfig() *anypb.Any {
	if x != nil {
		return x.RouteConfig
	}
	return nil
}

func (x *RoutesConfigDump_StaticRouteConfig) GetLastUpdated() *timestamppb.Timestamp {
	if x != nil {
		return x.LastUpdated
	}
	return nil
}

// [#next-free-field: 6]
type RoutesConfigDump_DynamicRouteConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// This is the per-resource version information. This version is currently taken from the
	// :ref:`version_info <envoy_v3_api_field_service.discovery.v3.DiscoveryResponse.version_info>` field at the time that
	// the route configuration was loaded.
	VersionInfo string `protobuf:"bytes,1,opt,name=version_info,json=versionInfo,proto3" json:"version_info,omitempty"`
	// The route config.
	RouteConfig *anypb.Any `protobuf:"bytes,2,opt,name=route_config,json=routeConfig,proto3" json:"route_config,omitempty"`
	// The timestamp when the Route was last updated.
	LastUpdated *timestamppb.Timestamp `protobuf:"bytes,3,opt,name=last_updated,json=lastUpdated,proto3" json:"last_updated,omitempty"`
	// Set if the last update failed, cleared after the next successful update.
	// The “error_state“ field contains the rejected version of this particular
	// resource along with the reason and timestamp. For successfully updated or
	// acknowledged resource, this field should be empty.
	// [#not-implemented-hide:]
	ErrorState *UpdateFailureState `protobuf:"bytes,4,opt,name=error_state,json=errorState,proto3" json:"error_state,omitempty"`
	// The client status of this resource.
	// [#not-implemented-hide:]
	ClientStatus  ClientResourceStatus `protobuf:"varint,5,opt,name=client_status,json=clientStatus,proto3,enum=envoy.admin.v3.ClientResourceStatus" json:"client_status,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RoutesConfigDump_DynamicRouteConfig) Reset() {
	*x = RoutesConfigDump_DynamicRouteConfig{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[13]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RoutesConfigDump_DynamicRouteConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RoutesConfigDump_DynamicRouteConfig) ProtoMessage() {}

func (x *RoutesConfigDump_DynamicRouteConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[13]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RoutesConfigDump_DynamicRouteConfig.ProtoReflect.Descriptor instead.
func (*RoutesConfigDump_DynamicRouteConfig) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{3, 1}
}

func (x *RoutesConfigDump_DynamicRouteConfig) GetVersionInfo() string {
	if x != nil {
		return x.VersionInfo
	}
	return ""
}

func (x *RoutesConfigDump_DynamicRouteConfig) GetRouteConfig() *anypb.Any {
	if x != nil {
		return x.RouteConfig
	}
	return nil
}

func (x *RoutesConfigDump_DynamicRouteConfig) GetLastUpdated() *timestamppb.Timestamp {
	if x != nil {
		return x.LastUpdated
	}
	return nil
}

func (x *RoutesConfigDump_DynamicRouteConfig) GetErrorState() *UpdateFailureState {
	if x != nil {
		return x.ErrorState
	}
	return nil
}

func (x *RoutesConfigDump_DynamicRouteConfig) GetClientStatus() ClientResourceStatus {
	if x != nil {
		return x.ClientStatus
	}
	return ClientResourceStatus_UNKNOWN
}

type ScopedRoutesConfigDump_InlineScopedRouteConfigs struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name assigned to the scoped route configurations.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// The scoped route configurations.
	ScopedRouteConfigs []*anypb.Any `protobuf:"bytes,2,rep,name=scoped_route_configs,json=scopedRouteConfigs,proto3" json:"scoped_route_configs,omitempty"`
	// The timestamp when the scoped route config set was last updated.
	LastUpdated   *timestamppb.Timestamp `protobuf:"bytes,3,opt,name=last_updated,json=lastUpdated,proto3" json:"last_updated,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ScopedRoutesConfigDump_InlineScopedRouteConfigs) Reset() {
	*x = ScopedRoutesConfigDump_InlineScopedRouteConfigs{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[14]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ScopedRoutesConfigDump_InlineScopedRouteConfigs) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ScopedRoutesConfigDump_InlineScopedRouteConfigs) ProtoMessage() {}

func (x *ScopedRoutesConfigDump_InlineScopedRouteConfigs) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[14]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ScopedRoutesConfigDump_InlineScopedRouteConfigs.ProtoReflect.Descriptor instead.
func (*ScopedRoutesConfigDump_InlineScopedRouteConfigs) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{4, 0}
}

func (x *ScopedRoutesConfigDump_InlineScopedRouteConfigs) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *ScopedRoutesConfigDump_InlineScopedRouteConfigs) GetScopedRouteConfigs() []*anypb.Any {
	if x != nil {
		return x.ScopedRouteConfigs
	}
	return nil
}

func (x *ScopedRoutesConfigDump_InlineScopedRouteConfigs) GetLastUpdated() *timestamppb.Timestamp {
	if x != nil {
		return x.LastUpdated
	}
	return nil
}

// [#next-free-field: 7]
type ScopedRoutesConfigDump_DynamicScopedRouteConfigs struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name assigned to the scoped route configurations.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// This is the per-resource version information. This version is currently taken from the
	// :ref:`version_info <envoy_v3_api_field_service.discovery.v3.DiscoveryResponse.version_info>` field at the time that
	// the scoped routes configuration was loaded.
	VersionInfo string `protobuf:"bytes,2,opt,name=version_info,json=versionInfo,proto3" json:"version_info,omitempty"`
	// The scoped route configurations.
	ScopedRouteConfigs []*anypb.Any `protobuf:"bytes,3,rep,name=scoped_route_configs,json=scopedRouteConfigs,proto3" json:"scoped_route_configs,omitempty"`
	// The timestamp when the scoped route config set was last updated.
	LastUpdated *timestamppb.Timestamp `protobuf:"bytes,4,opt,name=last_updated,json=lastUpdated,proto3" json:"last_updated,omitempty"`
	// Set if the last update failed, cleared after the next successful update.
	// The “error_state“ field contains the rejected version of this particular
	// resource along with the reason and timestamp. For successfully updated or
	// acknowledged resource, this field should be empty.
	// [#not-implemented-hide:]
	ErrorState *UpdateFailureState `protobuf:"bytes,5,opt,name=error_state,json=errorState,proto3" json:"error_state,omitempty"`
	// The client status of this resource.
	// [#not-implemented-hide:]
	ClientStatus  ClientResourceStatus `protobuf:"varint,6,opt,name=client_status,json=clientStatus,proto3,enum=envoy.admin.v3.ClientResourceStatus" json:"client_status,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ScopedRoutesConfigDump_DynamicScopedRouteConfigs) Reset() {
	*x = ScopedRoutesConfigDump_DynamicScopedRouteConfigs{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[15]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ScopedRoutesConfigDump_DynamicScopedRouteConfigs) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ScopedRoutesConfigDump_DynamicScopedRouteConfigs) ProtoMessage() {}

func (x *ScopedRoutesConfigDump_DynamicScopedRouteConfigs) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[15]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ScopedRoutesConfigDump_DynamicScopedRouteConfigs.ProtoReflect.Descriptor instead.
func (*ScopedRoutesConfigDump_DynamicScopedRouteConfigs) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{4, 1}
}

func (x *ScopedRoutesConfigDump_DynamicScopedRouteConfigs) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *ScopedRoutesConfigDump_DynamicScopedRouteConfigs) GetVersionInfo() string {
	if x != nil {
		return x.VersionInfo
	}
	return ""
}

func (x *ScopedRoutesConfigDump_DynamicScopedRouteConfigs) GetScopedRouteConfigs() []*anypb.Any {
	if x != nil {
		return x.ScopedRouteConfigs
	}
	return nil
}

func (x *ScopedRoutesConfigDump_DynamicScopedRouteConfigs) GetLastUpdated() *timestamppb.Timestamp {
	if x != nil {
		return x.LastUpdated
	}
	return nil
}

func (x *ScopedRoutesConfigDump_DynamicScopedRouteConfigs) GetErrorState() *UpdateFailureState {
	if x != nil {
		return x.ErrorState
	}
	return nil
}

func (x *ScopedRoutesConfigDump_DynamicScopedRouteConfigs) GetClientStatus() ClientResourceStatus {
	if x != nil {
		return x.ClientStatus
	}
	return ClientResourceStatus_UNKNOWN
}

type EndpointsConfigDump_StaticEndpointConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The endpoint config.
	EndpointConfig *anypb.Any `protobuf:"bytes,1,opt,name=endpoint_config,json=endpointConfig,proto3" json:"endpoint_config,omitempty"`
	// [#not-implemented-hide:] The timestamp when the Endpoint was last updated.
	LastUpdated   *timestamppb.Timestamp `protobuf:"bytes,2,opt,name=last_updated,json=lastUpdated,proto3" json:"last_updated,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EndpointsConfigDump_StaticEndpointConfig) Reset() {
	*x = EndpointsConfigDump_StaticEndpointConfig{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[16]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EndpointsConfigDump_StaticEndpointConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EndpointsConfigDump_StaticEndpointConfig) ProtoMessage() {}

func (x *EndpointsConfigDump_StaticEndpointConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[16]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EndpointsConfigDump_StaticEndpointConfig.ProtoReflect.Descriptor instead.
func (*EndpointsConfigDump_StaticEndpointConfig) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{5, 0}
}

func (x *EndpointsConfigDump_StaticEndpointConfig) GetEndpointConfig() *anypb.Any {
	if x != nil {
		return x.EndpointConfig
	}
	return nil
}

func (x *EndpointsConfigDump_StaticEndpointConfig) GetLastUpdated() *timestamppb.Timestamp {
	if x != nil {
		return x.LastUpdated
	}
	return nil
}

// [#next-free-field: 6]
type EndpointsConfigDump_DynamicEndpointConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// [#not-implemented-hide:] This is the per-resource version information. This version is currently taken from the
	// :ref:`version_info <envoy_v3_api_field_service.discovery.v3.DiscoveryResponse.version_info>` field at the time that
	// the endpoint configuration was loaded.
	VersionInfo string `protobuf:"bytes,1,opt,name=version_info,json=versionInfo,proto3" json:"version_info,omitempty"`
	// The endpoint config.
	EndpointConfig *anypb.Any `protobuf:"bytes,2,opt,name=endpoint_config,json=endpointConfig,proto3" json:"endpoint_config,omitempty"`
	// [#not-implemented-hide:] The timestamp when the Endpoint was last updated.
	LastUpdated *timestamppb.Timestamp `protobuf:"bytes,3,opt,name=last_updated,json=lastUpdated,proto3" json:"last_updated,omitempty"`
	// Set if the last update failed, cleared after the next successful update.
	// The “error_state“ field contains the rejected version of this particular
	// resource along with the reason and timestamp. For successfully updated or
	// acknowledged resource, this field should be empty.
	// [#not-implemented-hide:]
	ErrorState *UpdateFailureState `protobuf:"bytes,4,opt,name=error_state,json=errorState,proto3" json:"error_state,omitempty"`
	// The client status of this resource.
	// [#not-implemented-hide:]
	ClientStatus  ClientResourceStatus `protobuf:"varint,5,opt,name=client_status,json=clientStatus,proto3,enum=envoy.admin.v3.ClientResourceStatus" json:"client_status,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EndpointsConfigDump_DynamicEndpointConfig) Reset() {
	*x = EndpointsConfigDump_DynamicEndpointConfig{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[17]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EndpointsConfigDump_DynamicEndpointConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EndpointsConfigDump_DynamicEndpointConfig) ProtoMessage() {}

func (x *EndpointsConfigDump_DynamicEndpointConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[17]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EndpointsConfigDump_DynamicEndpointConfig.ProtoReflect.Descriptor instead.
func (*EndpointsConfigDump_DynamicEndpointConfig) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{5, 1}
}

func (x *EndpointsConfigDump_DynamicEndpointConfig) GetVersionInfo() string {
	if x != nil {
		return x.VersionInfo
	}
	return ""
}

func (x *EndpointsConfigDump_DynamicEndpointConfig) GetEndpointConfig() *anypb.Any {
	if x != nil {
		return x.EndpointConfig
	}
	return nil
}

func (x *EndpointsConfigDump_DynamicEndpointConfig) GetLastUpdated() *timestamppb.Timestamp {
	if x != nil {
		return x.LastUpdated
	}
	return nil
}

func (x *EndpointsConfigDump_DynamicEndpointConfig) GetErrorState() *UpdateFailureState {
	if x != nil {
		return x.ErrorState
	}
	return nil
}

func (x *EndpointsConfigDump_DynamicEndpointConfig) GetClientStatus() ClientResourceStatus {
	if x != nil {
		return x.ClientStatus
	}
	return ClientResourceStatus_UNKNOWN
}

// [#next-free-field: 6]
type EcdsConfigDump_EcdsFilterConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// This is the per-resource version information. This version is currently
	// taken from the :ref:`version_info
	// <envoy_v3_api_field_service.discovery.v3.DiscoveryResponse.version_info>`
	// field at the time that the ECDS filter was loaded.
	VersionInfo string `protobuf:"bytes,1,opt,name=version_info,json=versionInfo,proto3" json:"version_info,omitempty"`
	// The ECDS filter config.
	EcdsFilter *anypb.Any `protobuf:"bytes,2,opt,name=ecds_filter,json=ecdsFilter,proto3" json:"ecds_filter,omitempty"`
	// The timestamp when the ECDS filter was last updated.
	LastUpdated *timestamppb.Timestamp `protobuf:"bytes,3,opt,name=last_updated,json=lastUpdated,proto3" json:"last_updated,omitempty"`
	// Set if the last update failed, cleared after the next successful update.
	// The “error_state“ field contains the rejected version of this
	// particular resource along with the reason and timestamp. For successfully
	// updated or acknowledged resource, this field should be empty.
	// [#not-implemented-hide:]
	ErrorState *UpdateFailureState `protobuf:"bytes,4,opt,name=error_state,json=errorState,proto3" json:"error_state,omitempty"`
	// The client status of this resource.
	// [#not-implemented-hide:]
	ClientStatus  ClientResourceStatus `protobuf:"varint,5,opt,name=client_status,json=clientStatus,proto3,enum=envoy.admin.v3.ClientResourceStatus" json:"client_status,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EcdsConfigDump_EcdsFilterConfig) Reset() {
	*x = EcdsConfigDump_EcdsFilterConfig{}
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[18]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EcdsConfigDump_EcdsFilterConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EcdsConfigDump_EcdsFilterConfig) ProtoMessage() {}

func (x *EcdsConfigDump_EcdsFilterConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_config_dump_shared_proto_msgTypes[18]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EcdsConfigDump_EcdsFilterConfig.ProtoReflect.Descriptor instead.
func (*EcdsConfigDump_EcdsFilterConfig) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP(), []int{6, 0}
}

func (x *EcdsConfigDump_EcdsFilterConfig) GetVersionInfo() string {
	if x != nil {
		return x.VersionInfo
	}
	return ""
}

func (x *EcdsConfigDump_EcdsFilterConfig) GetEcdsFilter() *anypb.Any {
	if x != nil {
		return x.EcdsFilter
	}
	return nil
}

func (x *EcdsConfigDump_EcdsFilterConfig) GetLastUpdated() *timestamppb.Timestamp {
	if x != nil {
		return x.LastUpdated
	}
	return nil
}

func (x *EcdsConfigDump_EcdsFilterConfig) GetErrorState() *UpdateFailureState {
	if x != nil {
		return x.ErrorState
	}
	return nil
}

func (x *EcdsConfigDump_EcdsFilterConfig) GetClientStatus() ClientResourceStatus {
	if x != nil {
		return x.ClientStatus
	}
	return ClientResourceStatus_UNKNOWN
}

var File_envoy_admin_v3_config_dump_shared_proto protoreflect.FileDescriptor

const file_envoy_admin_v3_config_dump_shared_proto_rawDesc = "" +
	"\n" +
	"'envoy/admin/v3/config_dump_shared.proto\x12\x0eenvoy.admin.v3\x1a\x19google/protobuf/any.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"\x95\x02\n" +
	"\x12UpdateFailureState\x12G\n" +
	"\x14failed_configuration\x18\x01 \x01(\v2\x14.google.protobuf.AnyR\x13failedConfiguration\x12J\n" +
	"\x13last_update_attempt\x18\x02 \x01(\v2\x1a.google.protobuf.TimestampR\x11lastUpdateAttempt\x12\x18\n" +
	"\adetails\x18\x03 \x01(\tR\adetails\x12!\n" +
	"\fversion_info\x18\x04 \x01(\tR\vversionInfo:-\x9aň\x1e(\n" +
	"&envoy.admin.v2alpha.UpdateFailureState\"\xf3\t\n" +
	"\x13ListenersConfigDump\x12!\n" +
	"\fversion_info\x18\x01 \x01(\tR\vversionInfo\x12]\n" +
	"\x10static_listeners\x18\x02 \x03(\v22.envoy.admin.v3.ListenersConfigDump.StaticListenerR\x0fstaticListeners\x12`\n" +
	"\x11dynamic_listeners\x18\x03 \x03(\v23.envoy.admin.v3.ListenersConfigDump.DynamicListenerR\x10dynamicListeners\x1a\xc0\x01\n" +
	"\x0eStaticListener\x120\n" +
	"\blistener\x18\x01 \x01(\v2\x14.google.protobuf.AnyR\blistener\x12=\n" +
	"\flast_updated\x18\x02 \x01(\v2\x1a.google.protobuf.TimestampR\vlastUpdated:=\x9aň\x1e8\n" +
	"6envoy.admin.v2alpha.ListenersConfigDump.StaticListener\x1a\xef\x01\n" +
	"\x14DynamicListenerState\x12!\n" +
	"\fversion_info\x18\x01 \x01(\tR\vversionInfo\x120\n" +
	"\blistener\x18\x02 \x01(\v2\x14.google.protobuf.AnyR\blistener\x12=\n" +
	"\flast_updated\x18\x03 \x01(\v2\x1a.google.protobuf.TimestampR\vlastUpdated:C\x9aň\x1e>\n" +
	"<envoy.admin.v2alpha.ListenersConfigDump.DynamicListenerState\x1a\x92\x04\n" +
	"\x0fDynamicListener\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12[\n" +
	"\factive_state\x18\x02 \x01(\v28.envoy.admin.v3.ListenersConfigDump.DynamicListenerStateR\vactiveState\x12]\n" +
	"\rwarming_state\x18\x03 \x01(\v28.envoy.admin.v3.ListenersConfigDump.DynamicListenerStateR\fwarmingState\x12_\n" +
	"\x0edraining_state\x18\x04 \x01(\v28.envoy.admin.v3.ListenersConfigDump.DynamicListenerStateR\rdrainingState\x12C\n" +
	"\verror_state\x18\x05 \x01(\v2\".envoy.admin.v3.UpdateFailureStateR\n" +
	"errorState\x12I\n" +
	"\rclient_status\x18\x06 \x01(\x0e2$.envoy.admin.v3.ClientResourceStatusR\fclientStatus:>\x9aň\x1e9\n" +
	"7envoy.admin.v2alpha.ListenersConfigDump.DynamicListener:.\x9aň\x1e)\n" +
	"'envoy.admin.v2alpha.ListenersConfigDump\"\xca\a\n" +
	"\x12ClustersConfigDump\x12!\n" +
	"\fversion_info\x18\x01 \x01(\tR\vversionInfo\x12Y\n" +
	"\x0fstatic_clusters\x18\x02 \x03(\v20.envoy.admin.v3.ClustersConfigDump.StaticClusterR\x0estaticClusters\x12i\n" +
	"\x17dynamic_active_clusters\x18\x03 \x03(\v21.envoy.admin.v3.ClustersConfigDump.DynamicClusterR\x15dynamicActiveClusters\x12k\n" +
	"\x18dynamic_warming_clusters\x18\x04 \x03(\v21.envoy.admin.v3.ClustersConfigDump.DynamicClusterR\x16dynamicWarmingClusters\x1a\xbb\x01\n" +
	"\rStaticCluster\x12.\n" +
	"\acluster\x18\x01 \x01(\v2\x14.google.protobuf.AnyR\acluster\x12=\n" +
	"\flast_updated\x18\x02 \x01(\v2\x1a.google.protobuf.TimestampR\vlastUpdated:;\x9aň\x1e6\n" +
	"4envoy.admin.v2alpha.ClustersConfigDump.StaticCluster\x1a\xf0\x02\n" +
	"\x0eDynamicCluster\x12!\n" +
	"\fversion_info\x18\x01 \x01(\tR\vversionInfo\x12.\n" +
	"\acluster\x18\x02 \x01(\v2\x14.google.protobuf.AnyR\acluster\x12=\n" +
	"\flast_updated\x18\x03 \x01(\v2\x1a.google.protobuf.TimestampR\vlastUpdated\x12C\n" +
	"\verror_state\x18\x04 \x01(\v2\".envoy.admin.v3.UpdateFailureStateR\n" +
	"errorState\x12I\n" +
	"\rclient_status\x18\x05 \x01(\x0e2$.envoy.admin.v3.ClientResourceStatusR\fclientStatus:<\x9aň\x1e7\n" +
	"5envoy.admin.v2alpha.ClustersConfigDump.DynamicCluster:-\x9aň\x1e(\n" +
	"&envoy.admin.v2alpha.ClustersConfigDump\"\xdd\x06\n" +
	"\x10RoutesConfigDump\x12d\n" +
	"\x14static_route_configs\x18\x02 \x03(\v22.envoy.admin.v3.RoutesConfigDump.StaticRouteConfigR\x12staticRouteConfigs\x12g\n" +
	"\x15dynamic_route_configs\x18\x03 \x03(\v23.envoy.admin.v3.RoutesConfigDump.DynamicRouteConfigR\x13dynamicRouteConfigs\x1a\xca\x01\n" +
	"\x11StaticRouteConfig\x127\n" +
	"\froute_config\x18\x01 \x01(\v2\x14.google.protobuf.AnyR\vrouteConfig\x12=\n" +
	"\flast_updated\x18\x02 \x01(\v2\x1a.google.protobuf.TimestampR\vlastUpdated:=\x9aň\x1e8\n" +
	"6envoy.admin.v2alpha.RoutesConfigDump.StaticRouteConfig\x1a\xff\x02\n" +
	"\x12DynamicRouteConfig\x12!\n" +
	"\fversion_info\x18\x01 \x01(\tR\vversionInfo\x127\n" +
	"\froute_config\x18\x02 \x01(\v2\x14.google.protobuf.AnyR\vrouteConfig\x12=\n" +
	"\flast_updated\x18\x03 \x01(\v2\x1a.google.protobuf.TimestampR\vlastUpdated\x12C\n" +
	"\verror_state\x18\x04 \x01(\v2\".envoy.admin.v3.UpdateFailureStateR\n" +
	"errorState\x12I\n" +
	"\rclient_status\x18\x05 \x01(\x0e2$.envoy.admin.v3.ClientResourceStatusR\fclientStatus:>\x9aň\x1e9\n" +
	"7envoy.admin.v2alpha.RoutesConfigDump.DynamicRouteConfig:+\x9aň\x1e&\n" +
	"$envoy.admin.v2alpha.RoutesConfigDump\"\x8c\b\n" +
	"\x16ScopedRoutesConfigDump\x12~\n" +
	"\x1binline_scoped_route_configs\x18\x01 \x03(\v2?.envoy.admin.v3.ScopedRoutesConfigDump.InlineScopedRouteConfigsR\x18inlineScopedRouteConfigs\x12\x81\x01\n" +
	"\x1cdynamic_scoped_route_configs\x18\x02 \x03(\v2@.envoy.admin.v3.ScopedRoutesConfigDump.DynamicScopedRouteConfigsR\x19dynamicScopedRouteConfigs\x1a\x81\x02\n" +
	"\x18InlineScopedRouteConfigs\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12F\n" +
	"\x14scoped_route_configs\x18\x02 \x03(\v2\x14.google.protobuf.AnyR\x12scopedRouteConfigs\x12=\n" +
	"\flast_updated\x18\x03 \x01(\v2\x1a.google.protobuf.TimestampR\vlastUpdated:J\x9aň\x1eE\n" +
	"Cenvoy.admin.v2alpha.ScopedRoutesConfigDump.InlineScopedRouteConfigs\x1a\xb6\x03\n" +
	"\x19DynamicScopedRouteConfigs\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12!\n" +
	"\fversion_info\x18\x02 \x01(\tR\vversionInfo\x12F\n" +
	"\x14scoped_route_configs\x18\x03 \x03(\v2\x14.google.protobuf.AnyR\x12scopedRouteConfigs\x12=\n" +
	"\flast_updated\x18\x04 \x01(\v2\x1a.google.protobuf.TimestampR\vlastUpdated\x12C\n" +
	"\verror_state\x18\x05 \x01(\v2\".envoy.admin.v3.UpdateFailureStateR\n" +
	"errorState\x12I\n" +
	"\rclient_status\x18\x06 \x01(\x0e2$.envoy.admin.v3.ClientResourceStatusR\fclientStatus:K\x9aň\x1eF\n" +
	"Denvoy.admin.v2alpha.ScopedRoutesConfigDump.DynamicScopedRouteConfigs:1\x9aň\x1e,\n" +
	"*envoy.admin.v2alpha.ScopedRoutesConfigDump\"\xde\x05\n" +
	"\x13EndpointsConfigDump\x12p\n" +
	"\x17static_endpoint_configs\x18\x02 \x03(\v28.envoy.admin.v3.EndpointsConfigDump.StaticEndpointConfigR\x15staticEndpointConfigs\x12s\n" +
	"\x18dynamic_endpoint_configs\x18\x03 \x03(\v29.envoy.admin.v3.EndpointsConfigDump.DynamicEndpointConfigR\x16dynamicEndpointConfigs\x1a\x94\x01\n" +
	"\x14StaticEndpointConfig\x12=\n" +
	"\x0fendpoint_config\x18\x01 \x01(\v2\x14.google.protobuf.AnyR\x0eendpointConfig\x12=\n" +
	"\flast_updated\x18\x02 \x01(\v2\x1a.google.protobuf.TimestampR\vlastUpdated\x1a\xc8\x02\n" +
	"\x15DynamicEndpointConfig\x12!\n" +
	"\fversion_info\x18\x01 \x01(\tR\vversionInfo\x12=\n" +
	"\x0fendpoint_config\x18\x02 \x01(\v2\x14.google.protobuf.AnyR\x0eendpointConfig\x12=\n" +
	"\flast_updated\x18\x03 \x01(\v2\x1a.google.protobuf.TimestampR\vlastUpdated\x12C\n" +
	"\verror_state\x18\x04 \x01(\v2\".envoy.admin.v3.UpdateFailureStateR\n" +
	"errorState\x12I\n" +
	"\rclient_status\x18\x05 \x01(\x0e2$.envoy.admin.v3.ClientResourceStatusR\fclientStatus\"\x89\x04\n" +
	"\x0eEcdsConfigDump\x12R\n" +
	"\fecds_filters\x18\x01 \x03(\v2/.envoy.admin.v3.EcdsConfigDump.EcdsFilterConfigR\vecdsFilters\x1a\xf7\x02\n" +
	"\x10EcdsFilterConfig\x12!\n" +
	"\fversion_info\x18\x01 \x01(\tR\vversionInfo\x125\n" +
	"\vecds_filter\x18\x02 \x01(\v2\x14.google.protobuf.AnyR\n" +
	"ecdsFilter\x12=\n" +
	"\flast_updated\x18\x03 \x01(\v2\x1a.google.protobuf.TimestampR\vlastUpdated\x12C\n" +
	"\verror_state\x18\x04 \x01(\v2\".envoy.admin.v3.UpdateFailureStateR\n" +
	"errorState\x12I\n" +
	"\rclient_status\x18\x05 \x01(\x0e2$.envoy.admin.v3.ClientResourceStatusR\fclientStatus::\x9aň\x1e5\n" +
	"3envoy.admin.v2alpha.EcdsConfigDump.EcdsFilterConfig:)\x9aň\x1e$\n" +
	"\"envoy.admin.v2alpha.EcdsConfigDump*~\n" +
	"\x14ClientResourceStatus\x12\v\n" +
	"\aUNKNOWN\x10\x00\x12\r\n" +
	"\tREQUESTED\x10\x01\x12\x12\n" +
	"\x0eDOES_NOT_EXIST\x10\x02\x12\t\n" +
	"\x05ACKED\x10\x03\x12\n" +
	"\n" +
	"\x06NACKED\x10\x04\x12\x12\n" +
	"\x0eRECEIVED_ERROR\x10\x05\x12\v\n" +
	"\aTIMEOUT\x10\x06B~\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"\x1cio.envoyproxy.envoy.admin.v3B\x15ConfigDumpSharedProtoP\x01Z=github.com/envoyproxy/go-control-plane/envoy/admin/v3;adminv3b\x06proto3"

var (
	file_envoy_admin_v3_config_dump_shared_proto_rawDescOnce sync.Once
	file_envoy_admin_v3_config_dump_shared_proto_rawDescData []byte
)

func file_envoy_admin_v3_config_dump_shared_proto_rawDescGZIP() []byte {
	file_envoy_admin_v3_config_dump_shared_proto_rawDescOnce.Do(func() {
		file_envoy_admin_v3_config_dump_shared_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_admin_v3_config_dump_shared_proto_rawDesc), len(file_envoy_admin_v3_config_dump_shared_proto_rawDesc)))
	})
	return file_envoy_admin_v3_config_dump_shared_proto_rawDescData
}

var file_envoy_admin_v3_config_dump_shared_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_envoy_admin_v3_config_dump_shared_proto_msgTypes = make([]protoimpl.MessageInfo, 19)
var file_envoy_admin_v3_config_dump_shared_proto_goTypes = []any{
	(ClientResourceStatus)(0),                                // 0: envoy.admin.v3.ClientResourceStatus
	(*UpdateFailureState)(nil),                               // 1: envoy.admin.v3.UpdateFailureState
	(*ListenersConfigDump)(nil),                              // 2: envoy.admin.v3.ListenersConfigDump
	(*ClustersConfigDump)(nil),                               // 3: envoy.admin.v3.ClustersConfigDump
	(*RoutesConfigDump)(nil),                                 // 4: envoy.admin.v3.RoutesConfigDump
	(*ScopedRoutesConfigDump)(nil),                           // 5: envoy.admin.v3.ScopedRoutesConfigDump
	(*EndpointsConfigDump)(nil),                              // 6: envoy.admin.v3.EndpointsConfigDump
	(*EcdsConfigDump)(nil),                                   // 7: envoy.admin.v3.EcdsConfigDump
	(*ListenersConfigDump_StaticListener)(nil),               // 8: envoy.admin.v3.ListenersConfigDump.StaticListener
	(*ListenersConfigDump_DynamicListenerState)(nil),         // 9: envoy.admin.v3.ListenersConfigDump.DynamicListenerState
	(*ListenersConfigDump_DynamicListener)(nil),              // 10: envoy.admin.v3.ListenersConfigDump.DynamicListener
	(*ClustersConfigDump_StaticCluster)(nil),                 // 11: envoy.admin.v3.ClustersConfigDump.StaticCluster
	(*ClustersConfigDump_DynamicCluster)(nil),                // 12: envoy.admin.v3.ClustersConfigDump.DynamicCluster
	(*RoutesConfigDump_StaticRouteConfig)(nil),               // 13: envoy.admin.v3.RoutesConfigDump.StaticRouteConfig
	(*RoutesConfigDump_DynamicRouteConfig)(nil),              // 14: envoy.admin.v3.RoutesConfigDump.DynamicRouteConfig
	(*ScopedRoutesConfigDump_InlineScopedRouteConfigs)(nil),  // 15: envoy.admin.v3.ScopedRoutesConfigDump.InlineScopedRouteConfigs
	(*ScopedRoutesConfigDump_DynamicScopedRouteConfigs)(nil), // 16: envoy.admin.v3.ScopedRoutesConfigDump.DynamicScopedRouteConfigs
	(*EndpointsConfigDump_StaticEndpointConfig)(nil),         // 17: envoy.admin.v3.EndpointsConfigDump.StaticEndpointConfig
	(*EndpointsConfigDump_DynamicEndpointConfig)(nil),        // 18: envoy.admin.v3.EndpointsConfigDump.DynamicEndpointConfig
	(*EcdsConfigDump_EcdsFilterConfig)(nil),                  // 19: envoy.admin.v3.EcdsConfigDump.EcdsFilterConfig
	(*anypb.Any)(nil),                                        // 20: google.protobuf.Any
	(*timestamppb.Timestamp)(nil),                            // 21: google.protobuf.Timestamp
}
var file_envoy_admin_v3_config_dump_shared_proto_depIdxs = []int32{
	20, // 0: envoy.admin.v3.UpdateFailureState.failed_configuration:type_name -> google.protobuf.Any
	21, // 1: envoy.admin.v3.UpdateFailureState.last_update_attempt:type_name -> google.protobuf.Timestamp
	8,  // 2: envoy.admin.v3.ListenersConfigDump.static_listeners:type_name -> envoy.admin.v3.ListenersConfigDump.StaticListener
	10, // 3: envoy.admin.v3.ListenersConfigDump.dynamic_listeners:type_name -> envoy.admin.v3.ListenersConfigDump.DynamicListener
	11, // 4: envoy.admin.v3.ClustersConfigDump.static_clusters:type_name -> envoy.admin.v3.ClustersConfigDump.StaticCluster
	12, // 5: envoy.admin.v3.ClustersConfigDump.dynamic_active_clusters:type_name -> envoy.admin.v3.ClustersConfigDump.DynamicCluster
	12, // 6: envoy.admin.v3.ClustersConfigDump.dynamic_warming_clusters:type_name -> envoy.admin.v3.ClustersConfigDump.DynamicCluster
	13, // 7: envoy.admin.v3.RoutesConfigDump.static_route_configs:type_name -> envoy.admin.v3.RoutesConfigDump.StaticRouteConfig
	14, // 8: envoy.admin.v3.RoutesConfigDump.dynamic_route_configs:type_name -> envoy.admin.v3.RoutesConfigDump.DynamicRouteConfig
	15, // 9: envoy.admin.v3.ScopedRoutesConfigDump.inline_scoped_route_configs:type_name -> envoy.admin.v3.ScopedRoutesConfigDump.InlineScopedRouteConfigs
	16, // 10: envoy.admin.v3.ScopedRoutesConfigDump.dynamic_scoped_route_configs:type_name -> envoy.admin.v3.ScopedRoutesConfigDump.DynamicScopedRouteConfigs
	17, // 11: envoy.admin.v3.EndpointsConfigDump.static_endpoint_configs:type_name -> envoy.admin.v3.EndpointsConfigDump.StaticEndpointConfig
	18, // 12: envoy.admin.v3.EndpointsConfigDump.dynamic_endpoint_configs:type_name -> envoy.admin.v3.EndpointsConfigDump.DynamicEndpointConfig
	19, // 13: envoy.admin.v3.EcdsConfigDump.ecds_filters:type_name -> envoy.admin.v3.EcdsConfigDump.EcdsFilterConfig
	20, // 14: envoy.admin.v3.ListenersConfigDump.StaticListener.listener:type_name -> google.protobuf.Any
	21, // 15: envoy.admin.v3.ListenersConfigDump.StaticListener.last_updated:type_name -> google.protobuf.Timestamp
	20, // 16: envoy.admin.v3.ListenersConfigDump.DynamicListenerState.listener:type_name -> google.protobuf.Any
	21, // 17: envoy.admin.v3.ListenersConfigDump.DynamicListenerState.last_updated:type_name -> google.protobuf.Timestamp
	9,  // 18: envoy.admin.v3.ListenersConfigDump.DynamicListener.active_state:type_name -> envoy.admin.v3.ListenersConfigDump.DynamicListenerState
	9,  // 19: envoy.admin.v3.ListenersConfigDump.DynamicListener.warming_state:type_name -> envoy.admin.v3.ListenersConfigDump.DynamicListenerState
	9,  // 20: envoy.admin.v3.ListenersConfigDump.DynamicListener.draining_state:type_name -> envoy.admin.v3.ListenersConfigDump.DynamicListenerState
	1,  // 21: envoy.admin.v3.ListenersConfigDump.DynamicListener.error_state:type_name -> envoy.admin.v3.UpdateFailureState
	0,  // 22: envoy.admin.v3.ListenersConfigDump.DynamicListener.client_status:type_name -> envoy.admin.v3.ClientResourceStatus
	20, // 23: envoy.admin.v3.ClustersConfigDump.StaticCluster.cluster:type_name -> google.protobuf.Any
	21, // 24: envoy.admin.v3.ClustersConfigDump.StaticCluster.last_updated:type_name -> google.protobuf.Timestamp
	20, // 25: envoy.admin.v3.ClustersConfigDump.DynamicCluster.cluster:type_name -> google.protobuf.Any
	21, // 26: envoy.admin.v3.ClustersConfigDump.DynamicCluster.last_updated:type_name -> google.protobuf.Timestamp
	1,  // 27: envoy.admin.v3.ClustersConfigDump.DynamicCluster.error_state:type_name -> envoy.admin.v3.UpdateFailureState
	0,  // 28: envoy.admin.v3.ClustersConfigDump.DynamicCluster.client_status:type_name -> envoy.admin.v3.ClientResourceStatus
	20, // 29: envoy.admin.v3.RoutesConfigDump.StaticRouteConfig.route_config:type_name -> google.protobuf.Any
	21, // 30: envoy.admin.v3.RoutesConfigDump.StaticRouteConfig.last_updated:type_name -> google.protobuf.Timestamp
	20, // 31: envoy.admin.v3.RoutesConfigDump.DynamicRouteConfig.route_config:type_name -> google.protobuf.Any
	21, // 32: envoy.admin.v3.RoutesConfigDump.DynamicRouteConfig.last_updated:type_name -> google.protobuf.Timestamp
	1,  // 33: envoy.admin.v3.RoutesConfigDump.DynamicRouteConfig.error_state:type_name -> envoy.admin.v3.UpdateFailureState
	0,  // 34: envoy.admin.v3.RoutesConfigDump.DynamicRouteConfig.client_status:type_name -> envoy.admin.v3.ClientResourceStatus
	20, // 35: envoy.admin.v3.ScopedRoutesConfigDump.InlineScopedRouteConfigs.scoped_route_configs:type_name -> google.protobuf.Any
	21, // 36: envoy.admin.v3.ScopedRoutesConfigDump.InlineScopedRouteConfigs.last_updated:type_name -> google.protobuf.Timestamp
	20, // 37: envoy.admin.v3.ScopedRoutesConfigDump.DynamicScopedRouteConfigs.scoped_route_configs:type_name -> google.protobuf.Any
	21, // 38: envoy.admin.v3.ScopedRoutesConfigDump.DynamicScopedRouteConfigs.last_updated:type_name -> google.protobuf.Timestamp
	1,  // 39: envoy.admin.v3.ScopedRoutesConfigDump.DynamicScopedRouteConfigs.error_state:type_name -> envoy.admin.v3.UpdateFailureState
	0,  // 40: envoy.admin.v3.ScopedRoutesConfigDump.DynamicScopedRouteConfigs.client_status:type_name -> envoy.admin.v3.ClientResourceStatus
	20, // 41: envoy.admin.v3.EndpointsConfigDump.StaticEndpointConfig.endpoint_config:type_name -> google.protobuf.Any
	21, // 42: envoy.admin.v3.EndpointsConfigDump.StaticEndpointConfig.last_updated:type_name -> google.protobuf.Timestamp
	20, // 43: envoy.admin.v3.EndpointsConfigDump.DynamicEndpointConfig.endpoint_config:type_name -> google.protobuf.Any
	21, // 44: envoy.admin.v3.EndpointsConfigDump.DynamicEndpointConfig.last_updated:type_name -> google.protobuf.Timestamp
	1,  // 45: envoy.admin.v3.EndpointsConfigDump.DynamicEndpointConfig.error_state:type_name -> envoy.admin.v3.UpdateFailureState
	0,  // 46: envoy.admin.v3.EndpointsConfigDump.DynamicEndpointConfig.client_status:type_name -> envoy.admin.v3.ClientResourceStatus
	20, // 47: envoy.admin.v3.EcdsConfigDump.EcdsFilterConfig.ecds_filter:type_name -> google.protobuf.Any
	21, // 48: envoy.admin.v3.EcdsConfigDump.EcdsFilterConfig.last_updated:type_name -> google.protobuf.Timestamp
	1,  // 49: envoy.admin.v3.EcdsConfigDump.EcdsFilterConfig.error_state:type_name -> envoy.admin.v3.UpdateFailureState
	0,  // 50: envoy.admin.v3.EcdsConfigDump.EcdsFilterConfig.client_status:type_name -> envoy.admin.v3.ClientResourceStatus
	51, // [51:51] is the sub-list for method output_type
	51, // [51:51] is the sub-list for method input_type
	51, // [51:51] is the sub-list for extension type_name
	51, // [51:51] is the sub-list for extension extendee
	0,  // [0:51] is the sub-list for field type_name
}

func init() { file_envoy_admin_v3_config_dump_shared_proto_init() }
func file_envoy_admin_v3_config_dump_shared_proto_init() {
	if File_envoy_admin_v3_config_dump_shared_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_admin_v3_config_dump_shared_proto_rawDesc), len(file_envoy_admin_v3_config_dump_shared_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   19,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_admin_v3_config_dump_shared_proto_goTypes,
		DependencyIndexes: file_envoy_admin_v3_config_dump_shared_proto_depIdxs,
		EnumInfos:         file_envoy_admin_v3_config_dump_shared_proto_enumTypes,
		MessageInfos:      file_envoy_admin_v3_config_dump_shared_proto_msgTypes,
	}.Build()
	File_envoy_admin_v3_config_dump_shared_proto = out.File
	file_envoy_admin_v3_config_dump_shared_proto_goTypes = nil
	file_envoy_admin_v3_config_dump_shared_proto_depIdxs = nil
}
