// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/service/status/v3/csds.proto

package statusv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v32 "github.com/envoyproxy/go-control-plane/envoy/admin/v3"
	_ "github.com/envoyproxy/go-control-plane/envoy/annotations"
	v31 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/type/matcher/v3"
	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	anypb "google.golang.org/protobuf/types/known/anypb"
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Status of a config from a management server view.
type ConfigStatus int32

const (
	// Status info is not available/unknown.
	ConfigStatus_UNKNOWN ConfigStatus = 0
	// Management server has sent the config to client and received ACK.
	ConfigStatus_SYNCED ConfigStatus = 1
	// Config is not sent.
	ConfigStatus_NOT_SENT ConfigStatus = 2
	// Management server has sent the config to client but hasn’t received
	// ACK/NACK.
	ConfigStatus_STALE ConfigStatus = 3
	// Management server has sent the config to client but received NACK. The
	// attached config dump will be the latest config (the rejected one), since
	// it is the persisted version in the management server.
	ConfigStatus_ERROR ConfigStatus = 4
)

// Enum value maps for ConfigStatus.
var (
	ConfigStatus_name = map[int32]string{
		0: "UNKNOWN",
		1: "SYNCED",
		2: "NOT_SENT",
		3: "STALE",
		4: "ERROR",
	}
	ConfigStatus_value = map[string]int32{
		"UNKNOWN":  0,
		"SYNCED":   1,
		"NOT_SENT": 2,
		"STALE":    3,
		"ERROR":    4,
	}
)

func (x ConfigStatus) Enum() *ConfigStatus {
	p := new(ConfigStatus)
	*p = x
	return p
}

func (x ConfigStatus) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ConfigStatus) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_service_status_v3_csds_proto_enumTypes[0].Descriptor()
}

func (ConfigStatus) Type() protoreflect.EnumType {
	return &file_envoy_service_status_v3_csds_proto_enumTypes[0]
}

func (x ConfigStatus) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ConfigStatus.Descriptor instead.
func (ConfigStatus) EnumDescriptor() ([]byte, []int) {
	return file_envoy_service_status_v3_csds_proto_rawDescGZIP(), []int{0}
}

// Config status from a client-side view.
type ClientConfigStatus int32

const (
	// Config status is not available/unknown.
	ClientConfigStatus_CLIENT_UNKNOWN ClientConfigStatus = 0
	// Client requested the config but hasn't received any config from management
	// server yet.
	ClientConfigStatus_CLIENT_REQUESTED ClientConfigStatus = 1
	// Client received the config and replied with ACK.
	ClientConfigStatus_CLIENT_ACKED ClientConfigStatus = 2
	// Client received the config and replied with NACK. Notably, the attached
	// config dump is not the NACKed version, but the most recent accepted one. If
	// no config is accepted yet, the attached config dump will be empty.
	ClientConfigStatus_CLIENT_NACKED ClientConfigStatus = 3
	// Client received an error from the control plane. The attached config
	// dump is the most recent accepted one. If no config is accepted yet,
	// the attached config dump will be empty.
	ClientConfigStatus_CLIENT_RECEIVED_ERROR ClientConfigStatus = 4
)

// Enum value maps for ClientConfigStatus.
var (
	ClientConfigStatus_name = map[int32]string{
		0: "CLIENT_UNKNOWN",
		1: "CLIENT_REQUESTED",
		2: "CLIENT_ACKED",
		3: "CLIENT_NACKED",
		4: "CLIENT_RECEIVED_ERROR",
	}
	ClientConfigStatus_value = map[string]int32{
		"CLIENT_UNKNOWN":        0,
		"CLIENT_REQUESTED":      1,
		"CLIENT_ACKED":          2,
		"CLIENT_NACKED":         3,
		"CLIENT_RECEIVED_ERROR": 4,
	}
)

func (x ClientConfigStatus) Enum() *ClientConfigStatus {
	p := new(ClientConfigStatus)
	*p = x
	return p
}

func (x ClientConfigStatus) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ClientConfigStatus) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_service_status_v3_csds_proto_enumTypes[1].Descriptor()
}

func (ClientConfigStatus) Type() protoreflect.EnumType {
	return &file_envoy_service_status_v3_csds_proto_enumTypes[1]
}

func (x ClientConfigStatus) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ClientConfigStatus.Descriptor instead.
func (ClientConfigStatus) EnumDescriptor() ([]byte, []int) {
	return file_envoy_service_status_v3_csds_proto_rawDescGZIP(), []int{1}
}

// Request for client status of clients identified by a list of NodeMatchers.
type ClientStatusRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Management server can use these match criteria to identify clients.
	// The match follows OR semantics.
	NodeMatchers []*v3.NodeMatcher `protobuf:"bytes,1,rep,name=node_matchers,json=nodeMatchers,proto3" json:"node_matchers,omitempty"`
	// The node making the csds request.
	Node *v31.Node `protobuf:"bytes,2,opt,name=node,proto3" json:"node,omitempty"`
	// If true, the server will not include the resource contents in the response
	// (i.e., the generic_xds_configs.xds_config field will not be populated).
	// [#not-implemented-hide:]
	ExcludeResourceContents bool `protobuf:"varint,3,opt,name=exclude_resource_contents,json=excludeResourceContents,proto3" json:"exclude_resource_contents,omitempty"`
	unknownFields           protoimpl.UnknownFields
	sizeCache               protoimpl.SizeCache
}

func (x *ClientStatusRequest) Reset() {
	*x = ClientStatusRequest{}
	mi := &file_envoy_service_status_v3_csds_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClientStatusRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClientStatusRequest) ProtoMessage() {}

func (x *ClientStatusRequest) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_status_v3_csds_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClientStatusRequest.ProtoReflect.Descriptor instead.
func (*ClientStatusRequest) Descriptor() ([]byte, []int) {
	return file_envoy_service_status_v3_csds_proto_rawDescGZIP(), []int{0}
}

func (x *ClientStatusRequest) GetNodeMatchers() []*v3.NodeMatcher {
	if x != nil {
		return x.NodeMatchers
	}
	return nil
}

func (x *ClientStatusRequest) GetNode() *v31.Node {
	if x != nil {
		return x.Node
	}
	return nil
}

func (x *ClientStatusRequest) GetExcludeResourceContents() bool {
	if x != nil {
		return x.ExcludeResourceContents
	}
	return false
}

// Detailed config (per xDS) with status.
// [#next-free-field: 8]
type PerXdsConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Config status generated by management servers. Will not be present if the
	// CSDS server is an xDS client.
	Status ConfigStatus `protobuf:"varint,1,opt,name=status,proto3,enum=envoy.service.status.v3.ConfigStatus" json:"status,omitempty"`
	// Client config status is populated by xDS clients. Will not be present if
	// the CSDS server is an xDS server. No matter what the client config status
	// is, xDS clients should always dump the most recent accepted xDS config.
	//
	// .. attention::
	//
	//	This field is deprecated. Use :ref:`ClientResourceStatus
	//	<envoy_v3_api_enum_admin.v3.ClientResourceStatus>` for per-resource
	//	config status instead.
	//
	// Deprecated: Marked as deprecated in envoy/service/status/v3/csds.proto.
	ClientStatus ClientConfigStatus `protobuf:"varint,7,opt,name=client_status,json=clientStatus,proto3,enum=envoy.service.status.v3.ClientConfigStatus" json:"client_status,omitempty"`
	// Types that are valid to be assigned to PerXdsConfig:
	//
	//	*PerXdsConfig_ListenerConfig
	//	*PerXdsConfig_ClusterConfig
	//	*PerXdsConfig_RouteConfig
	//	*PerXdsConfig_ScopedRouteConfig
	//	*PerXdsConfig_EndpointConfig
	PerXdsConfig  isPerXdsConfig_PerXdsConfig `protobuf_oneof:"per_xds_config"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PerXdsConfig) Reset() {
	*x = PerXdsConfig{}
	mi := &file_envoy_service_status_v3_csds_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PerXdsConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PerXdsConfig) ProtoMessage() {}

func (x *PerXdsConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_status_v3_csds_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PerXdsConfig.ProtoReflect.Descriptor instead.
func (*PerXdsConfig) Descriptor() ([]byte, []int) {
	return file_envoy_service_status_v3_csds_proto_rawDescGZIP(), []int{1}
}

func (x *PerXdsConfig) GetStatus() ConfigStatus {
	if x != nil {
		return x.Status
	}
	return ConfigStatus_UNKNOWN
}

// Deprecated: Marked as deprecated in envoy/service/status/v3/csds.proto.
func (x *PerXdsConfig) GetClientStatus() ClientConfigStatus {
	if x != nil {
		return x.ClientStatus
	}
	return ClientConfigStatus_CLIENT_UNKNOWN
}

func (x *PerXdsConfig) GetPerXdsConfig() isPerXdsConfig_PerXdsConfig {
	if x != nil {
		return x.PerXdsConfig
	}
	return nil
}

func (x *PerXdsConfig) GetListenerConfig() *v32.ListenersConfigDump {
	if x != nil {
		if x, ok := x.PerXdsConfig.(*PerXdsConfig_ListenerConfig); ok {
			return x.ListenerConfig
		}
	}
	return nil
}

func (x *PerXdsConfig) GetClusterConfig() *v32.ClustersConfigDump {
	if x != nil {
		if x, ok := x.PerXdsConfig.(*PerXdsConfig_ClusterConfig); ok {
			return x.ClusterConfig
		}
	}
	return nil
}

func (x *PerXdsConfig) GetRouteConfig() *v32.RoutesConfigDump {
	if x != nil {
		if x, ok := x.PerXdsConfig.(*PerXdsConfig_RouteConfig); ok {
			return x.RouteConfig
		}
	}
	return nil
}

func (x *PerXdsConfig) GetScopedRouteConfig() *v32.ScopedRoutesConfigDump {
	if x != nil {
		if x, ok := x.PerXdsConfig.(*PerXdsConfig_ScopedRouteConfig); ok {
			return x.ScopedRouteConfig
		}
	}
	return nil
}

func (x *PerXdsConfig) GetEndpointConfig() *v32.EndpointsConfigDump {
	if x != nil {
		if x, ok := x.PerXdsConfig.(*PerXdsConfig_EndpointConfig); ok {
			return x.EndpointConfig
		}
	}
	return nil
}

type isPerXdsConfig_PerXdsConfig interface {
	isPerXdsConfig_PerXdsConfig()
}

type PerXdsConfig_ListenerConfig struct {
	ListenerConfig *v32.ListenersConfigDump `protobuf:"bytes,2,opt,name=listener_config,json=listenerConfig,proto3,oneof"`
}

type PerXdsConfig_ClusterConfig struct {
	ClusterConfig *v32.ClustersConfigDump `protobuf:"bytes,3,opt,name=cluster_config,json=clusterConfig,proto3,oneof"`
}

type PerXdsConfig_RouteConfig struct {
	RouteConfig *v32.RoutesConfigDump `protobuf:"bytes,4,opt,name=route_config,json=routeConfig,proto3,oneof"`
}

type PerXdsConfig_ScopedRouteConfig struct {
	ScopedRouteConfig *v32.ScopedRoutesConfigDump `protobuf:"bytes,5,opt,name=scoped_route_config,json=scopedRouteConfig,proto3,oneof"`
}

type PerXdsConfig_EndpointConfig struct {
	EndpointConfig *v32.EndpointsConfigDump `protobuf:"bytes,6,opt,name=endpoint_config,json=endpointConfig,proto3,oneof"`
}

func (*PerXdsConfig_ListenerConfig) isPerXdsConfig_PerXdsConfig() {}

func (*PerXdsConfig_ClusterConfig) isPerXdsConfig_PerXdsConfig() {}

func (*PerXdsConfig_RouteConfig) isPerXdsConfig_PerXdsConfig() {}

func (*PerXdsConfig_ScopedRouteConfig) isPerXdsConfig_PerXdsConfig() {}

func (*PerXdsConfig_EndpointConfig) isPerXdsConfig_PerXdsConfig() {}

// All xds configs for a particular client.
type ClientConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Node for a particular client.
	Node *v31.Node `protobuf:"bytes,1,opt,name=node,proto3" json:"node,omitempty"`
	// This field is deprecated in favor of generic_xds_configs which is
	// much simpler and uniform in structure.
	//
	// Deprecated: Marked as deprecated in envoy/service/status/v3/csds.proto.
	XdsConfig []*PerXdsConfig `protobuf:"bytes,2,rep,name=xds_config,json=xdsConfig,proto3" json:"xds_config,omitempty"`
	// Represents generic xDS config and the exact config structure depends on
	// the type URL (like Cluster if it is CDS)
	GenericXdsConfigs []*ClientConfig_GenericXdsConfig `protobuf:"bytes,3,rep,name=generic_xds_configs,json=genericXdsConfigs,proto3" json:"generic_xds_configs,omitempty"`
	// For xDS clients, the scope in which the data is used.
	// For example, gRPC indicates the data plane target or that the data is
	// associated with gRPC server(s).
	ClientScope   string `protobuf:"bytes,4,opt,name=client_scope,json=clientScope,proto3" json:"client_scope,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ClientConfig) Reset() {
	*x = ClientConfig{}
	mi := &file_envoy_service_status_v3_csds_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClientConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClientConfig) ProtoMessage() {}

func (x *ClientConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_status_v3_csds_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClientConfig.ProtoReflect.Descriptor instead.
func (*ClientConfig) Descriptor() ([]byte, []int) {
	return file_envoy_service_status_v3_csds_proto_rawDescGZIP(), []int{2}
}

func (x *ClientConfig) GetNode() *v31.Node {
	if x != nil {
		return x.Node
	}
	return nil
}

// Deprecated: Marked as deprecated in envoy/service/status/v3/csds.proto.
func (x *ClientConfig) GetXdsConfig() []*PerXdsConfig {
	if x != nil {
		return x.XdsConfig
	}
	return nil
}

func (x *ClientConfig) GetGenericXdsConfigs() []*ClientConfig_GenericXdsConfig {
	if x != nil {
		return x.GenericXdsConfigs
	}
	return nil
}

func (x *ClientConfig) GetClientScope() string {
	if x != nil {
		return x.ClientScope
	}
	return ""
}

type ClientStatusResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Client configs for the clients specified in the ClientStatusRequest.
	Config        []*ClientConfig `protobuf:"bytes,1,rep,name=config,proto3" json:"config,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ClientStatusResponse) Reset() {
	*x = ClientStatusResponse{}
	mi := &file_envoy_service_status_v3_csds_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClientStatusResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClientStatusResponse) ProtoMessage() {}

func (x *ClientStatusResponse) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_status_v3_csds_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClientStatusResponse.ProtoReflect.Descriptor instead.
func (*ClientStatusResponse) Descriptor() ([]byte, []int) {
	return file_envoy_service_status_v3_csds_proto_rawDescGZIP(), []int{3}
}

func (x *ClientStatusResponse) GetConfig() []*ClientConfig {
	if x != nil {
		return x.Config
	}
	return nil
}

// GenericXdsConfig is used to specify the config status and the dump
// of any xDS resource identified by their type URL. It is the generalized
// version of the now deprecated ListenersConfigDump, ClustersConfigDump etc
// [#next-free-field: 10]
type ClientConfig_GenericXdsConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Type_url represents the fully qualified name of xDS resource type
	// like envoy.v3.Cluster, envoy.v3.ClusterLoadAssignment etc.
	TypeUrl string `protobuf:"bytes,1,opt,name=type_url,json=typeUrl,proto3" json:"type_url,omitempty"`
	// Name of the xDS resource
	Name string `protobuf:"bytes,2,opt,name=name,proto3" json:"name,omitempty"`
	// This is the :ref:`version_info <envoy_v3_api_field_service.discovery.v3.DiscoveryResponse.version_info>`
	// in the last processed xDS discovery response. If there are only
	// static bootstrap listeners, this field will be ""
	VersionInfo string `protobuf:"bytes,3,opt,name=version_info,json=versionInfo,proto3" json:"version_info,omitempty"`
	// The xDS resource config. Actual content depends on the type
	XdsConfig *anypb.Any `protobuf:"bytes,4,opt,name=xds_config,json=xdsConfig,proto3" json:"xds_config,omitempty"`
	// Timestamp when the xDS resource was last updated
	LastUpdated *timestamppb.Timestamp `protobuf:"bytes,5,opt,name=last_updated,json=lastUpdated,proto3" json:"last_updated,omitempty"`
	// Per xDS resource config status. It is generated by management servers.
	// It will not be present if the CSDS server is an xDS client.
	ConfigStatus ConfigStatus `protobuf:"varint,6,opt,name=config_status,json=configStatus,proto3,enum=envoy.service.status.v3.ConfigStatus" json:"config_status,omitempty"`
	// Per xDS resource status from the view of a xDS client
	ClientStatus v32.ClientResourceStatus `protobuf:"varint,7,opt,name=client_status,json=clientStatus,proto3,enum=envoy.admin.v3.ClientResourceStatus" json:"client_status,omitempty"`
	// Set if the last update failed, cleared after the next successful
	// update. The *error_state* field contains the rejected version of
	// this particular resource along with the reason and timestamp. For
	// successfully updated or acknowledged resource, this field should
	// be empty.
	// [#not-implemented-hide:]
	ErrorState *v32.UpdateFailureState `protobuf:"bytes,8,opt,name=error_state,json=errorState,proto3" json:"error_state,omitempty"`
	// Is static resource is true if it is specified in the config supplied
	// through the file at the startup.
	IsStaticResource bool `protobuf:"varint,9,opt,name=is_static_resource,json=isStaticResource,proto3" json:"is_static_resource,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *ClientConfig_GenericXdsConfig) Reset() {
	*x = ClientConfig_GenericXdsConfig{}
	mi := &file_envoy_service_status_v3_csds_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClientConfig_GenericXdsConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClientConfig_GenericXdsConfig) ProtoMessage() {}

func (x *ClientConfig_GenericXdsConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_status_v3_csds_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClientConfig_GenericXdsConfig.ProtoReflect.Descriptor instead.
func (*ClientConfig_GenericXdsConfig) Descriptor() ([]byte, []int) {
	return file_envoy_service_status_v3_csds_proto_rawDescGZIP(), []int{2, 0}
}

func (x *ClientConfig_GenericXdsConfig) GetTypeUrl() string {
	if x != nil {
		return x.TypeUrl
	}
	return ""
}

func (x *ClientConfig_GenericXdsConfig) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *ClientConfig_GenericXdsConfig) GetVersionInfo() string {
	if x != nil {
		return x.VersionInfo
	}
	return ""
}

func (x *ClientConfig_GenericXdsConfig) GetXdsConfig() *anypb.Any {
	if x != nil {
		return x.XdsConfig
	}
	return nil
}

func (x *ClientConfig_GenericXdsConfig) GetLastUpdated() *timestamppb.Timestamp {
	if x != nil {
		return x.LastUpdated
	}
	return nil
}

func (x *ClientConfig_GenericXdsConfig) GetConfigStatus() ConfigStatus {
	if x != nil {
		return x.ConfigStatus
	}
	return ConfigStatus_UNKNOWN
}

func (x *ClientConfig_GenericXdsConfig) GetClientStatus() v32.ClientResourceStatus {
	if x != nil {
		return x.ClientStatus
	}
	return v32.ClientResourceStatus(0)
}

func (x *ClientConfig_GenericXdsConfig) GetErrorState() *v32.UpdateFailureState {
	if x != nil {
		return x.ErrorState
	}
	return nil
}

func (x *ClientConfig_GenericXdsConfig) GetIsStaticResource() bool {
	if x != nil {
		return x.IsStaticResource
	}
	return false
}

var File_envoy_service_status_v3_csds_proto protoreflect.FileDescriptor

const file_envoy_service_status_v3_csds_proto_rawDesc = "" +
	"\n" +
	"\"envoy/service/status/v3/csds.proto\x12\x17envoy.service.status.v3\x1a'envoy/admin/v3/config_dump_shared.proto\x1a\x1fenvoy/config/core/v3/base.proto\x1a envoy/type/matcher/v3/node.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x19google/protobuf/any.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"\xfe\x01\n" +
	"\x13ClientStatusRequest\x12G\n" +
	"\rnode_matchers\x18\x01 \x03(\v2\".envoy.type.matcher.v3.NodeMatcherR\fnodeMatchers\x12.\n" +
	"\x04node\x18\x02 \x01(\v2\x1a.envoy.config.core.v3.NodeR\x04node\x12:\n" +
	"\x19exclude_resource_contents\x18\x03 \x01(\bR\x17excludeResourceContents:2\x9aň\x1e-\n" +
	"+envoy.service.status.v2.ClientStatusRequest\"\xf9\x04\n" +
	"\fPerXdsConfig\x12=\n" +
	"\x06status\x18\x01 \x01(\x0e2%.envoy.service.status.v3.ConfigStatusR\x06status\x12]\n" +
	"\rclient_status\x18\a \x01(\x0e2+.envoy.service.status.v3.ClientConfigStatusB\v\x92ǆ\xd8\x04\x033.0\x18\x01R\fclientStatus\x12N\n" +
	"\x0flistener_config\x18\x02 \x01(\v2#.envoy.admin.v3.ListenersConfigDumpH\x00R\x0elistenerConfig\x12K\n" +
	"\x0ecluster_config\x18\x03 \x01(\v2\".envoy.admin.v3.ClustersConfigDumpH\x00R\rclusterConfig\x12E\n" +
	"\froute_config\x18\x04 \x01(\v2 .envoy.admin.v3.RoutesConfigDumpH\x00R\vrouteConfig\x12X\n" +
	"\x13scoped_route_config\x18\x05 \x01(\v2&.envoy.admin.v3.ScopedRoutesConfigDumpH\x00R\x11scopedRouteConfig\x12N\n" +
	"\x0fendpoint_config\x18\x06 \x01(\v2#.envoy.admin.v3.EndpointsConfigDumpH\x00R\x0eendpointConfig:+\x9aň\x1e&\n" +
	"$envoy.service.status.v2.PerXdsConfigB\x10\n" +
	"\x0eper_xds_config\"\xae\x06\n" +
	"\fClientConfig\x12.\n" +
	"\x04node\x18\x01 \x01(\v2\x1a.envoy.config.core.v3.NodeR\x04node\x12Q\n" +
	"\n" +
	"xds_config\x18\x02 \x03(\v2%.envoy.service.status.v3.PerXdsConfigB\v\x92ǆ\xd8\x04\x033.0\x18\x01R\txdsConfig\x12f\n" +
	"\x13generic_xds_configs\x18\x03 \x03(\v26.envoy.service.status.v3.ClientConfig.GenericXdsConfigR\x11genericXdsConfigs\x12!\n" +
	"\fclient_scope\x18\x04 \x01(\tR\vclientScope\x1a\xe2\x03\n" +
	"\x10GenericXdsConfig\x12\x19\n" +
	"\btype_url\x18\x01 \x01(\tR\atypeUrl\x12\x12\n" +
	"\x04name\x18\x02 \x01(\tR\x04name\x12!\n" +
	"\fversion_info\x18\x03 \x01(\tR\vversionInfo\x123\n" +
	"\n" +
	"xds_config\x18\x04 \x01(\v2\x14.google.protobuf.AnyR\txdsConfig\x12=\n" +
	"\flast_updated\x18\x05 \x01(\v2\x1a.google.protobuf.TimestampR\vlastUpdated\x12J\n" +
	"\rconfig_status\x18\x06 \x01(\x0e2%.envoy.service.status.v3.ConfigStatusR\fconfigStatus\x12I\n" +
	"\rclient_status\x18\a \x01(\x0e2$.envoy.admin.v3.ClientResourceStatusR\fclientStatus\x12C\n" +
	"\verror_state\x18\b \x01(\v2\".envoy.admin.v3.UpdateFailureStateR\n" +
	"errorState\x12,\n" +
	"\x12is_static_resource\x18\t \x01(\bR\x10isStaticResource:+\x9aň\x1e&\n" +
	"$envoy.service.status.v2.ClientConfig\"\x8a\x01\n" +
	"\x14ClientStatusResponse\x12=\n" +
	"\x06config\x18\x01 \x03(\v2%.envoy.service.status.v3.ClientConfigR\x06config:3\x9aň\x1e.\n" +
	",envoy.service.status.v2.ClientStatusResponse*K\n" +
	"\fConfigStatus\x12\v\n" +
	"\aUNKNOWN\x10\x00\x12\n" +
	"\n" +
	"\x06SYNCED\x10\x01\x12\f\n" +
	"\bNOT_SENT\x10\x02\x12\t\n" +
	"\x05STALE\x10\x03\x12\t\n" +
	"\x05ERROR\x10\x04*~\n" +
	"\x12ClientConfigStatus\x12\x12\n" +
	"\x0eCLIENT_UNKNOWN\x10\x00\x12\x14\n" +
	"\x10CLIENT_REQUESTED\x10\x01\x12\x10\n" +
	"\fCLIENT_ACKED\x10\x02\x12\x11\n" +
	"\rCLIENT_NACKED\x10\x03\x12\x19\n" +
	"\x15CLIENT_RECEIVED_ERROR\x10\x042\xb2\x02\n" +
	"\x1cClientStatusDiscoveryService\x12w\n" +
	"\x12StreamClientStatus\x12,.envoy.service.status.v3.ClientStatusRequest\x1a-.envoy.service.status.v3.ClientStatusResponse\"\x00(\x010\x01\x12\x98\x01\n" +
	"\x11FetchClientStatus\x12,.envoy.service.status.v3.ClientStatusRequest\x1a-.envoy.service.status.v3.ClientStatusResponse\"&\x82\xd3\xe4\x93\x02 :\x01*\"\x1b/v3/discovery:client_statusB\x85\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"%io.envoyproxy.envoy.service.status.v3B\tCsdsProtoP\x01ZGgithub.com/envoyproxy/go-control-plane/envoy/service/status/v3;statusv3b\x06proto3"

var (
	file_envoy_service_status_v3_csds_proto_rawDescOnce sync.Once
	file_envoy_service_status_v3_csds_proto_rawDescData []byte
)

func file_envoy_service_status_v3_csds_proto_rawDescGZIP() []byte {
	file_envoy_service_status_v3_csds_proto_rawDescOnce.Do(func() {
		file_envoy_service_status_v3_csds_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_service_status_v3_csds_proto_rawDesc), len(file_envoy_service_status_v3_csds_proto_rawDesc)))
	})
	return file_envoy_service_status_v3_csds_proto_rawDescData
}

var file_envoy_service_status_v3_csds_proto_enumTypes = make([]protoimpl.EnumInfo, 2)
var file_envoy_service_status_v3_csds_proto_msgTypes = make([]protoimpl.MessageInfo, 5)
var file_envoy_service_status_v3_csds_proto_goTypes = []any{
	(ConfigStatus)(0),                     // 0: envoy.service.status.v3.ConfigStatus
	(ClientConfigStatus)(0),               // 1: envoy.service.status.v3.ClientConfigStatus
	(*ClientStatusRequest)(nil),           // 2: envoy.service.status.v3.ClientStatusRequest
	(*PerXdsConfig)(nil),                  // 3: envoy.service.status.v3.PerXdsConfig
	(*ClientConfig)(nil),                  // 4: envoy.service.status.v3.ClientConfig
	(*ClientStatusResponse)(nil),          // 5: envoy.service.status.v3.ClientStatusResponse
	(*ClientConfig_GenericXdsConfig)(nil), // 6: envoy.service.status.v3.ClientConfig.GenericXdsConfig
	(*v3.NodeMatcher)(nil),                // 7: envoy.type.matcher.v3.NodeMatcher
	(*v31.Node)(nil),                      // 8: envoy.config.core.v3.Node
	(*v32.ListenersConfigDump)(nil),       // 9: envoy.admin.v3.ListenersConfigDump
	(*v32.ClustersConfigDump)(nil),        // 10: envoy.admin.v3.ClustersConfigDump
	(*v32.RoutesConfigDump)(nil),          // 11: envoy.admin.v3.RoutesConfigDump
	(*v32.ScopedRoutesConfigDump)(nil),    // 12: envoy.admin.v3.ScopedRoutesConfigDump
	(*v32.EndpointsConfigDump)(nil),       // 13: envoy.admin.v3.EndpointsConfigDump
	(*anypb.Any)(nil),                     // 14: google.protobuf.Any
	(*timestamppb.Timestamp)(nil),         // 15: google.protobuf.Timestamp
	(v32.ClientResourceStatus)(0),         // 16: envoy.admin.v3.ClientResourceStatus
	(*v32.UpdateFailureState)(nil),        // 17: envoy.admin.v3.UpdateFailureState
}
var file_envoy_service_status_v3_csds_proto_depIdxs = []int32{
	7,  // 0: envoy.service.status.v3.ClientStatusRequest.node_matchers:type_name -> envoy.type.matcher.v3.NodeMatcher
	8,  // 1: envoy.service.status.v3.ClientStatusRequest.node:type_name -> envoy.config.core.v3.Node
	0,  // 2: envoy.service.status.v3.PerXdsConfig.status:type_name -> envoy.service.status.v3.ConfigStatus
	1,  // 3: envoy.service.status.v3.PerXdsConfig.client_status:type_name -> envoy.service.status.v3.ClientConfigStatus
	9,  // 4: envoy.service.status.v3.PerXdsConfig.listener_config:type_name -> envoy.admin.v3.ListenersConfigDump
	10, // 5: envoy.service.status.v3.PerXdsConfig.cluster_config:type_name -> envoy.admin.v3.ClustersConfigDump
	11, // 6: envoy.service.status.v3.PerXdsConfig.route_config:type_name -> envoy.admin.v3.RoutesConfigDump
	12, // 7: envoy.service.status.v3.PerXdsConfig.scoped_route_config:type_name -> envoy.admin.v3.ScopedRoutesConfigDump
	13, // 8: envoy.service.status.v3.PerXdsConfig.endpoint_config:type_name -> envoy.admin.v3.EndpointsConfigDump
	8,  // 9: envoy.service.status.v3.ClientConfig.node:type_name -> envoy.config.core.v3.Node
	3,  // 10: envoy.service.status.v3.ClientConfig.xds_config:type_name -> envoy.service.status.v3.PerXdsConfig
	6,  // 11: envoy.service.status.v3.ClientConfig.generic_xds_configs:type_name -> envoy.service.status.v3.ClientConfig.GenericXdsConfig
	4,  // 12: envoy.service.status.v3.ClientStatusResponse.config:type_name -> envoy.service.status.v3.ClientConfig
	14, // 13: envoy.service.status.v3.ClientConfig.GenericXdsConfig.xds_config:type_name -> google.protobuf.Any
	15, // 14: envoy.service.status.v3.ClientConfig.GenericXdsConfig.last_updated:type_name -> google.protobuf.Timestamp
	0,  // 15: envoy.service.status.v3.ClientConfig.GenericXdsConfig.config_status:type_name -> envoy.service.status.v3.ConfigStatus
	16, // 16: envoy.service.status.v3.ClientConfig.GenericXdsConfig.client_status:type_name -> envoy.admin.v3.ClientResourceStatus
	17, // 17: envoy.service.status.v3.ClientConfig.GenericXdsConfig.error_state:type_name -> envoy.admin.v3.UpdateFailureState
	2,  // 18: envoy.service.status.v3.ClientStatusDiscoveryService.StreamClientStatus:input_type -> envoy.service.status.v3.ClientStatusRequest
	2,  // 19: envoy.service.status.v3.ClientStatusDiscoveryService.FetchClientStatus:input_type -> envoy.service.status.v3.ClientStatusRequest
	5,  // 20: envoy.service.status.v3.ClientStatusDiscoveryService.StreamClientStatus:output_type -> envoy.service.status.v3.ClientStatusResponse
	5,  // 21: envoy.service.status.v3.ClientStatusDiscoveryService.FetchClientStatus:output_type -> envoy.service.status.v3.ClientStatusResponse
	20, // [20:22] is the sub-list for method output_type
	18, // [18:20] is the sub-list for method input_type
	18, // [18:18] is the sub-list for extension type_name
	18, // [18:18] is the sub-list for extension extendee
	0,  // [0:18] is the sub-list for field type_name
}

func init() { file_envoy_service_status_v3_csds_proto_init() }
func file_envoy_service_status_v3_csds_proto_init() {
	if File_envoy_service_status_v3_csds_proto != nil {
		return
	}
	file_envoy_service_status_v3_csds_proto_msgTypes[1].OneofWrappers = []any{
		(*PerXdsConfig_ListenerConfig)(nil),
		(*PerXdsConfig_ClusterConfig)(nil),
		(*PerXdsConfig_RouteConfig)(nil),
		(*PerXdsConfig_ScopedRouteConfig)(nil),
		(*PerXdsConfig_EndpointConfig)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_service_status_v3_csds_proto_rawDesc), len(file_envoy_service_status_v3_csds_proto_rawDesc)),
			NumEnums:      2,
			NumMessages:   5,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_envoy_service_status_v3_csds_proto_goTypes,
		DependencyIndexes: file_envoy_service_status_v3_csds_proto_depIdxs,
		EnumInfos:         file_envoy_service_status_v3_csds_proto_enumTypes,
		MessageInfos:      file_envoy_service_status_v3_csds_proto_msgTypes,
	}.Build()
	File_envoy_service_status_v3_csds_proto = out.File
	file_envoy_service_status_v3_csds_proto_goTypes = nil
	file_envoy_service_status_v3_csds_proto_depIdxs = nil
}
