// Copyright 2026 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package storage

// trackedFeature represents a specific client feature being tracked, represented
// as a bit in a bitmask. Each feature corresponds to a specific bit position.
type trackedFeature uint32

const (
	featureMultistreamInMRD trackedFeature = 0
	featurePCU              trackedFeature = 1
)
