// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.1
// source: xds/type/matcher/v3/regex.proto

package v3

import (
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type RegexMatcher struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to EngineType:
	//
	//	*RegexMatcher_GoogleRe2
	EngineType    isRegexMatcher_EngineType `protobuf_oneof:"engine_type"`
	Regex         string                    `protobuf:"bytes,2,opt,name=regex,proto3" json:"regex,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RegexMatcher) Reset() {
	*x = RegexMatcher{}
	mi := &file_xds_type_matcher_v3_regex_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RegexMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RegexMatcher) ProtoMessage() {}

func (x *RegexMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_regex_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RegexMatcher.ProtoReflect.Descriptor instead.
func (*RegexMatcher) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_regex_proto_rawDescGZIP(), []int{0}
}

func (x *RegexMatcher) GetEngineType() isRegexMatcher_EngineType {
	if x != nil {
		return x.EngineType
	}
	return nil
}

func (x *RegexMatcher) GetGoogleRe2() *RegexMatcher_GoogleRE2 {
	if x != nil {
		if x, ok := x.EngineType.(*RegexMatcher_GoogleRe2); ok {
			return x.GoogleRe2
		}
	}
	return nil
}

func (x *RegexMatcher) GetRegex() string {
	if x != nil {
		return x.Regex
	}
	return ""
}

type isRegexMatcher_EngineType interface {
	isRegexMatcher_EngineType()
}

type RegexMatcher_GoogleRe2 struct {
	GoogleRe2 *RegexMatcher_GoogleRE2 `protobuf:"bytes,1,opt,name=google_re2,json=googleRe2,proto3,oneof"`
}

func (*RegexMatcher_GoogleRe2) isRegexMatcher_EngineType() {}

type RegexMatcher_GoogleRE2 struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RegexMatcher_GoogleRE2) Reset() {
	*x = RegexMatcher_GoogleRE2{}
	mi := &file_xds_type_matcher_v3_regex_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RegexMatcher_GoogleRE2) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RegexMatcher_GoogleRE2) ProtoMessage() {}

func (x *RegexMatcher_GoogleRE2) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_regex_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RegexMatcher_GoogleRE2.ProtoReflect.Descriptor instead.
func (*RegexMatcher_GoogleRE2) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_regex_proto_rawDescGZIP(), []int{0, 0}
}

var File_xds_type_matcher_v3_regex_proto protoreflect.FileDescriptor

const file_xds_type_matcher_v3_regex_proto_rawDesc = "" +
	"\n" +
	"\x1fxds/type/matcher/v3/regex.proto\x12\x13xds.type.matcher.v3\x1a\x17validate/validate.proto\"\xa6\x01\n" +
	"\fRegexMatcher\x12V\n" +
	"\n" +
	"google_re2\x18\x01 \x01(\v2+.xds.type.matcher.v3.RegexMatcher.GoogleRE2B\b\xfaB\x05\x8a\x01\x02\x10\x01H\x00R\tgoogleRe2\x12\x1d\n" +
	"\x05regex\x18\x02 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x05regex\x1a\v\n" +
	"\tGoogleRE2B\x12\n" +
	"\vengine_type\x12\x03\xf8B\x01BZ\n" +
	"\x1ecom.github.xds.type.matcher.v3B\n" +
	"RegexProtoP\x01Z*github.com/cncf/xds/go/xds/type/matcher/v3b\x06proto3"

var (
	file_xds_type_matcher_v3_regex_proto_rawDescOnce sync.Once
	file_xds_type_matcher_v3_regex_proto_rawDescData []byte
)

func file_xds_type_matcher_v3_regex_proto_rawDescGZIP() []byte {
	file_xds_type_matcher_v3_regex_proto_rawDescOnce.Do(func() {
		file_xds_type_matcher_v3_regex_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_xds_type_matcher_v3_regex_proto_rawDesc), len(file_xds_type_matcher_v3_regex_proto_rawDesc)))
	})
	return file_xds_type_matcher_v3_regex_proto_rawDescData
}

var file_xds_type_matcher_v3_regex_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_xds_type_matcher_v3_regex_proto_goTypes = []any{
	(*RegexMatcher)(nil),           // 0: xds.type.matcher.v3.RegexMatcher
	(*RegexMatcher_GoogleRE2)(nil), // 1: xds.type.matcher.v3.RegexMatcher.GoogleRE2
}
var file_xds_type_matcher_v3_regex_proto_depIdxs = []int32{
	1, // 0: xds.type.matcher.v3.RegexMatcher.google_re2:type_name -> xds.type.matcher.v3.RegexMatcher.GoogleRE2
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_xds_type_matcher_v3_regex_proto_init() }
func file_xds_type_matcher_v3_regex_proto_init() {
	if File_xds_type_matcher_v3_regex_proto != nil {
		return
	}
	file_xds_type_matcher_v3_regex_proto_msgTypes[0].OneofWrappers = []any{
		(*RegexMatcher_GoogleRe2)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_xds_type_matcher_v3_regex_proto_rawDesc), len(file_xds_type_matcher_v3_regex_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_xds_type_matcher_v3_regex_proto_goTypes,
		DependencyIndexes: file_xds_type_matcher_v3_regex_proto_depIdxs,
		MessageInfos:      file_xds_type_matcher_v3_regex_proto_msgTypes,
	}.Build()
	File_xds_type_matcher_v3_regex_proto = out.File
	file_xds_type_matcher_v3_regex_proto_goTypes = nil
	file_xds_type_matcher_v3_regex_proto_depIdxs = nil
}
