// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/core/v3/backoff.proto

package corev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration defining a jittered exponential back off strategy.
type BackoffStrategy struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The base interval to be used for the next back off computation. It should
	// be greater than zero and less than or equal to :ref:`max_interval
	// <envoy_v3_api_field_config.core.v3.BackoffStrategy.max_interval>`.
	BaseInterval *durationpb.Duration `protobuf:"bytes,1,opt,name=base_interval,json=baseInterval,proto3" json:"base_interval,omitempty"`
	// Specifies the maximum interval between retries. This parameter is optional,
	// but must be greater than or equal to the :ref:`base_interval
	// <envoy_v3_api_field_config.core.v3.BackoffStrategy.base_interval>` if set. The default
	// is 10 times the :ref:`base_interval
	// <envoy_v3_api_field_config.core.v3.BackoffStrategy.base_interval>`.
	MaxInterval   *durationpb.Duration `protobuf:"bytes,2,opt,name=max_interval,json=maxInterval,proto3" json:"max_interval,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *BackoffStrategy) Reset() {
	*x = BackoffStrategy{}
	mi := &file_envoy_config_core_v3_backoff_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *BackoffStrategy) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*BackoffStrategy) ProtoMessage() {}

func (x *BackoffStrategy) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_core_v3_backoff_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use BackoffStrategy.ProtoReflect.Descriptor instead.
func (*BackoffStrategy) Descriptor() ([]byte, []int) {
	return file_envoy_config_core_v3_backoff_proto_rawDescGZIP(), []int{0}
}

func (x *BackoffStrategy) GetBaseInterval() *durationpb.Duration {
	if x != nil {
		return x.BaseInterval
	}
	return nil
}

func (x *BackoffStrategy) GetMaxInterval() *durationpb.Duration {
	if x != nil {
		return x.MaxInterval
	}
	return nil
}

var File_envoy_config_core_v3_backoff_proto protoreflect.FileDescriptor

const file_envoy_config_core_v3_backoff_proto_rawDesc = "" +
	"\n" +
	"\"envoy/config/core/v3/backoff.proto\x12\x14envoy.config.core.v3\x1a\x1egoogle/protobuf/duration.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xd3\x01\n" +
	"\x0fBackoffStrategy\x12N\n" +
	"\rbase_interval\x18\x01 \x01(\v2\x19.google.protobuf.DurationB\x0e\xfaB\v\xaa\x01\b\b\x012\x04\x10\xc0\x84=R\fbaseInterval\x12F\n" +
	"\fmax_interval\x18\x02 \x01(\v2\x19.google.protobuf.DurationB\b\xfaB\x05\xaa\x01\x02*\x00R\vmaxInterval:(\x9aň\x1e#\n" +
	"!envoy.api.v2.core.BackoffStrategyB\x80\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"\"io.envoyproxy.envoy.config.core.v3B\fBackoffProtoP\x01ZBgithub.com/envoyproxy/go-control-plane/envoy/config/core/v3;corev3b\x06proto3"

var (
	file_envoy_config_core_v3_backoff_proto_rawDescOnce sync.Once
	file_envoy_config_core_v3_backoff_proto_rawDescData []byte
)

func file_envoy_config_core_v3_backoff_proto_rawDescGZIP() []byte {
	file_envoy_config_core_v3_backoff_proto_rawDescOnce.Do(func() {
		file_envoy_config_core_v3_backoff_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_core_v3_backoff_proto_rawDesc), len(file_envoy_config_core_v3_backoff_proto_rawDesc)))
	})
	return file_envoy_config_core_v3_backoff_proto_rawDescData
}

var file_envoy_config_core_v3_backoff_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_config_core_v3_backoff_proto_goTypes = []any{
	(*BackoffStrategy)(nil),     // 0: envoy.config.core.v3.BackoffStrategy
	(*durationpb.Duration)(nil), // 1: google.protobuf.Duration
}
var file_envoy_config_core_v3_backoff_proto_depIdxs = []int32{
	1, // 0: envoy.config.core.v3.BackoffStrategy.base_interval:type_name -> google.protobuf.Duration
	1, // 1: envoy.config.core.v3.BackoffStrategy.max_interval:type_name -> google.protobuf.Duration
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_config_core_v3_backoff_proto_init() }
func file_envoy_config_core_v3_backoff_proto_init() {
	if File_envoy_config_core_v3_backoff_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_core_v3_backoff_proto_rawDesc), len(file_envoy_config_core_v3_backoff_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_core_v3_backoff_proto_goTypes,
		DependencyIndexes: file_envoy_config_core_v3_backoff_proto_depIdxs,
		MessageInfos:      file_envoy_config_core_v3_backoff_proto_msgTypes,
	}.Build()
	File_envoy_config_core_v3_backoff_proto = out.File
	file_envoy_config_core_v3_backoff_proto_goTypes = nil
	file_envoy_config_core_v3_backoff_proto_depIdxs = nil
}
