// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/type/matcher/v3/path.proto

package matcherv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Specifies the way to match a path on HTTP request.
type PathMatcher struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to Rule:
	//
	//	*PathMatcher_Path
	Rule          isPathMatcher_Rule `protobuf_oneof:"rule"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PathMatcher) Reset() {
	*x = PathMatcher{}
	mi := &file_envoy_type_matcher_v3_path_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PathMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PathMatcher) ProtoMessage() {}

func (x *PathMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_type_matcher_v3_path_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PathMatcher.ProtoReflect.Descriptor instead.
func (*PathMatcher) Descriptor() ([]byte, []int) {
	return file_envoy_type_matcher_v3_path_proto_rawDescGZIP(), []int{0}
}

func (x *PathMatcher) GetRule() isPathMatcher_Rule {
	if x != nil {
		return x.Rule
	}
	return nil
}

func (x *PathMatcher) GetPath() *StringMatcher {
	if x != nil {
		if x, ok := x.Rule.(*PathMatcher_Path); ok {
			return x.Path
		}
	}
	return nil
}

type isPathMatcher_Rule interface {
	isPathMatcher_Rule()
}

type PathMatcher_Path struct {
	// The “path“ must match the URL path portion of the :path header. The query and fragment
	// string (if present) are removed in the URL path portion.
	// For example, the path “/data“ will match the “:path“ header “/data#fragment?param=value“.
	Path *StringMatcher `protobuf:"bytes,1,opt,name=path,proto3,oneof"`
}

func (*PathMatcher_Path) isPathMatcher_Rule() {}

var File_envoy_type_matcher_v3_path_proto protoreflect.FileDescriptor

const file_envoy_type_matcher_v3_path_proto_rawDesc = "" +
	"\n" +
	" envoy/type/matcher/v3/path.proto\x12\x15envoy.type.matcher.v3\x1a\"envoy/type/matcher/v3/string.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\x87\x01\n" +
	"\vPathMatcher\x12D\n" +
	"\x04path\x18\x01 \x01(\v2$.envoy.type.matcher.v3.StringMatcherB\b\xfaB\x05\x8a\x01\x02\x10\x01H\x00R\x04path:%\x9aň\x1e \n" +
	"\x1eenvoy.type.matcher.PathMatcherB\v\n" +
	"\x04rule\x12\x03\xf8B\x01B\x82\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"#io.envoyproxy.envoy.type.matcher.v3B\tPathProtoP\x01ZFgithub.com/envoyproxy/go-control-plane/envoy/type/matcher/v3;matcherv3b\x06proto3"

var (
	file_envoy_type_matcher_v3_path_proto_rawDescOnce sync.Once
	file_envoy_type_matcher_v3_path_proto_rawDescData []byte
)

func file_envoy_type_matcher_v3_path_proto_rawDescGZIP() []byte {
	file_envoy_type_matcher_v3_path_proto_rawDescOnce.Do(func() {
		file_envoy_type_matcher_v3_path_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_type_matcher_v3_path_proto_rawDesc), len(file_envoy_type_matcher_v3_path_proto_rawDesc)))
	})
	return file_envoy_type_matcher_v3_path_proto_rawDescData
}

var file_envoy_type_matcher_v3_path_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_type_matcher_v3_path_proto_goTypes = []any{
	(*PathMatcher)(nil),   // 0: envoy.type.matcher.v3.PathMatcher
	(*StringMatcher)(nil), // 1: envoy.type.matcher.v3.StringMatcher
}
var file_envoy_type_matcher_v3_path_proto_depIdxs = []int32{
	1, // 0: envoy.type.matcher.v3.PathMatcher.path:type_name -> envoy.type.matcher.v3.StringMatcher
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_type_matcher_v3_path_proto_init() }
func file_envoy_type_matcher_v3_path_proto_init() {
	if File_envoy_type_matcher_v3_path_proto != nil {
		return
	}
	file_envoy_type_matcher_v3_string_proto_init()
	file_envoy_type_matcher_v3_path_proto_msgTypes[0].OneofWrappers = []any{
		(*PathMatcher_Path)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_type_matcher_v3_path_proto_rawDesc), len(file_envoy_type_matcher_v3_path_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_type_matcher_v3_path_proto_goTypes,
		DependencyIndexes: file_envoy_type_matcher_v3_path_proto_depIdxs,
		MessageInfos:      file_envoy_type_matcher_v3_path_proto_msgTypes,
	}.Build()
	File_envoy_type_matcher_v3_path_proto = out.File
	file_envoy_type_matcher_v3_path_proto_goTypes = nil
	file_envoy_type_matcher_v3_path_proto_depIdxs = nil
}
