// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.31.1
// source: v1alpha1/meta.proto

// Meta package defines protobuf serialization of standard COSI resources from the 'meta' namespace.

package v1alpha1

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type ResourceDefinitionSpec_Sensitivity int32

const (
	ResourceDefinitionSpec_NON_SENSITIVE ResourceDefinitionSpec_Sensitivity = 0
	ResourceDefinitionSpec_SENSITIVE     ResourceDefinitionSpec_Sensitivity = 1
)

// Enum value maps for ResourceDefinitionSpec_Sensitivity.
var (
	ResourceDefinitionSpec_Sensitivity_name = map[int32]string{
		0: "NON_SENSITIVE",
		1: "SENSITIVE",
	}
	ResourceDefinitionSpec_Sensitivity_value = map[string]int32{
		"NON_SENSITIVE": 0,
		"SENSITIVE":     1,
	}
)

func (x ResourceDefinitionSpec_Sensitivity) Enum() *ResourceDefinitionSpec_Sensitivity {
	p := new(ResourceDefinitionSpec_Sensitivity)
	*p = x
	return p
}

func (x ResourceDefinitionSpec_Sensitivity) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ResourceDefinitionSpec_Sensitivity) Descriptor() protoreflect.EnumDescriptor {
	return file_v1alpha1_meta_proto_enumTypes[0].Descriptor()
}

func (ResourceDefinitionSpec_Sensitivity) Type() protoreflect.EnumType {
	return &file_v1alpha1_meta_proto_enumTypes[0]
}

func (x ResourceDefinitionSpec_Sensitivity) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ResourceDefinitionSpec_Sensitivity.Descriptor instead.
func (ResourceDefinitionSpec_Sensitivity) EnumDescriptor() ([]byte, []int) {
	return file_v1alpha1_meta_proto_rawDescGZIP(), []int{1, 0}
}

// NamespaceSpec is the protobuf serialization of the Namespace resource.
type NamespaceSpec struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Description of the namespace.
	Description   string `protobuf:"bytes,1,opt,name=description,proto3" json:"description,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *NamespaceSpec) Reset() {
	*x = NamespaceSpec{}
	mi := &file_v1alpha1_meta_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *NamespaceSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*NamespaceSpec) ProtoMessage() {}

func (x *NamespaceSpec) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_meta_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use NamespaceSpec.ProtoReflect.Descriptor instead.
func (*NamespaceSpec) Descriptor() ([]byte, []int) {
	return file_v1alpha1_meta_proto_rawDescGZIP(), []int{0}
}

func (x *NamespaceSpec) GetDescription() string {
	if x != nil {
		return x.Description
	}
	return ""
}

// ResourceDefinitionSpec is the protobuf serialization of the ResourceDefinition resource.
type ResourceDefinitionSpec struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Canonical type name.
	ResourceType string `protobuf:"bytes,1,opt,name=resource_type,json=resourceType,proto3" json:"resource_type,omitempty"`
	// Displayed human-readable type name.
	DisplayType string `protobuf:"bytes,2,opt,name=display_type,json=displayType,proto3" json:"display_type,omitempty"`
	// Default namespace to look for the resource if no namespace is given.
	DefaultNamespace string `protobuf:"bytes,3,opt,name=default_namespace,json=defaultNamespace,proto3" json:"default_namespace,omitempty"`
	// Human-readable aliases.
	Aliases []string `protobuf:"bytes,4,rep,name=aliases,proto3" json:"aliases,omitempty"`
	// All aliases for automatic matching.
	AllAliases []string `protobuf:"bytes,5,rep,name=all_aliases,json=allAliases,proto3" json:"all_aliases,omitempty"`
	// Additional columns to print in table output.
	PrintColumns []*ResourceDefinitionSpec_PrintColumn `protobuf:"bytes,6,rep,name=print_columns,json=printColumns,proto3" json:"print_columns,omitempty"`
	// Sensitivity indicates how secret resource of this type is.
	// The empty value represents a non-sensitive resource.
	Sensitivity   ResourceDefinitionSpec_Sensitivity `protobuf:"varint,7,opt,name=sensitivity,proto3,enum=cosi.resource.meta.ResourceDefinitionSpec_Sensitivity" json:"sensitivity,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ResourceDefinitionSpec) Reset() {
	*x = ResourceDefinitionSpec{}
	mi := &file_v1alpha1_meta_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ResourceDefinitionSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ResourceDefinitionSpec) ProtoMessage() {}

func (x *ResourceDefinitionSpec) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_meta_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ResourceDefinitionSpec.ProtoReflect.Descriptor instead.
func (*ResourceDefinitionSpec) Descriptor() ([]byte, []int) {
	return file_v1alpha1_meta_proto_rawDescGZIP(), []int{1}
}

func (x *ResourceDefinitionSpec) GetResourceType() string {
	if x != nil {
		return x.ResourceType
	}
	return ""
}

func (x *ResourceDefinitionSpec) GetDisplayType() string {
	if x != nil {
		return x.DisplayType
	}
	return ""
}

func (x *ResourceDefinitionSpec) GetDefaultNamespace() string {
	if x != nil {
		return x.DefaultNamespace
	}
	return ""
}

func (x *ResourceDefinitionSpec) GetAliases() []string {
	if x != nil {
		return x.Aliases
	}
	return nil
}

func (x *ResourceDefinitionSpec) GetAllAliases() []string {
	if x != nil {
		return x.AllAliases
	}
	return nil
}

func (x *ResourceDefinitionSpec) GetPrintColumns() []*ResourceDefinitionSpec_PrintColumn {
	if x != nil {
		return x.PrintColumns
	}
	return nil
}

func (x *ResourceDefinitionSpec) GetSensitivity() ResourceDefinitionSpec_Sensitivity {
	if x != nil {
		return x.Sensitivity
	}
	return ResourceDefinitionSpec_NON_SENSITIVE
}

type ResourceDefinitionSpec_PrintColumn struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Name          string                 `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	JsonPath      string                 `protobuf:"bytes,2,opt,name=json_path,json=jsonPath,proto3" json:"json_path,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ResourceDefinitionSpec_PrintColumn) Reset() {
	*x = ResourceDefinitionSpec_PrintColumn{}
	mi := &file_v1alpha1_meta_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ResourceDefinitionSpec_PrintColumn) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ResourceDefinitionSpec_PrintColumn) ProtoMessage() {}

func (x *ResourceDefinitionSpec_PrintColumn) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_meta_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ResourceDefinitionSpec_PrintColumn.ProtoReflect.Descriptor instead.
func (*ResourceDefinitionSpec_PrintColumn) Descriptor() ([]byte, []int) {
	return file_v1alpha1_meta_proto_rawDescGZIP(), []int{1, 0}
}

func (x *ResourceDefinitionSpec_PrintColumn) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *ResourceDefinitionSpec_PrintColumn) GetJsonPath() string {
	if x != nil {
		return x.JsonPath
	}
	return ""
}

var File_v1alpha1_meta_proto protoreflect.FileDescriptor

const file_v1alpha1_meta_proto_rawDesc = "" +
	"\n" +
	"\x13v1alpha1/meta.proto\x12\x12cosi.resource.meta\"1\n" +
	"\rNamespaceSpec\x12 \n" +
	"\vdescription\x18\x01 \x01(\tR\vdescription\"\xf0\x03\n" +
	"\x16ResourceDefinitionSpec\x12#\n" +
	"\rresource_type\x18\x01 \x01(\tR\fresourceType\x12!\n" +
	"\fdisplay_type\x18\x02 \x01(\tR\vdisplayType\x12+\n" +
	"\x11default_namespace\x18\x03 \x01(\tR\x10defaultNamespace\x12\x18\n" +
	"\aaliases\x18\x04 \x03(\tR\aaliases\x12\x1f\n" +
	"\vall_aliases\x18\x05 \x03(\tR\n" +
	"allAliases\x12[\n" +
	"\rprint_columns\x18\x06 \x03(\v26.cosi.resource.meta.ResourceDefinitionSpec.PrintColumnR\fprintColumns\x12X\n" +
	"\vsensitivity\x18\a \x01(\x0e26.cosi.resource.meta.ResourceDefinitionSpec.SensitivityR\vsensitivity\x1a>\n" +
	"\vPrintColumn\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12\x1b\n" +
	"\tjson_path\x18\x02 \x01(\tR\bjsonPath\"/\n" +
	"\vSensitivity\x12\x11\n" +
	"\rNON_SENSITIVE\x10\x00\x12\r\n" +
	"\tSENSITIVE\x10\x01B.Z,github.com/cosi-project/runtime/api/v1alpha1b\x06proto3"

var (
	file_v1alpha1_meta_proto_rawDescOnce sync.Once
	file_v1alpha1_meta_proto_rawDescData []byte
)

func file_v1alpha1_meta_proto_rawDescGZIP() []byte {
	file_v1alpha1_meta_proto_rawDescOnce.Do(func() {
		file_v1alpha1_meta_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_v1alpha1_meta_proto_rawDesc), len(file_v1alpha1_meta_proto_rawDesc)))
	})
	return file_v1alpha1_meta_proto_rawDescData
}

var file_v1alpha1_meta_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_v1alpha1_meta_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_v1alpha1_meta_proto_goTypes = []any{
	(ResourceDefinitionSpec_Sensitivity)(0),    // 0: cosi.resource.meta.ResourceDefinitionSpec.Sensitivity
	(*NamespaceSpec)(nil),                      // 1: cosi.resource.meta.NamespaceSpec
	(*ResourceDefinitionSpec)(nil),             // 2: cosi.resource.meta.ResourceDefinitionSpec
	(*ResourceDefinitionSpec_PrintColumn)(nil), // 3: cosi.resource.meta.ResourceDefinitionSpec.PrintColumn
}
var file_v1alpha1_meta_proto_depIdxs = []int32{
	3, // 0: cosi.resource.meta.ResourceDefinitionSpec.print_columns:type_name -> cosi.resource.meta.ResourceDefinitionSpec.PrintColumn
	0, // 1: cosi.resource.meta.ResourceDefinitionSpec.sensitivity:type_name -> cosi.resource.meta.ResourceDefinitionSpec.Sensitivity
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_v1alpha1_meta_proto_init() }
func file_v1alpha1_meta_proto_init() {
	if File_v1alpha1_meta_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_v1alpha1_meta_proto_rawDesc), len(file_v1alpha1_meta_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_v1alpha1_meta_proto_goTypes,
		DependencyIndexes: file_v1alpha1_meta_proto_depIdxs,
		EnumInfos:         file_v1alpha1_meta_proto_enumTypes,
		MessageInfos:      file_v1alpha1_meta_proto_msgTypes,
	}.Build()
	File_v1alpha1_meta_proto = out.File
	file_v1alpha1_meta_proto_goTypes = nil
	file_v1alpha1_meta_proto_depIdxs = nil
}
