// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/trace/v3/skywalking.proto

package tracev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the SkyWalking tracer. Please note that if SkyWalking tracer is used as the
// provider of tracing, then
// :ref:`spawn_upstream_span <envoy_v3_api_field_extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.Tracing.spawn_upstream_span>`
// in the tracing config must be set to true to get the correct topology and tracing data. Moreover, SkyWalking
// Tracer does not support SkyWalking extension header (“sw8-x“) temporarily.
// [#extension: envoy.tracers.skywalking]
type SkyWalkingConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// SkyWalking collector service.
	GrpcService   *v3.GrpcService `protobuf:"bytes,1,opt,name=grpc_service,json=grpcService,proto3" json:"grpc_service,omitempty"`
	ClientConfig  *ClientConfig   `protobuf:"bytes,2,opt,name=client_config,json=clientConfig,proto3" json:"client_config,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *SkyWalkingConfig) Reset() {
	*x = SkyWalkingConfig{}
	mi := &file_envoy_config_trace_v3_skywalking_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SkyWalkingConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SkyWalkingConfig) ProtoMessage() {}

func (x *SkyWalkingConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_trace_v3_skywalking_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SkyWalkingConfig.ProtoReflect.Descriptor instead.
func (*SkyWalkingConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_trace_v3_skywalking_proto_rawDescGZIP(), []int{0}
}

func (x *SkyWalkingConfig) GetGrpcService() *v3.GrpcService {
	if x != nil {
		return x.GrpcService
	}
	return nil
}

func (x *SkyWalkingConfig) GetClientConfig() *ClientConfig {
	if x != nil {
		return x.ClientConfig
	}
	return nil
}

// Client config for SkyWalking tracer.
type ClientConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Service name for SkyWalking tracer. If this field is empty, then local service cluster name
	// that configured by :ref:`Bootstrap node <envoy_v3_api_field_config.bootstrap.v3.Bootstrap.node>`
	// message's :ref:`cluster <envoy_v3_api_field_config.core.v3.Node.cluster>` field or command line
	// option :option:`--service-cluster` will be used. If both this field and local service cluster
	// name are empty, “EnvoyProxy“ is used as the service name by default.
	ServiceName string `protobuf:"bytes,1,opt,name=service_name,json=serviceName,proto3" json:"service_name,omitempty"`
	// Service instance name for SkyWalking tracer. If this field is empty, then local service node
	// that configured by :ref:`Bootstrap node <envoy_v3_api_field_config.bootstrap.v3.Bootstrap.node>`
	// message's :ref:`id <envoy_v3_api_field_config.core.v3.Node.id>` field or command line  option
	// :option:`--service-node` will be used. If both this field and local service node are empty,
	// “EnvoyProxy“ is used as the instance name by default.
	InstanceName string `protobuf:"bytes,2,opt,name=instance_name,json=instanceName,proto3" json:"instance_name,omitempty"`
	// Authentication token config for SkyWalking. SkyWalking can use token authentication to secure
	// that monitoring application data can be trusted. In current version, Token is considered as a
	// simple string.
	// [#comment:TODO(wbpcode): Get backend token through the SDS API.]
	//
	// Types that are valid to be assigned to BackendTokenSpecifier:
	//
	//	*ClientConfig_BackendToken
	BackendTokenSpecifier isClientConfig_BackendTokenSpecifier `protobuf_oneof:"backend_token_specifier"`
	// Envoy caches the segment in memory when the SkyWalking backend service is temporarily unavailable.
	// This field specifies the maximum number of segments that can be cached. If not specified, the
	// default is 1024.
	MaxCacheSize  *wrapperspb.UInt32Value `protobuf:"bytes,4,opt,name=max_cache_size,json=maxCacheSize,proto3" json:"max_cache_size,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ClientConfig) Reset() {
	*x = ClientConfig{}
	mi := &file_envoy_config_trace_v3_skywalking_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClientConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClientConfig) ProtoMessage() {}

func (x *ClientConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_trace_v3_skywalking_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClientConfig.ProtoReflect.Descriptor instead.
func (*ClientConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_trace_v3_skywalking_proto_rawDescGZIP(), []int{1}
}

func (x *ClientConfig) GetServiceName() string {
	if x != nil {
		return x.ServiceName
	}
	return ""
}

func (x *ClientConfig) GetInstanceName() string {
	if x != nil {
		return x.InstanceName
	}
	return ""
}

func (x *ClientConfig) GetBackendTokenSpecifier() isClientConfig_BackendTokenSpecifier {
	if x != nil {
		return x.BackendTokenSpecifier
	}
	return nil
}

func (x *ClientConfig) GetBackendToken() string {
	if x != nil {
		if x, ok := x.BackendTokenSpecifier.(*ClientConfig_BackendToken); ok {
			return x.BackendToken
		}
	}
	return ""
}

func (x *ClientConfig) GetMaxCacheSize() *wrapperspb.UInt32Value {
	if x != nil {
		return x.MaxCacheSize
	}
	return nil
}

type isClientConfig_BackendTokenSpecifier interface {
	isClientConfig_BackendTokenSpecifier()
}

type ClientConfig_BackendToken struct {
	// Inline authentication token string.
	BackendToken string `protobuf:"bytes,3,opt,name=backend_token,json=backendToken,proto3,oneof"`
}

func (*ClientConfig_BackendToken) isClientConfig_BackendTokenSpecifier() {}

var File_envoy_config_trace_v3_skywalking_proto protoreflect.FileDescriptor

const file_envoy_config_trace_v3_skywalking_proto_rawDesc = "" +
	"\n" +
	"&envoy/config/trace/v3/skywalking.proto\x12\x15envoy.config.trace.v3\x1a'envoy/config/core/v3/grpc_service.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1eudpa/annotations/migrate.proto\x1a udpa/annotations/sensitive.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xac\x01\n" +
	"\x10SkyWalkingConfig\x12N\n" +
	"\fgrpc_service\x18\x01 \x01(\v2!.envoy.config.core.v3.GrpcServiceB\b\xfaB\x05\x8a\x01\x02\x10\x01R\vgrpcService\x12H\n" +
	"\rclient_config\x18\x02 \x01(\v2#.envoy.config.trace.v3.ClientConfigR\fclientConfig\"\xe4\x01\n" +
	"\fClientConfig\x12!\n" +
	"\fservice_name\x18\x01 \x01(\tR\vserviceName\x12#\n" +
	"\rinstance_name\x18\x02 \x01(\tR\finstanceName\x12-\n" +
	"\rbackend_token\x18\x03 \x01(\tB\x06\xb8\xb7\x8b\xa4\x02\x01H\x00R\fbackendToken\x12B\n" +
	"\x0emax_cache_size\x18\x04 \x01(\v2\x1c.google.protobuf.UInt32ValueR\fmaxCacheSizeB\x19\n" +
	"\x17backend_token_specifierB\xb9\x01\xf2\x98\xfe\x8f\x05-\x12+envoy.extensions.tracers.skywalking.v4alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"#io.envoyproxy.envoy.config.trace.v3B\x0fSkywalkingProtoP\x01ZDgithub.com/envoyproxy/go-control-plane/envoy/config/trace/v3;tracev3b\x06proto3"

var (
	file_envoy_config_trace_v3_skywalking_proto_rawDescOnce sync.Once
	file_envoy_config_trace_v3_skywalking_proto_rawDescData []byte
)

func file_envoy_config_trace_v3_skywalking_proto_rawDescGZIP() []byte {
	file_envoy_config_trace_v3_skywalking_proto_rawDescOnce.Do(func() {
		file_envoy_config_trace_v3_skywalking_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_trace_v3_skywalking_proto_rawDesc), len(file_envoy_config_trace_v3_skywalking_proto_rawDesc)))
	})
	return file_envoy_config_trace_v3_skywalking_proto_rawDescData
}

var file_envoy_config_trace_v3_skywalking_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_config_trace_v3_skywalking_proto_goTypes = []any{
	(*SkyWalkingConfig)(nil),       // 0: envoy.config.trace.v3.SkyWalkingConfig
	(*ClientConfig)(nil),           // 1: envoy.config.trace.v3.ClientConfig
	(*v3.GrpcService)(nil),         // 2: envoy.config.core.v3.GrpcService
	(*wrapperspb.UInt32Value)(nil), // 3: google.protobuf.UInt32Value
}
var file_envoy_config_trace_v3_skywalking_proto_depIdxs = []int32{
	2, // 0: envoy.config.trace.v3.SkyWalkingConfig.grpc_service:type_name -> envoy.config.core.v3.GrpcService
	1, // 1: envoy.config.trace.v3.SkyWalkingConfig.client_config:type_name -> envoy.config.trace.v3.ClientConfig
	3, // 2: envoy.config.trace.v3.ClientConfig.max_cache_size:type_name -> google.protobuf.UInt32Value
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() { file_envoy_config_trace_v3_skywalking_proto_init() }
func file_envoy_config_trace_v3_skywalking_proto_init() {
	if File_envoy_config_trace_v3_skywalking_proto != nil {
		return
	}
	file_envoy_config_trace_v3_skywalking_proto_msgTypes[1].OneofWrappers = []any{
		(*ClientConfig_BackendToken)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_trace_v3_skywalking_proto_rawDesc), len(file_envoy_config_trace_v3_skywalking_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_trace_v3_skywalking_proto_goTypes,
		DependencyIndexes: file_envoy_config_trace_v3_skywalking_proto_depIdxs,
		MessageInfos:      file_envoy_config_trace_v3_skywalking_proto_msgTypes,
	}.Build()
	File_envoy_config_trace_v3_skywalking_proto = out.File
	file_envoy_config_trace_v3_skywalking_proto_goTypes = nil
	file_envoy_config_trace_v3_skywalking_proto_depIdxs = nil
}
