// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/type/v3/semantic_version.proto

package typev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Envoy uses SemVer (https://semver.org/). Major/minor versions indicate
// expected behaviors and APIs, the patch version field is used only
// for security fixes and can be generally ignored.
type SemanticVersion struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	MajorNumber   uint32                 `protobuf:"varint,1,opt,name=major_number,json=majorNumber,proto3" json:"major_number,omitempty"`
	MinorNumber   uint32                 `protobuf:"varint,2,opt,name=minor_number,json=minorNumber,proto3" json:"minor_number,omitempty"`
	Patch         uint32                 `protobuf:"varint,3,opt,name=patch,proto3" json:"patch,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *SemanticVersion) Reset() {
	*x = SemanticVersion{}
	mi := &file_envoy_type_v3_semantic_version_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SemanticVersion) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SemanticVersion) ProtoMessage() {}

func (x *SemanticVersion) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_type_v3_semantic_version_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SemanticVersion.ProtoReflect.Descriptor instead.
func (*SemanticVersion) Descriptor() ([]byte, []int) {
	return file_envoy_type_v3_semantic_version_proto_rawDescGZIP(), []int{0}
}

func (x *SemanticVersion) GetMajorNumber() uint32 {
	if x != nil {
		return x.MajorNumber
	}
	return 0
}

func (x *SemanticVersion) GetMinorNumber() uint32 {
	if x != nil {
		return x.MinorNumber
	}
	return 0
}

func (x *SemanticVersion) GetPatch() uint32 {
	if x != nil {
		return x.Patch
	}
	return 0
}

var File_envoy_type_v3_semantic_version_proto protoreflect.FileDescriptor

const file_envoy_type_v3_semantic_version_proto_rawDesc = "" +
	"\n" +
	"$envoy/type/v3/semantic_version.proto\x12\renvoy.type.v3\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"\x90\x01\n" +
	"\x0fSemanticVersion\x12!\n" +
	"\fmajor_number\x18\x01 \x01(\rR\vmajorNumber\x12!\n" +
	"\fminor_number\x18\x02 \x01(\rR\vminorNumber\x12\x14\n" +
	"\x05patch\x18\x03 \x01(\rR\x05patch:!\x9aň\x1e\x1c\n" +
	"\x1aenvoy.type.SemanticVersionBz\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"\x1bio.envoyproxy.envoy.type.v3B\x14SemanticVersionProtoP\x01Z;github.com/envoyproxy/go-control-plane/envoy/type/v3;typev3b\x06proto3"

var (
	file_envoy_type_v3_semantic_version_proto_rawDescOnce sync.Once
	file_envoy_type_v3_semantic_version_proto_rawDescData []byte
)

func file_envoy_type_v3_semantic_version_proto_rawDescGZIP() []byte {
	file_envoy_type_v3_semantic_version_proto_rawDescOnce.Do(func() {
		file_envoy_type_v3_semantic_version_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_type_v3_semantic_version_proto_rawDesc), len(file_envoy_type_v3_semantic_version_proto_rawDesc)))
	})
	return file_envoy_type_v3_semantic_version_proto_rawDescData
}

var file_envoy_type_v3_semantic_version_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_type_v3_semantic_version_proto_goTypes = []any{
	(*SemanticVersion)(nil), // 0: envoy.type.v3.SemanticVersion
}
var file_envoy_type_v3_semantic_version_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_envoy_type_v3_semantic_version_proto_init() }
func file_envoy_type_v3_semantic_version_proto_init() {
	if File_envoy_type_v3_semantic_version_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_type_v3_semantic_version_proto_rawDesc), len(file_envoy_type_v3_semantic_version_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_type_v3_semantic_version_proto_goTypes,
		DependencyIndexes: file_envoy_type_v3_semantic_version_proto_depIdxs,
		MessageInfos:      file_envoy_type_v3_semantic_version_proto_msgTypes,
	}.Build()
	File_envoy_type_v3_semantic_version_proto = out.File
	file_envoy_type_v3_semantic_version_proto_goTypes = nil
	file_envoy_type_v3_semantic_version_proto_depIdxs = nil
}
