// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/listener/v3/api_listener.proto

package listenerv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	anypb "google.golang.org/protobuf/types/known/anypb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Describes a type of API listener, which is used in non-proxy clients. The type of API
// exposed to the non-proxy application depends on the type of API listener.
type ApiListener struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The type in this field determines the type of API listener. At present, the following
	// types are supported:
	// envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager (HTTP)
	// envoy.extensions.filters.network.http_connection_manager.v3.EnvoyMobileHttpConnectionManager (HTTP)
	// [#next-major-version: In the v3 API, replace this Any field with a oneof containing the
	// specific config message for each type of API listener. We could not do this in v2 because
	// it would have caused circular dependencies for go protos: lds.proto depends on this file,
	// and http_connection_manager.proto depends on rds.proto, which is in the same directory as
	// lds.proto, so lds.proto cannot depend on this file.]
	ApiListener   *anypb.Any `protobuf:"bytes,1,opt,name=api_listener,json=apiListener,proto3" json:"api_listener,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ApiListener) Reset() {
	*x = ApiListener{}
	mi := &file_envoy_config_listener_v3_api_listener_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ApiListener) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ApiListener) ProtoMessage() {}

func (x *ApiListener) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_listener_v3_api_listener_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ApiListener.ProtoReflect.Descriptor instead.
func (*ApiListener) Descriptor() ([]byte, []int) {
	return file_envoy_config_listener_v3_api_listener_proto_rawDescGZIP(), []int{0}
}

func (x *ApiListener) GetApiListener() *anypb.Any {
	if x != nil {
		return x.ApiListener
	}
	return nil
}

var File_envoy_config_listener_v3_api_listener_proto protoreflect.FileDescriptor

const file_envoy_config_listener_v3_api_listener_proto_rawDesc = "" +
	"\n" +
	"+envoy/config/listener/v3/api_listener.proto\x12\x18envoy.config.listener.v3\x1a\x19google/protobuf/any.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"s\n" +
	"\vApiListener\x127\n" +
	"\fapi_listener\x18\x01 \x01(\v2\x14.google.protobuf.AnyR\vapiListener:+\x9aň\x1e&\n" +
	"$envoy.config.listener.v2.ApiListenerB\x90\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"&io.envoyproxy.envoy.config.listener.v3B\x10ApiListenerProtoP\x01ZJgithub.com/envoyproxy/go-control-plane/envoy/config/listener/v3;listenerv3b\x06proto3"

var (
	file_envoy_config_listener_v3_api_listener_proto_rawDescOnce sync.Once
	file_envoy_config_listener_v3_api_listener_proto_rawDescData []byte
)

func file_envoy_config_listener_v3_api_listener_proto_rawDescGZIP() []byte {
	file_envoy_config_listener_v3_api_listener_proto_rawDescOnce.Do(func() {
		file_envoy_config_listener_v3_api_listener_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_listener_v3_api_listener_proto_rawDesc), len(file_envoy_config_listener_v3_api_listener_proto_rawDesc)))
	})
	return file_envoy_config_listener_v3_api_listener_proto_rawDescData
}

var file_envoy_config_listener_v3_api_listener_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_config_listener_v3_api_listener_proto_goTypes = []any{
	(*ApiListener)(nil), // 0: envoy.config.listener.v3.ApiListener
	(*anypb.Any)(nil),   // 1: google.protobuf.Any
}
var file_envoy_config_listener_v3_api_listener_proto_depIdxs = []int32{
	1, // 0: envoy.config.listener.v3.ApiListener.api_listener:type_name -> google.protobuf.Any
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_config_listener_v3_api_listener_proto_init() }
func file_envoy_config_listener_v3_api_listener_proto_init() {
	if File_envoy_config_listener_v3_api_listener_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_listener_v3_api_listener_proto_rawDesc), len(file_envoy_config_listener_v3_api_listener_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_listener_v3_api_listener_proto_goTypes,
		DependencyIndexes: file_envoy_config_listener_v3_api_listener_proto_depIdxs,
		MessageInfos:      file_envoy_config_listener_v3_api_listener_proto_msgTypes,
	}.Build()
	File_envoy_config_listener_v3_api_listener_proto = out.File
	file_envoy_config_listener_v3_api_listener_proto_goTypes = nil
	file_envoy_config_listener_v3_api_listener_proto_depIdxs = nil
}
