// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11-devel
// 	protoc        (unknown)
// source: machine/lvm.proto

package machine

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	emptypb "google.golang.org/protobuf/types/known/emptypb"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// LVMServiceLogicalVolumeRemoveRequest identifies a single LV to remove.
type LVMServiceLogicalVolumeRemoveRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// VolumeGroup is the name of the parent VG (e.g. "vg0").
	VolumeGroup string `protobuf:"bytes,1,opt,name=volume_group,json=volumeGroup,proto3" json:"volume_group,omitempty"`
	// LogicalVolume is the name of the LV to remove (e.g. "lv0").
	LogicalVolume string `protobuf:"bytes,2,opt,name=logical_volume,json=logicalVolume,proto3" json:"logical_volume,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *LVMServiceLogicalVolumeRemoveRequest) Reset() {
	*x = LVMServiceLogicalVolumeRemoveRequest{}
	mi := &file_machine_lvm_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *LVMServiceLogicalVolumeRemoveRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*LVMServiceLogicalVolumeRemoveRequest) ProtoMessage() {}

func (x *LVMServiceLogicalVolumeRemoveRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_lvm_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use LVMServiceLogicalVolumeRemoveRequest.ProtoReflect.Descriptor instead.
func (*LVMServiceLogicalVolumeRemoveRequest) Descriptor() ([]byte, []int) {
	return file_machine_lvm_proto_rawDescGZIP(), []int{0}
}

func (x *LVMServiceLogicalVolumeRemoveRequest) GetVolumeGroup() string {
	if x != nil {
		return x.VolumeGroup
	}
	return ""
}

func (x *LVMServiceLogicalVolumeRemoveRequest) GetLogicalVolume() string {
	if x != nil {
		return x.LogicalVolume
	}
	return ""
}

// LVMServiceVolumeGroupRemoveRequest identifies a single VG to remove.
type LVMServiceVolumeGroupRemoveRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// VolumeGroup is the name of the VG to remove (e.g. "vg0").
	VolumeGroup   string `protobuf:"bytes,1,opt,name=volume_group,json=volumeGroup,proto3" json:"volume_group,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *LVMServiceVolumeGroupRemoveRequest) Reset() {
	*x = LVMServiceVolumeGroupRemoveRequest{}
	mi := &file_machine_lvm_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *LVMServiceVolumeGroupRemoveRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*LVMServiceVolumeGroupRemoveRequest) ProtoMessage() {}

func (x *LVMServiceVolumeGroupRemoveRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_lvm_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use LVMServiceVolumeGroupRemoveRequest.ProtoReflect.Descriptor instead.
func (*LVMServiceVolumeGroupRemoveRequest) Descriptor() ([]byte, []int) {
	return file_machine_lvm_proto_rawDescGZIP(), []int{1}
}

func (x *LVMServiceVolumeGroupRemoveRequest) GetVolumeGroup() string {
	if x != nil {
		return x.VolumeGroup
	}
	return ""
}

// LVMServicePhysicalVolumeRemoveRequest identifies a single PV label to wipe.
type LVMServicePhysicalVolumeRemoveRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Device is the block device path of the PV (e.g. "/dev/sda1").
	Device        string `protobuf:"bytes,1,opt,name=device,proto3" json:"device,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *LVMServicePhysicalVolumeRemoveRequest) Reset() {
	*x = LVMServicePhysicalVolumeRemoveRequest{}
	mi := &file_machine_lvm_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *LVMServicePhysicalVolumeRemoveRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*LVMServicePhysicalVolumeRemoveRequest) ProtoMessage() {}

func (x *LVMServicePhysicalVolumeRemoveRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_lvm_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use LVMServicePhysicalVolumeRemoveRequest.ProtoReflect.Descriptor instead.
func (*LVMServicePhysicalVolumeRemoveRequest) Descriptor() ([]byte, []int) {
	return file_machine_lvm_proto_rawDescGZIP(), []int{2}
}

func (x *LVMServicePhysicalVolumeRemoveRequest) GetDevice() string {
	if x != nil {
		return x.Device
	}
	return ""
}

var File_machine_lvm_proto protoreflect.FileDescriptor

const file_machine_lvm_proto_rawDesc = "" +
	"\n" +
	"\x11machine/lvm.proto\x12\amachine\x1a\x1bgoogle/protobuf/empty.proto\"p\n" +
	"$LVMServiceLogicalVolumeRemoveRequest\x12!\n" +
	"\fvolume_group\x18\x01 \x01(\tR\vvolumeGroup\x12%\n" +
	"\x0elogical_volume\x18\x02 \x01(\tR\rlogicalVolume\"G\n" +
	"\"LVMServiceVolumeGroupRemoveRequest\x12!\n" +
	"\fvolume_group\x18\x01 \x01(\tR\vvolumeGroup\"?\n" +
	"%LVMServicePhysicalVolumeRemoveRequest\x12\x16\n" +
	"\x06device\x18\x01 \x01(\tR\x06device2\xa4\x02\n" +
	"\n" +
	"LVMService\x12\\\n" +
	"\x13LogicalVolumeRemove\x12-.machine.LVMServiceLogicalVolumeRemoveRequest\x1a\x16.google.protobuf.Empty\x12X\n" +
	"\x11VolumeGroupRemove\x12+.machine.LVMServiceVolumeGroupRemoveRequest\x1a\x16.google.protobuf.Empty\x12^\n" +
	"\x14PhysicalVolumeRemove\x12..machine.LVMServicePhysicalVolumeRemoveRequest\x1a\x16.google.protobuf.EmptyBN\n" +
	"\x15dev.talos.api.machineZ5github.com/siderolabs/talos/pkg/machinery/api/machineb\x06proto3"

var (
	file_machine_lvm_proto_rawDescOnce sync.Once
	file_machine_lvm_proto_rawDescData []byte
)

func file_machine_lvm_proto_rawDescGZIP() []byte {
	file_machine_lvm_proto_rawDescOnce.Do(func() {
		file_machine_lvm_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_machine_lvm_proto_rawDesc), len(file_machine_lvm_proto_rawDesc)))
	})
	return file_machine_lvm_proto_rawDescData
}

var file_machine_lvm_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_machine_lvm_proto_goTypes = []any{
	(*LVMServiceLogicalVolumeRemoveRequest)(nil),  // 0: machine.LVMServiceLogicalVolumeRemoveRequest
	(*LVMServiceVolumeGroupRemoveRequest)(nil),    // 1: machine.LVMServiceVolumeGroupRemoveRequest
	(*LVMServicePhysicalVolumeRemoveRequest)(nil), // 2: machine.LVMServicePhysicalVolumeRemoveRequest
	(*emptypb.Empty)(nil),                         // 3: google.protobuf.Empty
}
var file_machine_lvm_proto_depIdxs = []int32{
	0, // 0: machine.LVMService.LogicalVolumeRemove:input_type -> machine.LVMServiceLogicalVolumeRemoveRequest
	1, // 1: machine.LVMService.VolumeGroupRemove:input_type -> machine.LVMServiceVolumeGroupRemoveRequest
	2, // 2: machine.LVMService.PhysicalVolumeRemove:input_type -> machine.LVMServicePhysicalVolumeRemoveRequest
	3, // 3: machine.LVMService.LogicalVolumeRemove:output_type -> google.protobuf.Empty
	3, // 4: machine.LVMService.VolumeGroupRemove:output_type -> google.protobuf.Empty
	3, // 5: machine.LVMService.PhysicalVolumeRemove:output_type -> google.protobuf.Empty
	3, // [3:6] is the sub-list for method output_type
	0, // [0:3] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_machine_lvm_proto_init() }
func file_machine_lvm_proto_init() {
	if File_machine_lvm_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_machine_lvm_proto_rawDesc), len(file_machine_lvm_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_machine_lvm_proto_goTypes,
		DependencyIndexes: file_machine_lvm_proto_depIdxs,
		MessageInfos:      file_machine_lvm_proto_msgTypes,
	}.Build()
	File_machine_lvm_proto = out.File
	file_machine_lvm_proto_goTypes = nil
	file_machine_lvm_proto_depIdxs = nil
}
