// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

// Code generated by "deep-copy -type ImageVerificationConfigV1Alpha1 -type TrustedRootsConfigV1Alpha1 -pointer-receiver -header-file ../../../../../hack/boilerplate.txt -o deep_copy.generated.go ."; DO NOT EDIT.

package security

// DeepCopy generates a deep copy of *ImageVerificationConfigV1Alpha1.
func (o *ImageVerificationConfigV1Alpha1) DeepCopy() *ImageVerificationConfigV1Alpha1 {
	var cp ImageVerificationConfigV1Alpha1 = *o
	if o.ConfigRules != nil {
		cp.ConfigRules = make([]ImageVerificationRuleV1Alpha1, len(o.ConfigRules))
		copy(cp.ConfigRules, o.ConfigRules)
		for i2 := range o.ConfigRules {
			if o.ConfigRules[i2].RuleSkip != nil {
				cp.ConfigRules[i2].RuleSkip = new(bool)
				*cp.ConfigRules[i2].RuleSkip = *o.ConfigRules[i2].RuleSkip
			}
			if o.ConfigRules[i2].RuleDeny != nil {
				cp.ConfigRules[i2].RuleDeny = new(bool)
				*cp.ConfigRules[i2].RuleDeny = *o.ConfigRules[i2].RuleDeny
			}
			if o.ConfigRules[i2].RuleKeylessVerifier != nil {
				cp.ConfigRules[i2].RuleKeylessVerifier = new(ImageKeylessVerifierV1Alpha1)
				*cp.ConfigRules[i2].RuleKeylessVerifier = *o.ConfigRules[i2].RuleKeylessVerifier
			}
			if o.ConfigRules[i2].RulePublicKeyVerifier != nil {
				cp.ConfigRules[i2].RulePublicKeyVerifier = new(ImagePublicKeyVerifierV1Alpha1)
				*cp.ConfigRules[i2].RulePublicKeyVerifier = *o.ConfigRules[i2].RulePublicKeyVerifier
			}
		}
	}
	return &cp
}

// DeepCopy generates a deep copy of *TrustedRootsConfigV1Alpha1.
func (o *TrustedRootsConfigV1Alpha1) DeepCopy() *TrustedRootsConfigV1Alpha1 {
	var cp TrustedRootsConfigV1Alpha1 = *o
	return &cp
}
