// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/transport_sockets/tls/v3/cert.proto

package tlsv3

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

var File_envoy_extensions_transport_sockets_tls_v3_cert_proto protoreflect.FileDescriptor

const file_envoy_extensions_transport_sockets_tls_v3_cert_proto_rawDesc = "" +
	"\n" +
	"4envoy/extensions/transport_sockets/tls/v3/cert.proto\x12)envoy.extensions.transport_sockets.tls.v3\x1a6envoy/extensions/transport_sockets/tls/v3/common.proto\x1a6envoy/extensions/transport_sockets/tls/v3/secret.proto\x1a3envoy/extensions/transport_sockets/tls/v3/tls.protoB\x9e\x01\n" +
	"7io.envoyproxy.envoy.extensions.transport_sockets.tls.v3B\tCertProtoP\x01ZVgithub.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/tls/v3;tlsv3P\x00P\x01P\x02b\x06proto3"

var file_envoy_extensions_transport_sockets_tls_v3_cert_proto_goTypes = []any{}
var file_envoy_extensions_transport_sockets_tls_v3_cert_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_envoy_extensions_transport_sockets_tls_v3_cert_proto_init() }
func file_envoy_extensions_transport_sockets_tls_v3_cert_proto_init() {
	if File_envoy_extensions_transport_sockets_tls_v3_cert_proto != nil {
		return
	}
	file_envoy_extensions_transport_sockets_tls_v3_common_proto_init()
	file_envoy_extensions_transport_sockets_tls_v3_secret_proto_init()
	file_envoy_extensions_transport_sockets_tls_v3_tls_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_tls_v3_cert_proto_rawDesc), len(file_envoy_extensions_transport_sockets_tls_v3_cert_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   0,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_transport_sockets_tls_v3_cert_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_transport_sockets_tls_v3_cert_proto_depIdxs,
	}.Build()
	File_envoy_extensions_transport_sockets_tls_v3_cert_proto = out.File
	file_envoy_extensions_transport_sockets_tls_v3_cert_proto_goTypes = nil
	file_envoy_extensions_transport_sockets_tls_v3_cert_proto_depIdxs = nil
}
