// Copyright 2026 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.33.2
// source: google/cloud/kms/v1/autokey.proto

package kmspb

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	longrunningpb "cloud.google.com/go/longrunning/autogen/longrunningpb"
	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Request message for
// [Autokey.CreateKeyHandle][google.cloud.kms.v1.Autokey.CreateKeyHandle].
type CreateKeyHandleRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Name of the resource project and location to create the
	// [KeyHandle][google.cloud.kms.v1.KeyHandle] in, e.g.
	// `projects/{PROJECT_ID}/locations/{LOCATION}`.
	Parent string `protobuf:"bytes,1,opt,name=parent,proto3" json:"parent,omitempty"`
	// Optional. Id of the [KeyHandle][google.cloud.kms.v1.KeyHandle]. Must be
	// unique to the resource project and location. If not provided by the caller,
	// a new UUID is used.
	KeyHandleId string `protobuf:"bytes,2,opt,name=key_handle_id,json=keyHandleId,proto3" json:"key_handle_id,omitempty"`
	// Required. [KeyHandle][google.cloud.kms.v1.KeyHandle] to create.
	KeyHandle     *KeyHandle `protobuf:"bytes,3,opt,name=key_handle,json=keyHandle,proto3" json:"key_handle,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateKeyHandleRequest) Reset() {
	*x = CreateKeyHandleRequest{}
	mi := &file_google_cloud_kms_v1_autokey_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateKeyHandleRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateKeyHandleRequest) ProtoMessage() {}

func (x *CreateKeyHandleRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_kms_v1_autokey_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateKeyHandleRequest.ProtoReflect.Descriptor instead.
func (*CreateKeyHandleRequest) Descriptor() ([]byte, []int) {
	return file_google_cloud_kms_v1_autokey_proto_rawDescGZIP(), []int{0}
}

func (x *CreateKeyHandleRequest) GetParent() string {
	if x != nil {
		return x.Parent
	}
	return ""
}

func (x *CreateKeyHandleRequest) GetKeyHandleId() string {
	if x != nil {
		return x.KeyHandleId
	}
	return ""
}

func (x *CreateKeyHandleRequest) GetKeyHandle() *KeyHandle {
	if x != nil {
		return x.KeyHandle
	}
	return nil
}

// Request message for [GetKeyHandle][google.cloud.kms.v1.Autokey.GetKeyHandle].
type GetKeyHandleRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Name of the [KeyHandle][google.cloud.kms.v1.KeyHandle] resource,
	// e.g.
	// `projects/{PROJECT_ID}/locations/{LOCATION}/keyHandles/{KEY_HANDLE_ID}`.
	Name          string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetKeyHandleRequest) Reset() {
	*x = GetKeyHandleRequest{}
	mi := &file_google_cloud_kms_v1_autokey_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetKeyHandleRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetKeyHandleRequest) ProtoMessage() {}

func (x *GetKeyHandleRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_kms_v1_autokey_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetKeyHandleRequest.ProtoReflect.Descriptor instead.
func (*GetKeyHandleRequest) Descriptor() ([]byte, []int) {
	return file_google_cloud_kms_v1_autokey_proto_rawDescGZIP(), []int{1}
}

func (x *GetKeyHandleRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

// Resource-oriented representation of a request to Cloud KMS Autokey and the
// resulting provisioning of a [CryptoKey][google.cloud.kms.v1.CryptoKey].
type KeyHandle struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Identifier. Name of the [KeyHandle][google.cloud.kms.v1.KeyHandle]
	// resource, e.g.
	// `projects/{PROJECT_ID}/locations/{LOCATION}/keyHandles/{KEY_HANDLE_ID}`.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// Output only. Name of a [CryptoKey][google.cloud.kms.v1.CryptoKey] that has
	// been provisioned for Customer Managed Encryption Key (CMEK) use in the
	// [KeyHandle][google.cloud.kms.v1.KeyHandle] project and location for the
	// requested resource type. The [CryptoKey][google.cloud.kms.v1.CryptoKey]
	// project will reflect the value configured in the
	// [AutokeyConfig][google.cloud.kms.v1.AutokeyConfig] on the resource
	// project's ancestor folder at the time of the
	// [KeyHandle][google.cloud.kms.v1.KeyHandle] creation. If more than one
	// ancestor folder has a configured
	// [AutokeyConfig][google.cloud.kms.v1.AutokeyConfig], the nearest of these
	// configurations is used.
	KmsKey string `protobuf:"bytes,3,opt,name=kms_key,json=kmsKey,proto3" json:"kms_key,omitempty"`
	// Required. Indicates the resource type that the resulting
	// [CryptoKey][google.cloud.kms.v1.CryptoKey] is meant to protect, e.g.
	// `{SERVICE}.googleapis.com/{TYPE}`. See documentation for supported resource
	// types.
	ResourceTypeSelector string `protobuf:"bytes,4,opt,name=resource_type_selector,json=resourceTypeSelector,proto3" json:"resource_type_selector,omitempty"`
	unknownFields        protoimpl.UnknownFields
	sizeCache            protoimpl.SizeCache
}

func (x *KeyHandle) Reset() {
	*x = KeyHandle{}
	mi := &file_google_cloud_kms_v1_autokey_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *KeyHandle) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*KeyHandle) ProtoMessage() {}

func (x *KeyHandle) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_kms_v1_autokey_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use KeyHandle.ProtoReflect.Descriptor instead.
func (*KeyHandle) Descriptor() ([]byte, []int) {
	return file_google_cloud_kms_v1_autokey_proto_rawDescGZIP(), []int{2}
}

func (x *KeyHandle) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *KeyHandle) GetKmsKey() string {
	if x != nil {
		return x.KmsKey
	}
	return ""
}

func (x *KeyHandle) GetResourceTypeSelector() string {
	if x != nil {
		return x.ResourceTypeSelector
	}
	return ""
}

// Metadata message for
// [CreateKeyHandle][google.cloud.kms.v1.Autokey.CreateKeyHandle] long-running
// operation response.
type CreateKeyHandleMetadata struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateKeyHandleMetadata) Reset() {
	*x = CreateKeyHandleMetadata{}
	mi := &file_google_cloud_kms_v1_autokey_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateKeyHandleMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateKeyHandleMetadata) ProtoMessage() {}

func (x *CreateKeyHandleMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_kms_v1_autokey_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateKeyHandleMetadata.ProtoReflect.Descriptor instead.
func (*CreateKeyHandleMetadata) Descriptor() ([]byte, []int) {
	return file_google_cloud_kms_v1_autokey_proto_rawDescGZIP(), []int{3}
}

// Request message for
// [Autokey.ListKeyHandles][google.cloud.kms.v1.Autokey.ListKeyHandles].
type ListKeyHandlesRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Name of the resource project and location from which to list
	// [KeyHandles][google.cloud.kms.v1.KeyHandle], e.g.
	// `projects/{PROJECT_ID}/locations/{LOCATION}`.
	Parent string `protobuf:"bytes,1,opt,name=parent,proto3" json:"parent,omitempty"`
	// Optional. Optional limit on the number of
	// [KeyHandles][google.cloud.kms.v1.KeyHandle] to include in the response. The
	// service may return fewer than this value. Further
	// [KeyHandles][google.cloud.kms.v1.KeyHandle] can subsequently be obtained by
	// including the
	// [ListKeyHandlesResponse.next_page_token][google.cloud.kms.v1.ListKeyHandlesResponse.next_page_token]
	// in a subsequent request.  If unspecified, at most 100
	// [KeyHandles][google.cloud.kms.v1.KeyHandle] will be returned.
	PageSize int32 `protobuf:"varint,2,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// Optional. Optional pagination token, returned earlier via
	// [ListKeyHandlesResponse.next_page_token][google.cloud.kms.v1.ListKeyHandlesResponse.next_page_token].
	PageToken string `protobuf:"bytes,3,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	// Optional. Filter to apply when listing
	// [KeyHandles][google.cloud.kms.v1.KeyHandle], e.g.
	// `resource_type_selector="{SERVICE}.googleapis.com/{TYPE}"`.
	Filter        string `protobuf:"bytes,4,opt,name=filter,proto3" json:"filter,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListKeyHandlesRequest) Reset() {
	*x = ListKeyHandlesRequest{}
	mi := &file_google_cloud_kms_v1_autokey_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListKeyHandlesRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListKeyHandlesRequest) ProtoMessage() {}

func (x *ListKeyHandlesRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_kms_v1_autokey_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListKeyHandlesRequest.ProtoReflect.Descriptor instead.
func (*ListKeyHandlesRequest) Descriptor() ([]byte, []int) {
	return file_google_cloud_kms_v1_autokey_proto_rawDescGZIP(), []int{4}
}

func (x *ListKeyHandlesRequest) GetParent() string {
	if x != nil {
		return x.Parent
	}
	return ""
}

func (x *ListKeyHandlesRequest) GetPageSize() int32 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListKeyHandlesRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

func (x *ListKeyHandlesRequest) GetFilter() string {
	if x != nil {
		return x.Filter
	}
	return ""
}

// Response message for
// [Autokey.ListKeyHandles][google.cloud.kms.v1.Autokey.ListKeyHandles].
type ListKeyHandlesResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Resulting [KeyHandles][google.cloud.kms.v1.KeyHandle].
	KeyHandles []*KeyHandle `protobuf:"bytes,1,rep,name=key_handles,json=keyHandles,proto3" json:"key_handles,omitempty"`
	// A token to retrieve next page of results. Pass this value in
	// [ListKeyHandlesRequest.page_token][google.cloud.kms.v1.ListKeyHandlesRequest.page_token]
	// to retrieve the next page of results.
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListKeyHandlesResponse) Reset() {
	*x = ListKeyHandlesResponse{}
	mi := &file_google_cloud_kms_v1_autokey_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListKeyHandlesResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListKeyHandlesResponse) ProtoMessage() {}

func (x *ListKeyHandlesResponse) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_kms_v1_autokey_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListKeyHandlesResponse.ProtoReflect.Descriptor instead.
func (*ListKeyHandlesResponse) Descriptor() ([]byte, []int) {
	return file_google_cloud_kms_v1_autokey_proto_rawDescGZIP(), []int{5}
}

func (x *ListKeyHandlesResponse) GetKeyHandles() []*KeyHandle {
	if x != nil {
		return x.KeyHandles
	}
	return nil
}

func (x *ListKeyHandlesResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

var File_google_cloud_kms_v1_autokey_proto protoreflect.FileDescriptor

const file_google_cloud_kms_v1_autokey_proto_rawDesc = "" +
	"\n" +
	"!google/cloud/kms/v1/autokey.proto\x12\x13google.cloud.kms.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x17google/api/client.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a#google/longrunning/operations.proto\"\xc8\x01\n" +
	"\x16CreateKeyHandleRequest\x12A\n" +
	"\x06parent\x18\x01 \x01(\tB)\xe0A\x02\xfaA#\n" +
	"!locations.googleapis.com/LocationR\x06parent\x12'\n" +
	"\rkey_handle_id\x18\x02 \x01(\tB\x03\xe0A\x01R\vkeyHandleId\x12B\n" +
	"\n" +
	"key_handle\x18\x03 \x01(\v2\x1e.google.cloud.kms.v1.KeyHandleB\x03\xe0A\x02R\tkeyHandle\"T\n" +
	"\x13GetKeyHandleRequest\x12=\n" +
	"\x04name\x18\x01 \x01(\tB)\xe0A\x02\xfaA#\n" +
	"!cloudkms.googleapis.com/KeyHandleR\x04name\"\xa3\x02\n" +
	"\tKeyHandle\x12\x17\n" +
	"\x04name\x18\x01 \x01(\tB\x03\xe0A\bR\x04name\x12B\n" +
	"\akms_key\x18\x03 \x01(\tB)\xe0A\x03\xfaA#\n" +
	"!cloudkms.googleapis.com/CryptoKeyR\x06kmsKey\x129\n" +
	"\x16resource_type_selector\x18\x04 \x01(\tB\x03\xe0A\x02R\x14resourceTypeSelector:~\xeaA{\n" +
	"!cloudkms.googleapis.com/KeyHandle\x12?projects/{project}/locations/{location}/keyHandles/{key_handle}*\n" +
	"keyHandles2\tkeyHandle\"\x19\n" +
	"\x17CreateKeyHandleMetadata\"\xbd\x01\n" +
	"\x15ListKeyHandlesRequest\x12A\n" +
	"\x06parent\x18\x01 \x01(\tB)\xe0A\x02\xfaA#\n" +
	"!locations.googleapis.com/LocationR\x06parent\x12 \n" +
	"\tpage_size\x18\x02 \x01(\x05B\x03\xe0A\x01R\bpageSize\x12\"\n" +
	"\n" +
	"page_token\x18\x03 \x01(\tB\x03\xe0A\x01R\tpageToken\x12\x1b\n" +
	"\x06filter\x18\x04 \x01(\tB\x03\xe0A\x01R\x06filter\"\x81\x01\n" +
	"\x16ListKeyHandlesResponse\x12?\n" +
	"\vkey_handles\x18\x01 \x03(\v2\x1e.google.cloud.kms.v1.KeyHandleR\n" +
	"keyHandles\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\xb4\x05\n" +
	"\aAutokey\x12\xeb\x01\n" +
	"\x0fCreateKeyHandle\x12+.google.cloud.kms.v1.CreateKeyHandleRequest\x1a\x1d.google.longrunning.Operation\"\x8b\x01\xcaA$\n" +
	"\tKeyHandle\x12\x17CreateKeyHandleMetadata\xdaA\x1fparent,key_handle,key_handle_id\x82\xd3\xe4\x93\x02<:\n" +
	"key_handle\"./v1/{parent=projects/*/locations/*}/keyHandles\x12\x97\x01\n" +
	"\fGetKeyHandle\x12(.google.cloud.kms.v1.GetKeyHandleRequest\x1a\x1e.google.cloud.kms.v1.KeyHandle\"=\xdaA\x04name\x82\xd3\xe4\x93\x020\x12./v1/{name=projects/*/locations/*/keyHandles/*}\x12\xaa\x01\n" +
	"\x0eListKeyHandles\x12*.google.cloud.kms.v1.ListKeyHandlesRequest\x1a+.google.cloud.kms.v1.ListKeyHandlesResponse\"?\xdaA\x06parent\x82\xd3\xe4\x93\x020\x12./v1/{parent=projects/*/locations/*}/keyHandles\x1at\xcaA\x17cloudkms.googleapis.com\xd2AWhttps://www.googleapis.com/auth/cloud-platform,https://www.googleapis.com/auth/cloudkmsBT\n" +
	"\x17com.google.cloud.kms.v1B\fAutokeyProtoP\x01Z)cloud.google.com/go/kms/apiv1/kmspb;kmspbb\x06proto3"

var (
	file_google_cloud_kms_v1_autokey_proto_rawDescOnce sync.Once
	file_google_cloud_kms_v1_autokey_proto_rawDescData []byte
)

func file_google_cloud_kms_v1_autokey_proto_rawDescGZIP() []byte {
	file_google_cloud_kms_v1_autokey_proto_rawDescOnce.Do(func() {
		file_google_cloud_kms_v1_autokey_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_google_cloud_kms_v1_autokey_proto_rawDesc), len(file_google_cloud_kms_v1_autokey_proto_rawDesc)))
	})
	return file_google_cloud_kms_v1_autokey_proto_rawDescData
}

var file_google_cloud_kms_v1_autokey_proto_msgTypes = make([]protoimpl.MessageInfo, 6)
var file_google_cloud_kms_v1_autokey_proto_goTypes = []any{
	(*CreateKeyHandleRequest)(nil),  // 0: google.cloud.kms.v1.CreateKeyHandleRequest
	(*GetKeyHandleRequest)(nil),     // 1: google.cloud.kms.v1.GetKeyHandleRequest
	(*KeyHandle)(nil),               // 2: google.cloud.kms.v1.KeyHandle
	(*CreateKeyHandleMetadata)(nil), // 3: google.cloud.kms.v1.CreateKeyHandleMetadata
	(*ListKeyHandlesRequest)(nil),   // 4: google.cloud.kms.v1.ListKeyHandlesRequest
	(*ListKeyHandlesResponse)(nil),  // 5: google.cloud.kms.v1.ListKeyHandlesResponse
	(*longrunningpb.Operation)(nil), // 6: google.longrunning.Operation
}
var file_google_cloud_kms_v1_autokey_proto_depIdxs = []int32{
	2, // 0: google.cloud.kms.v1.CreateKeyHandleRequest.key_handle:type_name -> google.cloud.kms.v1.KeyHandle
	2, // 1: google.cloud.kms.v1.ListKeyHandlesResponse.key_handles:type_name -> google.cloud.kms.v1.KeyHandle
	0, // 2: google.cloud.kms.v1.Autokey.CreateKeyHandle:input_type -> google.cloud.kms.v1.CreateKeyHandleRequest
	1, // 3: google.cloud.kms.v1.Autokey.GetKeyHandle:input_type -> google.cloud.kms.v1.GetKeyHandleRequest
	4, // 4: google.cloud.kms.v1.Autokey.ListKeyHandles:input_type -> google.cloud.kms.v1.ListKeyHandlesRequest
	6, // 5: google.cloud.kms.v1.Autokey.CreateKeyHandle:output_type -> google.longrunning.Operation
	2, // 6: google.cloud.kms.v1.Autokey.GetKeyHandle:output_type -> google.cloud.kms.v1.KeyHandle
	5, // 7: google.cloud.kms.v1.Autokey.ListKeyHandles:output_type -> google.cloud.kms.v1.ListKeyHandlesResponse
	5, // [5:8] is the sub-list for method output_type
	2, // [2:5] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_google_cloud_kms_v1_autokey_proto_init() }
func file_google_cloud_kms_v1_autokey_proto_init() {
	if File_google_cloud_kms_v1_autokey_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_cloud_kms_v1_autokey_proto_rawDesc), len(file_google_cloud_kms_v1_autokey_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   6,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_google_cloud_kms_v1_autokey_proto_goTypes,
		DependencyIndexes: file_google_cloud_kms_v1_autokey_proto_depIdxs,
		MessageInfos:      file_google_cloud_kms_v1_autokey_proto_msgTypes,
	}.Build()
	File_google_cloud_kms_v1_autokey_proto = out.File
	file_google_cloud_kms_v1_autokey_proto_goTypes = nil
	file_google_cloud_kms_v1_autokey_proto_depIdxs = nil
}
