// Copyright 2025 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.31.0
// source: google/monitoring/v3/mutation_record.proto

package monitoringpb

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Describes a change made to a configuration.
type MutationRecord struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// When the change occurred.
	MutateTime *timestamppb.Timestamp `protobuf:"bytes,1,opt,name=mutate_time,json=mutateTime,proto3" json:"mutate_time,omitempty"`
	// The email address of the user making the change.
	MutatedBy     string `protobuf:"bytes,2,opt,name=mutated_by,json=mutatedBy,proto3" json:"mutated_by,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MutationRecord) Reset() {
	*x = MutationRecord{}
	mi := &file_google_monitoring_v3_mutation_record_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MutationRecord) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MutationRecord) ProtoMessage() {}

func (x *MutationRecord) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_mutation_record_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MutationRecord.ProtoReflect.Descriptor instead.
func (*MutationRecord) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_mutation_record_proto_rawDescGZIP(), []int{0}
}

func (x *MutationRecord) GetMutateTime() *timestamppb.Timestamp {
	if x != nil {
		return x.MutateTime
	}
	return nil
}

func (x *MutationRecord) GetMutatedBy() string {
	if x != nil {
		return x.MutatedBy
	}
	return ""
}

var File_google_monitoring_v3_mutation_record_proto protoreflect.FileDescriptor

const file_google_monitoring_v3_mutation_record_proto_rawDesc = "" +
	"\n" +
	"*google/monitoring/v3/mutation_record.proto\x12\x14google.monitoring.v3\x1a\x1fgoogle/protobuf/timestamp.proto\"l\n" +
	"\x0eMutationRecord\x12;\n" +
	"\vmutate_time\x18\x01 \x01(\v2\x1a.google.protobuf.TimestampR\n" +
	"mutateTime\x12\x1d\n" +
	"\n" +
	"mutated_by\x18\x02 \x01(\tR\tmutatedByB\xce\x01\n" +
	"\x18com.google.monitoring.v3B\x13MutationRecordProtoP\x01ZAcloud.google.com/go/monitoring/apiv3/v2/monitoringpb;monitoringpb\xaa\x02\x1aGoogle.Cloud.Monitoring.V3\xca\x02\x1aGoogle\\Cloud\\Monitoring\\V3\xea\x02\x1dGoogle::Cloud::Monitoring::V3b\x06proto3"

var (
	file_google_monitoring_v3_mutation_record_proto_rawDescOnce sync.Once
	file_google_monitoring_v3_mutation_record_proto_rawDescData []byte
)

func file_google_monitoring_v3_mutation_record_proto_rawDescGZIP() []byte {
	file_google_monitoring_v3_mutation_record_proto_rawDescOnce.Do(func() {
		file_google_monitoring_v3_mutation_record_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_google_monitoring_v3_mutation_record_proto_rawDesc), len(file_google_monitoring_v3_mutation_record_proto_rawDesc)))
	})
	return file_google_monitoring_v3_mutation_record_proto_rawDescData
}

var file_google_monitoring_v3_mutation_record_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_google_monitoring_v3_mutation_record_proto_goTypes = []any{
	(*MutationRecord)(nil),        // 0: google.monitoring.v3.MutationRecord
	(*timestamppb.Timestamp)(nil), // 1: google.protobuf.Timestamp
}
var file_google_monitoring_v3_mutation_record_proto_depIdxs = []int32{
	1, // 0: google.monitoring.v3.MutationRecord.mutate_time:type_name -> google.protobuf.Timestamp
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_google_monitoring_v3_mutation_record_proto_init() }
func file_google_monitoring_v3_mutation_record_proto_init() {
	if File_google_monitoring_v3_mutation_record_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_monitoring_v3_mutation_record_proto_rawDesc), len(file_google_monitoring_v3_mutation_record_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_google_monitoring_v3_mutation_record_proto_goTypes,
		DependencyIndexes: file_google_monitoring_v3_mutation_record_proto_depIdxs,
		MessageInfos:      file_google_monitoring_v3_mutation_record_proto_msgTypes,
	}.Build()
	File_google_monitoring_v3_mutation_record_proto = out.File
	file_google_monitoring_v3_mutation_record_proto_goTypes = nil
	file_google_monitoring_v3_mutation_record_proto_depIdxs = nil
}
