// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/annotations/resource.proto

package annotations

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	descriptorpb "google.golang.org/protobuf/types/descriptorpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type ResourceAnnotation struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Annotation for xDS services that indicates the fully-qualified Protobuf type for the resource
	// type.
	Type          string `protobuf:"bytes,1,opt,name=type,proto3" json:"type,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ResourceAnnotation) Reset() {
	*x = ResourceAnnotation{}
	mi := &file_envoy_annotations_resource_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ResourceAnnotation) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ResourceAnnotation) ProtoMessage() {}

func (x *ResourceAnnotation) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_annotations_resource_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ResourceAnnotation.ProtoReflect.Descriptor instead.
func (*ResourceAnnotation) Descriptor() ([]byte, []int) {
	return file_envoy_annotations_resource_proto_rawDescGZIP(), []int{0}
}

func (x *ResourceAnnotation) GetType() string {
	if x != nil {
		return x.Type
	}
	return ""
}

var file_envoy_annotations_resource_proto_extTypes = []protoimpl.ExtensionInfo{
	{
		ExtendedType:  (*descriptorpb.ServiceOptions)(nil),
		ExtensionType: (*ResourceAnnotation)(nil),
		Field:         265073217,
		Name:          "envoy.annotations.resource",
		Tag:           "bytes,265073217,opt,name=resource",
		Filename:      "envoy/annotations/resource.proto",
	},
}

// Extension fields to descriptorpb.ServiceOptions.
var (
	// optional envoy.annotations.ResourceAnnotation resource = 265073217;
	E_Resource = &file_envoy_annotations_resource_proto_extTypes[0]
)

var File_envoy_annotations_resource_proto protoreflect.FileDescriptor

const file_envoy_annotations_resource_proto_rawDesc = "" +
	"\n" +
	" envoy/annotations/resource.proto\x12\x11envoy.annotations\x1a google/protobuf/descriptor.proto\"(\n" +
	"\x12ResourceAnnotation\x12\x12\n" +
	"\x04type\x18\x01 \x01(\tR\x04type:e\n" +
	"\bresource\x12\x1f.google.protobuf.ServiceOptions\x18\xc1\xe4\xb2~ \x01(\v2%.envoy.annotations.ResourceAnnotationR\bresourceB:Z8github.com/envoyproxy/go-control-plane/envoy/annotationsb\x06proto3"

var (
	file_envoy_annotations_resource_proto_rawDescOnce sync.Once
	file_envoy_annotations_resource_proto_rawDescData []byte
)

func file_envoy_annotations_resource_proto_rawDescGZIP() []byte {
	file_envoy_annotations_resource_proto_rawDescOnce.Do(func() {
		file_envoy_annotations_resource_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_annotations_resource_proto_rawDesc), len(file_envoy_annotations_resource_proto_rawDesc)))
	})
	return file_envoy_annotations_resource_proto_rawDescData
}

var file_envoy_annotations_resource_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_annotations_resource_proto_goTypes = []any{
	(*ResourceAnnotation)(nil),          // 0: envoy.annotations.ResourceAnnotation
	(*descriptorpb.ServiceOptions)(nil), // 1: google.protobuf.ServiceOptions
}
var file_envoy_annotations_resource_proto_depIdxs = []int32{
	1, // 0: envoy.annotations.resource:extendee -> google.protobuf.ServiceOptions
	0, // 1: envoy.annotations.resource:type_name -> envoy.annotations.ResourceAnnotation
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	1, // [1:2] is the sub-list for extension type_name
	0, // [0:1] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_envoy_annotations_resource_proto_init() }
func file_envoy_annotations_resource_proto_init() {
	if File_envoy_annotations_resource_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_annotations_resource_proto_rawDesc), len(file_envoy_annotations_resource_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 1,
			NumServices:   0,
		},
		GoTypes:           file_envoy_annotations_resource_proto_goTypes,
		DependencyIndexes: file_envoy_annotations_resource_proto_depIdxs,
		MessageInfos:      file_envoy_annotations_resource_proto_msgTypes,
		ExtensionInfos:    file_envoy_annotations_resource_proto_extTypes,
	}.Build()
	File_envoy_annotations_resource_proto = out.File
	file_envoy_annotations_resource_proto_goTypes = nil
	file_envoy_annotations_resource_proto_depIdxs = nil
}
