// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/trace/v3/lightstep.proto

package tracev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/go-control-plane/envoy/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Available propagation modes
type LightstepConfig_PropagationMode int32

const (
	// Propagate trace context in the single header x-ot-span-context.
	LightstepConfig_ENVOY LightstepConfig_PropagationMode = 0
	// Propagate trace context using LightStep's native format.
	LightstepConfig_LIGHTSTEP LightstepConfig_PropagationMode = 1
	// Propagate trace context using the b3 format.
	LightstepConfig_B3 LightstepConfig_PropagationMode = 2
	// Propagation trace context using the w3 trace-context standard.
	LightstepConfig_TRACE_CONTEXT LightstepConfig_PropagationMode = 3
)

// Enum value maps for LightstepConfig_PropagationMode.
var (
	LightstepConfig_PropagationMode_name = map[int32]string{
		0: "ENVOY",
		1: "LIGHTSTEP",
		2: "B3",
		3: "TRACE_CONTEXT",
	}
	LightstepConfig_PropagationMode_value = map[string]int32{
		"ENVOY":         0,
		"LIGHTSTEP":     1,
		"B3":            2,
		"TRACE_CONTEXT": 3,
	}
)

func (x LightstepConfig_PropagationMode) Enum() *LightstepConfig_PropagationMode {
	p := new(LightstepConfig_PropagationMode)
	*p = x
	return p
}

func (x LightstepConfig_PropagationMode) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (LightstepConfig_PropagationMode) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_config_trace_v3_lightstep_proto_enumTypes[0].Descriptor()
}

func (LightstepConfig_PropagationMode) Type() protoreflect.EnumType {
	return &file_envoy_config_trace_v3_lightstep_proto_enumTypes[0]
}

func (x LightstepConfig_PropagationMode) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use LightstepConfig_PropagationMode.Descriptor instead.
func (LightstepConfig_PropagationMode) EnumDescriptor() ([]byte, []int) {
	return file_envoy_config_trace_v3_lightstep_proto_rawDescGZIP(), []int{0, 0}
}

// Configuration for the LightStep tracer.
// [#extension: envoy.tracers.lightstep]
// [#not-implemented-hide:]
type LightstepConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The cluster manager cluster that hosts the LightStep collectors.
	CollectorCluster string `protobuf:"bytes,1,opt,name=collector_cluster,json=collectorCluster,proto3" json:"collector_cluster,omitempty"`
	// File containing the access token to the `LightStep
	// <https://lightstep.com/>`_ API.
	//
	// Deprecated: Marked as deprecated in envoy/config/trace/v3/lightstep.proto.
	AccessTokenFile string `protobuf:"bytes,2,opt,name=access_token_file,json=accessTokenFile,proto3" json:"access_token_file,omitempty"`
	// Access token to the `LightStep <https://lightstep.com/>`_ API.
	AccessToken *v3.DataSource `protobuf:"bytes,4,opt,name=access_token,json=accessToken,proto3" json:"access_token,omitempty"`
	// Propagation modes to use by LightStep's tracer.
	PropagationModes []LightstepConfig_PropagationMode `protobuf:"varint,3,rep,packed,name=propagation_modes,json=propagationModes,proto3,enum=envoy.config.trace.v3.LightstepConfig_PropagationMode" json:"propagation_modes,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *LightstepConfig) Reset() {
	*x = LightstepConfig{}
	mi := &file_envoy_config_trace_v3_lightstep_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *LightstepConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*LightstepConfig) ProtoMessage() {}

func (x *LightstepConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_trace_v3_lightstep_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use LightstepConfig.ProtoReflect.Descriptor instead.
func (*LightstepConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_trace_v3_lightstep_proto_rawDescGZIP(), []int{0}
}

func (x *LightstepConfig) GetCollectorCluster() string {
	if x != nil {
		return x.CollectorCluster
	}
	return ""
}

// Deprecated: Marked as deprecated in envoy/config/trace/v3/lightstep.proto.
func (x *LightstepConfig) GetAccessTokenFile() string {
	if x != nil {
		return x.AccessTokenFile
	}
	return ""
}

func (x *LightstepConfig) GetAccessToken() *v3.DataSource {
	if x != nil {
		return x.AccessToken
	}
	return nil
}

func (x *LightstepConfig) GetPropagationModes() []LightstepConfig_PropagationMode {
	if x != nil {
		return x.PropagationModes
	}
	return nil
}

var File_envoy_config_trace_v3_lightstep_proto protoreflect.FileDescriptor

const file_envoy_config_trace_v3_lightstep_proto_rawDesc = "" +
	"\n" +
	"%envoy/config/trace/v3/lightstep.proto\x12\x15envoy.config.trace.v3\x1a\x1fenvoy/config/core/v3/base.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xaf\x03\n" +
	"\x0fLightstepConfig\x124\n" +
	"\x11collector_cluster\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x10collectorCluster\x127\n" +
	"\x11access_token_file\x18\x02 \x01(\tB\v\x92ǆ\xd8\x04\x033.0\x18\x01R\x0faccessTokenFile\x12C\n" +
	"\faccess_token\x18\x04 \x01(\v2 .envoy.config.core.v3.DataSourceR\vaccessToken\x12r\n" +
	"\x11propagation_modes\x18\x03 \x03(\x0e26.envoy.config.trace.v3.LightstepConfig.PropagationModeB\r\xfaB\n" +
	"\x92\x01\a\"\x05\x82\x01\x02\x10\x01R\x10propagationModes\"F\n" +
	"\x0fPropagationMode\x12\t\n" +
	"\x05ENVOY\x10\x00\x12\r\n" +
	"\tLIGHTSTEP\x10\x01\x12\x06\n" +
	"\x02B3\x10\x02\x12\x11\n" +
	"\rTRACE_CONTEXT\x10\x03:,\x9aň\x1e'\n" +
	"%envoy.config.trace.v2.LightstepConfigB\xb7\x01\xf2\x98\xfe\x8f\x05,\x12*envoy.extensions.tracers.lightstep.v4alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"#io.envoyproxy.envoy.config.trace.v3B\x0eLightstepProtoP\x01ZDgithub.com/envoyproxy/go-control-plane/envoy/config/trace/v3;tracev3b\x06proto3"

var (
	file_envoy_config_trace_v3_lightstep_proto_rawDescOnce sync.Once
	file_envoy_config_trace_v3_lightstep_proto_rawDescData []byte
)

func file_envoy_config_trace_v3_lightstep_proto_rawDescGZIP() []byte {
	file_envoy_config_trace_v3_lightstep_proto_rawDescOnce.Do(func() {
		file_envoy_config_trace_v3_lightstep_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_trace_v3_lightstep_proto_rawDesc), len(file_envoy_config_trace_v3_lightstep_proto_rawDesc)))
	})
	return file_envoy_config_trace_v3_lightstep_proto_rawDescData
}

var file_envoy_config_trace_v3_lightstep_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_envoy_config_trace_v3_lightstep_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_config_trace_v3_lightstep_proto_goTypes = []any{
	(LightstepConfig_PropagationMode)(0), // 0: envoy.config.trace.v3.LightstepConfig.PropagationMode
	(*LightstepConfig)(nil),              // 1: envoy.config.trace.v3.LightstepConfig
	(*v3.DataSource)(nil),                // 2: envoy.config.core.v3.DataSource
}
var file_envoy_config_trace_v3_lightstep_proto_depIdxs = []int32{
	2, // 0: envoy.config.trace.v3.LightstepConfig.access_token:type_name -> envoy.config.core.v3.DataSource
	0, // 1: envoy.config.trace.v3.LightstepConfig.propagation_modes:type_name -> envoy.config.trace.v3.LightstepConfig.PropagationMode
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_config_trace_v3_lightstep_proto_init() }
func file_envoy_config_trace_v3_lightstep_proto_init() {
	if File_envoy_config_trace_v3_lightstep_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_trace_v3_lightstep_proto_rawDesc), len(file_envoy_config_trace_v3_lightstep_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_trace_v3_lightstep_proto_goTypes,
		DependencyIndexes: file_envoy_config_trace_v3_lightstep_proto_depIdxs,
		EnumInfos:         file_envoy_config_trace_v3_lightstep_proto_enumTypes,
		MessageInfos:      file_envoy_config_trace_v3_lightstep_proto_msgTypes,
	}.Build()
	File_envoy_config_trace_v3_lightstep_proto = out.File
	file_envoy_config_trace_v3_lightstep_proto_goTypes = nil
	file_envoy_config_trace_v3_lightstep_proto_depIdxs = nil
}
