// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/type/matcher/v3/address.proto

package matcherv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/cncf/xds/go/xds/core/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Match an IP against a repeated CIDR range. This matcher is intended to be
// used in other matchers, for example in the filter state matcher to match a
// filter state object as an IP.
type AddressMatcher struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Ranges        []*v3.CidrRange        `protobuf:"bytes,1,rep,name=ranges,proto3" json:"ranges,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AddressMatcher) Reset() {
	*x = AddressMatcher{}
	mi := &file_envoy_type_matcher_v3_address_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AddressMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AddressMatcher) ProtoMessage() {}

func (x *AddressMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_type_matcher_v3_address_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AddressMatcher.ProtoReflect.Descriptor instead.
func (*AddressMatcher) Descriptor() ([]byte, []int) {
	return file_envoy_type_matcher_v3_address_proto_rawDescGZIP(), []int{0}
}

func (x *AddressMatcher) GetRanges() []*v3.CidrRange {
	if x != nil {
		return x.Ranges
	}
	return nil
}

var File_envoy_type_matcher_v3_address_proto protoreflect.FileDescriptor

const file_envoy_type_matcher_v3_address_proto_rawDesc = "" +
	"\n" +
	"#envoy/type/matcher/v3/address.proto\x12\x15envoy.type.matcher.v3\x1a\x16xds/core/v3/cidr.proto\x1a\x1dudpa/annotations/status.proto\"@\n" +
	"\x0eAddressMatcher\x12.\n" +
	"\x06ranges\x18\x01 \x03(\v2\x16.xds.core.v3.CidrRangeR\x06rangesB\x85\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"#io.envoyproxy.envoy.type.matcher.v3B\fAddressProtoP\x01ZFgithub.com/envoyproxy/go-control-plane/envoy/type/matcher/v3;matcherv3b\x06proto3"

var (
	file_envoy_type_matcher_v3_address_proto_rawDescOnce sync.Once
	file_envoy_type_matcher_v3_address_proto_rawDescData []byte
)

func file_envoy_type_matcher_v3_address_proto_rawDescGZIP() []byte {
	file_envoy_type_matcher_v3_address_proto_rawDescOnce.Do(func() {
		file_envoy_type_matcher_v3_address_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_type_matcher_v3_address_proto_rawDesc), len(file_envoy_type_matcher_v3_address_proto_rawDesc)))
	})
	return file_envoy_type_matcher_v3_address_proto_rawDescData
}

var file_envoy_type_matcher_v3_address_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_type_matcher_v3_address_proto_goTypes = []any{
	(*AddressMatcher)(nil), // 0: envoy.type.matcher.v3.AddressMatcher
	(*v3.CidrRange)(nil),   // 1: xds.core.v3.CidrRange
}
var file_envoy_type_matcher_v3_address_proto_depIdxs = []int32{
	1, // 0: envoy.type.matcher.v3.AddressMatcher.ranges:type_name -> xds.core.v3.CidrRange
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_type_matcher_v3_address_proto_init() }
func file_envoy_type_matcher_v3_address_proto_init() {
	if File_envoy_type_matcher_v3_address_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_type_matcher_v3_address_proto_rawDesc), len(file_envoy_type_matcher_v3_address_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_type_matcher_v3_address_proto_goTypes,
		DependencyIndexes: file_envoy_type_matcher_v3_address_proto_depIdxs,
		MessageInfos:      file_envoy_type_matcher_v3_address_proto_msgTypes,
	}.Build()
	File_envoy_type_matcher_v3_address_proto = out.File
	file_envoy_type_matcher_v3_address_proto_goTypes = nil
	file_envoy_type_matcher_v3_address_proto_depIdxs = nil
}
