// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/trace/v3/datadog.proto

package tracev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the Remote Configuration feature.
type DatadogRemoteConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Frequency at which new configuration updates are queried.
	// If no value is provided, the default value is delegated to the Datadog tracing library.
	PollingInterval *durationpb.Duration `protobuf:"bytes,1,opt,name=polling_interval,json=pollingInterval,proto3" json:"polling_interval,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *DatadogRemoteConfig) Reset() {
	*x = DatadogRemoteConfig{}
	mi := &file_envoy_config_trace_v3_datadog_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DatadogRemoteConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DatadogRemoteConfig) ProtoMessage() {}

func (x *DatadogRemoteConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_trace_v3_datadog_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DatadogRemoteConfig.ProtoReflect.Descriptor instead.
func (*DatadogRemoteConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_trace_v3_datadog_proto_rawDescGZIP(), []int{0}
}

func (x *DatadogRemoteConfig) GetPollingInterval() *durationpb.Duration {
	if x != nil {
		return x.PollingInterval
	}
	return nil
}

// Configuration for the Datadog tracer.
// [#extension: envoy.tracers.datadog]
type DatadogConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The cluster to use for submitting traces to the Datadog agent.
	CollectorCluster string `protobuf:"bytes,1,opt,name=collector_cluster,json=collectorCluster,proto3" json:"collector_cluster,omitempty"`
	// The name used for the service when traces are generated by envoy.
	ServiceName string `protobuf:"bytes,2,opt,name=service_name,json=serviceName,proto3" json:"service_name,omitempty"`
	// Optional hostname to use when sending spans to the collector_cluster. Useful for collectors
	// that require a specific hostname. Defaults to :ref:`collector_cluster <envoy_v3_api_field_config.trace.v3.DatadogConfig.collector_cluster>` above.
	CollectorHostname string `protobuf:"bytes,3,opt,name=collector_hostname,json=collectorHostname,proto3" json:"collector_hostname,omitempty"`
	// Enables and configures remote configuration.
	// Remote Configuration allows to configure the tracer from Datadog's user interface.
	// This feature can drastically increase the number of connections to the Datadog Agent.
	// Each tracer regularly polls for configuration updates, and the number of tracers is the product
	// of the number of listeners and worker threads.
	RemoteConfig  *DatadogRemoteConfig `protobuf:"bytes,4,opt,name=remote_config,json=remoteConfig,proto3" json:"remote_config,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DatadogConfig) Reset() {
	*x = DatadogConfig{}
	mi := &file_envoy_config_trace_v3_datadog_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DatadogConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DatadogConfig) ProtoMessage() {}

func (x *DatadogConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_trace_v3_datadog_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DatadogConfig.ProtoReflect.Descriptor instead.
func (*DatadogConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_trace_v3_datadog_proto_rawDescGZIP(), []int{1}
}

func (x *DatadogConfig) GetCollectorCluster() string {
	if x != nil {
		return x.CollectorCluster
	}
	return ""
}

func (x *DatadogConfig) GetServiceName() string {
	if x != nil {
		return x.ServiceName
	}
	return ""
}

func (x *DatadogConfig) GetCollectorHostname() string {
	if x != nil {
		return x.CollectorHostname
	}
	return ""
}

func (x *DatadogConfig) GetRemoteConfig() *DatadogRemoteConfig {
	if x != nil {
		return x.RemoteConfig
	}
	return nil
}

var File_envoy_config_trace_v3_datadog_proto protoreflect.FileDescriptor

const file_envoy_config_trace_v3_datadog_proto_rawDesc = "" +
	"\n" +
	"#envoy/config/trace/v3/datadog.proto\x12\x15envoy.config.trace.v3\x1a\x1egoogle/protobuf/duration.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"[\n" +
	"\x13DatadogRemoteConfig\x12D\n" +
	"\x10polling_interval\x18\x01 \x01(\v2\x19.google.protobuf.DurationR\x0fpollingInterval\"\x9d\x02\n" +
	"\rDatadogConfig\x124\n" +
	"\x11collector_cluster\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x10collectorCluster\x12*\n" +
	"\fservice_name\x18\x02 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\vserviceName\x12-\n" +
	"\x12collector_hostname\x18\x03 \x01(\tR\x11collectorHostname\x12O\n" +
	"\rremote_config\x18\x04 \x01(\v2*.envoy.config.trace.v3.DatadogRemoteConfigR\fremoteConfig:*\x9aň\x1e%\n" +
	"#envoy.config.trace.v2.DatadogConfigB\xb3\x01\xf2\x98\xfe\x8f\x05*\x12(envoy.extensions.tracers.datadog.v4alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"#io.envoyproxy.envoy.config.trace.v3B\fDatadogProtoP\x01ZDgithub.com/envoyproxy/go-control-plane/envoy/config/trace/v3;tracev3b\x06proto3"

var (
	file_envoy_config_trace_v3_datadog_proto_rawDescOnce sync.Once
	file_envoy_config_trace_v3_datadog_proto_rawDescData []byte
)

func file_envoy_config_trace_v3_datadog_proto_rawDescGZIP() []byte {
	file_envoy_config_trace_v3_datadog_proto_rawDescOnce.Do(func() {
		file_envoy_config_trace_v3_datadog_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_trace_v3_datadog_proto_rawDesc), len(file_envoy_config_trace_v3_datadog_proto_rawDesc)))
	})
	return file_envoy_config_trace_v3_datadog_proto_rawDescData
}

var file_envoy_config_trace_v3_datadog_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_config_trace_v3_datadog_proto_goTypes = []any{
	(*DatadogRemoteConfig)(nil), // 0: envoy.config.trace.v3.DatadogRemoteConfig
	(*DatadogConfig)(nil),       // 1: envoy.config.trace.v3.DatadogConfig
	(*durationpb.Duration)(nil), // 2: google.protobuf.Duration
}
var file_envoy_config_trace_v3_datadog_proto_depIdxs = []int32{
	2, // 0: envoy.config.trace.v3.DatadogRemoteConfig.polling_interval:type_name -> google.protobuf.Duration
	0, // 1: envoy.config.trace.v3.DatadogConfig.remote_config:type_name -> envoy.config.trace.v3.DatadogRemoteConfig
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_config_trace_v3_datadog_proto_init() }
func file_envoy_config_trace_v3_datadog_proto_init() {
	if File_envoy_config_trace_v3_datadog_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_trace_v3_datadog_proto_rawDesc), len(file_envoy_config_trace_v3_datadog_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_trace_v3_datadog_proto_goTypes,
		DependencyIndexes: file_envoy_config_trace_v3_datadog_proto_depIdxs,
		MessageInfos:      file_envoy_config_trace_v3_datadog_proto_msgTypes,
	}.Build()
	File_envoy_config_trace_v3_datadog_proto = out.File
	file_envoy_config_trace_v3_datadog_proto_goTypes = nil
	file_envoy_config_trace_v3_datadog_proto_depIdxs = nil
}
